@@ -1,7 +1,7 @@
 ;;;
 ;;; ndbm - ndbm interface
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -232,7 +232,7 @@
        (Scm_SysError "couldn't open ndbm file %S" name))
      (result (SCM_OBJ z))))
 
- (define-cproc ndbm-close (ndbm::<ndbm-file>)
+ (define-cproc ndbm-close (ndbm::<ndbm-file>) ::<void>
    (when (-> ndbm dbf)
      (dbm_close (-> ndbm dbf))
      (set! (-> ndbm dbf) NULL)))
