@@ -58,17 +58,15 @@
 (define-constant *md5-unit-len* 4096)
 
 (define (md5-digest)
-  (let ((md5 (make <md5-context>))
-        (buf (make-u8vector *md5-unit-len*)))
-    (port-for-each
-     (lambda (x) (%md5-update md5 x))
-     (lambda ()
-       (let1 count (read-block! buf)
-         (if (eof-object? count)
-           count
-           (if (< count *md5-unit-len*)
-             (uvector-alias <u8vector> buf 0 count)
-             buf)))))
+  (let ([md5 (make <md5-context>)]
+        [buf (make-u8vector *md5-unit-len*)])
+    (generator-for-each
+     (^x (%md5-update md5 x))
+     (^[] (let1 count (read-block! buf)
+            (cond [(eof-object? count) count]
+                  [(< count *md5-unit-len*)
+                   (uvector-alias <u8vector> buf 0 count)]
+                  [else buf]))))
     (%md5-final md5)))
 
 (define (md5-digest-string string)
