@@ -2,7 +2,7 @@
 ;;; sha - SHA-1/SHA-224/SHA-256/SHA-384/SHA-512 message-digest
 ;;;
 ;;;   Copyright (c) 2002-2003 Kimura Fuyuki, All rights reserved.
-;;;   Copyright (c) 2008-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2008-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -80,18 +80,19 @@
 ;;; Digest framework
 ;;;
 
-(define-macro (define-framework n)
-  (let ([meta   (string->symbol #`"<sha,|n|-meta>")]
-        [cls    (string->symbol #`"<sha,|n|>")]
-        [init   (string->symbol #`"%sha,|n|-init")]
-        [update (string->symbol #`"%sha,|n|-update")]
-        [final  (string->symbol #`"%sha,|n|-final")]
-        [digest (string->symbol #`"sha,|n|-digest")])
+(define-macro (define-framework n block-size)
+  (let ([meta   (string->symbol #"<sha~|n|-meta>")]
+        [cls    (string->symbol #"<sha~|n|>")]
+        [init   (string->symbol #"%sha~|n|-init")]
+        [update (string->symbol #"%sha~|n|-update")]
+        [final  (string->symbol #"%sha~|n|-final")]
+        [digest (string->symbol #"sha~|n|-digest")])
     `(begin
        (define-class ,meta (<message-digest-algorithm-meta>) ())
        (define-class ,cls (<message-digest-algorithm>)
          (context)
-         :metaclass ,meta)
+         :metaclass ,meta
+         :hmac-block-size ,block-size)
        (define-method initialize ((self ,cls) initargs)
          (next-method)
          (let1 ctx (make <sha-context>)
@@ -104,11 +105,11 @@
        (define-method digest ((class ,meta))
          (,digest)))))
 
-(define-framework 1)
-(define-framework 224)
-(define-framework 256)
-(define-framework 384)
-(define-framework 512)
+(define-framework 1    64)
+(define-framework 224  64)
+(define-framework 256  64)
+(define-framework 384  128)
+(define-framework 512  128)
 
 ;;;
 ;;; Low-level bindings
