@@ -1,3 +1,4 @@
+(use gauche.uvector)
 (use gauche.test)
 (test-start "records")
 
@@ -240,31 +241,75 @@
 ;;--------------------------------------------------------------------
 (test-section "pseudo record")
 
-(define-record-type (vec-rec1 (pseudo-rtd <vector>)) #t #f x y z)
-(let1 v #f
-  (test* "vector record ctor" '#(1 2 3)
-         (begin (set! v (make-vec-rec1 1 2 3)) v))
-  (test* "accessor" '(1 2 3)
-         (list (vec-rec1-x v) (vec-rec1-y v) (vec-rec1-z v)))
-  (test* "can't mutate" (test-error)
-         (set! (vec-rec1-x v) 10)))
+;; we make this a macro only because define-inline (generated by define-
+;; record-type) need to be in toplevel in the current version of Gauche.
+(define-macro (pseudo-record-test base-class maker)
+  (define p1  (string->symbol #"~|base-class|-pseudo-record1"))
+  (define p1? (string->symbol #"~|base-class|-pseudo-record1?"))
+  (define make-p1 (string->symbol #"make-~|p1|"))
+  (define p1-x (string->symbol #"~|p1|-x"))
+  (define p1-y (string->symbol #"~|p1|-y"))
+  (define p1-z (string->symbol #"~|p1|-z"))
+  (define p2 (string->symbol #"~|base-class|-pseudo-record2"))
+  (define make-p2 (string->symbol #"make-~|p2|"))
+  (define p2-x (string->symbol #"~|p2|-x"))
+  (define p2-y (string->symbol #"~|p2|-y"))
+  (define p2-z (string->symbol #"~|p2|-z"))
+  (define p2-x-set! (string->symbol #"~|p2|-x-set!"))
+  (define p2-y-set! (string->symbol #"~|p2|-y-set!"))
+  (define p2-z-set! (string->symbol #"~|p2|-z-set!"))
+  `(begin
+     (define-record-type (,p1 (pseudo-rtd ,base-class)) #t #t x y z)
+     (let1 v #f
+       (test* (format "~a record ctor" (class-name ,base-class))
+              (,maker 1 2 3)
+              (begin (set! v (,make-p1 1 2 3)) v))
+       (test* (format "~a record predicate" (class-name ,base-class))
+              '(#t #f #t)
+              (list (,p1? (,maker 1 2 3))
+                    (,p1? (,maker 1 2))
+                    (,p1? (,maker 1 2 3 4))))
+       (test* (format "~a record accessor" (class-name ,base-class))
+              (coerce-to <list> v)
+              (list (,p1-x v) (,p1-y v) (,p1-z v)))
+       (test* (format "~a can't mutate" (class-name ,base-class))
+              (test-error)
+              (set! (,p1-x v) 10)))
+     (define-record-type (,p2 (pseudo-rtd ,base-class)) #t #f (x) (y) (z))
+     (let1 v #f
+       (test* (format "~a record ctor" (class-name ,base-class))
+              (,maker 1 2 3)
+              (begin (set! v (,make-p2 1 2 3)) v))
+       (test* (format "~a record accessor" (class-name ,base-class))
+              (coerce-to <list> v)
+              (list (,p2-x v) (,p2-y v) (,p2-z v)))
+       (test* (format "~a mutator" (class-name ,base-class))
+              (,maker 10 20 30)
+              (begin (,p2-x-set! v 10)
+                     (,p2-y-set! v 20)
+                     (,p2-z-set! v 30)
+                     v))
+       (test* (format "~a record srfi-17 mutator" (class-name ,base-class))
+              (coerce-to <list> (,maker 15 25 35))
+              (begin (set! (,p2-x v) 15)
+                     (set! (,p2-y v) 25)
+                     (set! (,p2-z v) 35)
+                     (list (,p2-x v) (,p2-y v) (,p2-z v))))
+       )))
+  
+(pseudo-record-test <list> list)
+(pseudo-record-test <vector> vector)
+(pseudo-record-test <u8vector> u8vector)
+(pseudo-record-test <s8vector> s8vector)
+(pseudo-record-test <u16vector> u16vector)
+(pseudo-record-test <s16vector> s16vector)
+(pseudo-record-test <u32vector> u32vector)
+(pseudo-record-test <s32vector> s32vector)
+(pseudo-record-test <u64vector> u64vector)
+(pseudo-record-test <s64vector> s64vector)
+(pseudo-record-test <f16vector> f16vector)
+(pseudo-record-test <f32vector> f32vector)
+(pseudo-record-test <f64vector> f64vector)
 
-(define-record-type (vec-rec2 (pseudo-rtd <vector>)) #t #f (x) (y) (z))
-(let1 v #f
-  (test* "vector record ctor" '#(1 2 3)
-         (begin (set! v (make-vec-rec2 1 2 3)) v))
-  (test* "accessor" '(1 2 3)
-         (list (vec-rec2-x v) (vec-rec2-y v) (vec-rec2-z v)))
-  (test* "mutator"  '(10 20 30)
-         (begin (vec-rec2-x-set! v 10)
-                (vec-rec2-y-set! v 20)
-                (vec-rec2-z-set! v 30)
-                (list (vec-rec2-x v) (vec-rec2-y v) (vec-rec2-z v))))
-  (test* "srfi-17 mutator" '(100 200 300)
-         (begin (set! (vec-rec2-x v) 100)
-                (set! (vec-rec2-y v) 200)
-                (set! (vec-rec2-z v) 300)
-                (list (vec-rec2-x v) (vec-rec2-y v) (vec-rec2-z v))))
-  )
 
 (test-end)
