@@ -1,7 +1,7 @@
 ;;;
 ;;; netaux.scm - network interface
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -212,11 +212,11 @@
 ;; accessor methods
 
 (define-method sockaddr-name ((addr <sockaddr-in>))
-  #`",(inet-address->string (sockaddr-addr addr) AF_INET):,(sockaddr-port addr)")
+  #"~(inet-address->string (sockaddr-addr addr) AF_INET):~(sockaddr-port addr)")
 
 ;; NB: this should be conditionally defined by cond-expand at compile-time,
 ;; instead of load-time dispatch.  We need to clean up cond feature management
 ;; more to do so.
 (if ipv6-capable
   (define-method sockaddr-name ((addr <sockaddr-in6>))
-    #`"[,(inet-address->string (sockaddr-addr addr) AF_INET6)]:,(sockaddr-port addr)"))
+    #"[~(inet-address->string (sockaddr-addr addr) AF_INET6)]:~(sockaddr-port addr)"))
