@@ -1,7 +1,7 @@
 /*
  * ctrie.c - Compact Trie
  *
- *   Copyright (c) 2009-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2009-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -91,14 +91,13 @@ static Node *node_insert(Node *orig, u_long ind, void *entry, int leafp)
 {
     int size = NODE_NCHILDREN(orig);
     int insertpoint = Scm__CountBitsBelow(orig->emap, ind);
-    int i;
 
     if (size&(NODE_SIZE_INCR-1)) {
         /* we have one more room */
         NODE_ARC_SET(orig, ind);
         if (leafp) NODE_LEAF_SET(orig, ind);
         if (insertpoint < size) {
-            for (i=size-1; i>=insertpoint; i--) {
+            for (int i=size-1; i>=insertpoint; i--) {
                 orig->entries[i+1] = orig->entries[i];
             }
         }
@@ -111,7 +110,8 @@ static Node *node_insert(Node *orig, u_long ind, void *entry, int leafp)
         newn->lmap = orig->lmap;
         NODE_ARC_SET(newn, ind);
         if (leafp) NODE_LEAF_SET(newn, ind);
-        for (i=0; i<insertpoint; i++) newn->entries[i] = orig->entries[i];
+        int i = 0;
+        for (; i<insertpoint; i++) newn->entries[i] = orig->entries[i];
         newn->entries[insertpoint] = entry;
         for (; i<size; i++) newn->entries[i+1] = orig->entries[i];
         return newn;
@@ -123,12 +123,13 @@ static int node_delete(Node *orig, u_long ind)
 {
     int size = NODE_NCHILDREN(orig);
     int deletepoint = Scm__CountBitsBelow(orig->emap, ind);
-    int i;
 
     /* TODO: shrink node */
     NODE_ARC_RESET(orig, ind);
     NODE_LEAF_RESET(orig, ind);
-    for (i=deletepoint; i<size-1; i++) orig->entries[i] = orig->entries[i+1];
+    for (int i=deletepoint; i<size-1; i++) {
+      orig->entries[i] = orig->entries[i+1];
+    }
     return size-1;
 }
 
@@ -175,11 +176,11 @@ static Node *add_rec(CompactTrie *ct, Node *n, u_long key, int level,
                      Leaf **result, Leaf *(*creator)(void*), void *data)
 
 {
-    Leaf *l;
     u_long ind = KEY2INDEX(key, level);
 
     if (!NODE_HAS_ARC(n, ind)) {
-        *result = l = new_leaf(key, creator, data);
+        Leaf *l = new_leaf(key, creator, data);
+        *result = l;
         ct->numEntries++;
         return node_insert(n, ind, (void*)l, TRUE);
     }
@@ -193,10 +194,10 @@ static Node *add_rec(CompactTrie *ct, Node *n, u_long key, int level,
     else {
         u_long off = NODE_INDEX2OFF(n, ind);
         Leaf *l0 = (Leaf*)NODE_ENTRY(n, off);
-        u_long k0 = LEAF_KEY(l0), i0;
+        u_long k0 = LEAF_KEY(l0);
 
         if (key == k0) { *result = l0; return n; }
-        i0 = KEY2INDEX(LEAF_KEY(l0), level+1);
+        u_long i0 = KEY2INDEX(LEAF_KEY(l0), level+1);
         Node *m = make_node(NODE_SIZE_INCR);
         NODE_ARC_SET(m, i0);
         NODE_LEAF_SET(m, i0);
@@ -289,17 +290,16 @@ static void clear_rec(CompactTrie *ct, Node *n,
                       void (*clearer)(Leaf*, void*),
                       void *data)
 {
-    int i, off;
     int size = Scm__CountBitsInWord(n->emap);
     char is_leaf[MAX_NODE_SIZE];
 
-    for (i=0, off=0; i<MAX_NODE_SIZE; i++) {
+    for (int i=0, off=0; i<MAX_NODE_SIZE; i++) {
         if (NODE_HAS_ARC(n, i)) {
             if (NODE_ARC_IS_LEAF(n, i)) is_leaf[off++] = TRUE;
             else is_leaf[off++] = FALSE;
         }
     }
-    for (i=0; i<size; i++) {
+    for (int i=0; i<size; i++) {
         if (is_leaf[i]) clearer((Leaf*)NODE_ENTRY(n, i), data);
         else clear_rec(ct, (Node*)NODE_ENTRY(n, i), clearer, data);
         NODE_ENTRY(n, i) = NULL;
@@ -322,9 +322,9 @@ void CompactTrieClear(CompactTrie *ct,
  */
 static Leaf *next_rec(Node *n, u_long key, int level, int over)
 {
-    u_int i, ind = over? 0 : KEY2INDEX(key, level);
+    u_int ind = over? 0 : KEY2INDEX(key, level);
 
-    for (i = ind; i < MAX_NODE_SIZE; i++) {
+    for (u_int i = ind; i < MAX_NODE_SIZE; i++) {
         if (!NODE_HAS_ARC(n, i)) continue;
         if (NODE_ARC_IS_LEAF(n, i)) {
             if (!over && i == ind) continue;
@@ -370,9 +370,8 @@ Leaf *CompactTrieFirstLeaf(CompactTrie *ct)
 
 static Leaf *last_rec(Node *n)
 {
-    int maxkey;
     if (n->emap == 0) return NULL; /* for safety in MT situation */
-    maxkey = Scm__HighestBitNumber(n->emap);
+    int maxkey = Scm__HighestBitNumber(n->emap);
     if (NODE_ARC_IS_LEAF(n, maxkey)) {
         return (Leaf*)NODE_ENTRY(n, NODE_INDEX2OFF(n, maxkey));
     } else {
@@ -394,12 +393,11 @@ Leaf *CompactTrieLastLeaf(CompactTrie *ct)
  */
 static Node *copy_rec(const Node *s, Leaf *(*copy)(Leaf*, void*), void *data)
 {
-    int i, off;
     int size = Scm__CountBitsInWord(s->emap);
     Node *d = make_node(size);
     d->emap = s->emap;
     d->lmap = s->lmap;
-    for (i=0, off=0; i<MAX_NODE_SIZE && off < size; i++) {
+    for (int i=0, off=0; i<MAX_NODE_SIZE && off < size; i++) {
         if (!NODE_HAS_ARC(s, i)) continue;
         if (NODE_ARC_IS_LEAF(s, i)) {
             NODE_ENTRY(d,off) = copy((Leaf*)NODE_ENTRY(s,off), data);
@@ -457,9 +455,8 @@ static char digit32(u_int n)
 
 static char *key_dump(u_long key, char *buf) /* buf must be of length 8 */
 {
-    int i;
     buf[BUF_SIZE-1] = '\0';
-    for (i=0; i<BUF_SIZE-1; i++) {
+    for (int i=0; i<BUF_SIZE-1; i++) {
         buf[BUF_SIZE-i-2] = digit32(key&TRIE_MASK);
         key >>= TRIE_SHIFT;
     }
@@ -479,10 +476,8 @@ static void leaf_dump(ScmPort *out, Leaf *self, int indent,
 static void node_dump(ScmPort *out, Node *n, int level,
                       void (*dumper)(ScmPort*, Leaf*, int, void*), void *data)
 {
-    int i;
-
     Scm_Printf(out, "NODE(%p)\n", n);
-    for (i=0; i<MAX_NODE_SIZE; i++) {
+    for (int i=0; i<MAX_NODE_SIZE; i++) {
         if (!NODE_HAS_ARC(n, i)) continue;
         Scm_Printf(out, " %*s%c:", level*2, "", digit32(i));
         if (NODE_ARC_IS_LEAF(n, i)) {
@@ -510,8 +505,9 @@ void CompactTrieDump(ScmPort *out, const CompactTrie *ct,
 static int check_rec(Node *n, int level,
                      void (*checker)(Leaf*, ScmObj), ScmObj obj)
 {
-    int direct_leaves = 0, total_leaves = 0, i, off;
-    for (i=0, off=0; i<MAX_NODE_SIZE; i++) {
+    int direct_leaves = 0, total_leaves = 0;
+    int off = 0;
+    for (int i=0; i<MAX_NODE_SIZE; i++) {
         if (NODE_HAS_ARC(n, i)) {
             if (NODE_ARC_IS_LEAF(n, i)) {
                 direct_leaves++;
