@@ -1,7 +1,7 @@
 /*
  * termios.h - termios interface
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -39,16 +39,23 @@
 #if !defined(GAUCHE_WINDOWS)
 
 #include <termios.h>
-#ifdef HAVE_PTY_H
+
+#if   defined(HAVE_PTY_H)
 #include <pty.h>
 #endif
-#ifdef HAVE_UTIL_H
+
+#if   defined(HAVE_UTIL_H)
 #include <util.h>
 #endif
-#ifdef HAVE_LIBUTIL_H
+
+/* libutil.h is superseded by bsd/libutil.h */
+#if   defined(HAVE_BSD_LIBUTIL_H)
+#include <bsd/libutil.h>
+#elif defined(HAVE_LIBUTIL_h)
 #include <libutil.h>
 #endif
-#ifdef HAVE_UNISTD_H
+
+#if   defined(HAVE_UNISTD_H)
 #include <unistd.h>
 #endif
 
