@@ -1,7 +1,7 @@
 ;;;
 ;;; text.unicode - Various Unicode-specific operations
 ;;;
-;;;   Copyright (c) 2011-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,6 +39,7 @@
   (use srfi-42)
   (export ucs4->utf8  utf8-length  utf8->ucs4
           ucs4->utf16 utf16-length utf16->ucs4
+          utf8->string string->utf8
 
           make-word-breaker
           make-word-reader
@@ -50,10 +51,14 @@
           string-upcase string-downcase string-titlecase string-foldcase
           codepoints-upcase codepoints-downcase codepoints-titlecase
           codepoints-foldcase
+
+          string-ci=? string-ci<? string-ci<=? string-ci>? string-ci>=?
           )
   )
 (select-module text.unicode)
 
+(autoload gauche.charconv ces-convert)
+
 ;; This module implements some higher-level Unicode operations.
 ;; Most of them do not depend on the internal encodings---some
 ;; operates on integer values of codepoints or encoded octets,
@@ -159,7 +164,7 @@
 
 ;; generate a state transition procedure of k-th octet of n-octet utf8 sequence
 (define-macro (decode-utf8-gen-1 strictness)
-  (define (name prefix) (string->symbol #`",|prefix|-,|strictness|"))
+  (define (name prefix) (string->symbol #"~|prefix|-~|strictness|"))
   `(define (,(name "u1") b val)
      (if (eof-object? b)
        (values b #f)
@@ -174,7 +179,7 @@
 
 (define-macro (decode-utf8-gen-n n limit strictness)
   (define (rec n k)
-    (let1 name (string->symbol #`"u,|n|-,|k|-,|strictness|")
+    (let1 name (string->symbol #"u~|n|-~|k|-~|strictness|")
       (if (= k (- n 1))
         ;; last byte
         `((define (,name b val)
@@ -193,7 +198,7 @@
                           [else         ,(out-of-range strictness)])))]
              [else ,(bad-octet strictness)])))
         ;; intermediate byte
-        (let1 next-name (string->symbol #`"u,|n|-,(+ k 1)-,|strictness|")
+        (let1 next-name (string->symbol #"u~|n|-~(+ k 1)-~|strictness|")
           `((define (,name b val)
               (cond [(eof-object? b) ,(end-input strictness)]
                     [(<= #x80 b #xbf) (values (+ (ash val 6) (logand b #x3f))
@@ -204,7 +209,7 @@
     (case strictness
       [(strict)     `(values 'bad #f)]
       [(permissive) `(values v #f)]
-      [(ignore)     `(values 0 ,(string->symbol #`"u1-,|strictness|"))]))
+      [(ignore)     `(values 0 ,(string->symbol #"u1-~|strictness|"))]))
   (define (bad-octet strictness)
     (case strictness
       [(strict permissive) `(values 'bad #f)]
@@ -350,6 +355,30 @@
   (set! utf16->ucs4 %utf16->ucs4)
   )
 
+;; R7RS procedures
+
+;; This module may be precompiled by different platforms, so we dispatch
+;; at runtime.
+;; At this moment, we don't worry much about performance when the native
+;; encoding isn't utf8.
+(define (utf8->string bvec :optional (start 0) (end -1))
+  (check-arg u8vector? bvec)
+  (if (eq? (gauche-character-encoding) 'utf-8)
+    (rlet1 s (u8vector->string bvec start end)
+      (when (string-incomplete? s)
+        (error "invalid utf-8 sequence in u8vector:" bvec)))
+    (ces-convert (u8vector->string bvec start end) 'utf-8)))
+
+(define (string->utf8 str :optional (start 0) (end -1))
+  (check-arg string? str)
+  (if (eq? (gauche-character-encoding) 'utf-8)
+    (string->u8vector str start end)
+    (let ([start (or start 0)]
+          [end (if (and (number? end) (>= end 0)) end (string-length str))])
+      (string->u8vector (ces-convert (substring str start end)
+                                     (gauche-character-encoding)
+                                     'utf-8)))))
+
 ;;;
 ;;;
 ;;; Grapheme cluster break and word break
@@ -951,6 +980,20 @@
   (let1 buf (make-vector SCM_CHAR_FULL_CASE_MAPPING_SIZE)
     (generator-for-each (^[ch] (%tr ch buf CHAR_UPCASE sink char?)) generator)))
 
+;; Greek capital sigma U+03a3 and final sigma U+03c2.
+;; We can't use character literal #\u03a3 etc., for they're not valid
+;; when internal encoding is 'none'.  We intend to make utf-8 support
+;; mandatory in future, and this limitation will be lifted.
+;; For now, they'll be #f when the internal encoding is 'none'.
+(define-constant .capital-sigma.
+  (cond-expand
+   [gauche.ces.none #f]
+   [else (ucs->char #x03a3)]))
+(define-constant .final-sigma.
+  (cond-expand
+   [gauche.ces.none #f]
+   [else (ucs->char #x03c2)]))
+
 ;; Greek capital sigma U+03a3 needs context-sensitive conversion.
 (define (%downcase generator sink char?)
   (let ([breaker (make-word-breaker generator)]
@@ -959,10 +1002,10 @@
       (let loop ([ch ch] [prev-break? break?])
         (unless (eof-object? ch)
           (receive (next break?) (breaker)
-            (if (and (memv ch '(#\u03a3 #x03a3)) ; capital sigma
+            (if (and (memv ch `(,.capital-sigma. #x03a3))
                      (not prev-break?)
                      break?)
-              (sink `(,(if char? #\u03c2 #x03c2)) ch) ; final sigma
+              (sink `(,(if char? .final-sigma. #x03c2)) ch)
               (%tr ch buf CHAR_DOWNCASE sink char?))
             (loop next break?)))))))
 
@@ -974,28 +1017,25 @@
         (unless (eof-object? ch)
           (receive (next break?) (breaker)
             (cond [prev-break? (%tr ch buf CHAR_TITLECASE sink char?)]
-                  [(and (memv ch '(#\u03a3 #x03a3)) ; capital sigma
+                  [(and (memv ch `(,.capital-sigma. #x03a3))
                         (not prev-break?)
                         break?)
-                   (sink `(,(if char? #\u03c2 #x03c2)) ch)]
+                   (sink `(,(if char? .final-sigma. #x03c2)) ch)]
                   [else (%tr ch buf CHAR_DOWNCASE sink char?)])
             (loop next break?)))))))
 
 (define (%foldcase generator sink char?)
-  (let ([buf1 (make-vector SCM_CHAR_FULL_CASE_MAPPING_SIZE)]
+  (let ([breaker (make-word-breaker generator)]
+        [buf1 (make-vector SCM_CHAR_FULL_CASE_MAPPING_SIZE)]
         [buf2 (make-vector SCM_CHAR_FULL_CASE_MAPPING_SIZE)])
-    (generator-for-each
-     (^[ch]
-       (let1 c (%char-xcase-extended ch buf1 CHAR_UPCASE char?)
-         (sink (append-ec
-                (: i c)
-                (let1 c2 (%char-xcase-extended (vector-ref buf1 i)
-                                               buf2
-                                               CHAR_DOWNCASE
-                                               char?)
-                  (list-ec (: j c2) (vector-ref buf2 j))))
-               ch)))
-     generator)))
+    (receive (ch break?) (breaker)
+      (let loop ([ch ch] [prev-break? break?])
+        (unless (eof-object? ch)
+          (receive (next break?) (breaker)
+            (do-ec
+             (: i (%char-xcase-extended ch buf1 CHAR_UPCASE char?))
+             (%tr (vector-ref buf1 i) buf2 CHAR_DOWNCASE sink char?))
+            (loop next break?)))))))
 
 (define string-xcase
   (case (gauche-character-encoding)
@@ -1027,3 +1067,25 @@
 (define (codepoints-downcase seq)  (codepoints-xcase seq %downcase))
 (define (codepoints-titlecase seq) (codepoints-xcase seq %titlecase))
 (define (codepoints-foldcase seq)  (codepoints-xcase seq %foldcase))
+
+;;
+;; R7RS-compatible case-insensitive string comparison
+;;
+
+;; NB: Builtin string=? etc. can take more than two args, so the simplest
+;; definition would be:
+;;  (define (string-ci=? . args) (apply string=? (map string-foldcase args)))
+;; We compare pairwise instead, so that we don't need to foldcase
+;; more than necessary when we find the condition doesn't meet.
+
+(define (string-ci-cmp =? s0 s1 more)
+  (let loop ([s0 (string-foldcase s0)] [s1 (string-foldcase s1)] [more more])
+    (if (null? more)
+      (=? s0 s1)
+      (and (=? s0 s1) (loop s1 (string-foldcase (car more)) (cdr more))))))
+
+(define (string-ci=? s0 s1 . more)  (string-ci-cmp string=? s0 s1 more))
+(define (string-ci<? s0 s1 . more)  (string-ci-cmp string<? s0 s1 more))
+(define (string-ci<=? s0 s1 . more) (string-ci-cmp string<=? s0 s1 more))
+(define (string-ci>? s0 s1 . more)  (string-ci-cmp string>? s0 s1 more))
+(define (string-ci>=? s0 s1 . more) (string-ci-cmp string>=? s0 s1 more))
