@@ -100,6 +100,18 @@
            (thread-terminate! t1)
            (thread-join! t1))))
 
+;; this SEGVs on 0.9.3.3. test code from @cryks.
+(test* "thread termination before running" 'terminated
+       (let1 t1 (make-thread (^[] #f))
+         (thread-terminate! t1)
+         (thread-state t1)))
+
+(test* "thread termination while being stopped" 'terminated
+       (let1 t1 (thread-start! (make-thread (^[] (let loop () (loop)))))
+         (thread-stop! t1)
+         (thread-terminate! t1)
+         (thread-state t1)))
+
 ;;---------------------------------------------------------------------
 (test-section "thread and error")
 
@@ -482,9 +494,9 @@
 
 (test* "atom" #t (atom? (atom 0 1 2)))
 
-(test* "atomic counting" 1000
+(test* "atomic counting" 300
        (let ([a (atom 0)] [ts '()])
-         (dotimes [n 100]
+         (dotimes [n 30]
            (push! ts
                   (thread-start! (make-thread
                                   (^[] (dotimes [m 10]
@@ -561,7 +573,7 @@
         (sys-nanosleep #e1e7)
         (loop (dequeue/wait! qs)))))
 
-  (test* #`"synchronized queue ,name" data
+  (test* #"synchronized queue ~name" data
          (let* ([cs (map (^_ (thread-start! (make-thread consumer)))
                          (iota nthreads))]
                 [p1 (make-thread producer)])
