@@ -1,7 +1,7 @@
 ;;;
 ;;; util.queue - queue (fifo) implementation
 ;;;
-;;;   Copyright (c) 2010-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2010-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -324,7 +324,7 @@
 
  ;; caller must hold lock
  (define-cproc %mtqueue-overflow? (q::<mtqueue> cnt::<int>) ::<boolean>
-   (SCM_MAKE_BOOL (mtq-overflows q cnt)))
+   (result (mtq-overflows q cnt)))
 
  ;; API
  (define-cproc mtqueue-room (q::<mtqueue>) ::<number>
