@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.uvector - uniform vectors
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,4 +44,10 @@
   (export-all)
   )
 (select-module gauche.uvector)
+
 (dynamic-load "libgauche-uvector")
+
+;; for the bakcward compatibility
+(define read-block! read-uvector!)
+(define write-block write-uvector)
+
