@@ -1,7 +1,7 @@
 /*
  * uvinit.c - initialize routine for uvector extension
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -35,6 +35,7 @@
 #include <gauche/extend.h>
 #define EXTUVECTOR_EXPORTS
 #include "gauche/uvector.h"
+#include "gauche/priv/readerP.h"
 #include "uvectorP.h"
 
 /*
@@ -44,11 +45,11 @@ static ScmObj read_uvector(ScmPort *port, const char *tag,
                            ScmReadContext *ctx)
 {
     ScmChar c;
-    ScmObj list, uv = SCM_UNDEFINED;
+    ScmObj uv = SCM_UNDEFINED;
 
     SCM_GETC(c, port);
     if (c != '(') Scm_Error("bad uniform vector syntax for %s", tag);
-    list = Scm_ReadList(SCM_OBJ(port), ')');
+    ScmObj list = Scm_ReadList(SCM_OBJ(port), ')');
     if (strcmp(tag, "s8") == 0)  uv = Scm_ListToS8Vector(list, 0);
     else if (strcmp(tag, "u8") == 0)  uv = Scm_ListToU8Vector(list, 0);
     else if (strcmp(tag, "s16") == 0) uv = Scm_ListToS16Vector(list, 0);
@@ -62,7 +63,7 @@ static ScmObj read_uvector(ScmPort *port, const char *tag,
     else if (strcmp(tag, "f64") == 0) uv = Scm_ListToF64Vector(list, 0);
     else Scm_Error("invalid unform vector tag: %s", tag);
     /* If we are reading source file, let literal uvectors be immutable. */
-    if (ctx->flags & SCM_READ_LITERAL_IMMUTABLE) {
+    if (Scm_ReadContextLiteralImmutable(ctx)) {
         SCM_UVECTOR_IMMUTABLE_P(uv) = TRUE;
     }
     return uv;
@@ -77,10 +78,8 @@ extern void Scm_Init_uvseq(void);
 
 SCM_EXTENSION_ENTRY void Scm_Init_libgauche_uvector(void)
 {
-    ScmModule *m;
-
     SCM_INIT_EXTENSION(uvector);
-    m = SCM_FIND_MODULE("gauche.uvector", SCM_FIND_MODULE_CREATE);
+    ScmModule *m = SCM_FIND_MODULE("gauche.uvector", SCM_FIND_MODULE_CREATE);
     Scm_InitStaticClassWithMeta(&Scm_UVectorClass,   "<uvector>", m, NULL, SCM_NIL, NULL, 0);
     Scm_InitStaticClassWithMeta(&Scm_S8VectorClass,  "<s8vector>", m, NULL, SCM_NIL, NULL, 0);
     Scm_InitStaticClassWithMeta(&Scm_U8VectorClass,  "<u8vector>", m, NULL, SCM_NIL, NULL, 0);
