@@ -1,7 +1,7 @@
 ;;;
 ;;; uvutil - miscellaneous utility procedures for uvector
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,7 +40,7 @@
 ;; Experimental - compile-time inlining *-ref
 ;; The TYPE constant must be in sync with ScmUVectorType in gauche/vector.h
 
-(define %uvector-ref (with-module gauche.internal %uvector-ref))
+(define-inline %uvector-ref (with-module gauche.internal %uvector-ref))
 
 (define-macro (set-reference-inliner ref type)
   `(define-compiler-macro ,ref
@@ -61,23 +61,53 @@
 (set-reference-inliner f32vector-ref 9)
 (set-reference-inliner f64vector-ref 10)
 
+;;-------------------------------------------------------------
+;; Appending vectors
+;;
+
+(define-macro (define-appender tag)
+  (let ([app   (string->symbol #"~|tag|vector-append")]
+        [len   (string->symbol #"~|tag|vector-length")]
+        [make  (string->symbol #"make-~|tag|vector")]
+        [copy! (string->symbol #"~|tag|vector-copy!")])
+    `(define (,app . vs)
+       (let* ([size (apply + (map ,len vs))]
+              [dest (,make size)])
+         (do ([vs vs (cdr vs)]
+              [k  0  (+ k (,len (car vs)))])
+             [(null? vs) dest]
+           (,copy! dest k (car vs)))))
+    ))
+
+(define-appender s8)
+(define-appender u8)
+(define-appender s16)
+(define-appender u16)
+(define-appender s32)
+(define-appender u32)
+(define-appender s64)
+(define-appender u64)
+(define-appender f16)
+(define-appender f32)
+(define-appender f64)
+
 ;;-------------------------------------------------------------
 ;; Sequence protocol implementation
 ;;
 
 (define-macro (%define-srfi-4-collection-interface tag)
-  (let* ([tagvector (string->symbol #`",|tag|vector")]
-         [class     (string->symbol #`"<,|tagvector|>")]
-         [meta      (string->symbol #`"<,|tagvector|-meta>")]
-         [len       (string->symbol #`",|tagvector|-length")]
-         [ref       (string->symbol #`",|tagvector|-ref")]
-         [set       (string->symbol #`",|tagvector|-set!")]
-         [copy      (string->symbol #`",|tagvector|-copy")]
-         [->list    (string->symbol #`",|tagvector|->list")]
-         [list->    (string->symbol #`"list->,|tagvector|")]
-         [->vec     (string->symbol #`",|tagvector|->vector")]
-         [vec->     (string->symbol #`"vector->,|tagvector|")]
-         [make      (string->symbol #`"make-,|tagvector|")])
+  (let* ([tagvector (string->symbol #"~|tag|vector")]
+         [class     (string->symbol #"<~|tagvector|>")]
+         [meta      (string->symbol #"<~|tagvector|-meta>")]
+         [len       (string->symbol #"~|tagvector|-length")]
+         [ref       (string->symbol #"~|tagvector|-ref")]
+         [set       (string->symbol #"~|tagvector|-set!")]
+         [copy      (string->symbol #"~|tagvector|-copy")]
+         [->list    (string->symbol #"~|tagvector|->list")]
+         [list->    (string->symbol #"list->~|tagvector|")]
+         [->vec     (string->symbol #"~|tagvector|->vector")]
+         [vec->     (string->symbol #"vector->~|tagvector|")]
+         [make      (string->symbol #"make-~|tagvector|")])
     `(begin
        (define-method call-with-iterator ((v ,class) proc :key (start #f))
          (let* ([len (,len v)] [i (or start 0)])
