@@ -139,7 +139,7 @@
        (call-with-output-string
          (lambda (o)
            (let* ((p (make <virtual-output-port>
-                       :putc (lambda (c) (write-char c o)))))
+                       :putc (^c (write-char c o)))))
              (write-char #\a p)
              (display "bcd" p)
              (write-byte 101 p)
@@ -150,7 +150,7 @@
        (call-with-output-string
          (lambda (o)
            (let* ((p (make <virtual-output-port>
-                       :putb (lambda (b) (write-byte b o)))))
+                       :putb (^b (write-byte b o)))))
              (write-char #\a p)
              (display "bcd" p)
              (write-byte 101 p)
@@ -161,9 +161,9 @@
        (call-with-output-string
          (lambda (o)
            (let* ((p (make <virtual-output-port>
-                       :putb (lambda (b) #f)
-                       :putc (lambda (c) #f)
-                       :puts (lambda (s) (display s o)))))
+                       :putb (^b #f)
+                       :putc (^c #f)
+                       :puts (^s (display s o)))))
              (write-char #\a p)
              (display "bcd" p)
              (write-byte 101 p)
@@ -176,7 +176,7 @@
 
 (let ()
   (define (test-biport file size)
-    (let* ((ifile #`",(sys-dirname (current-load-path))/,file")
+    (let* ((ifile #"~(sys-dirname (current-load-path))/~file")
            (src (open-input-file ifile))
            (p (apply make <buffered-input-port>
                      :fill  (lambda (buf) (read-block! buf src))
@@ -234,7 +234,7 @@
 
 (let ()
   (define (test-boport file size)
-    (let* ((ifile #`",(sys-dirname (current-load-path))/,file")
+    (let* ((ifile #"~(sys-dirname (current-load-path))/~file")
            (src  (file->string ifile))
            (sink (open-output-string))
            (closed? #f)
@@ -246,7 +246,7 @@
                      (if size
                        (list :buffer-size size)
                        '()))))
-      (string-for-each (lambda (c) (write-char c p)) src)
+      (string-for-each (^c (write-char c p)) src)
       (close-output-port p)
       (list (equal? src (get-output-string sink))
             closed?)))
@@ -273,7 +273,7 @@
 
 (let ()
   (define (tester size)
-    (test* #`"size=,size" #t
+    (test* #"size=~size" #t
            (let1 v (make-u8vector size 0)
              (dotimes (i size) (u8vector-set! v i (modulo i 256)))
              (let* ((p (open-input-uvector v))
@@ -314,7 +314,7 @@
 
 (let ()
   (define (tester size)
-    (test* #`"size=,size" #t
+    (test* #"size=~size" #t
            (let1 v (make-u8vector size 0)
              (dotimes (i size) (u8vector-set! v i (modulo i 256)))
              (let* ((dst (make-u8vector size 0))
@@ -345,9 +345,9 @@
 
 (let ()
   (define (tester size limit)
-    (test* #`"size=,size limit=,limit" #t
+    (test* #"size=~size limit=~limit" #t
            (let* ((source (string-tabulate
-                           (lambda (i) (integer->char (modulo i 128)))
+                           (^i (integer->char (modulo i 128)))
                            size))
                   (expected (if (<= limit size)
                               (string-take source limit)
