@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.cise - C in S expression
 ;;;
-;;;   Copyright (c) 2004-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -98,15 +98,15 @@
 (define (ensure-stmt-ctx form env)
   (unless (stmt-ctx? env)
     (if (expr-ctx? env)
-      (error "cise: statment appears in an expression context:" form)
-      (error "cise: statment appears in a toplevel context:" form))))
+      (error "cise: statement appears in an expression context:" form)
+      (error "cise: statement appears in a toplevel context:" form))))
 
 (define (ensure-toplevel-ctx form env)
   (unless (toplevel-ctx? env)
     (error "cise: form can only appear in toplevel:" form)))
 (define (ensure-stmt-or-toplevel-ctx form env)
   (unless (or (toplevel-ctx? env) (stmt-ctx? env))
-    (error "cise: form can only appear in toplevel or statment context:" form)))
+    (error "cise: form can only appear in toplevel or statement context:" form)))
 (define (env-decl-add! env decl)
   (push! (~ env'decls) decl))
 
@@ -188,6 +188,75 @@
 ;; define-cise-expr OP [ENV] CLAUSE ... [:where DEFINITION ...]
 ;; define-cise-toplevel OP [ENV] CLAUSE ... [:where DEFINITION ...]
 ;;
+
+;; Temporary: Pre-0.9.4 Gauche dones't allow keyword in the literal of
+;; syntax-rules, but we need to include :where if keyword-symbol integration
+;; is turned on.  
+(cond-expand
+ [gauche.unified-keyword-symbol
+(define-syntax define-cise-stmt
+  (syntax-rules (:where)
+    ;; recursion
+    [(_ "clauses" op env clauses (:where defs ...))
+     (define-cise-macro (op form env)
+       defs ...
+       (ensure-stmt-ctx form env)
+       (match form . clauses))]
+    [(_ "clauses" op env clauses ())
+     (define-cise-stmt "clauses" op env clauses (:where))]
+    [(_ "clauses" op env (clause ...) (x . y))
+     (define-cise-stmt "clauses" op env (clause ... x) y)]
+    ;; entry
+    [(_ (op . args) . body)       ; single pattern case
+     (define-cise-stmt "clauses" op env (((_ . args) . body)) ())]
+    [(_ op (pat . body) .  clauses) ; (pat . body) rules out a single symbol
+     (define-cise-stmt "clauses" op env ((pat . body)) clauses)]
+    [(_ op env . clauses)
+     (define-cise-stmt "clauses" op env () clauses)]))
+
+(define-syntax define-cise-expr
+  (syntax-rules (:where)
+    ;; recursion
+    [(_ "clauses" op env clauses (:where defs ...))
+     (define-cise-macro (op form env)
+       defs ...
+       (let1 expanded (match form . clauses)
+         (if (and (pair? expanded) (symbol? (car expanded)))
+           (render-rec expanded env)
+           (wrap-expr expanded env))))]
+    [(_ "clauses" op env clauses ())
+     (define-cise-expr "clauses" op env clauses (:where))]
+    [(_ "clauses" op env (clause ...) (x . y))
+     (define-cise-expr "clauses" op env (clause ... x) y)]
+    ;; entry
+    [(_ (op . args) . body)       ; single pattern case
+     (define-cise-expr "clauses" op env (((_ . args) . body)) ())]
+    [(_ op (pat . body) .  clauses)
+     (define-cise-expr "clauses" op env ((pat . body)) clauses)]
+    [(_ op env . clauses)
+     (define-cise-expr "clauses" op env () clauses)]))
+
+(define-syntax define-cise-toplevel
+  (syntax-rules (:where)
+    ;; recursion
+    [(_ "clauses" op env clauses (:where defs ...))
+     (define-cise-macro (op form env)
+       defs ...
+       (ensure-toplevel-ctx form env)
+       (match form . clauses))]
+    [(_ "clauses" op env clauses ())
+     (define-cise-toplevel "clauses" op env clauses (:where))]
+    [(_ "clauses" op env (clause ...) (x . y))
+     (define-cise-toplevel "clauses" op env (clause ... x) y)]
+    ;; entry
+    [(_ (op . args) . body)       ; single pattern case
+     (define-cise-toplevel "clauses" op env (((_ . args) . body)) ())]
+    [(_ op (pat . body) .  clauses) ; (pat . body) rules out a single symbol
+     (define-cise-toplevel "clauses" op env ((pat . body)) clauses)]
+    [(_ op env . clauses)
+     (define-cise-toplevel "clauses" op env () clauses)]))
+]
+[else
 (define-syntax define-cise-stmt
   (syntax-rules ()
     ;; recursion
@@ -249,6 +318,7 @@
      (define-cise-toplevel "clauses" op env ((pat . body)) clauses)]
     [(_ op env . clauses)
      (define-cise-toplevel "clauses" op env () clauses)]))
+])
 
 ;;
 ;; cise-render cise &optional port context
@@ -337,10 +407,13 @@
     [#\newline   (wrap-expr "'\\n'"  env)]
     [#\return    (wrap-expr "'\\r'"  env)]
     [#\tab       (wrap-expr "'\\t'"  env)]
-    [[? char?]   (wrap-expr `("'" ,(if (char-set-contains? #[[:alnum:]] form)
-                                     (string form)
-                                     (format "\\x~2'0x" (char->integer form)))
-                              "'") env)]
+    [[? char?]
+     (if (>= (char->integer form) 128)
+       (error "CISE: Cannot embed non-ASCII character literal (yet):" form)
+       (wrap-expr `("'" ,(if (char-set-contains? #[[:alnum:]] form)
+                           (string form)
+                           (format "\\x~2,'0x" (char->integer form)))
+                    "'") env))]
     [_           (error "Invalid CISE form: " form)]))
 
 ;;
@@ -804,6 +877,11 @@
 ;;
 ;;   cast.
 ;;
+;; [cise expr] .type TYPE
+;;
+;;   not a C expression, but useful to place a type name (e.g. an argument
+;;   of sizeof etc.)
+;;
 ;; [cise expr] ?: TEST-EXPR THEN-EXPR ELSE-EXPR
 ;;
 ;;   conditional.
@@ -947,6 +1025,11 @@
         `("((",(cise-render-typed-var type "" env)")(",(render-rec expr eenv)"))")])
      env)))
 
+(define-cise-macro (.type form env)
+  (match form
+    [(_ typenames ...)
+     `(,(cise-render-typed-var typenames "" env))]))
+
 ;;------------------------------------------------------------
 ;; Convenience expression macros
 ;;
@@ -1028,16 +1111,16 @@
 (define (canonicalize-vardecl vardecls)
   (define (expand-type elt seed)
     (cond
+     [(keyword? elt)  ;; The case of (var ::type)
+      (rxmatch-case (keyword->string elt)
+        [#/^:(.+)$/ (_ t) `(:: ,(string->symbol t) ,@seed)]
+        [else (cons elt seed)])]
      [(symbol? elt)
       (rxmatch-case (symbol->string elt)
         [#/^(.+)::$/ (_ v) `(,(string->symbol v) :: ,@seed)]
         [#/^(.+)::(.+)$/ (_ v t)
             `(,(string->symbol v) :: ,(string->symbol t) ,@seed)]
         [else (cons elt seed)])]
-     [(keyword? elt)  ;; The case of (var ::type)
-      (rxmatch-case (keyword->string elt)
-        [#/^:(.+)$/ (_ t) `(:: ,(string->symbol t) ,@seed)]
-        [else (cons elt seed)])]
      [else (cons elt seed)]))
 
   (define (err decl) (error "invlaid variable declaration:" decl))
@@ -1045,7 +1128,7 @@
   (define (scan in r)
     (match in
       [() (reverse r)]
-      [([? symbol? var] :: type . rest)
+      [([? symbol? var] ':: type . rest)
        (scan rest `((,var :: ,type) ,@r))]
       [([? symbol? var] . rest)
        (scan rest `((,var :: ScmObj) ,@r))]
