@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.literal - static literal data
 ;;;
-;;;   Copyright (c) 2004-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,6 +44,7 @@
           cgen-literal-static?
           define-cgen-literal cgen-literal
           cgen-allocate-static-datum
+          c-safe-string-literal         ;useful to emit C string constant
           )
   )
 (select-module gauche.cgen.literal)
@@ -476,12 +477,13 @@
   (with-string-io value
     (lambda ()
       (display "\"")
-      (port-for-each (^b (if (or (= #x20 b) (= #x21 b) ; #x22 = #\"
-                                 (<= #x23 b #x5b)      ; #x5c = #\\
-                                 (<= #x5d b #x7e))
-                           (write-byte b)
-                           (format #t "\\~3,'0o" b)))
-                     read-byte)
+      (port-for-each
+       (^b (if (or (= #x20 b) (= #x21 b) ; #x22 = #\"
+                   (<= #x23 b #x3e)      ; #x3f = #\?  - avoid trigraph trap
+                   (<= #x40 b #x5b)      ; #x5c = #\\
+                   (<= #x5d b #x7e))
+             (write-byte b)
+             (format #t "\\~3,'0o" b))) read-byte)
       (display "\""))))
 
 ;; symbol ------------------------------------------------------
@@ -528,7 +530,7 @@
     (cond
      [(fixnum? value)
       (make <cgen-scheme-integer> :value value :c-name #f)]
-     [(< (- (expt 2 31)) value (- (expt 2 32)))
+     [(< (- (%expt 2 31)) value (- (%expt 2 32)))
       (make <cgen-scheme-integer> :value value
             :c-name (cgen-allocate-static-datum))]
      [else
@@ -547,13 +549,13 @@
       ;; we can use 64bit literal, but we'll leave it for later revision.
       (let ([val   (~ self'value)]
             [cname (cgen-c-name self)])
-        (cond [(< (- (expt 2 31)) val 0)
+        (cond [(< (- (%expt 2 31)) val 0)
                (print "  " cname " = Scm_MakeInteger("val");")]
-              [(<= 0 val (- (expt 2 32) 1))
+              [(<= 0 val (- (%expt 2 32) 1))
                (print "  " cname " = Scm_MakeIntegerU("val"U);")]
               [else
                (print "  " cname " = Scm_StringToNumber(SCM_STRING("
-                      (cgen-cexpr (~ self'string-rep))"), 16, TRUE);")]))))
+                      (cgen-cexpr (~ self'string-rep))"), 16, 0);")]))))
   (static (self)
     (if (cgen-c-name self) #f #t))
   )
@@ -593,7 +595,13 @@
   (init (self)
     (let ([real (real-part (~ self'value))]
           [imag (imag-part (~ self'value))])
-      (print "  "(cgen-c-name self)" = Scm_MakeComplex("real", "imag");")))
+      (define (double-literal v)
+        (cond [(finite? v) v]
+              [(nan? v) "SCM_DBL_NAN"]
+              [(positive? v) "SCM_DBL_POSITIVE_INFINITY"]
+              [else "SCM_DBL_NEGATIVE_INFINITY"]))
+      (print "  "(cgen-c-name self)" = "
+             "Scm_MakeComplex("(double-literal real)", "(double-literal imag)");")))
   (static (self) #f))
 
 ;; pair ---------------------------------------------------------
@@ -720,7 +728,7 @@
     (print "#endif /*!WORDS_BIGENDIAN*/")
     (print "#else  /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")
     (format #t " (((int64_t)~dl << 32)|~dl),\n"
-            (ash v -32) (logand v (- (expt 2 32) 1)))
+            (ash v -32) (logand v (- (%expt 2 32) 1)))
     (print "#endif /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")]
    [(eq? class <u64vector>)
     (print "#if SIZEOF_LONG == 8")
@@ -731,7 +739,7 @@
     (print "#endif /*!WORDS_BIGENDIAN*/")
     (print "#else  /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")
     (format #t " (((int64_t)~dlu << 32)|~dlu),\n"
-            (ash v -32) (logand v (- (expt 2 32) 1)))
+            (ash v -32) (logand v (- (%expt 2 32) 1)))
     (print "#endif /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")]
    [(eq? class <f16vector>)
     (error "gauche.cgen.literal: literal f16vector isn't supported yet")]
