@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.precomp - Precompile Scheme into C data
 ;;;
-;;;   Copyright (c) 2004-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,7 +44,6 @@
   (use gauche.parameter)
   (use gauche.sequence)
   (use gauche.experimental.lamb)
-  (use gauche.experimental.app)
   (use file.util)
   (use util.match)
   (use util.list)
@@ -53,6 +52,10 @@
   (export cgen-precompile cgen-precompile-multi))
 (select-module gauche.cgen.precomp)
 
+;; Do not load optimizer unless explicitly requested.  To compile
+;; 0.9.4 with 0.9.3 compiler, we can't depend on the optimizer module.
+(autoload gauche.cgen.optimizer optimize-compiled-code)
+
 ;;================================================================
 ;; Main Entry point
 ;;
@@ -157,12 +160,13 @@
    (apply %cgen-precompile src keys)))
 
 (define (do-it src ext-initializer sub-initializers)
-  (setup ext-initializer sub-initializers)
-  (with-input-from-file src
-    (cut emit-toplevel-executor
-         (reverse (port-fold compile-toplevel-form '() read))))
-  (finalize sub-initializers)
-  (cgen-emit-c (cgen-current-unit)))
+  (parameterize ([omitted-code '()])
+    (setup ext-initializer sub-initializers)
+    (with-input-from-file src
+      (cut emit-toplevel-executor
+           (reverse (port-fold compile-toplevel-form '() read))))
+    (finalize sub-initializers)
+    (cgen-emit-c (cgen-current-unit))))
 
 ;; Precompile multiple Scheme sources that are to be linked into
 ;; single DSO.  Need to check dependency.  The name of the first
@@ -172,7 +176,8 @@
                                     ((:strip-prefix prefix) #f)
                                     ((:dso-name dso) #f)
                                     (predef-syms '())
-                                    (macros-to-keep '()))
+                                    (macros-to-keep '())
+                                    (extra-optimization #f))
   (match srcs
     [() #f]
     [(main . subs)
@@ -189,6 +194,7 @@
                             :predef-syms predef-syms
                             :strip-prefix prefix
                             :macros-to-keep macros-to-keep
+                            :extra-optimization extra-optimization
                             :ext-initializer (and (equal? src main)
                                                   ext-initializer)
                             :initializer-name #`"Scm_Init_,initname"))))]
@@ -204,7 +210,8 @@
                                (initializer-name #f)
                                (sub-initializers '())
                                (predef-syms '())
-                               (macros-to-keep '()))
+                               (macros-to-keep '())
+                               (extra-optimization #f))
   (let ([out.c   (or out.c (path-swap-extension (sys-basename src) "c"))]
         [out.sci (or out.sci
                      (and (check-first-form-is-define-module src)
@@ -219,7 +226,8 @@
                                        (basename-sans-extension out.c)))
                               initializer-name)]
                    [vm-eval-situation SCM_VM_COMPILING]
-                   [private-macros-to-keep macros-to-keep])
+                   [private-macros-to-keep macros-to-keep]
+                   [run-extra-optimization-passes extra-optimization])
       (select-tmodule 'gauche)
       (cond [out.sci
              (make-directory* (sys-dirname out.sci))
@@ -251,6 +259,7 @@
 (define (eval-in-current-tmodule expr)
   (eval expr (~ (current-tmodule)'module)))
 
+;; Expr -> CompiledCode
 (define (compile-in-current-tmodule expr)
   (compile expr (~ (current-tmodule)'module)))
 
@@ -277,10 +286,13 @@
 ;; (--keep-private-macro=name,name,...)
 ;; usually private macros (macros bound to a variable which isn't exported)
 ;; are discarded, but sometimes hygienic public macros expands to a call
-;; of private macros.  gencomp cannot detect such dependency yet, and
+;; of private macros.  precomp cannot detect such dependency yet, and
 ;; so they need to be explicitly listed for the time being.
 (define private-macros-to-keep (make-parameter '()))
 
+;; Experimental: Run extra optimization during AOT compilation.
+(define run-extra-optimization-passes (make-parameter #f))
+
 ;;================================================================
 ;; Bridge to the internal stuff
 ;;
@@ -397,12 +409,12 @@
 
 ;; NOTE:
 ;;   The code is compiled in the version of the compiler currently
-;;   running gencomp (host compiler).  It may differ from the version
+;;   running precomp (host compiler).  It may differ from the version
 ;;   of the compiler we're compiling (target compiler), and it becomes
 ;;   a problem if the two versions of compilers are using different
 ;;   mappings between mnemonics and codes.
 ;;
-;;   When gencomp generates the C literals for the compiled code, it
+;;   When precomp generates the C literals for the compiled code, it
 ;;   uses the following mapping scheme.
 ;;
 ;;    1. use vm-code->list to extract mnemonics from the code
@@ -444,15 +456,12 @@
 (define-global-pred =lambda?          lambda)
 (define-global-pred =declare?         declare)
 
-;; KLUDGE: Include needs to refer to the path of including source file, which
-;; is taken from current-load-path in 0.9.2 and before.  If we were able to
-;; override that, we wouldn't need to intercept 'include'.  That should be the
-;; right thing, since 'include' doesn't need to appear in toplevel.
-;; For 0.9.3, however, we need to keep things compilable by 0.9.2 compiler
-;; whose behavior we can't change.  Hence this hack allows us to make 'include'
-;; work for the time being.
-(define-global-pred =include?         include)
-
+;; A parameter that holds the list of 'omitted' #<compiled-code> - for
+;; example, the cliche of (CLOSURE #<compiled-code> DEFINE #<identifier> RET)
+;; will be generated as Scm_Define() in the initialization, not as a code
+;; vector.  We need to suppress emitting code vector for those, since they
+;; can be reachable via parent link in the inner closure.
+(define omitted-code (make-parameter '()))
 
 ;; compile FORM, and conses the toplevel code (something to be
 ;; executed at toplevel).
@@ -483,7 +492,16 @@
        (when (list? (compile-module-exports))
          (compile-module-exports
           (lset-union eq? syms (compile-module-exports))))
-       (eval-in-current-tmodule `(export ,@syms)) seed]
+       (eval-in-current-tmodule `(export ,@syms))
+       (unless (ext-module-file)
+         ;; If generate .sci file, 'export' form will be in it (as a part of
+         ;; define-module form).  Otherwise we need to do export during
+         ;; initialization.
+         (let1 exp-specs (cgen-literal syms)
+           (cgen-init
+            (format "  (void)Scm_ExportSymbols(Scm_CurrentModule(), ~a);"
+                    (cgen-cexpr exp-specs)))))
+       seed]
       [((? =export-all?)) (compile-module-exports #t)]
       [((? =export-if-defined?) . _) (write-ext-module form) seed]
       [((? =provide?) arg) (write-ext-module form) seed]
@@ -496,44 +514,67 @@
             (private-macros-to-keep (append (private-macros-to-keep) macros))]
            [other (error "Unknown declaration:" other)]))
        seed]
-      ;; TODO: Temporary kludge---remove in the future release.  See the
-      ;; above comment about 'include' kludge.
-      [((? =include?) path)
-       (let* ([spath (if (absolute-path? path)
-                       path
-                       (build-path ($ sys-dirname $ port-name
-                                      $ current-input-port)
-                                   path))]
-              [source (any (^[ext] (let1 p #`",|spath|,|ext|"
-                                     (and (file-is-readable? p) p)))
-                           (cons "" *load-suffixes*))])
-         (unless source
-           (errorf "Couldn't read included file ~s" path))
-         (compile-toplevel-form `(begin ,@(file->sexp-list source)) seed))]
       ;; Finally, ordinary expressions.
       [else
-       (let1 compiled-code (compile-in-current-tmodule form)
+       (let* ([compiled-code (compile-in-current-tmodule form)]
+              [toplevel-code (and (eq? (~ compiled-code 'name) '%toplevel)
+                                  (vm-code->list compiled-code))])
          ;; We exclude a compiled code with only CONSTU-RET, which appears
          ;; as the result of macro expansion sometimes.
-         (if (toplevel-constu-ret-code? compiled-code)
-           seed
-           (cons (cgen-literal compiled-code) seed)))]
+         (cond [(toplevel-constu-ret-code? toplevel-code) seed]
+               [(toplevel-definition-code? toplevel-code)
+                => (match-lambda
+                     ([inner-code id flags]
+                      (push! (omitted-code) compiled-code)
+                      (emit-toplevel-definition inner-code id flags)
+                      seed))]
+               [else (cons (cgen-literal compiled-code) seed)]))]
       )))
 
 ;; check to see the compiled code only contains CONSTU-RET insn.
-(define (toplevel-constu-ret-code? compiled-code)
-  (and (eq? (~ compiled-code'name) '%toplevel)
-       (= (~ compiled-code'size) 1)
-       (let1 code (vm-code->list compiled-code)
-         (null? (cdr code))
-         (eq? (caar code) 'CONSTU-RET))))
+(define (toplevel-constu-ret-code? toplevel-code)
+  (and (null? (cdr toplevel-code))
+       (eq? (caar toplevel-code) 'CONSTU-RET)))
+
+;; check to see if the compiled code has the cliche of toplevel
+;; definition (CLOSURE #<code> DEFINE #<id> RET).  If we find it, returns
+;; the internal closure code, identifier, and define flags.
+(define (toplevel-definition-code? toplevel-code)
+  (and (eq? (car (~ toplevel-code 0)) 'CLOSURE)
+       (eq? (car (~ toplevel-code 2)) 'DEFINE)
+       (eq? (car (~ toplevel-code 4)) 'RET)
+       (list (~ toplevel-code 1)           ; #<compiled-code> of CLOSURE
+             (~ toplevel-code 3)           ; identifier
+             (cadr (~ toplevel-code 2))))) ; define flags
+
+(define (emit-toplevel-definition inner-code id flags)
+  (let ([sym  (cgen-literal (~ id'name))]
+        ;; NB: Currently, the main tmodule has no name.  It's better to give
+        ;; it a proper name.  Then the following Scm_CurrentModule hack
+        ;; will be unnecessary.
+        [mod  (and-let* ([n (module-name (~ id'module))])
+                (find-tmodule n))]
+        [code (cgen-literal inner-code)])
+    (cgen-init (format "  Scm_MakeBinding(SCM_MODULE(~a) /* ~a */, \
+                                          SCM_SYMBOL(~a) /* ~a */, \
+                                          Scm_MakeClosure(~a, NULL),\
+                                          ~a);\n"
+                       (if mod (tmodule-cname mod) "Scm_CurrentModule()")
+                       (if mod (cgen-safe-comment (~ mod'name)) "")
+                       (cgen-cexpr sym)
+                       (cgen-safe-comment (~ id'name))
+                       (cgen-cexpr code)
+                       (case flags
+                         [(2) 'SCM_BINDING_CONST]
+                         [(4) 'SCM_BINDING_INLINABLE]
+                         [else 0])))))
 
 ;; given list of toplevel compiled codes, generate code in init
 ;; that calls them.  This is assumed to be the last procedure before
 ;; calling cgen-emit.
 (define (emit-toplevel-executor topcodes)
   (cgen-body "static ScmCompiledCode *toplevels[] = {")
-  (dolist (t topcodes)
+  (dolist [t topcodes]
     (cgen-body (format "  SCM_COMPILED_CODE(~a)," (cgen-cexpr t))))
   (cgen-body " NULL /*termination*/" "};")
 
@@ -638,31 +679,37 @@
    (literals    :init-keyword :literals)
    )
   (make (value)
-    (let* ([cv  (vm-code->list value)]
+    (let* ([code (if (run-extra-optimization-passes)
+                   (optimize-compiled-code value)
+                   value)]
+           [cv  (vm-code->list code)]
            [lv  (extract-literals cv)]
-           [cvn (allocate-code-vector cv lv (~ value'full-name))]
-           [code-name (cgen-literal (~ value'name))]
-           [arg-info (cgen-literal (~ value'arg-info))]
-           [inliner (check-packed-inliner value)])
+           [cvn (allocate-code-vector cv lv (~ code'full-name))]
+           [code-name (cgen-literal (~ code'name))]
+           [arg-info (cgen-literal (~ code'arg-info))]
+           [inliner (check-packed-inliner code)])
       (define (init-thunk)
         (format #t "    SCM_COMPILED_CODE_CONST_INITIALIZER(  /* ~a */\n"
-                (cgen-safe-comment (~ value'name)))
+                (cgen-safe-comment (~ code'name)))
         (format #t "            (ScmWord*)(~a), ~a,\n"
                 cvn (length cv))
         (format #t "            ~a, ~a, ~a, ~a, SCM_NIL, ~a,\n"
-                (~ value'max-stack)
-                (~ value'required-args)
-                (~ value'optional-args)
+                (~ code'max-stack)
+                (~ code'required-args)
+                (~ code'optional-args)
                 (if (cgen-literal-static? code-name)
                   (cgen-cexpr code-name)
                   "SCM_FALSE")
                 (cgen-cexpr arg-info))
         (format #t "            ~a, ~a)"
-                (cgen-cexpr (cgen-literal (~ value'parent)))
+                (let1 parent-code (~ code'parent)
+                  (if (memq parent-code (omitted-code))
+                    "SCM_FALSE"
+                    (cgen-cexpr (cgen-literal (~ code'parent)))))
                 (if inliner
                   (cgen-cexpr inliner)
                   "SCM_FALSE")))
-      (make <cgen-scheme-code> :value value
+      (make <cgen-scheme-code> :value code
             :c-name (cgen-allocate-static-datum 'runtime 'ScmCompiledCode
                                                 init-thunk)
             :code-vector-c-name cvn
@@ -838,7 +885,7 @@
       :c-name  (cgen-allocate-static-datum)
       :gf-name (cgen-literal (ref value 'name))))
   (init (self)
-    (format #t "  ~a = Scm_SymbolValue(~a, SCM_SYMBOL(~a));\n"
+    (format #t "  ~a = Scm_GlobalVariableRef(SCM_MODULE(~a), SCM_SYMBOL(~a), FALSE);\n"
             (~ self'c-name)
             (current-tmodule-cname)
             (~ self'gf-name'c-name)))
