@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.cgen.type - type management
-;;;  
-;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,13 +29,12 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.cgen.type
   (use srfi-13)
   (use text.tr)
   (use gauche.mop.instance-pool)
-  (use gauche.experimental.ref)
   (export <cgen-type> cgen-type-from-name make-cgen-type
           cgen-box-expr cgen-unbox-expr cgen-pred-expr cgen-return-stmt)
   )
@@ -75,7 +74,7 @@
 ;;
 ;;   stub type    Scheme       C           Notes
 ;;  -----------------------------------------------------------------
-;;   <fixnum>     <integer>    int         Integers within fixnum range
+;;   <fixnum>     <integer>    ScmSmallInt Integers within fixnum range
 ;;   <integer>    <integer>    ScmObj      Any exact integers
 ;;   <real>       <real>       double
 ;;   <number>     <number>     ScmObj      Any numbers
@@ -92,7 +91,7 @@
 ;;   <uint32>     <integer>    uint
 ;;   <ushort>     <integer>    ushort
 ;;   <ulong>      <integer>    ulong
-;;   <float>      <real>       float       Unboxed value casted to float
+;;   <float>      <real>       float       Unboxed value cast to float
 ;;   <double>     <real>       double      Alias of <real>
 ;;
 ;;   <boolean>    <boolean>    int         Boolean value
@@ -184,30 +183,30 @@
 (for-each
  (cut apply make-cgen-type <>)
  '(;; Numeric types
-   (<fixnum>  "int" "small integer" "SCM_INTP" "SCM_INT_VALUE" "SCM_MAKE_INT")
-   (<integer> "ScmObj" "exact integer" "SCM_EXACTP" "")
+   (<fixnum>  "ScmSmallInt" "small integer" "SCM_INTP" "SCM_INT_VALUE" "SCM_MAKE_INT")
+   (<integer> "ScmObj" "exact integer" "SCM_INTEGERP" "")
    (<real>    "double" "real number" "SCM_REALP" "Scm_GetDouble" "Scm_VMReturnFlonum")
    (<number>  "ScmObj" "number" "SCM_NUMBERP" "")
-   (<int>     "int" "C integer" "SCM_EXACTP" "Scm_GetInteger" "Scm_MakeInteger")
-   (<long>    "long" "C long integer" "SCM_EXACTP" "Scm_GetInteger" "Scm_MakeInteger")
+   (<int>     "int" "C integer" "SCM_INTEGERP" "Scm_GetInteger" "Scm_MakeInteger")
+   (<long>    "long" "C long integer" "SCM_INTEGERP" "Scm_GetInteger" "Scm_MakeInteger")
    (<short>   "short" "C short integer" "SCM_INTP" "(short)SCM_INT_VALUE" "SCM_MAKE_INT")
-   (<int8>    "int" "C integer" "SCM_EXACTP" "Scm_GetInteger" "Scm_MakeInteger")
-   (<int16>   "int" "C integer" "SCM_EXACTP" "Scm_GetInteger" "Scm_MakeInteger")
-   (<int32>   "int" "C integer" "SCM_EXACTP" "Scm_GetInteger" "Scm_MakeInteger")
+   (<int8>    "int" "8bit signed integer" "SCM_INTEGERP" "Scm_GetInteger8" "Scm_MakeInteger")
+   (<int16>   "int" "16bit signed integer" "SCM_INTEGERP" "Scm_GetInteger16" "Scm_MakeInteger")
+   (<int32>   "int" "32bit signed integer" "SCM_INTEGERP" "Scm_GetInteger32" "Scm_MakeInteger")
    (<uint>    "u_int" "C integer" "SCM_UINTEGERP" "Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
    (<ulong>   "u_long" "C integer" "SCM_UINTEGERP" "Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
-   (<ushort>  "u_short" "C short integer" "SCM_EXACTP" "(unsigned short)Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
-   (<uint8>   "u_int" "C integer" "SCM_UINTP" "Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
-   (<uint16>  "u_int" "C integer" "SCM_UINTP" "Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
-   (<uint32>  "u_int" "C integer" "SCM_UINTEGERP" "Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
+   (<ushort>  "u_short" "C short integer" "SCM_INTEGERP" "(unsigned short)Scm_GetIntegerU" "Scm_MakeIntegerFromUI")
+   (<uint8>   "u_int" "8bit unsigned integer" "SCM_UINTP" "Scm_GetIntegerU8" "Scm_MakeIntegerFromUI")
+   (<uint16>  "u_int" "16bit unsigned integer" "SCM_UINTP" "Scm_GetIntegerU16" "Scm_MakeIntegerFromUI")
+   (<uint32>  "u_int" "32bit unsigned integer" "SCM_UINTEGERP" "Scm_GetIntegerU32" "Scm_MakeIntegerFromUI")
    (<float>   "float" "real number" "SCM_REALP" "(float)Scm_GetDouble" "Scm_VMReturnFlonum")
    (<double>  "double" "real number" "SCM_REALP" "Scm_GetDouble" "Scm_VMReturnFlonum")
-   
+
    ;; Basic immediate types
    (<boolean> "int" "boolean" "SCM_BOOLP"   "SCM_BOOL_VALUE" "SCM_MAKE_BOOL")
    (<char>    "ScmChar" "character" "SCM_CHARP" "SCM_CHAR_VALUE" "SCM_MAKE_CHAR")
    (<void>    "void" "void" "" "" "SCM_VOID_RETURN_VALUE")
-   (<top>     "ScmObj" "scheme object" "" "") 
+   (<top>     "ScmObj" "scheme object" "" "")
    ;; C string
    (<const-cstring> "const char *" "const C string"
                     "SCM_STRINGP" "SCM_STRING_CONST_CSTRING" "SCM_MAKE_STR_COPYING")
@@ -241,6 +240,7 @@
    (<procedure> "ScmProcedure*" "procedure" "SCM_PROCEDUREP" "SCM_PROCEDURE")
    (<closure> "ScmClosure*" "closure" "SCM_CLOSUREP" "SCM_CLOSURE")
    (<promise> "ScmPromise*" "promise" "SCM_PROMISEP" "SCM_PROMISE")
+   (<comparator> "ScmComparator*" "comparator" "SCM_COMPARATORP" "SCM_COMPARATOR")
    (<hash-table> "ScmHashTable*" "hash table" "SCM_HASH_TABLE_P" "SCM_HASH_TABLE")
    (<tree-map> "ScmTreeMap*" "tree map" "SCM_TREE_MAP_P" "SCM_TREE_MAP")
    (<class> "ScmClass*" "class" "SCM_CLASSP" "SCM_CLASS")
