@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.cgen.unit - cgen-unit
-;;;  
-;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,27 +29,26 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.cgen.unit
   (use srfi-1)
   (use srfi-13)
   (use srfi-42)
   (use util.match)
+  (use util.list)
   (use gauche.parameter)
   (use gauche.sequence)
-  (use gauche.experimental.ref)
-  (use gauche.experimental.lamb)
   (export <cgen-unit> cgen-current-unit
           cgen-unit-c-file cgen-unit-init-name cgen-unit-h-file
           cgen-unit-toplevel-nodes cgen-add!
           cgen-emit-h cgen-emit-c
 
-          <cgen-node> cgen-cpp-condition cgen-emit
+          <cgen-node> cgen-with-cpp-condition cgen-cpp-conditions cgen-emit
           cgen-emit-xtrn cgen-emit-decl cgen-emit-body cgen-emit-init
           cgen-extern cgen-decl cgen-body cgen-init
           cgen-include cgen-define
-          
+
           cgen-safe-name cgen-safe-name-friendly cgen-safe-comment
 
           ;; semi-private routines
@@ -67,36 +66,43 @@
 ;; cgen-current-unit, and most cgen APIs implicitly work to it.
 
 (define-class <cgen-unit> ()
-  ((name     :init-keyword :name   :init-value "cgen")
+  (;; public
+   (name     :init-keyword :name   :init-value "cgen")
    (c-file   :init-keyword :c-file :init-value #f)
    (h-file   :init-keyword :h-file :init-value #f)
    (preamble :init-keyword :preamble
-             :init-value '("/* Generated by gauche.cgen $Revision: 1.7 $ */"))
-   (pre-decl :init-keyword :pre-decl :init-value '())
+             :init-value '("/* Generated by gauche.cgen */"))
    (init-prologue :init-keyword :init-prologue :init-value #f)
    (init-epilogue :init-keyword :init-epilogue :init-value #f)
+   ;; private
    (toplevels :init-value '())   ;; toplevel nodes to be realized
    (transients :init-value '())  ;; transient variables
    (literals  :init-form #f)     ;; literals. see gauche.cgen.literal
    (static-data-list :init-value '()) ;; static C data, see below
    ))
 
+;; API
 (define cgen-current-unit (make-parameter #f))
 
+;; API
 (define-method cgen-unit-c-file ((unit <cgen-unit>))
   (or (~ unit'c-file)
       #`",(~ unit 'name).c"))
 
+;; API
 (define-method cgen-unit-init-name ((unit <cgen-unit>))
   (format "Scm__Init_~a"
           (or (~ unit'init-name) (cgen-safe-name (~ unit'name)))))
 
+;; API
 (define-method cgen-unit-h-file ((unit <cgen-unit>))
   (~ unit'h-file))
 
+;; API
 (define-method cgen-unit-toplevel-nodes ((unit <cgen-unit>))
   (~ unit'toplevels))
 
+;; API
 (define (cgen-add! node)
   (and-let* ((unit (cgen-current-unit)))
     (slot-push! unit 'toplevels node))
@@ -111,25 +117,25 @@
         (cgen-emit node part)))
     (for-each walker (reverse (~ unit'toplevels)))))
 
+;; API
 (define-method cgen-emit-h ((unit <cgen-unit>))
   (and-let* ([h-file (cgen-unit-h-file unit)])
     (cgen-with-output-file h-file
-      (lambda ()
+      (^[]
         (cond [(~ unit'preamble) => emit-raw])
         (cgen-emit-part unit 'extern)))))
 
+;; API
 (define-method cgen-emit-c ((unit <cgen-unit>))
   (cgen-with-output-file (cgen-unit-c-file unit)
-    (lambda ()
+    (^[]
       (cond [(~ unit'preamble) => emit-raw])
-      (cond [(~ unit'pre-decl) => emit-raw])
-      (print "#include <gauche.h>")
       (cgen-emit-part unit 'decl)
       (cgen-emit-static-data unit)
       (cgen-emit-part unit 'body)
       (cond [(~ unit'init-prologue) => emit-raw]
             [else
-             (print "Scm__Init_"(cgen-safe-name (~ unit'name))"(void)")
+             (print "void Scm__Init_"(cgen-safe-name (~ unit'name))"(void)")
              (print "{")])
       (cgen-emit-part unit 'init)
       (cond [(~ unit'init-epilogue) => emit-raw]
@@ -144,12 +150,46 @@
 ;; Base node class
 ;;
 (define-class <cgen-node> ()
-  ((extern?        :init-keyword :extern? :init-value #f)
-   (cpp-condition  :init-keyword :cpp-condition
-                   :init-form (cgen-cpp-condition))
+  ([extern?        :init-keyword :extern? :init-value #f]
+   [cpp-conditions :init-keyword :cpp-condition
+                   :init-form (cgen-cpp-conditions)]
    ))
 
-(define cgen-cpp-condition (make-parameter #f))
+;; list of conditions.  internally used.
+(define %cgen-cpp-conditions (make-parameter '()))
+
+;; API
+(define (cgen-cpp-conditions) (%cgen-cpp-conditions))
+
+;; API
+(define-syntax cgen-with-cpp-condition
+  (syntax-rules ()
+    [(_ condition . body)
+     (let1 new-conditions (cons (cpp-condition->string condition)
+                                (%cgen-cpp-conditions))
+       (parameterize ([%cgen-cpp-conditions new-conditions])
+         . body))]))
+
+;; render cpp condition
+(define (cpp-condition->string condition)
+  (define (rec c)
+    (match c
+      [(and (or (? string?) (? symbol?) (? integer?)) c) (x->string c)]
+      [('defined c) #`"defined(,(rec c))"]
+      [('not c)     #`"!(,(rec c))"]
+      [('and c ...) (n-ary "&&" c)]
+      [('or c ...)  (n-ary "||" c)]
+      [((and (or '+ '*) op) c ...) (n-ary op c)]
+      [((and (or '- '/) op) c0 c1 ...)
+       (if (null? c1) #`",|op|(,(rec c0))" (n-ary op (cons c0 c1)))]
+      [((and (or '> '>= '== '< '<= '!= 'logand 'logior 'lognot '>> '<<) op) c0 c1)
+       (binary op c0 c1)]
+      [_ (error "Invalid C preprocessor condition expression:" condition)]))
+  (define (n-ary op cs)
+    (string-concatenate (intersperse (x->string op) (map (^c #`"(,c)") cs))))
+  (define (binary op c0 c1)
+    #`"(,(rec c0),|op|(,(rec c1))")
+  (rec condition))
 
 ;; fallback methods
 (define-method cgen-decl-common ((node <cgen-node>)) #f)
@@ -167,8 +207,8 @@
 (define-method cgen-node-traverse ((node <cgen-node>) walker)
   (do-ec (: slot (map slot-definition-name (class-slots (class-of node))))
          (if (slot-bound? node slot))
-         (and-let* ((var (slot-ref node slot))
-                    ( (is-a? var <cgen-node>) ))
+         (and-let* ([var (slot-ref node slot)]
+                    [ (is-a? var <cgen-node>) ])
            (walker var))))
 
 (define-method cgen-emit ((node <cgen-node>) part)
@@ -180,11 +220,12 @@
         [((? (cut eq? <> <cgen-node>))) #f]
         [_ #t])))
   (define (with-cpp-condition gf)
-    (cond [(~ node'cpp-condition)
-           => (^(cppc) (cond [(method-overridden? gf)
-                              (print "#if "cppc)
+    (cond [(~ node'cpp-conditions)
+           => (^[cppc] (cond [(method-overridden? gf)
+                              (for-each (cut print "#if " <>) cppc)
                               (gf node)
-                              (print "#endif /* "cppc" */")]
+                              (for-each (cut print "#endif /* "<>" */")
+                                        (reverse cppc))]
                              [else (gf node)]))]
           [else (gf node)]))
   (case part
@@ -223,7 +264,7 @@
 
 (define (cgen-decl . code)
   (cgen-add! (make <cgen-raw-decl> :code code)))
-   
+
 (define (cgen-body . code)
   (cgen-add! (make <cgen-raw-body> :code code)))
 
@@ -263,12 +304,12 @@
 
 (define-method cgen-emit-xtrn ((node <cgen-cpp-define>))
   (cpp-define-common node))
-(define-method cgen-emit-init ((node <cgen-cpp-define>))
+(define-method cgen-emit-decl ((node <cgen-cpp-define>))
   (cpp-define-common node))
 
-(define (cgen-define name . maybe-value)
+(define (cgen-define name :optional (value ""))
   (cgen-add!
-   (make <cgen-cpp-define> :name name :value (get-optional maybe-value ""))))
+   (make <cgen-cpp-define> :name name :value value)))
 
 ;;=============================================================
 ;; Utilities
@@ -276,10 +317,10 @@
 
 (define (cgen-with-output-file file thunk)
   (receive (port tmpfile) (sys-mkstemp file)
-    (guard (e (else 
+    (guard (e [else
                (close-output-port port)
                (sys-unlink tmpfile)
-               (raise e)))
+               (raise e)])
       (with-output-to-port port thunk)
       (close-output-port port)
       (sys-rename tmpfile file))))
@@ -292,15 +333,14 @@
 ;; Creates a C-safe name from Scheme string str
 (define (cgen-safe-name str)
   (with-string-io str
-    (lambda ()
-      (let loop ((b (read-byte)))
-        (cond ((eof-object? b))
-              ((or (<= 48 b 57)
-                   (<= 65 b 90)
-                   (<= 97 b 122))
-               (write-byte b) (loop (read-byte)))
-              (else
-               (format #t "_~2,'0x" b) (loop (read-byte))))))))
+    (^[] (let loop ((b (read-byte)))
+           (cond [(eof-object? b)]
+                 [(or (<= 48 b 57)
+                      (<= 65 b 90)
+                      (<= 97 b 122))
+                  (write-byte b) (loop (read-byte))]
+                 [else
+                  (format #t "_~2,'0x" b) (loop (read-byte))])))))
 
 ;; Like cgen-safe-name, but using more 'friendly' transliteration.
 ;; Used in genstub, since the transliterated name may be referred from
@@ -309,23 +349,22 @@
 ;; chance of name conflict.
 (define (cgen-safe-name-friendly str)
   (with-string-io str
-    (lambda ()
-      (let loop ((c (read-char)))
-        (unless (eof-object? c)
-          (case c
-            ((#\-) (let ((d (read-char)))
-                     (cond ((eqv? d #\>) (display "_TO") (loop (read-char)))
-                           (else         (display #\_) (loop d)))))
-            ((#\?) (display #\P) (loop (read-char)))
-            ((#\!) (display #\X) (loop (read-char)))
-            ((#\<) (display "_LT") (loop (read-char)))
-            ((#\>) (display "_GT") (loop (read-char)))
-            ((#\* #\> #\@ #\$ #\% #\^ #\& #\* #\+ #\= #\: #\. #\/ #\~)
-             (display #\_)
-             (display (number->string (char->integer c) 16))
-             (loop (read-char)))
-            (else (display c) (loop (read-char)))
-            ))))))
+    (^[] (let loop ([c (read-char)])
+           (unless (eof-object? c)
+             (case c
+               [(#\-) (let1 d (read-char)
+                        (cond [(eqv? d #\>) (display "_TO") (loop (read-char))]
+                              [else         (display #\_) (loop d)]))]
+               [(#\?) (display #\P) (loop (read-char))]
+               [(#\!) (display #\X) (loop (read-char))]
+               [(#\<) (display "_LT") (loop (read-char))]
+               [(#\>) (display "_GT") (loop (read-char))]
+               [(#\* #\> #\@ #\$ #\% #\^ #\& #\* #\+ #\= #\: #\. #\/ #\~)
+                (display #\_)
+                (display (number->string (char->integer c) 16))
+                (loop (read-char))]
+               [else (display c) (loop (read-char))]
+               ))))))
 
 ;; Escape  '*/' so that str can be inserted safely within a comment.
 (define (cgen-safe-comment str)
