@@ -1,23 +1,23 @@
 ;;;
 ;;; file related utility functions.  to be autoloaded.
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,13 +29,12 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 #!no-fold-case
 
 (define-module gauche.fileutil
-  (export file-exists? file-is-regular? file-is-directory?
-          glob glob-fold glob-component->regexp make-glob-fs-fold
+  (export glob glob-fold glob-component->regexp make-glob-fs-fold
           sys-stat->file-type sys-stat->mode sys-stat->ino
           sys-stat->dev sys-stat->rdev sys-stat->nlink
           sys-stat->size sys-stat->uid sys-stat->gid
@@ -44,16 +43,6 @@
   )
 (select-module gauche.fileutil)
 
-(define (file-exists? path)
-  (sys-access path F_OK))
-(define (file-is-regular? path)
-  (and (sys-access path F_OK)
-       (eq? (slot-ref (sys-stat path) 'type) 'regular)))
-(define (file-is-directory? path)
-  (and (sys-access path F_OK)
-       (eq? (slot-ref (sys-stat path) 'type) 'directory)))
-
-
 ;; system object accessors (for backward compatibility)
 (define (sys-stat->file-type s)  (slot-ref s 'type))
 (define (sys-stat->mode s)  (slot-ref s 'mode))
@@ -70,7 +59,7 @@
 (define (sys-stat->type s)  (slot-ref s 'type))
 
 (define (sys-tm->alist tm)
-  (map (lambda (n s) (cons n (slot-ref tm s)))
+  (map (^[n s] (cons n (slot-ref tm s)))
        '(tm_sec tm_min tm_hour tm_mday tm_mon tm_year tm_wday tm_yday tm_isdst)
        '(sec min hour mday mon year wday yday isdst)))
 
@@ -103,17 +92,14 @@
 
 ;; NB: we avoid util.match due to the hairy dependency problem.
 (define (glob-fold-1 pattern proc seed opts)
-  (let-keywords opts ((separator #[/])
-                      (folder glob-fs-folder))
+  (let-keywords opts ([separator #[/]]
+                      [folder glob-fs-folder])
     (define (rec node matcher seed)
       (cond [(null? matcher) seed]
-            [(eq? (car matcher) '**)
-             (rec* node (cdr matcher) seed)]
-            [(null? (cdr matcher))
-             (folder proc seed node (car matcher) #f)]
-            [else
-             (folder (lambda (node seed) (rec node (cdr matcher) seed))
-                     seed node (car matcher) #t)]))
+            [(eq? (car matcher) '**) (rec* node (cdr matcher) seed)]
+            [(null? (cdr matcher)) (folder proc seed node (car matcher) #f)]
+            [else (folder (^[node seed] (rec node (cdr matcher) seed))
+                          seed node (car matcher) #t)]))
     (define (rec* node matcher seed)
       (fold (cut rec* <> matcher <>)
             (rec node matcher seed)
@@ -139,20 +125,20 @@
 ;; Hopefully we'll put some optimization here, making single traversal
 ;; for the common prefix.
 ;;
-;; The treatment of backslashes is tricky. 
+;; The treatment of backslashes is tricky.
 ;;
 (define (glob-expand-braces pattern seed)
   (define (parse str pres level)
-    (let loop ((str str)
-               (segs pres))
+    (let loop ([str str]
+               [segs pres])
       (cond
        [(rxmatch #/[{}]/ str) =>
-        (lambda (m)
+        (^m
           (cond [(equal? (m 0) "{")
                  (receive (ins post) (parse (m'after) '("") (+ level 1))
                    (loop post
-                         (fold (lambda (seg seed)
-                                 (fold (lambda (in seed)
+                         (fold (^[seg seed]
+                                 (fold (^[in seed]
                                          (cons (string-append seg in) seed))
                                        seed ins))
                                '()
@@ -179,7 +165,7 @@
   (regexp-compile
    (regexp-optimize
     (with-input-from-string pattern
-      (lambda ()
+      (^[]
         (define (element0 ch ct)
           (case ch
             [(#\*) (element0* (n) ct)]
@@ -201,7 +187,7 @@
                    ;; we have to treat [!...] as [^...]
                    [(#\!) (n)
                     (let1 cs (read-char-set (current-input-port))
-                      (cons (%char-set-complement! cs) (element1 (n) ct)))]
+                      (cons (char-set-complement! cs) (element1 (n) ct)))]
                    [else
                     (let1 cs (read-char-set (current-input-port))
                       (cons cs (element1 (n) ct)))])]
@@ -215,19 +201,18 @@
 ;; if rx is just test perfect match, e.g. #/^string$/, returns
 ;; string portion.
 (define (fixed-regexp? rx)
-  (let ((ast (regexp-ast rx)))
+  (let1 ast (regexp-ast rx)
     (and (> (length ast) 4)
          (eq? (caddr ast) 'bol)
-         (let loop ((cs (cdddr ast)) (r '()))
+         (let loop ([cs (cdddr ast)] [r '()])
            (cond [(null? (cdr cs))
                   (and (eq? (car cs) 'eol) (list->string (reverse r)))]
                  [(char? (car cs)) (loop (cdr cs) (cons (car cs) r))]
                  [else #f])))))
 
 (define (make-glob-fs-fold :key (root-path #f) (current-path #f))
-  (let ((separ (cond-expand
-                (gauche.os.windows "\\")
-                (else "/"))))
+  (let1 separ (cond-expand [gauche.os.windows "\\"]
+                           [else "/"])
     (define (ensure-dirname s)
       (and s
            (or (and-let* ([len (string-length s)]
@@ -238,7 +223,7 @@
                s)))
     (define root-path/    (ensure-dirname root-path))
     (define current-path/ (ensure-dirname current-path))
-    (lambda (proc seed node regexp non-leaf?)
+    (^[proc seed node regexp non-leaf?]
       (let1 prefix (case node
                      [(#t) (or root-path/ separ)]
                      [(#f) (or current-path/ "")]
@@ -259,7 +244,7 @@
                           (proc full seed)
                           seed)))]
               [else
-               (fold (lambda (child seed)
+               (fold (^[child seed]
                        (or (and-let* ([ (regexp child) ]
                                       [full (string-append prefix child)]
                                       [ (or (not non-leaf?)
