@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.generator - generator framework
 ;;;
-;;;   Copyright (c) 2011-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -41,14 +41,15 @@
           bits->generator reverse-bits->generator
           file->generator file->sexp-generator file->char-generator
           file->line-generator file->byte-generator
-          port->char-generator port->byte-generator
+          port->sexp-generator port->char-generator
+          port->line-generator port->byte-generator
           x->generator generate
 
-          generator->list null-generator gcons* gappend
+          generator->list null-generator gcons* gappend gconcatenate gmerge
           circular-generator gunfold giota grange
-          gmap gmap-accum gfilter gfilter-map gstate-filter
-          gtake gdrop gtake-while gdrop-while grxmatch glet* glet1
-          do-generator
+          gmap gmap-accum gfilter gfilter-map gstate-filter gbuffer-filter
+          gtake gdrop gtake-while gdrop-while grxmatch gslices
+          glet* glet1 do-generator
           ))
 (select-module gauche.generator)
 
@@ -96,16 +97,14 @@
     (^[] (read-char p))))
 
 (define (bits->generator n :optional (start #f) (end #f))
-  (let* ([len-1 (- (integer-length n) 1)]
-         [k     (- len-1 (or start 0))]
-         [end   (if end (- len-1 end) -1)])
-    (^[] (if (<= k end) (eof-object) (%begin0 (logbit? k n) (dec! k))))))
+  (let* ([limit (or end (integer-length n))]
+         [k     (or start 0)])
+    (^[] (if (>= k limit) (eof-object) (%begin0 (logbit? k n) (inc! k))))))
 
 (define (reverse-bits->generator n :optional (start #f) (end #f))
-  (let* ([len-1 (- (integer-length n) 1)]
-         [start (- len-1 (or start 0))]
-         [k     (if end (- len-1 end -1) 0)])
-    (^[] (if (> k start) (eof-object) (%begin0 (logbit? k n) (inc! k))))))
+  (let* ([k     (- (or end (integer-length n)) 1)]
+         [limit (or start 0)])
+    (^[] (if (< k limit) (eof-object) (%begin0 (logbit? k n) (dec! k))))))
 
 (define (file->generator filename reader . open-args)
   ;; If the generator is abanboned before reaching EOF, we have to rely
@@ -128,6 +127,8 @@
   (apply file->generator filename read-byte open-args))
 
 ;; simple, but useful
+(define (port->sexp-generator port) (cut read port))
+(define (port->line-generator port) (cut read-line port))
 (define (port->char-generator port) (cut read-char port))
 (define (port->byte-generator port) (cut read-byte port))
 
@@ -155,7 +156,7 @@
 ;; function dispatch for common objects.
 (define (%->gen x)
   (cond [(procedure? x) x]
-        [(is-a? x <pair>) (list->generator x)] ;avoid pair? not to force lazy pairs
+        [(is-a? x <pair>) (list->generator x)] ;prevent pair? from forcing lazy pairs
         [(vector? x) (vector->generator x)]
         [(string? x) (string->generator x)]
         [(is-a? x <collection>) (x->generator x)]
@@ -207,22 +208,43 @@
                 (tail)]
                [else (%begin0 (f seed) (set! seed (g seed)))]))))
 
-(define (giota num :optional (start 0) (step 1))
-  (let1 val start
-    (^[] (if (<= num 0)
-           (eof-object)
-           (%begin0 val (inc! val step) (dec! num))))))
+(define (giota :optional (count +inf.0) (start 0) (step 1))
+  (if (and (exact? start) (exact? step))
+    (let1 val start
+      ;; NB: We allow count < 0 to mean "infinite", for the consistentcy
+      ;; of stream-iota.
+      (if (or (infinite? count) (< count 0))
+        (^[] (%begin0 val (inc! val step)))
+        (^[] (if (<= count 0)
+               (eof-object)
+               (%begin0 val (inc! val step) (dec! count))))))
+    (let ([val (inexact start)]
+          [k   0])
+      (if (or (infinite? count) (< count 0))
+        (^[] (%begin0 (+ val (* k step)) (inc! k)))
+        (^[] (if (<= count 0)
+               (eof-object)
+               (%begin0 (+ val (* k step)) (inc! k) (dec! count))))))))
 
 (define (grange :optional (start 0) (end +inf.0) (step 1))
-  (let1 val start
-    (^[] (if (>= val end)
-           (eof-object)
-           (%begin0 val (inc! val step))))))
+  (if (and (exact? start) (exact? step))
+    (let1 val start
+      (^[] (if (>= val end)
+             (eof-object)
+             (%begin0 val (inc! val step)))))
+    (let ([val (inexact start)]
+          [k   0])
+      (^[] (let1 v (+ val (* k step))
+             (if (>= v end)
+               (eof-object)
+               (begin (inc! k) v)))))))
 
 ;;;
 ;;; Generator operations
 ;;;
 
+;; data Generator a = () -> a
+
 ;; A monadic syntax sugar
 (define-syntax glet*
   (syntax-rules ()
@@ -257,7 +279,7 @@
            body ...
            (loop))))]))
 
-;; gcons* :: (a, ..., () -> a) -> (() -> a)
+;; gcons* :: (a, ..., Generator a) -> Generator a
 (define gcons*
   (case-lambda
     [() (error "gcons* needs at least one argument")]
@@ -269,7 +291,7 @@
      (let* ([lp (last-pair args)] [g (%->gen (car lp))])
        (^[] (if (eq? args lp) (g) (pop! args))))]))
 
-;; gappend :: [() -> a] -> (() -> a)
+;; gappend :: [Generator a] -> Generator a
 (define (gappend . gens)
   (let ([gs gens] [g #f])
     (if (null? gs)
@@ -281,7 +303,40 @@
                 [(null? gs) v]          ;exhausted
                 [else (set! g #f) (f)]))))))
 
-;; gmap :: (a -> b, () -> a) -> (() -> b)
+;; gconcatenate :: Generator Generator a -> Generator a
+(define (gconcatenate gen)
+  (let* ([gen (%->gen gen)]
+         [g (gen)])
+    (^[] (let loop ()
+           (if (eof-object? g)
+             g
+             (let1 v (g)
+               (if (eof-object? v)
+                 (begin (set! g (%->gen (gen))) (loop))
+                 v)))))))
+
+;; gmerge :: ((a, a)->Bool, Generator a, Generator a,...) -> Generator a
+(define gmerge
+  (case-lambda
+    [(prec gen) (%->gen gen)]
+    [(prec gen1 gen2)
+     (let ([gen1 (%->gen gen1)]
+           [gen2 (%->gen gen2)])
+       (let ([e1 (gen1)]
+             [e2 (gen2)])
+         (^[] (if (eof-object? e1)
+                (if (eof-object? e2)
+                  e2
+                  (begin0 e2 (set! e2 (gen2))))
+                (if (eof-object? e2)
+                  (begin0 e1 (set! e1 (gen1)))
+                  (if (prec e1 e2)
+                    (begin0 e1 (set! e1 (gen1)))
+                    (begin0 e2 (set! e2 (gen2)))))))))]
+    [(prec . gens)
+     (apply gmerge prec (map (cut apply gmerge prec <>) (slices gens 2)))]))
+
+;; gmap :: (a -> b, Generator a) -> Generator b
 (define gmap
   (case-lambda
     [(fn gen)
@@ -292,7 +347,7 @@
        (^[] (let1 vs (map (^f (f)) gs)
               (if (any eof-object? vs) (eof-object) (apply fn vs)))))]))
 
-;; gmap-accum :: ((a,b) -> (c,b), b, () -> a) -> (() -> c)
+;; gmap-accum :: ((a,b) -> (c,b), b, Generator a) -> Generator c
 (define gmap-accum
   (case-lambda
     [(fn seed gen)
@@ -311,7 +366,7 @@
                 (set! seed seed_)
                 v_))))]))
 
-;; gfilter :: (a -> Bool, () -> a) -> (() -> a)
+;; gfilter :: (a -> Bool, Generator a) -> Generator a
 (define (gfilter pred gen)
   (let1 gen (%->gen gen)
     (^[] (let loop ([v (gen)])
@@ -319,7 +374,7 @@
                  [(pred v) v]
                  [else (loop (gen))])))))
 
-;; gfilter-map :: (a -> b, () -> a) -> (() -> b)
+;; gfilter-map :: (a -> b, Generator a) -> Generator b
 (define gfilter-map
   (case-lambda
     [(fn gen)
@@ -335,19 +390,39 @@
                       [(apply fn vs)]
                       [else (loop)])))))]))
 
-;; gstate-filter :: ((a,b) -> (Bool,b), b, () -> a) -> (() -> a)
+;; gstate-filter :: ((a,b) -> (Bool,b), b, Generator a) -> Generator a
 (define (gstate-filter proc seed gen)
   (let1 gen (%->gen gen)
-    (^[] (let loop ()
-           (glet1 v (gen)
-             (receive (flag seed1) (proc v seed)
-               (set! seed seed1)
-               (if flag v (loop))))))))
+    (rec (loop)
+      (glet1 v (gen)
+        (receive (flag seed1) (proc v seed)
+          (set! seed seed1)
+          (if flag v (loop)))))))
 
-;; gtake :: (() -> a, Int) -> (() -> a)
-;; gdrop :: (() -> a, Int) -> (() -> a)
+;; gbuffer-filter :: ((a,b)->([c],b), b, Generator a, b->[c]) -> Generator c
+(define (gbuffer-filter proc seed gen :optional (tail-gen (^s '())))
+  (let ([gen (%->gen gen)]
+        [exhausted? #f]
+        [buffered '()]
+        [seed seed])
+    (rec (loop)
+      (cond [(pair? buffered) (pop! buffered)]
+            [exhausted? (eof-object)]
+            [else (let1 v (gen)
+                    (if (eof-object? v)
+                      (begin
+                        (set! buffered (tail-gen seed))
+                        (set! exhausted? #t)
+                        (loop))
+                      (receive (vals seed.) (proc v seed)
+                        (set! seed seed.)
+                        (set! buffered vals)
+                        (loop))))]))))
+
+;; gtake :: (Generator a, Int) -> Generator a
+;; gdrop :: (Generator a, Int) -> Generator a
 (define (gtake gen n :optional (fill? #f) (padding #f))
-  (let ([k 0] [gen (%->gen gen)])
+  (let ([k 0] [end #f] [gen (%->gen gen)])
     (if fill?
       (^[] (if (< k n)
              (let1 v (gen)
@@ -359,8 +434,8 @@
   (let ([k 0] [gen (%->gen gen)])
     (^[] (when (< k n) (dotimes [i n] (inc! k) (gen))) (gen))))
 
-;; gtake-while :: (a -> Bool, () -> a) -> (() -> a)
-;; gdrop-while :: (a -> Bool, () -> a) -> (() -> a)
+;; gtake-while :: (a -> Bool, Generator a) -> Generator a
+;; gdrop-while :: (a -> Bool, Generator a) -> Generator a
 (define (gtake-while pred gen)
   (let ([end? #f] [gen (%->gen gen)])
     (^[] (if end?
@@ -379,7 +454,7 @@
                  (loop)
                  (begin (set! found? #t) v))))))))
 
-;; generate :: ((a -> ()) -> ()) -> (() -> a)
+;; generate :: ((a -> ()) -> ()) -> Generator a
 (define (generate proc)
   (define (cont)
     (reset (proc (^[value] (shift k (set! cont k) value)))
@@ -387,8 +462,8 @@
            (eof-object)))
   (^[] (cont)))
 
-;; grxmatch :: (Regexp, (() -> char)) -> (() -> RegMatch)
-;;          |  (Regexp, String) -> (() -> RegMatch)
+;; grxmatch :: (Regexp, Generator Char) -> Generator RegMatch
+;;          |  (Regexp, String) -> Generator RegMatch
 (define (grxmatch rx gen)
   (if (string? gen)
     (let1 str gen
@@ -426,6 +501,21 @@
               (expand-buffer #f)))
           (eof-object))))))
 
+;; gslices :: (Generator a, Int) -> Generator [a]
+(define (gslices gen k :optional (fill? #f) (padding #f))
+  ;; It's tempting to do this:
+  ;; (let1 elts (generator->list (gtake gen k fill? padding))
+  ;;   (if (null? elts) (eof-object) elts)))))
+  ;; but it doesn't work, since gtake with fill?=#t always returns generator
+  ;; of k items, even the input generator is exhausted.
+  (let1 gen (%->gen gen)
+    (^[] (let* ([elts (generator->list gen k)]
+                [len (length elts)])
+           (cond [(null? elts) (eof-object)]
+                 [(= len k) elts]
+                 [fill? (append elts (make-list (- k len) padding))]
+                 [else elts])))))
+
 ;; TODO:
 ;;  (gen-ec (: i ...) ...)
 ;;    srfi-42-ish macro to create generator.  it's not "eager", so
