@@ -1,7 +1,7 @@
 ;;;
 ;;; library utilities - to be autoloaded.
 ;;;
-;;;   Copyright (c) 2003-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2003-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,7 @@
   (use srfi-1)
   (use srfi-13)
   (export library-exists? library-fold library-map library-for-each
-          library-has-module?))
+          library-has-module? library-name->module-name))
 (select-module gauche.libutil)
 
 ;; library-fold - iterate over the modules or libraries whose name matches
@@ -60,32 +60,30 @@
   ;; file - path components after prefix, e.g. gauche/mop
   ;; base - the last component of file, e.g. mop
   (define (search pats prefix file base seed)
-    (let* ((path (topath prefix file)))
-      (cond ((and (not (null? (cdr pats)))
+    (let1 path (topath prefix file)
+      (cond [(and (not (null? (cdr pats)))
                   (match? (car pats) base)
                   (file-is-directory? path))
-             (fold (lambda (subfile seed)
+             (fold (^[subfile seed]
                      (search (cdr pats) prefix #`",|file|/,|subfile|"
                              subfile seed))
-                   seed (readdir path)))
-            ((and (null? (cdr pats))
+                   seed (readdir path))]
+            [(and (null? (cdr pats))
                   (or (string-suffix? ".scm" base)
                       (string-suffix? ".sci" base))
                   (match? (car pats) (string-drop-right base 4))
                   (file-exists? path))
-             (cond (allow-duplicates?  (proc (ensure path file) path seed))
-                   ((member file seen) seed)
-                   ((ensure path file)
-                    => (lambda (mod)
-                         (push! seen file)
-                         (proc mod path seed)))
-                   (else seed)))
-            (else seed))))
+             (cond [allow-duplicates?  (proc (ensure path file) path seed)]
+                   [(member file seen) seed]
+                   [(ensure path file)
+                    => (^[mod] (push! seen file) (proc mod path seed))]
+                   [else seed])]
+            [else seed])))
 
   (define (match? pat component)
     (let1 rx (module-glob-pattern->regexp pat)
-      (cond ((rx component) => (lambda (m) (m 0)))
-            (else #f))))
+      (cond [(rx component) => (^m (m 0))]
+            [else #f])))
   (define (ensure path file)
     (if search-module?
       (let1 modname (path->module-name (string-drop-right file 4))
@@ -95,12 +93,11 @@
       (string-drop-right file 4)))
 
   ;; main body
-  (let ((pats (if search-module?
-                (string-split (x->string pattern) #\.)
-                (string-split pattern #\/))))
-    (fold (lambda (prefix seed)
-            (fold (lambda (file seed)
-                    (search pats prefix file file seed))
+  (let1 pats (if search-module?
+               (string-split (x->string pattern) #\.)
+               (string-split pattern #\/))
+    (fold (^[prefix seed]
+            (fold (^[file seed] (search pats prefix file file seed))
                   seed (readdir prefix)))
           seed paths))
 
@@ -136,33 +133,36 @@
            (or (and (string? mod/path) (provided? mod/path))
                (and (symbol? mod/path) (find-module mod/path))))
       ;; scan the filesystem
-      (call/cc
-       (lambda (found)
-         (library-fold mod/path
-                       (lambda (mod path seed) (found #t))
-                       #f
-                       :strict? strict? :paths paths)))))
+      (let/cc found
+        (library-fold mod/path
+                      (^[mod path seed] (found #t))
+                      #f
+                      :strict? strict? :paths paths))))
 
 ;; Convenience wrappers
 (define (library-map mod/path proc . opts)
   (reverse! (apply library-fold mod/path
-                   (lambda (mod path seed) (cons (proc mod path) seed))
+                   (^[mod path seed] (cons (proc mod path) seed))
                    '() opts)))
 
 (define (library-for-each mod/path proc . opts)
-  (apply library-fold mod/path (lambda (mod path seed) (proc mod path))
-         '() opts))
+  (apply library-fold mod/path (^[mod path seed] (proc mod path)) '() opts))
 
 ;; Try to determine the file is a module source file
 ;;  NB: this will be more involved when we allow more than one modules
 ;;  in a file.  Or should we?
 (define (library-has-module? file name)
-  (let1 exp (guard (e (else #f))
+  (let1 exp (guard (e [else #f])
               (with-input-from-file file read :if-does-not-exist #f))
     (and (pair? exp)
-         (eq? (car exp) 'define-module)
-         (pair? (cdr exp))
-         (eq? (cadr exp) name)
+         (or (and (eq? (car exp) 'define-module)
+                  (pair? (cdr exp))
+                  (eq? (cadr exp) name))
+             (and (eq? (car exp) 'define-library)
+                  (pair? (cdr exp))
+                  ;; This should be optimized---it's waste of time running
+                  ;; library-name->module-name for every library on the way.
+                  (eq? (library-name->module-name (cadr exp)) name)))
          file)))
 
 ;; Auxiliary procedures
@@ -171,10 +171,10 @@
 ;; readdir with removing "." and "..".
 (define (readdir dir)
   (if (file-is-directory? dir)
-    (let loop ((p (sys-readdir dir)) (r '()))
-      (cond ((null? p) (reverse! r))
-            ((member (car p) '("." "..")) (loop (cdr p) r))
-            (else (loop (cdr p) (cons (car p) r)))))
+    (let loop ([p (sys-readdir dir)] (r '()))
+      (cond [(null? p) (reverse! r)]
+            [(member (car p) '("." "..")) (loop (cdr p) r)]
+            [else (loop (cdr p) (cons (car p) r))]))
     '()))
 
 (define (topath prefix file)
@@ -182,20 +182,44 @@
 
 ;; NB: this is also used by gauche.reload
 (define (module-glob-pattern->regexp pat)
-  (guard (e (else (error "bad glob pattern" pat)))
+  (guard (e [else (error "bad glob pattern" pat)])
     (string->regexp
      (with-string-io pat
-       (lambda ()
+       (^[]
          (display "^")
-         (let loop ((c (read-char)))
+         (let loop ([c (read-char)])
            (unless (eof-object? c)
              (case c
-               ((#\?) (display "[^.]") (loop (read-char)))
-               ((#\*) (display "[^.]*") (loop (read-char)))
-               ((#\\) (let1 c2 (read-char)
+               [(#\?) (display "[^.]") (loop (read-char))]
+               [(#\*) (display "[^.]*") (loop (read-char))]
+               [(#\\) (let1 c2 (read-char)
                         (unless (eof-object? c2)
-                          (write-char c2) (loop (read-char)))))
-               ((#\.) (display "\\.") (loop (read-char)))
-               (else (write-char c) (loop (read-char))))))
+                          (write-char c2) (loop (read-char))))]
+               [(#\.) (display "\\.") (loop (read-char))]
+               [else (write-char c) (loop (read-char))])))
          (display "$"))))))
 
+;; mapping R7RS library name to Gauche module name
+;;
+;; We simply maps R7RS (foo bar baz) to Gauche's foo.bar.baz . The caveat
+;; is that R7RS library name component may include dots.  We map them in R7RS
+;; library names into consecutive dots in Gauche module name, which will be
+;; mapped to a single dot again in pathnames.
+;; (The latter translation is done by module-name->path and path->module-name
+;; in src/libmod.scm)
+;;
+;;  R7RS library name   Gauche module name      File pathname
+;;
+;;  (foo bar baz)       foo.bar.baz             foo/bar/baz
+;;  (foo b.r baz)       foo.b..r.baz            foo/b.r/baz
+;;
+;; TODO: R7RS library name can contain weird characters, and we need a rule
+;; to map them.
+(define (library-name->module-name libname)
+  (define (stringify x)
+    (cond [(keyword? x) (write-to-string x)]
+          [(symbol? x) (symbol->string x)]
+          [(and (integer? x) (exact? x) (>= x 0)) (number->string x)]
+          [else (error "Bad name component in library name:" x)]))    
+  ($ string->symbol $ (cut string-join <> ".")
+     $ map ($ (cut regexp-replace-all #/\./ <> "..") $ stringify $) libname))
