@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.package - package management
-;;;  
-;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; *EXPERIMENTAL*
 ;; The spec might be changed in future.  Do not count on the current version.
@@ -97,12 +97,12 @@
    ))
 
 (define (path->gauche-package-description path)
-  (guard (e ((or (<io-error> e) (<read-error> e))
+  (guard (e [(or (<io-error> e) (<read-error> e))
              (errorf "couldn't read the package description ~s: ~a"
-                     path (ref e 'message))))
+                     path (ref e 'message))])
     (call-with-input-file path
-      (lambda (in)
-        (let ((f (read in)))
+      (^[in]
+        (let1 f (read in)
           (if (and (list? f)
                    (>= (length f) 2)
                    (eq? (car f) 'define-gauche-package)
@@ -112,17 +112,15 @@
             (error "malformed define-gauche-package")))))))
 
 (define-method write-gauche-package-description
-    ((desc <gauche-package-description>) . maybe-port)
-  (let ((out (get-optional maybe-port (current-output-port))))
-    (format out ";; -*-Scheme-*-\n")
-    (format out ";; Gauche Package Description\n\n")
-    (format out "(define-gauche-package ~s\n" (ref desc 'name))
-    (for-each (lambda (slot)
-                (unless (eq? slot 'name)
-                  (format out "  :~a ~s\n" slot (ref desc slot))))
-              (map slot-definition-name
-                   (class-slots <gauche-package-description>)))
-    (format out ")\n")))
+    ((desc <gauche-package-description>) :optional (out (current-output-port)))
+  (format out ";; -*-Scheme-*-\n")
+  (format out ";; Gauche Package Description\n\n")
+  (format out "(define-gauche-package ~s\n" (ref desc 'name))
+  (for-each (^[slot] (unless (eq? slot 'name)
+                       (format out "  :~a ~s\n" slot (ref desc slot))))
+            (map slot-definition-name
+                 (class-slots <gauche-package-description>)))
+  (format out ")\n"))
 
 ;;;
 ;;; Searching .gpd files
@@ -145,31 +143,25 @@
 
 ;; scan the directory to find older verison of Gauche library directories.
 (define (get-all-version-paths)
-  (let ((xpaths '()))
-    (dolist (path *load-path*)
-      (cond ((#/\/\d+(\.\d+)*[^\/]*\/lib\/?$/ path)
-             => (lambda (m)
-                  (let* ((base (m 'before))
-                         (dirs (directory-list base
+  (apply append
+         *load-path*
+         (filter-map
+          (^p (and-let* ([m (#/\/\d+(\.\d+)*[^\/]*\/lib\/?$/ p)]
+                         [base (m 'before)]
+                         [ (file-is-directory? base) ]
+                         [dirs (directory-list base
                                                :children? #t :add-path? #t
-                                               :filter #/^\d+(\.\d+)*[^\/]*$/))
-                         (pdirs (map (cut string-append <> "/lib") dirs))
-                         )
-                    (set! xpaths
-                          (append (sort (delete path pdirs)
-                                        (lambda (a b)
-                                          (version>? (sys-basename a)
-                                                     (sys-basename b))))
-                                  xpaths)))))
-            ))
-    (append *load-path* xpaths)))
+                                               :filter #/^\d+(\.\d+)*[^\/]*$/)]
+                         [pdirs (map (cut string-append <> "/lib") dirs)])
+                (sort-by (delete p pdirs) sys-basename version>?)))
+          *load-path*)))
 
 ;; Iterator protocol
 (define-method call-with-iterator ((gpd <gauche-package-description-paths>)
                                    proc . args)
-  (let ((paths (ref gpd 'paths))
-        (files '())
-        (visited (make-hash-table 'string=?)))
+  (let ([paths (ref gpd 'paths)]
+        [files '()]
+        [visited (make-hash-table 'string=?)])
     (define (interesting? path)
       (and (#/\.gpd$/ path)
            (not (hash-table-get visited (sys-basename path) #f))
@@ -180,23 +172,22 @@
         (let loop ()
           (if (null? paths)
             #f
-            (let ((dir (build-path (pop! paths) ".packages")))
-              (cond ((file-is-directory? dir)
+            (let1 dir (build-path (pop! paths) ".packages")
+              (cond [(file-is-directory? dir)
                      (set! files
                            (directory-list dir
                                            :add-path? #t :filter interesting?))
-                     (pick-next))
-                    (else (loop))))))
+                     (pick-next)]
+                    [else (loop)]))))
         (pop! files)))
-    (let ((buf (pick-next)))
+    (let1 buf (pick-next)
       (define (end?) (not buf))
       (define (next) (begin0 buf (set! buf (pick-next))))
       (proc end? next))))
 
 ;; utility
 (define (find-gauche-package-description name :key (all-versions #f))
-  (and-let* ((path (find (string->regexp #`"/,|name|\\.gpd$")
+  (and-let* ([path (find (string->regexp #`"/,|name|\\.gpd$")
                          (gauche-package-description-paths
-                          :all-versions all-versions))))
+                          :all-versions all-versions))])
     (path->gauche-package-description path)))
-
