@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.package.compile - compile extensions
-;;;  
-;;;   Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2005-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; *EXPERIMENTAL*
 ;; gauche.package.compile kicks C compiler/linker to compile an
@@ -106,7 +106,7 @@
                  [in-place-dir gauche-builddir])
     (unless (and (file-exists? sofile)
                  (every (cut file-mtime>? sofile <>) ofiles))
-      (let1 all-ofiles (string-join (map (lambda (f) #`"',f'") ofiles) " ")
+      (let1 all-ofiles (string-join (map (^f #`"',f'") ofiles) " ")
         (run #`",(or ld CC) ,(or ldflags \"\") ,(LIBDIR) ,LDFLAGS ,sofile ,all-ofiles ,LIBS ,(or libs \"\")")))))
 
 (define (gauche-package-compile-and-link module-name files . args)
@@ -147,7 +147,7 @@
                    ;; NB: we wrap to-dir by closure to for the case if
                    ;; to-dir includes submatch replacement spec.
                    [new (regexp-replace-all (string->regexp orig-dir) olddirs
-                                            (lambda (m) #`",|to-dir|/src"))])
+                                            (^m #`",|to-dir|/src"))])
           (if (equal? dir-key "--sysincdir")
             #`",|new|,|sep|,|to-dir|/gc/include"
             new))
