@@ -1,23 +1,23 @@
 ;;;
 ;;; process.scm - process interface
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 #!no-fold-case
 
@@ -58,7 +58,7 @@
           ))
 (select-module gauche.process)
 
-;; Delay-load gauche.charconv 
+;; Delay-load gauche.charconv
 (autoload gauche.charconv
           wrap-with-input-conversion wrap-with-output-conversion)
 (autoload gauche.uvector
@@ -115,7 +115,7 @@
     (%run-process-old command args) ;; backward compatibility
     (%run-process-new command args)))
 
-;; Note: I/O redirection 
+;; Note: I/O redirection
 ;;  'Redirects' keyword argument is a generic way to wire child's I/Os.
 ;;  It takes a list of <io-spec>s, where each <io-spec> can be one of
 ;;  the followings:
@@ -164,9 +164,9 @@
 (define (%run-process-new command args)
   (let-keywords* args ((input  #f) (output #f) (error  #f)
                        (redirects '())
-                       (wait   #f) (fork   #t) 
+                       (wait   #f) (fork   #t)
                        (host   #f)    ;remote execution
-                       (sigmask #f) (directory #f))
+                       (sigmask #f) (directory #f) (detached #f))
     (let* ([redirs (%canon-redirects redirects input output error)]
            [argv (map x->string command)]
            [proc (make <process> :command (car argv))]
@@ -182,21 +182,23 @@
         (if fork
           (let1 pid (sys-fork-and-exec (car argv) argv
                                        :iomap iomap :directory dir
-                                       :sigmask (%ensure-mask sigmask))
+                                       :sigmask (%ensure-mask sigmask)
+                                       :detached detached)
             (push! (ref proc 'processes) proc)
             (set!  (ref proc 'pid) pid)
             (dolist (p toclose)
               (if (input-port? p)
                 (close-input-port p)
                 (close-output-port p)))
-            (when wait
+            (when (and wait (not detached))
               ;; the following expr waits until the child exits
               (set! (ref proc 'status) (values-ref (sys-waitpid pid) 1))
               (update! (ref proc 'processes) (cut delete proc <>)))
             proc)
           (sys-exec (car argv) argv
                     :iomap iomap :directory dir
-                    :sigmask (%ensure-mask sigmask)))))))
+                    :sigmask (%ensure-mask sigmask)
+                    :detached detached))))))
 
 (define (%canon-redirects redirects in out err)
   (rlet1 redirs
@@ -232,7 +234,7 @@
                  (car arg) a2 (cadr arg)))]
       [(<& >&) (unless (integer? a2)
                  (errorf "redirection '~a' requires an integer file \
-                          descriptor, but got ~s" a2))]
+                          descriptor, but got ~s" (car arg) a2))]
       [else (errorf "invalid redirection entry: ~s" arg)])))
 
 (define (%check-directory dir)
@@ -257,7 +259,7 @@
 
 ;; Prepare remote execution via ssh
 (define (%prepare-remote host argv dir)
-  (rxmatch-let (#/^(?:([\w-]+):)?(?:([\w-]+)@)?([\w._]+)(?::(\d+))?$/ host)
+  (rxmatch-let (#/^(?:([\w-]+):)?(?:([\w-]+)@)?([-\w.]+)(?::(\d+))?$/ host)
       (#f proto user server port)
     (unless (or (not proto) (equal? proto "ssh"))
       (error "Remote execution protocol other than 'ssh' is not supported:"
@@ -279,7 +281,7 @@
                                            :canonicalize #t)
             (set! val v))))))
 
-;; Build I/O map 
+;; Build I/O map
 (define (%setup-iomap proc redirs)
 
   (define toclose '())  ;list of ports to be closed in parent
@@ -309,7 +311,7 @@
     ;; child's fds.  fd1 may be remapped from the current process, so
     ;; we first search iomap.  If it hasn't remapped, we transfer the
     ;; current process's fd1 to the child.
-    ;; TODO: need to check the direction of 
+    ;; TODO: need to check the direction of
     (if-let1 p (assv fd1 iomap)
       (push! iomap `(,fd0 . ,(cdr p)))
       (guard (e [(<system-error> e)
@@ -320,12 +322,12 @@
         ;; try dup2 it into a dummy port to see if fd1 is valid.
         ((if (eq? dir '<&) call-with-input-file call-with-output-file)
          *nulldev*
-         (lambda (dummy-port)
-           (let ((p ((if (eq? dir '<&) open-input-fd-port open-output-fd-port)
-                     fd1)))
+         (^[dummy-port]
+           (let1 p ((if (eq? dir '<&) open-input-fd-port open-output-fd-port)
+                    fd1)
              (port-fd-dup! dummy-port p))))
         (push! iomap `(,fd0 . ,fd1)))))
-  
+
   (dolist [r redirs]
     (let ([dir (car r)] [fd (cadr r)] [arg (caddr r)])
       (when (memv fd seen)
@@ -374,7 +376,7 @@
   (cond
    [(is-a? mask <sys-sigset>) mask]
    [(and (list? mask) (every integer? mask))
-    (fold (lambda (sig m) (sys-sigset-add! m sig) m) (make <sys-sigset>) mask)]
+    (fold (^[sig m] (sys-sigset-add! m sig) m) (make <sys-sigset>) mask)]
    [(not mask) #f]
    [else (error "run-process: sigmask argument must be either #f, \
                  <sys-sigset>, or a list of integers, but got:" mask)]))
@@ -422,7 +424,7 @@
   (and (not (null? (process-list)))
        (receive (pid status) (sys-waitpid -1 :nohang nohang?)
          (and (not (eqv? pid 0))
-              (and-let* ((p (find (lambda (pp) (eqv? (process-pid pp) pid))
+              (and-let* ((p (find (^[pp] (eqv? (process-pid pp) pid))
                                   (process-list))))
                 (update! (ref p 'processes) (cut delete p <>))
                 (set! (ref p 'status) status)
@@ -519,10 +521,8 @@
 
 (define (process-output->string command . opts)
   (apply call-with-input-process command
-         (lambda (p)
-           (with-port-locking p
-             (lambda ()
-               (string-join (string-tokenize (port->string p)) " "))))
+         (^p (with-port-locking p
+               (^[] (string-join (string-tokenize (port->string p)) " "))))
          opts))
 
 (define (process-output->string-list command . opts)
@@ -535,7 +535,7 @@
 (define (shell-escape-string str)
   (cond-expand
    [gauche.os.windows
-    ;; This is supported in src/scmlib.scm.  See the comment in it.
+    ;; This is supported in src/libsys.scm.  See the comment in it.
     (%sys-escape-windows-command-line str)]
    [else
     ;; We follow standard unix shell convention: if STR contains special
@@ -557,30 +557,27 @@
 ;; provide consistent behavior.  Something to think about.
 (define (%apply-run-process command stdin stdout stderr host)
   (apply run-process
-         (cond ((string? command)
-                (cond-expand
-                 (gauche.os.windows
-                  `("cmd" "/c" ,command))
-                 (else
-                  `("/bin/sh" "-c" ,command))))
-               ((list? command) command)
-               (else (error "Bad command spec" command)))
+         (cond [(string? command)
+                (cond-expand [gauche.os.windows `("cmd" "/c" ,command)]
+                             [else        `("/bin/sh" "-c" ,command)])]
+               [(list? command) command]
+               [else (error "Bad command spec" command)])
          :input stdin :output stdout :host host
-         (cond ((string? stderr) `(:error ,stderr))
-               (else '()))))
+         (cond [(string? stderr) `(:error ,stderr)]
+               [else '()])))
 
 ;; Possibly wrap the process port by a conversion port
 (define (wrap-input-process-port process opts)
-  (let-keywords opts ((encoding #f)
-                      (conversion-buffer-size 0))
+  (let-keywords opts ([encoding #f]
+                      [conversion-buffer-size 0])
     (if encoding
       (wrap-with-input-conversion (process-output process) encoding
                                   :buffer-size conversion-buffer-size)
       (process-output process))))
 
 (define (wrap-output-process-port process opts)
-  (let-keywords opts ((encoding #f)
-                      (conversion-buffer-size 0))
+  (let-keywords opts ([encoding #f]
+                      [conversion-buffer-size 0])
     (if encoding
       (wrap-with-output-conversion (process-input process) encoding
                                   :buffer-size conversion-buffer-size)
@@ -588,8 +585,8 @@
 
 (define (handle-abnormal-exit on-abnormal-exit process)
   (case on-abnormal-exit
-    ((:error) (%check-normal-exit process))
-    ((:ignore))
-    (else (unless (zero? (process-exit-status process))
-            (on-abnormal-exit process)))))
+    [(:error) (%check-normal-exit process)]
+    [(:ignore)]
+    [else (unless (zero? (process-exit-status process))
+            (on-abnormal-exit process))]))
 
