@@ -1,7 +1,7 @@
 ;;;
 ;;; regexp.scm - auxiliary macros and procedures for regexp.  autoloaded.
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -101,6 +101,26 @@
             [strp (string? temp)])
        (rxmatch-case #t temp strp ?clause ...))]))
 
+;; Extract all matches (including entire match).  Allows match to be #f.
+(define (rxmatch-substrings match :optional (start 0) (end #f))
+  (if match
+    (do ([end (if (and end (>= end 0))
+                (min end (rxmatch-num-matches match))
+                (rxmatch-num-matches match))]
+         [k start (+ k 1)]
+         [r '() (cons (rxmatch-substring match k) r)])
+        [(>= k end) (reverse r)])
+    '()))
+(define (rxmatch-positions match :optional (start 0) (end #f))
+  (if match
+    (do ([end (if (and end (>= end 0))
+                (min end (rxmatch-num-matches match))
+                (rxmatch-num-matches match))]
+         [k start (+ k 1)]
+         [r '() (acons (rxmatch-start match k) (rxmatch-end match k) r)])
+        [(>= k end) (reverse r)])
+    '()))
+
 ;; NB: This should eventually be defined in regexp.c, and Scm_RegComp
 ;; should throw this on invalid AST tree.
 (define-condition-type <regexp-invalid-ast> <error> #f
@@ -130,7 +150,23 @@
                (disp ")"))))
 
   (define (unparse-rep op M N . ns)
-    (between "(?:" ns ")")
+    ;; special optimization - if the content is a simple character,
+    ;; or a single group, omit extra grouping
+    (if (and (not (null? ns)) (null? (cdr ns))
+             (let1 item (car ns)
+               (or (char? item)
+                   (eq? item 'any)
+                   (char-set? item)
+                   (and (pair? item)
+                        (or (integer? (car item)) ;capturing group
+                            (memq (car item)
+                                  '(alt comp seq-uncase seq-case cpat once
+                                        assert nassert))
+                            (and (eq? (car item) 'seq)
+                                 (pair? (cdr item))
+                                 (null? (cddr item))))))))
+      (unparse (car ns))
+      (between "(?:" ns ")"))
     (disp (cond [(not N) (case M
                            [(0 #f) "*"]
                            [(1) "+"]
@@ -154,7 +190,7 @@
         [(assert nassert)  (apply unparse-assert-like test)]
         [else (err "invalid AST in the test part of cpat" test)]))
     (seq yes)
-    (when no (disp "|") (seq no))
+    (unless (null? no) (disp "|") (seq no))
     (disp ")"))
 
   (define (unparse-assert-like op . asst)
