@@ -1,23 +1,23 @@
 ;;;
 ;;; serializer.scm - generic serializer framework
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; THIS MODULE IS UNDER DEVELOPMENT.
 ;;  This is a straightforward port from the module I wrote for STk before.
@@ -93,42 +93,42 @@
 (define (input-serializer? obj)
   (and (serializer? obj) (eq? (direction-of obj) :in)))
 (define (output-serializer? obj)
-  (and (serializer? obj) (eq? (direction-of self) :out)))
+  (and (serializer? obj) (eq? (direction-of obj) :out)))
 
 ;; Utility method
 
 (define-method write-to-string-with-serializer ((class <class>) obj . options)
-  (let ((out (open-output-string)))
+  (let1 out (open-output-string)
     (write-to-serializer
      (apply make class :port out :direction :out options)
      obj)
     (get-output-string out)))
 
 (define-method read-from-string-with-serializer ((class <class>) str . options)
-  (let ((in (open-input-string str)))
+  (let1 in (open-input-string str)
     (read-from-serializer
      (apply make class :port in :direction :in options))))
 
 (define-method write-to-file-with-serializer ((class <class>) obj file . options)
   (call-with-output-file file
-   (lambda (port)
+   (^[port]
      (write-to-serializer
       (apply make class :port port :direction :out options)
       obj))))
 
 (define-method read-from-file-with-serializer ((class <class>) file . options)
   (call-with-input-file file
-   (lambda (port)
+   (^[port]
      (read-from-serializer
       (apply make class :port port :direction :in options)))))
 
 ;; For generic object serializer
 (define-method get-serializable-slots ((obj <object>))
-  (let loop ((slots (class-slots (class-of obj)))
-             (result '()))
-    (cond ((null? slots) (reverse result))
-          ((not (eqv? (slot-definition-allocation (car slots)) :virtual))
-           (loop (cdr slots) (cons (slot-definition-name (car slots)) result)))
-          (else (loop (cdr slots) result)))))
+  (let loop ([slots (class-slots (class-of obj))]
+             [result '()])
+    (cond [(null? slots) (reverse result)]
+          [(not (eqv? (slot-definition-allocation (car slots)) :virtual))
+           (loop (cdr slots) (cons (slot-definition-name (car slots)) result))]
+          [else (loop (cdr slots) result)])))
 
 
