@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.test - test framework
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 #!no-fold-case
 
@@ -82,16 +82,12 @@
 ;;         into the named file instead of reporting it out immediately.
 
 (define-module gauche.test
-  (export test test* test-start test-end test-section
-          test-module test-error test-check test-record-file
+  (export test test* test-start test-end test-section test-log
+          test-module test-error test-one-of
+          test-check test-record-file test-summary-check
           *test-error* *test-report-error* test-error? prim-test))
 (select-module gauche.test)
 
-;; we cannot use srfi-1, since we use gauche.test to test srfi-1 itself.
-;; we count on the built-in fold work properly here.
-(define (%filter proc lis)
-  (reverse (fold (lambda (x r) (if (proc x) (cons x r) r)) '() lis)))
-
 ;; An object to represent error.
 (define-class <test-error> ()
   ((message :init-keyword :message :init-value #f)
@@ -105,10 +101,19 @@
 
 (define (test-error? obj) (is-a? obj <test-error>))
 
-(define (test-error . maybe-class)
-  (make <test-error> :class (and (pair? maybe-class) (car maybe-class))))
+(define (test-error :optional (class #f) (message #f))
+  (make <test-error> :class class :message message))
 
-(define (test-check expected result)
+;; An object to represet any one of
+(define-class <test-one-of> ()
+  ((choices :init-keyword :choices)))
+
+(define-method write-object ((obj <test-one-of>) out)
+  (format out "#<test-one-of: any one of ~s>" (slot-ref obj'choices)))
+
+(define (test-one-of . choices) (make <test-one-of> :choices choices))
+
+(define (test-check expected result :optional (fallback equal?))
   (cond [(test-error? expected)
          (and (test-error? result)
               (let ([ex (slot-ref expected'class)]
@@ -116,7 +121,13 @@
                 (or (not ex)
                     (memq (slot-ref expected'class)
                           (class-precedence-list (slot-ref result'class))))))]
-        [else (equal? expected result)]))
+        [(is-a? expected <test-one-of>)
+         ;; NB: not using srfi-1's any to avoid dependency
+         (let loop ([choices (slot-ref expected 'choices)])
+           (cond [(null? choices) #f]
+                 [(test-check (car choices) result fallback)]
+                 [else (loop (cdr choices))]))]
+        [else (fallback expected result)]))
 
 (define *test-error* (make <test-error>)) ;DEPRECATED
 
@@ -131,7 +142,7 @@
 
 (define *test-counts* (vector 0 0 0 0)) ; total/pass/fail/abort
 
-(define (test-count++) 
+(define (test-count++)
   (vector-set! *test-counts* 0 (+ (vector-ref *test-counts* 0) 1)))
 (define (test-pass++)
   (vector-set! *test-counts* 1 (+ (vector-ref *test-counts* 1) 1)))
@@ -148,60 +159,60 @@
              (sys-access *test-record-file* F_OK)) ; avoid file-exists? to trigger autoload
     (with-input-from-file *test-record-file*
       (lambda ()
-        (let ((m (rxmatch #/Total:\s+(\d+)\s+tests,\s+(\d+)\s+passed,\s+(\d+)\s+failed,\s+(\d+)\s+aborted/ (read-line))))
+        (let [(m (rxmatch #/Total:\s+(\d+)\s+tests,\s+(\d+)\s+passed,\s+(\d+)\s+failed,\s+(\d+)\s+aborted/ (read-line)))]
           (when m
             (for-each (lambda (i)
                         (vector-set! *test-counts* i
                                      (string->number
                                       (rxmatch-substring m (+ i 1)))))
-                      '(0 1 2 3)))))))
+                      '(0 1 2 3))))))))
+(define (prepare-summary)
   ;; We write out aborted+1, in case if the test process fails before test-end
   ;; For normal case, it will be overwritten by test-end.
-  (let ((orig-abort (vector-ref *test-counts* 3)))
+  (let ([orig-abort (vector-ref *test-counts* 3)])
     (vector-set! *test-counts* 3 (+ orig-abort 1))
     (write-summary)
     (vector-set! *test-counts* 3 orig-abort)))
-  
+
 (define (write-summary)
   (when (string? *test-record-file*)
-    (receive (p nam) (sys-mkstemp *test-record-file*)
+    (receive [p nam] (sys-mkstemp *test-record-file*)
       (display (format-summary) p)
       (close-output-port p)
       (sys-rename nam *test-record-file*))))
 
 ;; Tests ------------------------------------------------------------
 
-;; Primitive test.  This doesn't use neither with-error-handler nor
+;; Primitive test.  This uses neither with-error-handler nor the
 ;; object system, so it can be used _before_ those constructs are tested.
 (define (prim-test msg expect thunk . compare)
-  (let ((cmp (if (pair? compare) (car compare) test-check)))
+  (let ([cmp (if (pair? compare) (car compare) test-check)])
     (format/ss #t "test ~a, expects ~s ==> " msg expect)
     (flush)
     (test-count++)
-    (let ((r (thunk)))
-      (cond ((cmp expect r)
+    (let ([r (thunk)])
+      (cond [(cmp expect r)
              (format #t "ok\n")
-             (test-pass++))
-            (else
+             (test-pass++)]
+            [else
              (format/ss #t "ERROR: GOT ~S\n" r)
              (set! *discrepancy-list*
                    (cons (list msg expect r) *discrepancy-list*))
-             (test-fail++)))
-      (flush)
-      )))
+             (test-fail++)])
+      (flush))))
 
 ;; Normal test.
 (define (test msg expect thunk . compare)
   (apply prim-test msg expect
          (lambda ()
-           (guard (e (else
+           (guard (e [else
                       (when *test-report-error*
                         (report-error e))
                       (make <test-error>
                         :class (class-of e)
                         :message (if (is-a? e <message-condition>)
                                    (ref e 'message)
-                                   e))))
+                                   e))])
              (thunk)))
          compare))
 
@@ -213,10 +224,11 @@
 
 ;; Try to catch careless typos.  Suggested by Kimura Fuyuki.
 ;; The toplevel undefined variable screening is suggested by Kazuki Tsujimoto.
-;; Keyword argument :allow-undefined may take a list of symbols, which
-;; is excluded from undefined variable check.
+;; Keyword argument :allow-undefined takes a list of symbols, which
+;; is excluded from undefined variable check.  Keyword argument
+;; :bypass-arity-check takes a list of symbols that bypasses arity check.
 
-(define (test-module module :key (allow-undefined '()))
+(define (test-module module :key (allow-undefined '()) (bypass-arity-check '()))
   (test-count++)
   (let1 mod (cond [(module? module) module]
                   [(symbol? module)
@@ -226,10 +238,11 @@
                    (error "test-module requires module or symbol, but got"
                           module)])
     (format #t "testing bindings in ~a ... " mod) (flush)
-    (let ((bad-autoload '())
-          (bad-export '())
-          (bad-gref '())
-          (report '()))
+    (let ([bad-autoload '()]
+          [bad-export '()]
+          [bad-gref '()]
+          [bad-arity '()]
+          [report '()])
       ;; 1. Check if there's no dangling autoloads.
       (hash-table-for-each (module-table mod)
                            (lambda (sym val)
@@ -238,17 +251,22 @@
       ;; 2. Check if all exported symbols are properly defined.
       (when (pair? (module-exports mod))
         (for-each (lambda (sym)
-                    (guard (_ (else (push! bad-export sym)))
+                    (guard (_ [else (push! bad-export sym)])
                       (global-variable-ref mod sym)))
                   (module-exports mod)))
-      ;; 3. Check if all global references are resolvable.
+      ;; 3. Check if all global references are resolvable, and if it is
+      ;; called, gets valid number of arguments.
       (for-each
        (lambda (closure)
-         (for-each (lambda (gref)
-                     (cond ((and (not (memq (slot-ref gref 'name)
-                                            allow-undefined))
-                                 (dangling-gref? gref closure))
-                            => (lambda (bad) (push! bad-gref bad)))))
+         (for-each (lambda (arg)
+                     (let ([gref (car arg)]
+                           [numargs (cadr arg)])
+                       (cond [(memq (slot-ref gref 'name) allow-undefined)]
+                             [(dangling-gref? gref closure)
+                              => (lambda (bad) (push! bad-gref bad))]
+                             [(memq (slot-ref gref 'name) bypass-arity-check)]
+                             [(arity-invalid? gref numargs closure)
+                              => (lambda (bad) (push! bad-arity bad))])))
                    (closure-grefs closure)))
        (toplevel-closures mod))
       ;; report discrepancies
@@ -266,11 +284,20 @@
                                            (format "~a(~a)" (car z) (cdr z)))
                                          bad-gref)
                                     ", "))))
+      (unless (null? bad-arity)
+        (unless (null? report) (push! report " AND "))
+        (push! report
+               (format "procedures received wrong number of argument: ~a"
+                       (string-join (map (lambda (z)
+                                           (format "~a(~a) got ~a"
+                                                   (car z) (cadr z) (caddr z)))
+                                         bad-arity)
+                                    ", "))))
       (cond
        [(null? report) (test-pass++) (format #t "ok\n")]
        [else
         (test-fail++)
-        (let ((s (apply string-append report)))
+        (let ([s (apply string-append report)])
           (format #t "ERROR: ~a\n" s)
           (set! *discrepancy-list*
                 (cons (list (format #f "bindings in module ~a" (module-name mod))
@@ -284,34 +311,62 @@
 ;; operand of GREF or GSET.  (This is because identifiers are
 ;; introduced by macro expansion, but quoted identifiers are turned
 ;; back to ortinary symbols when expansion is done.)  However, it
-;; may not be impossible to embed identifiers within literals.  
+;; may not be impossible to embed identifiers within literals.
 ;; Eventually we need a builtin procedure that picks identifiers
 ;; used for GREF/GSET.
 ;;
-;; Note that these identifiers in operands are replaced by GLOCs 
+;; Note that these identifiers in operands are replaced by GLOCs
 ;; once the code is executed.  We don't need to consider them; since
 ;; if the identifier has successufully replaced by a GLOC, it couldn't
 ;; be an undefined reference.
 
 (define (toplevel-closures module)
-  (%filter closure?
-           (map (lambda (sym)
-                  (global-variable-ref module sym #f))
-                (hash-table-keys (module-table module)))))
+  (filter closure?
+          (map (lambda (sym)
+                 (global-variable-ref module sym #f))
+               (hash-table-keys (module-table module)))))
 
+;; Combs closure's instruction list to extract references for the global
+;; identifiers.  If it is a call to the global function, we also picks
+;; up the number of arguments, so that we can check it against arity.
+;; Returns ((<identifier> <num-args>) ...)
 (define (closure-grefs closure)
   (define code->list (with-module gauche.internal vm-code->list))
-  (let loop ((r '())
-             (code (code->list (closure-code closure))))
+  (define (gref-numargs code) (cadr code))
+  (define gref-call-insns
+    '(GREF-CALL PUSH-GREF-CALL GREF-TAIL-CALL PUSH-GREF-TAIL-CALL))
+  (let loop ([r '()] [code (code->list (closure-code closure))])
     (cond [(null? code) r]
+          [(and (pair? (car code))
+                (memq (caar code) gref-call-insns))
+           (if (pair? (cdr code))
+             (if (identifier? (cadr code))
+               (loop `((,(cadr code) ,(gref-numargs (car code))) ,@r)
+                     (cddr code))
+               (loop r (cddr code)))    ;skip #<gloc>
+             (loop r '()))]
           [(identifier? (car code))
-           (loop (cons (car code) r) (cdr code))]
+           (loop `((,(car code) #f) ,@r) (cdr code))]
           [(is-a? (car code) <compiled-code>)
            (loop (loop r (code->list (car code))) (cdr code))]
-          [(list? (car code)) ; for operand of LOCAL-ENV-CLOSURES
+          [(list? (car code)) ; for the operand of LOCAL-ENV-CLOSURES
            (loop (loop r (car code)) (cdr code))]
           [else (loop r (cdr code))])))
 
+(define (arity-invalid? gref numargs closure)
+  (and-let* ([ numargs ]
+             [proc (global-variable-ref
+                    (slot-ref gref 'module)
+                    (slot-ref gref 'name))]
+             ;; We exclude <generic> with no methods.  Such "placeholder"
+             ;; generic function may be used in the base module, expecting
+             ;; the other module adds methods to it.
+             [ (not (and (is-a? proc <generic>) (null? (~ proc'methods)))) ]
+             [ (not (apply applicable? proc (make-list numargs <bottom>))) ])
+    (list (slot-ref gref 'name)
+          (slot-ref closure 'info)
+          numargs)))
+
 (define (dangling-gref? ident closure)
   (and (not ((with-module gauche.internal find-binding)
              (slot-ref ident 'module)
@@ -321,36 +376,48 @@
 
 ;; Logging and bookkeeping -----------------------------------------
 (define (test-section msg)
-  (let ((msglen (string-length msg)))
+  (let ([msglen (string-length msg)])
     (format #t "<~a>~a\n" msg (make-string (max 5 (- 77 msglen)) #\-))))
 
 (define (test-start msg)
-  (let* ((s (format #f "Testing ~a ... " msg))
-         (pad (make-string (max 3 (- 65 (string-length s))) #\space)))
+  (let* ([s (format #f "Testing ~a ... " msg)]
+         [pad (make-string (max 3 (- 65 (string-length s))) #\space)])
     (display s (current-error-port))
     (display pad (current-error-port))
     (flush (current-error-port))
     (read-summary)
+    (prepare-summary)
     (when (and (sys-isatty (current-error-port))
                (sys-isatty (current-output-port)))
       (newline (current-error-port))))
   (set! *discrepancy-list* '())
   (unless (and (sys-isatty (current-error-port))
                (sys-isatty (current-output-port)))
-    (let ((msglen (string-length msg)))
+    (let ([msglen (string-length msg)])
       (format #t "Testing ~a ~a\n" msg (make-string (max 5 (- 70 msglen)) #\=)))
     (flush))
   )
 
-(define (test-end)
-  (let ((e (current-error-port))
-        (o (current-output-port)))
+;; test-log fmt arg ...
+;; The formatted output, prefixed by ";;",  goes to stdout for the logging.
+(define (test-log fmt . args)
+  (display ";; ")
+  (apply format #t fmt args)
+  (newline)
+  (flush))
+
+;; test-end :key :exit-on-failure
+;; avoid using extended formal list since we need to test it.
+(define (test-end . args)
+  (let ([e (current-error-port)]
+        [o (current-output-port)]
+        [exit-on-failure (get-keyword :exit-on-failure args #f)])
     (define (fmt . args)
       (if (and (sys-isatty e) (sys-isatty o))
         (apply format/ss o args)
         (begin (apply format/ss e args)
                (apply format/ss o args))))
-    
+
     (if (null? *discrepancy-list*)
       (fmt "passed.\n")
       (begin
@@ -362,6 +429,17 @@
     (when *test-record-file*
       (write-summary))
 
-    ;; Returns the number of failed tests.
-    (length *discrepancy-list*)))
+    ;; the number of failed tests.
+    (let ([num-failures (length *discrepancy-list*)])
+      (when (and (> num-failures 0)
+                 exit-on-failure)
+        (exit (if (fixnum? exit-on-failure) exit-on-failure 1)))
+      num-failures)))
 
+;; Read the test record file (if there's any), and exit with 1
+;; if there has been any failure.
+(define (test-summary-check)
+  (read-summary)
+  (unless (and (zero? (vector-ref *test-counts* 2))
+               (zero? (vector-ref *test-counts* 3)))
+    (exit 1)))
