@@ -1,23 +1,23 @@
 ;;;
 ;;; auxiliary treemap utilities.  to be autoloaded.
-;;;  
-;;;   Copyright (c) 2007-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2007-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.treeutil
   (export make-tree-map tree-map-empty?
@@ -41,11 +41,12 @@
   )
 (select-module gauche.treeutil)
 
-(define (make-tree-map key=? key<?)
-  (%make-tree-map (lambda (x y)
-                    (cond ((key=? x y) 0)
-                          ((key<? x y) -1)
-                          (else        1)))))
+(define make-tree-map
+  (case-lambda
+    [() (%make-tree-map compare)]
+    [(cmp) (%make-tree-map cmp)]
+    [(=? <?)
+     (%make-tree-map (^[x y](cond [(=? x y) 0] [(<? x y) -1] [else 1])))]))
 
 (define (tree-map-empty? tm) (zero? (tree-map-num-entries tm)))
 
