@@ -1,7 +1,7 @@
 ;;;
 ;;; base64.scm - base64 encoding/decoding routine
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -42,7 +42,7 @@
           base64-decode base64-decode-string))
 (select-module rfc.base64)
 
-(define *decode-table*
+(define *standard-decode-table*
   ;;    !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /
   #(#f  #f  #f  #f  #f  #f  #f  #f  #f  #f  #f  62  #f  #f  #f  63
   ;;0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?
@@ -57,7 +57,7 @@
     41  42  43  44  45  46  47  48  49  50  51  #f  #f  #f  #f  #f
   ))
 
-(define *encode-table*
+(define *standard-encode-table*
   ;;0   1   2   3   4   5   6   7   8   9   10  11  12  13  14  15
   #(#\A #\B #\C #\D #\E #\F #\G #\H #\I #\J #\K #\L #\M #\N #\O #\P
   ;;16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  31
@@ -70,53 +70,80 @@
     #\=
   ))
 
-(define (base64-decode)
-  (let-syntax ((lookup (syntax-rules ()
-                         ((_ c)
-                          (let ((i (char->integer c)))
+(define *url-safe-decode-table*
+  ;;    !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /
+  #(#f  #f  #f  #f  #f  #f  #f  #f  #f  #f  #f  #f  #f  62  #f  #f
+  ;;0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?
+    52  53  54  55  56  57  58  59  60  61  #f  #f  #f  #f  #f  #f
+  ;;@   A   B   C   D   E   F   G   H   I   J   K   L   M   N   O
+    #f  0   1   2   3   4   5   6   7   8   9   10  11  12  13  14
+  ;;P   Q   R   S   T   U   V   W   X   Y   Z   [   \   ]   ^   _
+    15  16  17  18  19  20  21  22  23  24  25  #f  #f  #f  #f  63
+  ;;`   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o
+    #f  26  27  28  29  30  31  32  33  34  35  36  37  38  39  40
+  ;;p   q   r   s   t   u   v   w   x   y   z   {   |   }   ~
+    41  42  43  44  45  46  47  48  49  50  51  #f  #f  #f  #f  #f
+  ))
+
+(define *url-safe-encode-table*
+  ;;0   1   2   3   4   5   6   7   8   9   10  11  12  13  14  15
+  #(#\A #\B #\C #\D #\E #\F #\G #\H #\I #\J #\K #\L #\M #\N #\O #\P
+  ;;16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  31
+    #\Q #\R #\S #\T #\U #\V #\W #\X #\Y #\Z #\a #\b #\c #\d #\e #\f
+  ;;32  33  34  35  36  37  38  39  40  41  42  43  44  45  46  47
+    #\g #\h #\i #\j #\k #\l #\m #\n #\o #\p #\q #\r #\s #\t #\u #\v
+  ;;48  49  50  51  52  53  54  55  56  57  58  59  60  61  62  63
+    #\w #\x #\y #\z #\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9 #\- #\_
+  ;;pad
+    #\=
+  ))
+
+(define (base64-decode :key (url-safe #f))
+  (define table (if url-safe *url-safe-decode-table* *standard-decode-table*))
+  (let-syntax ([lookup (syntax-rules ()
+                         [(_ c)
+                          (let1 i (char->integer c)
                             (and (< 32 i 128)
-                                 (vector-ref *decode-table* (- i 32)))))))
+                                 (vector-ref table (- i 32))))])]
                )
     (define (d0 c)
-      (cond ((eof-object? c))
-            ((eqv? c #\=))
-            ((lookup c) => (lambda (v) (d1 (read-char) v)))
-            (else (d0 (read-char)))))
+      (cond [(eof-object? c)]
+            [(eqv? c #\=)]
+            [(lookup c) => (^v (d1 (read-char) v))]
+            [else (d0 (read-char))]))
 
     (define (d1 c hi)
-      (cond ((eof-object? c))
-            ((eqv? c #\=))
-            ((lookup c) => (lambda (lo)
+      (cond [(eof-object? c)]
+            [(eqv? c #\=)]
+            [(lookup c) => (^[lo]
                              (write-byte (+ (* hi 4) (quotient lo 16)))
-                             (d2 (read-char) (modulo lo 16))))
-            (else (d1 (read-char) hi))))
+                             (d2 (read-char) (modulo lo 16)))]
+            [else (d1 (read-char) hi)]))
 
     (define (d2 c hi)
-      (cond ((eof-object? c))
-            ((eqv? c #\=))
-            ((lookup c) => (lambda (lo)
+      (cond [(eof-object? c)]
+            [(eqv? c #\=)]
+            [(lookup c) => (^[lo]
                              (write-byte (+ (* hi 16) (quotient lo 4)))
-                             (d3 (read-char) (modulo lo 4))))
-            (else (d2 (read-char) hi))))
+                             (d3 (read-char) (modulo lo 4)))]
+            [else (d2 (read-char) hi)]))
 
     (define (d3 c hi)
-      (cond ((eof-object? c))
-            ((eqv? c #\=))
-            ((lookup c) => (lambda (lo)
+      (cond [(eof-object? c)]
+            [(eqv? c #\=)]
+            [(lookup c) => (^[lo]
                              (write-byte (+ (* hi 64) lo))
-                             (d0 (read-char))))
-            (else (d3 (read-char) hi))))
+                             (d0 (read-char)))]
+            [else (d3 (read-char) hi)]))
 
     (d0 (read-char))))
 
-(define (base64-decode-string string)
+(define (base64-decode-string string . opts)
   (with-output-to-string
-    (lambda ()
-      (with-input-from-string string base64-decode))))
-
-
-(define (base64-encode :key (line-width 76))
+    (cut with-input-from-string string (cut apply base64-decode opts))))
 
+(define (base64-encode :key (line-width 76) (url-safe #f))
+  (define table (if url-safe *url-safe-encode-table* *standard-encode-table*))
   (define maxcol (and line-width (> line-width 0) (- line-width 1)))
 
   (letrec-syntax ([emit*
@@ -124,7 +151,7 @@
                      [(_ col) col]
                      [(_ col idx idx2 ...)
                       (begin
-                        (write-char (vector-ref *encode-table* idx))
+                        (write-char (vector-ref table idx))
                         (let1 col2 (cond [(eqv? col maxcol) (newline) 0]
                                          [else (+ col 1)])
                           (emit* col2 idx2 ...)))])])
@@ -152,5 +179,3 @@
 
 (define (base64-encode-string string . opts)
   (with-string-io string (cut apply base64-encode opts)))
-
-
