@@ -1,7 +1,7 @@
 ;;;
 ;;; http.scm - HTTP 1.1
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -50,12 +50,12 @@
   (use srfi-13)
   (use rfc.822)
   (use rfc.uri)
+  (use rfc.base64)
   (use gauche.net)
   (use gauche.parameter)
   (use gauche.charconv)
-  (use gauche.collection)
+  (use gauche.sequence)
   (use gauche.uvector)
-  (use gauche.partcont)
   (use util.match)
   (use util.list)
   (use text.tree)
@@ -234,40 +234,90 @@
                            (sender #f)
                            ((:request-encoding enc) (gauche-character-encoding))
                       :allow-other-keys opts)
-  (let1 conn (ensure-connection server auth-handler auth-user auth-password
-                                proxy secure extra-headers)
-    (let loop ([history '()]
-               [host host]
-               [method method]
-               [request-uri (ensure-request-uri request-uri enc)])
-      (receive (code headers body)
-          (request-response method conn host request-uri sender receiver
-                            `(:user-agent ,user-agent ,@opts) enc)
-        (or (and-let* ([ (not no-redirect) ]
-                       [ (string-prefix? "3" code) ]
-                       [h (case redirect-handler
-                            [(#t) (http-default-redirect-handler)]
-                            [(#f) #f]
-                            [else => identity])]
-                       [r (h method code headers body)]
-                       [method (car r)]
-                       [loc (cdr r)])
-              (receive (uri proto new-server path*)
-                  (canonical-uri conn loc (ref conn'server))
-                (when (or (member uri history)
-                          (> (length history) 20))
-                  (errorf <http-error> "redirection is looping via ~a" uri))
-                (loop (cons uri history)
-                      (ref (redirect conn proto new-server)'server)
-                      method
-                      path*)))
-            (values code headers body))))))
 
+  (define conn (ensure-connection server auth-handler auth-user auth-password
+                                  proxy secure extra-headers))
+  (define redirector (if no-redirect
+                       #f
+                       (case redirect-handler
+                         [(#t) (http-default-redirect-handler)]
+                         [(#f) #f]
+                         [else => identity])))
+  (define options `(:user-agent ,user-agent ,@(http-auth-headers conn) ,@opts))
+  (define no-body-replies '("204" "304"))
+
+  (define (get-body iport method code headers receiver)
+    (and (not (eq? method 'HEAD))
+         (not (member code no-body-replies))
+         (receive-body iport code headers receiver)))
+
+  ;; final touch of request headers
+  (define (req-headers host)
+    (cond-list [(~ conn'persistent) @ (if (~ conn'proxy)
+                                        '(:proxy-connection keep-alive)
+                                        '(:connection keep-alive))]
+               [#t @ `(:host ,host :user-agent ,user-agent
+                       ,@(http-auth-headers conn) ,@opts)]))
+
+  ;; If we decide to give up redirection, we read from already-retrieved
+  ;; body of 3xx reply.  This modifies reply headers if necessary.
+  (define (redirect-headers body rep-headers)
+    (if body
+      `(:content-length ,(string-size body)
+                        ,@(delete-keywords '(:content-length
+                                             :content-transfer-encoding)
+                                           rep-headers))
+      rep-headers))
+
+  ;; returns either one of:
+  ;;   (reply <code> <headers> <body>)
+  ;;   (redirect-to <method> <location>)
+  (define (request-response in out method uri host sender)
+    (send-request out method uri sender (req-headers host) enc)
+    (receive (code rep-headers) (receive-header in)
+      (if-let1 consider-redirect (and (string-prefix? "3" code) redirector)
+        ;; we retrieve body as string, not using caller-provided receiver
+        (let* ([body (get-body in method code rep-headers
+                               (http-string-receiver))]
+               [verdict (consider-redirect method code rep-headers body)])
+          (if verdict
+            `(redirect-to ,(car verdict) ,(cdr verdict))
+            (let1 hdrs (redirect-headers body rep-headers)
+              `(reply ,code ,hdrs
+                      ,(and body
+                            (receive-body (open-input-string body) code
+                                          hdrs receiver))))))
+        ;; no redirection
+        `(reply ,code ,rep-headers
+                ,(get-body in method code rep-headers receiver)))))
+
+  ;; main loop
+  (let loop ([history '()]
+             [host host]
+             [method method]
+             [request-uri (ensure-request-uri request-uri enc)])
+    (receive (host uri)
+        (consider-proxy conn (or host (~ conn'server)) request-uri)
+      (let1 result
+          (with-connection
+           conn
+           (^[i o] (request-response i o method uri host sender)))
+        (match result
+          [('reply code rep-headers body) (values code rep-headers body)]
+          [('redirect-to method location)
+           (receive (uri proto new-server path*)
+               (canonical-uri conn location (ref conn'server))
+             (when (or (member uri history)
+                       (> (length history) 20))
+               (errorf <http-error> "redirection is looping via ~a" uri))
+             (loop (cons uri history)
+                   (~ (redirect-connection! conn proto new-server)'server)
+                   method path*))])))))
 ;;
 ;; Pre-defined receivers
 ;;
 (define (http-string-receiver)
-  (lambda (code hdrs total retr)
+  (^[code hdrs total retr]
     ;; TODO: check headers for encoding
     (let loop ([sink (open-output-string)])
       (receive (remote size) (retr)
@@ -276,22 +326,21 @@
                (copy-port remote sink :size size) (loop sink)])))))
 
 (define (http-null-receiver)
-  (lambda (code hdrs total retr)
-    (let loop ([sink (open-output-file (cond-expand [gauche.os.windows "NUL"]
-                                                    [else "/dev/null"]))])
+  (^[code hdrs total retr]
+    (let loop ([sink (open-output-file (null-device))])
       (receive (remote size) (retr)
         (cond [(and size (<= size 0)) (close-output-port sink)]
               [else (copy-port remote sink :size size) (loop sink)])))))
 
 (define (http-oport-receiver sink flusher)
-  (lambda (code hdrs total retr)
+  (^[code hdrs total retr]
     (let loop ()
       (receive (remote size) (retr)
         (cond [(and size (<= size 0)) (flusher sink hdrs)]
               [else (copy-port remote sink :size size) (loop)])))))
 
 (define (http-file-receiver filename :key (temporary #f))
-  (lambda (code hdrs total retr)
+  (^[code hdrs total retr]
     (receive (port tmpname) (sys-mkstemp filename)
       (let loop ()
         (receive (remote size) (retr)
@@ -309,13 +358,13 @@
     [(_) (http-null-receiver)]
     [(_ [else . exprs]) (begin . exprs)]
     [(_ [cc => proc] . rest)
-     (lambda (code hdrs total retr)
+     (^[code hdrs total retr]
        ((if (match-status-code? cc code)
           proc
           (http-cond-receiver . rest))
         code hdrs total retr))]
     [(_ [cc . exprs] . rest)
-     (lambda (code hdrs total retr)
+     (^[code hdrs total retr]
        ((if (match-status-code? cc code '(cc . exprs))
           (begin . exprs)
           (http-cond-receiver . rest))
@@ -326,7 +375,7 @@
 (define (match-status-code? pattern code clause)
   (cond [(string? pattern) (equal? pattern code)]
         [(regexp? pattern) (rxmatch pattern code)]
-        [else (error "invalid pattern in a clause of http-cond-receiver"
+        [else (error "Invalid pattern in a clause of http-cond-receiver:"
                      clause)]))
 
 ;;
@@ -334,12 +383,12 @@
 ;;
 
 (define (http-null-sender)
-  (lambda (hdrs encoding header-sink)
-    (let ((body-sink (header-sink `(("content-length" "0") ,@hdrs))))
+  (^[hdrs encoding header-sink]
+    (let1 body-sink (header-sink `(("content-length" "0") ,@hdrs))
       (body-sink 0))))
 
 (define (http-string-sender string)     ;honors encoding
-  (lambda (hdrs encoding header-sink)
+  (^[hdrs encoding header-sink]
     (let* ([body (if (ces-equivalent? encoding (gauche-character-encoding))
                    string
                    (ces-convert string (gauche-character-encoding) encoding))]
@@ -351,7 +400,7 @@
       (body-sink 0))))
 
 (define (http-blob-sender blob)        ;blob may be a string or uvector
-  (lambda (hdrs encoding header-sink)
+  (^[hdrs encoding header-sink]
     (let* ([size (if (string? blob) (string-size blob) (uvector-size blob))]
            [body-sink (header-sink `(("content-length" ,(x->string size))
                                      ,@hdrs))]
@@ -365,18 +414,18 @@
 ;; TODO: The file size may be changed while sending out.  If it gets
 ;; bigger, we can just ignore the rest, but what if it gets shorter?
 (define (http-file-sender filename)
-  (lambda (hdrs encoding header-sink)
+  (^[hdrs encoding header-sink]
     (let* ([size (file-size filename)]
            [body-sink (header-sink `(("content-length" ,(x->string size))
                                      ,@hdrs))]
            [port (body-sink size)])
-      (call-with-input-file (cut copy-port <> port :size size))
+      (call-with-input-file filename (cut copy-port <> port :size size))
       (body-sink 0))))
 
 ;; See http-compose-form-data definition for params spec.
 ;; TODO: support chunked sending, instead of building entire body at once.
 (define (http-multipart-sender params)
-  (lambda (hdrs encoding header-sink)
+  (^[hdrs encoding header-sink]
     (receive (body boundary) (http-compose-form-data params #f encoding)
       (let* ([size (string-size body)]
              [hdrs `(("content-length" ,(x->string size))
@@ -409,13 +458,18 @@
 ;; Adaptor to the new API.  Converts :sink and :flusher arguments,
 ;; which are superseded by :receiver arguments.
 (define (%http-request-adaptor method server request-uri body
-                               :key receiver (sink #f) (flusher #f)
+                               :key (receiver #f) (sink #f) (flusher #f)
                                :allow-other-keys opts)
   (define recvr
-    (if (or sink flusher)
-      (http-oport-receiver (or sink (open-output-string))
-                           (or flusher (^(s h) (get-output-string s))))
-      receiver))
+    (cond [(and sink flusher)
+           (http-oport-receiver sink flusher)]
+          [(or sink flusher)
+           (errorf "You need to provide :sink and :flusher together to http-~a"
+                   (string-downcase (symbol->string method)))]
+          [receiver]
+          ;; fallback
+          [else (http-oport-receiver (open-output-string)
+                                     (^[s h] (get-output-string s)))]))
   (apply http-request method server request-uri
          :sender (cond [(not body) (http-null-sender)]
                        [(list? body) (http-multipart-sender body)]
@@ -441,9 +495,9 @@
    (secure-agent  :init-value #f)       ; When using secure connection via
                                         ; tls, this slot holds its handle.
    (persistent    :init-keyword :persistent)   ; true for persistent connection.
-   (auth-handler  :init-keyword :auth-handler) ; unused yet
-   (auth-user     :init-keyword :auth-user)    ; unused yet
-   (auth-password :init-keyword :auth-password); unused yet
+   (auth-handler  :init-keyword :auth-handler  :init-value #f)
+   (auth-user     :init-keyword :auth-user     :init-value #f)
+   (auth-password :init-keyword :auth-password :init-value #f)
    (proxy         :init-keyword :proxy)
    (extra-headers :init-keyword :extra-headers)
    (secure        :init-keyword :secure) ; boolean
@@ -451,18 +505,22 @@
 
 (define (make-http-connection server :key
                               (persistent #t)
-                              (auth-handler  http-default-auth-handler)
+                              (auth-handler  #f)
                               (auth-user     #f)
                               (auth-password #f)
                               (proxy #f)
                               (extra-headers '()))
   (make <http-connection>
     :persistent persistent
-    :server server :auth-handler auth-handler :auth-user auth-user
-    :auth-password auth-password :proxy proxy
+    :server server
+    :auth-handler (or auth-handler (http-default-auth-handler))
+    :auth-user auth-user
+    :auth-password auth-password
+    :proxy proxy
     :extra-headers extra-headers))
 
-(define (redirect conn proto new-server)
+;; This modifies CONN.
+(define (redirect-connection! conn proto new-server)
   (let1 orig-server (~ conn'server)
     (unless (and (string=? orig-server new-server)
                  (eq? (~ conn'secure) (equal? proto "https")))
@@ -492,7 +550,7 @@
   (define (query-1 n&v)
     (match n&v
       [(name value) #`",(esc name)=,(esc value)"]
-      [_ (error "invalid request-uri form: ~s" params)]))
+      [_ (error "Invalid request-uri form:" params)]))
   (define (query) (string-concatenate (intersperse "&" (map query-1 params))))
   (cond [(not path) (query)]
         [(null? params) path]
@@ -513,7 +571,7 @@
       [(name value) (translate-param `(,name :value ,value))]
       [(name . kvs)
        (unless (even? (length kvs))
-         (error "invalid parameter format to create multipart/form-data:" param))
+         (error "Invalid parameter format to create multipart/form-data:" param))
        (let-keywords kvs ([value ""]
                           [file  #f]
                           [content-type #f]
@@ -535,10 +593,11 @@
          `(,type ,subtype ("charset" . ,(x->string encoding)) ,@options))]))
   (define (make-content-disposition name file)
     (with-output-to-string
-      (lambda ()
+      (^[]
         (display "form-data")
-        (mime-compose-parameters `(("name" . ,name)
-                                   ,@(cond-list [file `("filename" . ,file)]))))))
+        (mime-compose-parameters
+         `(("name" . ,name)
+           ,@(cond-list [file `("filename" . ,file)]))))))
   (if (not port)
     (mime-compose-message-string (map translate-param params))
     (mime-compose-message (map translate-param params) port)))
@@ -617,27 +676,6 @@
       (when (~ conn'secure) (shutdown-secure-agent conn))
       (shutdown-socket-connection conn))))
 
-(define (request-response method conn host request-uri
-                          sender receiver options enc)
-  (define no-body-replies '("204" "304"))
-  (receive (host uri) (consider-proxy conn (or host (~ conn'server)) request-uri)
-    (let1 req-headers `(,@(if (~ conn'persistent)
-                            (if (~ conn'proxy)
-                              '(:proxy-connection keep-alive)
-                              '(:connection keep-alive))
-                            '())
-                        :host ,host ,@options)
-      (with-connection
-       conn
-       (lambda (in out)
-         (send-request out method uri sender req-headers enc)
-         (receive (code rep-headers) (receive-header in)
-           (values code
-                   rep-headers
-                   (and (not (eq? method 'HEAD))
-                        (not (member code no-body-replies))
-                        (receive-body in code rep-headers receiver)))))))))
-
 ;; canonicalize uri for the sake of redirection.
 ;; URI is a request-uri given to the API, or the redirect location specified
 ;; in 3xx response.  It can be a full URI or just a path w.r.t. the current
@@ -666,23 +704,25 @@
 ;; send
 (define (send-request out method uri sender headers enc)
   (define request-line #`",method ,uri HTTP/1.1\r\n")
+  (define request-headers
+    ($ map (cut map (^s (if (keyword? s) (keyword->string s) (x->string s))) <>)
+       $ slices headers 2))
   (case method
     [(POST PUT)
-     (sender (options->request-headers headers) enc
-             (lambda (hdrs)
+     (sender request-headers enc
+             (^[hdrs]
                (send-headers request-line hdrs out)
                (let ([chunked?
                       (equal? (rfc822-header-ref hdrs "transfer-encoding")
                               "chunked")]
                      [first-time #t])
-                 (lambda (size)
+                 (^[size]
                    (when chunked?
                      (unless first-time (display "\r\n" out))
                      (format out "~x\r\n" size))
                    (flush out)
                    out))))]
-    [else
-     (send-headers request-line (options->request-headers headers) out)]))
+    [else (send-headers request-line request-headers out)]))
 
 ;; NB: We try to send the request line and headers in one packet if possible,
 ;; since some http servers assumes important headers can be read in single
@@ -695,14 +735,6 @@
            out)
   (flush out))
 
-;; convert key-value list to (("key" "value") ...)
-(define (options->request-headers options)
-  (let loop ([options options] [r '()])
-    (if (or (null? options) (null? (cdr options))) ;be permissive
-      (reverse r)
-      (loop (cddr options)
-            `((,(x->string (car options)) ,(x->string (cadr options))) ,@r)))))
-
 ;; receive
 (define (receive-header remote)
   (receive (code reason) (parse-status-line (read-line remote))
@@ -715,41 +747,55 @@
         [else (error <http-error> "bad reply from server" line)]))
 
 (define (receive-body remote code headers receiver)
-  (let* ([total (and-let* ([p (assoc "content-length" headers)])
-                  (x->integer (cadr p)))]
-         [handler (reset (receiver code headers total (^() (shift k k))))])
-    (cond
-     [(assoc "transfer-encoding" headers)
-      => (^p (unless (equal? (cadr p) "chunked")
-               (error <http-error> "unsupported transfer-encoding" (cadr p)))
-             (receive-body-chunked remote handler))]
-     [total
-      (when (> total 0) (set! handler (handler remote total)))
-      (handler remote 0)]
-     [else      ; length is unknown.  we call the handler with size=#f.
-      (set! handler (handler remote #f))
-      (handler remote 0)])))
+  (let1 total (and-let* ([p (assoc "content-length" headers)])
+                (x->integer (cadr p)))
+    (if-let1 enc (assoc "transfer-encoding" headers)
+      (if (equal? (cadr enc) "chunked")
+        (receive-body-chunked remote code headers total receiver)
+        (error <http-error> "unsupported transfer-encoding:" (cadr enc)))
+      (receive-body-once remote code headers total receiver))))
+
+(define (receive-body-once remote code headers total receiver)
+  ;; Callback will be called twice (unless total is 0).  The first
+  ;; time we return # of total bytes, the second time zero.
+  (let1 rest total
+    (define (callback)
+      (if (equal? rest 0)
+        (values remote 0)
+        (begin (set! rest 0) (values remote total))))
+    (receiver code headers total callback)))
 
 ;; NB: chunk extension and trailer are ignored for now.
-(define (receive-body-chunked remote handler)
-  (guard (e [else (handler remote -1) (raise e)])
-    (let loop ([line (read-line remote)])
-      (when (eof-object? line)
-        (error <http-error> "chunked body ended prematurely"))
-      (rxmatch-if (#/^([[:xdigit:]]+)/ line) (#f digits)
-        (let1 chunk-size (string->number digits 16)
-          (if (zero? chunk-size)
-            ;; finish reading trailer
-            (do ([line (read-line remote) (read-line remote)])
-                [(or (eof-object? line) (string-null? line))
-                 (handler remote 0)]
-              #f)
+(define (receive-body-chunked remote code headers total receiver)
+  (define chunk-size #f)
+  (define condition #f)
+  (define (callback)
+    (if (equal? chunk-size 0)
+      (values remote 0) ;; finalize
+      ;; If we get an error during receiving from the server, we need
+      ;; to return -1 to give the chance to the receiver to clean up
+      ;; things.  After the receiver returns we reraise the condition.
+      (guard (e [else (set! condition e) (values remote -1)])
+        ;; If we've already handled some chunks, we need to skip
+        ;; the trailing CRLF of the previous chunk.
+        (when chunk-size
+          (read-line remote))
+        (let1 line (read-line remote)
+          (when (eof-object? line)
+            (error <http-error> "chunked body ended prematurely"))
+          (rxmatch-if (#/^([[:xdigit:]]+)/ line) (#f digits)
             (begin
-              (set! handler (handler remote chunk-size))
-              (read-line remote) ;skip the following CRLF
-              (loop (read-line remote)))))
-        ;; something's wrong
-        (error <http-error> "bad line in chunked data:" line)))))
+              (set! chunk-size (string->number digits 16))
+              (if (zero? chunk-size)
+                ;; finish reading trailer
+                (do ([line (read-line remote) (read-line remote)])
+                    [(or (eof-object? line) (string-null? line))
+                     (values remote 0)])
+                (values remote chunk-size)))
+            ;; something's wrong
+            (error <http-error> "bad line in chunked data:" line))))))
+  (begin0 (receiver code headers total callback)
+    (when condition (raise condition))))
 
 ;;==============================================================
 ;; secure agent handling
@@ -758,6 +804,7 @@
 (define (shutdown-secure-agent conn)
   (when (~ conn'secure-agent)
     (tls-close (~ conn'secure-agent))
+    (tls-destroy (~ conn'secure-agent))
     (set! (~ conn'secure-agent) #f)))
 
 (define (start-secure-agent conn)
@@ -778,5 +825,16 @@
 ;; authentication handling
 ;;
 
-;; dummy - to be written
-(define (http-default-auth-handler . _) #f)
+(define (http-auth-headers conn)
+  (or (and-let* ([auth-handler (~ conn'auth-handler)])
+        (auth-handler conn))
+      '()))
+
+(define (http-basic-auth-handler conn)
+  (and-let* ([user (~ conn 'auth-user)]
+             [pass (or (~ conn 'auth-password) "")])
+    `(:authorization ,($ format "Basic ~a"
+                         $ base64-encode-string #`",|user|:,|pass|"))))
+
+(define http-default-auth-handler
+  (make-parameter http-basic-auth-handler))
