@@ -1,7 +1,7 @@
 ;;;
 ;;; mime.scm - parsing MIME (rfc2045) message
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -43,15 +43,11 @@
 ;;            Character Sets, Languages, and Continuations
 
 (define-module rfc.mime
-  (use gauche.vport)
-  (use gauche.uvector)
   (use srfi-1)
   (use srfi-13)
   (use srfi-14)
   (use rfc.822)
   (use util.match)
-  (use util.queue)
-  (use util.list)
   (export mime-parse-version
           mime-parse-content-type mime-parse-content-disposition
           mime-parse-parameters mime-compose-parameters
@@ -71,6 +67,7 @@
           base64-encode-string base64-encode)
 (autoload gauche.charconv
           ces-upper-compatible? ces-conversion-supported? ces-convert)
+(autoload rfc.mime-port make-mime-port)
 (autoload srfi-27 random-integer)       ;for MIME boundary generation
 
 ;;===============================================================
@@ -83,17 +80,17 @@
              [s (string-concatenate
                  (map x->string (rfc822-field->tokens field)))]
              [m (#/^(\d+)\.(\d+)$/ s)])
-    (map (lambda (d) (x->integer (m d))) '(1 2))))
+    (map (^d (x->integer (m d))) '(1 2))))
 
 (define-constant *ct-token-chars*
-  (char-set-difference #[\x21-\x7e] #[()<>@,\;:\\\"/\[\]?=]))
+  (char-set-difference #[\u0021-\u007e] #[()<>@,\;:\\\"/\[\]?=]))
 
 ;; RFC2045 Content-Type header field
 ;; returns (<type> <subtype> (<attribute> . <value>) ...)
 (define (mime-parse-content-type field)
   (and field
        (call-with-input-string field
-         (lambda (input)
+         (^[input]
            (and-let* ([type (rfc822-next-token input `(,*ct-token-chars*))]
                       [ (string? type) ]
                       [ (eqv? #\/ (rfc822-next-token input '())) ]
@@ -108,7 +105,7 @@
 (define (mime-parse-content-disposition field)
   (and field
        (call-with-input-string field
-         (lambda (input)
+         (^[input]
            (and-let* ([token (rfc822-next-token input `(,*ct-token-chars*))]
                       [ (string? token) ])
              (cons (string-downcase token)
@@ -131,7 +128,7 @@
                                (#[\"] . ,rfc822-quoted-string)))]
                       [ (string? val) ])
              (cons attr val))
-           => (lambda (p) (loop (cons p r)))]
+           => (^p (loop (cons p r)))]
           [else (reverse! r)])))
 
 ;; Inverse of mime-parse-parameters.
@@ -149,7 +146,7 @@
       (unless (string-every *ct-token-chars* z)
         (error "invalid parameter value for rfc2822 header:" p))))
   (define (gen)
-    (fold (lambda (pv column)
+    (fold (^[pv column]
             (match pv
               [(p . v)
                (let* ([z #`",(valid-name p)=,(quote-value (x->string v))"]
@@ -190,10 +187,9 @@
 (define (mime-decode-word word)
   (cond [(string-incomplete? word) word] ;; safety net
         [(rxmatch *mime-encoded-header-rx* word)
-         => (lambda (m)
-              (if (equal? (m'after) "")
-                (%mime-decode-word word (m 1) (m 2) (m 3))
-                word))]
+         => (^m (if (equal? (m'after) "")
+                  (%mime-decode-word word (m 1) (m 2) (m 3))
+                  word))]
         [else word]))
 
 ;; Decode the entire header field body, possibly a mixture of
@@ -203,25 +199,22 @@
 ;; the parser.  The correct order is to parse first, then apply
 ;; mime-decode-word for each token.
 (define (mime-decode-text body)
-  (let loop ((s body))
+  (let loop ([s body])
     (receive (pre rest) (string-scan s "=?" 'before*)
       (cond [(not pre) s]
             [(rxmatch *mime-encoded-header-rx* rest)
-             => (lambda (m)
-                  (string-append pre
-                                 (%mime-decode-word (m 0) (m 1) (m 2) (m 3))
-                                 (loop (m'after))))]
+             => (^m (string-append pre
+                                   (%mime-decode-word (m 0) (m 1) (m 2) (m 3))
+                                   (loop (m'after))))]
             [else s]))))
 
 ;; Encode a single word.  This one always encode, even WORD contains
 ;; US-ASCII characters only.
 (define (mime-encode-word word :key (charset "utf-8") (transfer-encoding 'base64))
-  (let ((converted
-         (cond [(ces-upper-compatible? charset (gauche-character-encoding))
-                word]
-               [else
-                (ces-convert word (gauche-character-encoding) charset)]))
-        (enc (%canonical-encoding transfer-encoding)))
+  (let ([converted
+         (cond [(ces-upper-compatible? charset (gauche-character-encoding)) word]
+               [else (ces-convert word (gauche-character-encoding) charset)])]
+        [enc (%canonical-encoding transfer-encoding)])
     (format "=?~a?~a?~a?=" charset enc
             ((if (eq? enc 'B)
                base64-encode-string
@@ -238,17 +231,17 @@
                           (line-width 76)
                           (start-column 0)
                           (force #f))
-  (let ((enc (%canonical-encoding transfer-encoding))
-        (cslen (string-length (x->string charset)))
-        (pass-through? (and (not force)
+  (let ([enc (%canonical-encoding transfer-encoding)]
+        [cslen (string-length (x->string charset))]
+        [pass-through? (and (not force)
                             (ces-upper-compatible? charset 'ascii)
-                            (string-every #[\x00-\x7f] body))))
+                            (string-every #[\u0000-\u007f] body))])
     ;; estimates the length of an encoded word.  it is not trivial since
     ;; we have to ces-convert each segment separately, and there's no
     ;; general way to estimate the size of ces-converted string.
     ;; we throw in some heuristics here.
     (define (estimate-width s i)
-      (let1 na (string-count s #[\x00-\x7f] 0 i)
+      (let1 na (string-count s #[\u0000-\u007f] 0 i)
         (+ 6 cslen
            (if (eq? enc 'B)
              (ceiling (* (+ na (* (- i na) 3)) 4/3))
@@ -264,7 +257,7 @@
             (if (<= (string-length ew) width)
               `(,ew)
               (encode str width (* adj (/. (string-length ew) width)))))
-          (let loop ((k (min (string-length str) (quotient width 2))))
+          (let loop ([k (min (string-length str) (quotient width 2))])
             (let1 estim (* adj (estimate-width str k))
               (if (<= estim width)
                 (let1 ew (encode-word (string-take str k))
@@ -313,120 +306,6 @@
                  transfer-encoding)]))
 
 
-;;===============================================================
-;; Virtual port to recognize mime boundary
-;;
-
-(define-class <mime-port> (<buffered-input-port>)
-  ((state :init-form 'prologue)
-   ;; prologue -> boundary <-> body -> eof
-   ))
-
-;; Creates a procedural port, which reads from SRCPORT until it reaches
-;; either EOF or MIME boundary.  Basically it runs a DFA.
-(define (make-mime-port boundary srcport)
-  (define q (make-queue))
-  (define --boundary (string->u8vector #`"--,boundary"))
-
-  (define port (make <mime-port>))
-
-  (define eof (read-from-string ""))
-
-  (define (deq! q)
-    (if (queue-empty? q) eof (dequeue! q)))
-
-  (define (fifo! q b)
-    (enqueue! q b) (dequeue! q))
-
-  (define (getb)
-    (if (queue-empty? q)
-      (case (ref port 'state)
-        [(prologue) (skip-prologue)]
-        [(boundary eof) eof]
-        [else (newb)])
-      (dequeue! q)))
-
-  (define (newb)
-    (match (read-byte srcport)
-      [(and #x0d b) ;; CR, check to see LF
-       (let1 b2 (peek-byte srcport)
-         (if (eqv? b2 #x0a)
-           (begin
-             (read-byte srcport)
-             (enqueue! q b #x0a)
-             (check-boundary))
-           b))]
-      [(and #x0a b) ;; LF, check boundary
-       (enqueue! q b) (check-boundary)]
-      [(? eof-object?) (set! (ref port 'state) 'eof) eof]
-      [b b]))
-
-  (define (check-boundary)
-    (let loop ((b   (peek-byte srcport))
-               (ind 0)
-               (max (u8vector-length --boundary)))
-      (cond [(eof-object? b) (deq! q)]
-            [(= ind max)
-             (cond [(memv b '(#x0d #x0a)) ;;found boundary
-                    (read-byte srcport)   ;;consume LF or CRLF
-                    (when (and (eqv? #x0d b)
-                               (eqv? #x0a (peek-byte srcport)))
-                      (read-byte srcport))
-                    (dequeue-all! q)
-                    (set! (ref port 'state) 'boundary)
-                    eof]
-                   [(eqv? b #x2d) ;; maybe end boundary
-                    (enqueue! q (read-byte srcport))
-                    (cond [(eqv? (peek-byte srcport) #x2d);; yes, end boundary
-                           (read-byte srcport)
-                           (dequeue-all! q)
-                           (skip-epilogue)]
-                          [else (deq! q)])]
-                   [else (deq! q)])]
-            [(= b (u8vector-ref --boundary ind))
-             (enqueue! q (read-byte srcport))
-             (loop (peek-byte srcport) (+ ind 1) max)]
-            [(queue-empty? q) (newb)]
-            [else (dequeue! q)])))
-
-  ;; Reads past the first boundary.  The first boundary may appear
-  ;; at the beginning of the message (instead of after CRLF), so
-  ;; we need slightly different handling than the normal getb.
-  (define (skip-prologue)
-    (let loop ((b (check-boundary)))
-      (cond
-       [(eof-object? b)
-        (cond [(eq? (ref port 'state) 'boundary) ; we've found the boundary
-               (set! (ref port 'state) 'body) (getb)]
-              [else                              ; no boundary found
-               (set! (ref port 'state) 'eof) eof])]
-       [(queue-empty? q) (loop (newb))]
-       [else (dequeue-all! q) (loop (newb))])))
-
-  (define (skip-epilogue)
-    (let loop ((b (read-byte srcport)))
-      (if (eof-object? b)
-        (begin (set! (ref port 'state) 'eof) b)
-        (loop (read-byte srcport)))))
-
-  ;; fills vector, until it sees either
-  ;;   (1) vec got full
-  ;;   (2) srcport reaches EOF
-  ;;   (3) mime-boundary is read
-  (define (fill vec)
-    (let1 len (u8vector-length vec)
-      (let loop ((ind 0))
-        (if (= ind len)
-          len
-          (let1 b (getb)
-            (if (eof-object? b)
-              ind
-              (begin (u8vector-set! vec ind b)
-                     (loop (+ ind 1)))))))))
-
-  (set! (ref port 'fill) fill)
-  port)
-
 ;;===============================================================
 ;; Basic streaming parser
 ;;
@@ -481,8 +360,8 @@
                          '("message" "rfc822")
                          '("text" "plain" ("charset" . "us-ascii")))]
          [mime-port (make-mime-port boundary port)])
-    (let loop ((index 0)
-               (contents '()))
+    (let loop ([index 0]
+               [contents '()])
       (let* ([headers (rfc822-header->list mime-port)]
              [r (internal-parse mime-port headers handler
                                 packet index
@@ -503,9 +382,9 @@
     ))
 
 (define (message-parse port packet handler)
-  (let* ((headers (rfc822-header->list port))
-         (r (internal-parse port headers handler packet 0
-                            '("text" "plain" ("charset" . "us-ascii")))))
+  (let* ([headers (rfc822-header->list port)]
+         [r (internal-parse port headers handler packet 0
+                            '("text" "plain" ("charset" . "us-ascii")))])
     (set! (ref packet 'content) (list r)))
   packet)
 
@@ -516,8 +395,8 @@
 (define (mime-retrieve-body packet inp outp)
 
   (define (read-line/nl)
-    (let loop ((c (read-char inp))
-               (chars '()))
+    (let loop ([c (read-char inp)]
+               [chars '()])
       (cond [(eof-object? c)
              (if (null? chars) c  (list->string (reverse! chars)))]
             [(char=? c #\newline) (list->string (reverse! (cons c chars)))]
@@ -531,7 +410,7 @@
             [else (loop (read-char inp) (cons c chars))])))
 
   (define (read-text decoder)
-    (let loop ((line (read-line/nl)))
+    (let loop ([line (read-line/nl)])
       (unless (eof-object? line)
         (display (decoder line) outp)
         (loop (read-line/nl)))))
@@ -542,26 +421,25 @@
         (cut with-port-locking (current-input-port)
              (cut with-output-to-port out base64-decode))))
     (let1 buf (open-output-string :private? #t)
-      (let loop ((line (read-line inp)))
-	(unless (eof-object? line)
+      (let loop ([line (read-line inp)])
+        (unless (eof-object? line)
           (display line buf)
           (loop (read-line inp))))
       (base64-output (get-output-string buf) outp))
     )
 
   (with-port-locking inp
-    (lambda ()
-      (let1 enc (ref packet 'transfer-encoding)
-        (cond
-         [(string-ci=? enc "base64") (read-base64)]
-         [(string-ci=? enc "quoted-printable")
-          (read-text quoted-printable-decode-string)]
-         [(member enc '("7bit" "8bit" "binary"))
-          (let loop ((b (read-byte inp)))
-            (unless (eof-object? b)
-              (write-byte b outp)
-              (loop (read-byte inp))))]
-         ))))
+    (^[] (let1 enc (ref packet 'transfer-encoding)
+           (cond
+            [(string-ci=? enc "base64") (read-base64)]
+            [(string-ci=? enc "quoted-printable")
+             (read-text quoted-printable-decode-string)]
+            [(member enc '("7bit" "8bit" "binary"))
+             (let loop ((b (read-byte inp)))
+               (unless (eof-object? b)
+                 (write-byte b outp)
+                 (loop (read-byte inp))))]
+            ))))
   )
 
 (define (mime-body->string packet inp)
@@ -571,7 +449,7 @@
 
 (define (mime-body->file packet inp filename)
   (call-with-output-file filename
-    (lambda (outp)
+    (^[outp]
       (with-port-locking outp
         (cut mime-retrieve-body packet inp outp))))
   filename)
@@ -651,7 +529,7 @@
       (push! (ref part'parameters) (cons "boundary" (mime-make-boundary)))))
   (let1 cte (mime-generate-part-header part port)
     (with-output-to-port port
-      (lambda ()
+      (^[]
         (cond
          [(ref part'source) =>
           (cut with-input-from-file <> (cut mime-generate-part-body part cte))]
