@@ -1,7 +1,7 @@
 ;;;
 ;;; srfi-14/query - character set (querying contents)
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,13 +35,6 @@
 
 (select-module srfi-14)
 
-(define (char-set-size cs)
-  (let ((count 0))
-    (for-each (lambda (range)
-                (set! count (+ count (- (cdr range) (car range)) 1)))
-              (%char-set-ranges cs))
-    count))
-
 (define (char-set-count pred cs)
   (char-set-fold (lambda (c r) (if (pred c) (+ r 1) r)) 0 cs))
 
