@@ -2,7 +2,7 @@
 ;;; combinations.scm - combinations and that sort of stuff.
 ;;;
 ;;;  Copyright(C) 2003 by Alex Shinn (foof@synthcode.com)
-;;;  Copyright (c) 2003-2010  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2003-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
 ;;;  accompanying documentation for any purpose is hereby granted,
@@ -135,21 +135,21 @@
                        (but-kth set i))
                      (loop (+ i 1) (cons (car p) seen) (cdr p))]))]))
   (rec proc set))
-                          
+
 ;;----------------------------------------------------------------
 ;; combinations
 ;;
 
 (define (combinations set n)
-  (if (not (positive? n))
-    (list '())
-    (pair-fold-right
-     (lambda (pr acc)
-       (fold-right (cut acons (car pr) <> <>)
-                   acc
-                   (combinations (cdr pr) (- n 1))))
-     '()
-     set)))
+  (define (rec set tail)
+    (cond [(null? tail) (list set)]
+          [(eq? (cdr set) tail) (map list set)]
+          [else (fold-right (cut acons (car set) <> <>)
+                            (rec (cdr set) (cdr tail))
+                            (rec (cdr set) tail))]))
+  (cond [(not (positive? n)) (list '())]
+        [(list-tail set n #f) => (cut rec set <>)]
+        [else '()]))
 
 (define (combinations* set n :optional (eq eqv?))
   (define (rec set n)
@@ -205,7 +205,7 @@
       (list '())
       (let ((x (car set))
             (rest (power-set-binary (cdr set))))
-        (append rest (map (lambda (s) (cons x s)) rest)))))
+        (append rest (map (^s (cons x s)) rest)))))
 
 ;; use combinations for nice ordering
 (define (power-set set)
@@ -284,7 +284,7 @@
             (rest (cartesian-product (cdr lol))))
         (append-map!
          (lambda (sub-prod)
-           (map (lambda (x) (cons x sub-prod)) l))
+           (map (^x (cons x sub-prod)) l))
          rest))))
 
 (define (cartesian-product-right-for-each proc lol)
@@ -292,7 +292,7 @@
       (proc '())
       (cartesian-product-right-for-each
        (lambda (sub-prod)
-         (for-each (lambda (x) (proc (cons x sub-prod))) (car lol)))
+         (for-each (^x (proc (cons x sub-prod))) (car lol)))
        (cdr lol))))
 
 
