@@ -1,7 +1,7 @@
 ;;;
 ;;; isomorph.scm - check isomorphism
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,58 +33,46 @@
 
 (define-module util.isomorph
   (use srfi-1)
+  (use srfi-43)
   (export isomorphic? object-isomorphic?)
   )
 (select-module util.isomorph)
 
-(define (isomorphic? a b . args)
-  (define ctx (if (pair? args)
-                  (if (hash-table? (car args))
-                      (car args)
-                      (error "hash table required, but got" (car args)))
-                  (make-hash-table)))
-
+(define (isomorphic? a b :optional (ctx (make-hash-table)))
   (define (iso? a b)
-    (cond ((or (boolean? a) (char? a) (number? a) (undefined? a)
-               (eof-object? a))
-           (eqv? a b))
-          ((eq? a '()) (eq? b '()))
-          ((hash-table-get ctx a #f)   ;node has been appeared
-           => (lambda (bb) (eq? bb b)))
-          (else
+    (cond [(or (boolean? a) (null? a) (char? a) (number? a) (symbol? a)
+               (keyword? a) (undefined? a) (eof-object? a))
+           (eqv? a b)]
+          [(hash-table-get ctx a #f)   ;node has been appeared
+           => (^[bb] (eq? bb b))]
+          [else
            (hash-table-put! ctx a b)
-           (cond ((pair? a) (and (pair? b)
-                                 (iso? (car a) (car b))
-                                 (iso? (cdr a) (cdr b))))
-                 ((string? a) (and (string? b) (string=? a b)))
-                 ((symbol? a) (eq? a b))
-                 ((keyword? a) (eqv? a b))
-                 ((vector? a) (vector-iso? a b))
-                 (else (object-isomorphic? a b ctx))
-                 ))))
+           (cond [(pair? a)   (and (pair? b)
+                                   (iso? (car a) (car b))
+                                   (iso? (cdr a) (cdr b)))]
+                 [(string? a) (and (string? b) (string=? a b))]
+                 [(vector? a) (vector-iso? a b)]
+                 [else (object-isomorphic? a b ctx)])]))
 
   (define (vector-iso? a b)
     (and (vector? b)
-         (let loop ((la (vector->list a)) (lb (vector->list b)))
-           (cond ((null? la) (null? lb))
-                 ((null? lb) #f)
-                 ((iso? (car la) (car lb)) (loop (cdr la) (cdr lb)))
-                 (else #f)))))
+         (= (vector-length a) (vector-length b))
+         (vector-every iso? a b)))
 
   (define (hash-iso? a b)
     (and (hash-table? b)
-         (let loop ((la (hash-table-map a cons))
-                    (lb (hash-table-map b cons)))
-           (cond ((null? a) (null? b))
-                 ((null? b) #f)
-                 ((assq (caar la) lb)
-                  => (lambda (p)
-                       (and (iso? (cdar la) (cdr p))
-                            (loop (cdr la) (alist-delete (caar la) lb)))))
-                 (else #f)))))
+         (let loop ([la (hash-table-map a cons)]
+                    [lb (hash-table-map b cons)])
+           (cond [(null? a) (null? b)]
+                 [(null? b) #f]
+                 [(assq (caar la) lb)
+                  => (^p (and (iso? (cdar la) (cdr p))
+                              (loop (cdr la) (alist-delete (caar la) lb))))]
+                 [else #f]))))
 
-  (iso? a b)
-  )
+  (unless (hash-table? ctx)
+    (error "hash table required, but got" ctx))
+  (iso? a b))
 
 (define-method object-isomorphic? (a b context)
   (equal? a b))                         ;default
