@@ -1,23 +1,23 @@
 ;;;
 ;;; relation.scm - relational operations
 ;;;
-;;;  Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2005-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;;; Given set of values S1, S2, ..., Sn, a relation R is a set of tuples
 ;;; such that the first element of a tuple is from S1, the second from
@@ -145,12 +145,12 @@
 ;; the list of getters, for example.
 (define-method relation-column-getters ((r <relation>))
   (let1 accessor (relation-accessor r)
-    (map (lambda (c) (lambda (row) (accessor row c)))
+    (map (^c (lambda (row) (accessor row c)))
          (relation-column-names r))))
 
 (define-method relation-column-setters ((r <relation>))
   (let1 modifier (relation-modifier r)
-    (map (lambda (c) (lambda (row val) (modifier row c val)))
+    (map (^c (lambda (row val) (modifier row c val)))
          (relation-column-names r))))
 
 ;; Returns a procedure that coerces a row into a sequence.
@@ -193,7 +193,7 @@
 
 ;;
 ;; <simple-relation>
-;;   
+;;
 
 (define-class <simple-relation> (<relation> <collection>)
   ((columns :init-keyword :columns :init-value '()) ;; sequence of symbols
@@ -288,7 +288,7 @@
 
 (define-method relation-coercer ((r <object-set-relation>))
   (lambda (row)
-    (map (lambda (s) (slot-ref row s)) (relation-column-names r))))
+    (map (^s (slot-ref row s)) (relation-column-names r))))
 
 (define-method relation-insertable? ((r <object-set-relation>)) #t)
 (define-method relation-insert! ((r <object-set-relation>) row)
