@@ -14,7 +14,7 @@
 
 (define (tree-walk tree proc leaf? walker)
   (define (rec node)
-    (walker (lambda (n) (if (leaf? n) (proc n) (rec n))) node))
+    (walker (^n (if (leaf? n) (proc n) (rec n))) node))
   (if (leaf? tree) (proc tree) (rec tree)))
 
 (define (tree-fold tree proc knil leaf? folder)
@@ -26,10 +26,10 @@
 (define (tree-walk-bf tree proc leaf? walker)
   (define (rec node)
     (let* ((branches '()))
-      (walker (lambda (n) (if (leaf? n) (proc n) (push! branches n))) node)
+      (walker (^n (if (leaf? n) (proc n) (push! branches n))) node)
       (for-each rec (reverse branches))))
   (if (leaf? tree) (proc tree) (rec tree)))
-                    
+
 (define (tree-fold-bf tree proc knil leaf? folder)
   (define (rec node r)
     (let* ((branches '())
@@ -40,5 +40,5 @@
                       r node)))
       (fold rec r (reverse branches))))
   (if (leaf? tree) (proc tree knil) (rec tree knil)))
-                    
+
 
