@@ -1,7 +1,7 @@
 ;;;
 ;;; cgi.scm - CGI utility
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,19 +39,16 @@
 
 (define-module www.cgi
   (use srfi-1)
-  (use srfi-2)
   (use srfi-13)
   (use rfc.uri)
-  (use rfc.cookie)
   (use rfc.mime)
   (use rfc.822)
   (use gauche.parameter)
   (use gauche.charconv)
-  (use gauche.vport)
+  (use gauche.uvector)
   (use text.tree)
   (use text.html-lite)
   (use file.util)
-  (use util.list)
   (export cgi-metavariables
           cgi-get-metavariable
           cgi-output-character-encoding
@@ -68,6 +65,9 @@
   )
 (select-module www.cgi)
 
+(autoload rfc.cookie parse-cookie-string)
+(autoload gauche.vport open-input-limited-length-port)
+
 ;;----------------------------------------------------------------
 ;; A parameter cgi-metavariables can be used to supply metavariables,
 ;; instead of via environment variables.  It is handy for debugging,
@@ -76,8 +76,8 @@
 
 ;; Internal routine to get metavariable
 (define (get-meta name)
-  (or (and-let* ((mv (cgi-metavariables))
-                 (p  (assoc name mv)))
+  (or (and-let* ([mv (cgi-metavariables)]
+                 [p  (assoc name mv)])
         (cadr p))
       (sys-getenv name)))
 
@@ -127,9 +127,9 @@
 ;; without retrieving input.   cgi-parse-parameters handles the mime
 ;; message.
 (define (cgi-get-query content-length)
-  (let* ((type    (mime-parse-content-type (get-meta "CONTENT_TYPE")))
-         (typesig (and type (list (car type) (cadr type))))
-         (method  (get-meta "REQUEST_METHOD")))
+  (let* ([type    (mime-parse-content-type (get-meta "CONTENT_TYPE"))]
+         [typesig (and type (list (car type) (cadr type)))]
+         [method  (get-meta "REQUEST_METHOD")])
     (unless (or (not type)
                 (member typesig
                         '(("application" "x-www-form-urlencoded")
@@ -137,7 +137,7 @@
       (errorf <cgi-content-type-error>
               :content-type type
               "Unsupported CONTENT_TYPE: ~a" type))
-    (cond ((not method)  ;; interactive use.
+    (cond [(not method)  ;; interactive use.
            (if (sys-isatty (current-input-port))
              (begin
                (display "Enter parameters (name=value).  ^D to stop.\n")
@@ -152,25 +152,44 @@
                            (cons line params))))))
              (error <cgi-request-method-error>
                     :request-method #f
-                    "REQUEST_METHOD not defined")))
-          ((or (string-ci=? method "GET")
+                    "REQUEST_METHOD not defined"))]
+          [(or (string-ci=? method "GET")
                (string-ci=? method "HEAD"))
-           (or (get-meta "QUERY_STRING") ""))
-          ((string-ci=? method "POST")
+           (or (get-meta "QUERY_STRING") "")]
+          [(string-ci=? method "POST")
            (if (equal? typesig '("multipart" "form-data"))
              'mime
-             (or (and-let* ((lenp (or content-length
-                                      (get-meta "CONTENT_LENGTH")))
-                            (len  (x->integer lenp))
-                            ((<= 0 len)))
-                   (string-incomplete->complete (read-block len)))
-                 (port->string (current-input-port)))))
-          (else
+             (or (and-let* ([lenp (or content-length
+                                      (get-meta "CONTENT_LENGTH"))]
+                            [len  (x->integer lenp)]
+                            [ (<= 0 len) ]
+                            [data (read-complete-block len)])
+                   ;; NB: When we're here, we know content-type is
+                   ;; application/x-www-form-urlencoded, so buf should consist
+                   ;; of ASCII characters.
+                   (u8vector->string data))
+                 (port->string (current-input-port))))]
+          [else
            (errorf <cgi-request-method-error>
                    :request-method method
-                   "Unknown REQUEST_METHOD: ~a" method))
+                   "Unknown REQUEST_METHOD: ~a" method)]
           )))
 
+;; Read LEN octets from the current input port and returns u8vector.
+;; The port buffering may not be :full, so we should keep reading
+;; until all data is retrieved.
+(define (read-complete-block len)
+  (if (zero? len)
+    '#u8()
+    (let ([buf (make-u8vector len)]
+          [inp (current-input-port)])
+      (let loop ([i 0])
+        (let1 nread (read-block! buf inp i)
+          (cond [(eof-object? nread)
+                 (errorf <cgi-error> "POST request ends prematurely.  Expected content-length: ~a, but read only ~a octets." len i)]
+                [(< (+ nread i) len) (loop (+ nread i))]
+                [else buf]))))))
+
 ;;----------------------------------------------------------------
 ;; API: cgi-parse-parameters &keyword query-string merge-cookies
 ;;                                    part-handlers content-type
@@ -182,38 +201,38 @@
                               (content-type #f)
                               (content-length #f)
                               (mime-input #f))
-  (let* ((input (cond (query-string)
-                      ((or mime-input content-type) 'mime)
-                      (else
-                       (cgi-get-query content-length))))
-         (cookies (cond ((and merge-cookies (get-meta "HTTP_COOKIE"))
-                         => parse-cookie-string)
-                        (else '()))))
-    (append
+  (let* ([input (cond [ query-string ]
+                      [(or mime-input content-type) 'mime]
+                      [else (cgi-get-query content-length)])]
+         [cookies (if-let1 s (and merge-cookies (get-meta "HTTP_COOKIE"))
+                    (parse-cookie-string s)
+                    '())])
+    (fold-params-by-name
      (cond
-      ((eq? input 'mime)
+      [(eq? input 'mime)
        (get-mime-parts part-handlers
                        (or content-type (get-meta "CONTENT_TYPE"))
                        (or content-length (get-meta "CONTENT_LENGTH"))
-                       (or mime-input (current-input-port))))
-      ((string-null? input) '())
-      (else (split-query-string input)))
-     (map (lambda (cookie) (list (car cookie) (cadr cookie))) cookies))))
+                       (or mime-input (current-input-port)))]
+      [(string-null? input) '()]
+      [else (split-query-string input)])
+     (map (^[cookie] (list (car cookie) (cadr cookie))) cookies))))
+
+(define (fold-params-by-name alist tail)
+  (fold-right (^[kv params]
+                (if-let1 p (assoc (car kv) params)
+                  (begin (push! (cdr p) (cdr kv)) params)
+                  (cons (list (car kv) (cdr kv)) params)))
+              tail alist))
 
 (define (split-query-string input)
-  (fold-right (lambda (elt params)
-                (let* ((ss (string-split elt #\=))
-                       (n  (uri-decode-string (car ss) :cgi-decode #t))
-                       (p  (assoc n params))
-                       (v  (if (null? (cdr ss))
-                             #t
-                             (uri-decode-string (string-join (cdr ss) "=")
-                                                :cgi-decode #t))))
-                  (if p
-                    (begin (set! (cdr p) (cons v (cdr p))) params)
-                    (cons (list n v) params))))
-              '()
-              (string-split input #[&\;])))
+  (map (^[elt] (let1 ss (string-split elt #\=)
+                 (cons (uri-decode-string (car ss) :cgi-decode #t)
+                       (if (null? (cdr ss))
+                         #t
+                         (uri-decode-string (string-join (cdr ss) "=")
+                                            :cgi-decode #t)))))
+       (string-split input #[&\;])))
 
 ;; part-handlers: ((<name-list> <action> <options> ...) ...)
 ;;  <name-list> : list of field names (string, symbol, or regexp)
@@ -241,7 +260,7 @@
     (rfc822-header-ref (ref info 'headers) name))
 
   (define (make-file-handler prefix honor-origfile? mode)
-    (lambda (name filename part-info inp)
+    (^[name filename part-info inp]
       (receive (outp tmpfile) (sys-mkstemp prefix)
         (cgi-add-temporary-file tmpfile)
         (mime-retrieve-body part-info inp outp)
@@ -255,11 +274,11 @@
     (mime-body->string part-info inp))
 
   (define (ignore-handler name filename part-info inp)
-    (let loop ((ignore (read-line inp #t)))
+    (let loop ([ignore (read-line inp #t)])
       (if (eof-object? ignore) #f (loop (read-line inp #t)))))
 
   (define (get-action&opts part-name)
-    (let1 clause (find (lambda (entry)
+    (let1 clause (find (^[entry]
                          (or (eq? (car entry) #t)
                              (and (regexp? (car entry))
                                   (rxmatch (car entry) part-name))
@@ -299,7 +318,7 @@
     (read-char input)                   ; discard beginning DQUOTE
     (let1 r (open-output-string :private? #t)
       (define (finish) (get-output-string r))
-      (let loop ((c (read-char input)))
+      (let loop ([c (read-char input)])
         (cond [(eof-object? c) (finish)] ; tolerate missing closing DQUOTE
               [(char=? c #\")  (finish)] ; discard ending DQUOTE
               [(char=? c #\\)
@@ -338,14 +357,14 @@
         (ignore-handler name filename part-info inp)
         #f]
        [(not filename)  ;; this is not a file uploading field.
-        (list name (string-handler name filename part-info inp))]
+        (cons name (string-handler name filename part-info inp))]
        [(string-null? filename) ;; file field is empty
-        (list name (ignore-handler name filename part-info inp))]
+        (cons name (ignore-handler name filename part-info inp))]
        [else
         (let* ([action&opts (get-action&opts name)]
                [handler (apply get-handler action&opts)]
                [result (handler name filename part-info inp)])
-          (list name result))])))
+          (cons name result))])))
 
   (let* ([inp (if (and clength (<= 0 (x->integer clength)))
                 (open-input-limited-length-port inp (x->integer clength))
@@ -378,13 +397,11 @@
     (when status   (push! r #`"Status: ,status\r\n"))
     (when ct       (push! r #`"Content-type: ,ct\r\n"))
     (when location (push! r #`"Location: ,location\r\n"))
-    (for-each (lambda (cookie)
-                (push! r #`"Set-cookie: ,cookie\r\n"))
-              cookies)
-    (for-each (lambda (p)
-                (when (pair? (cdr p))
-                  (push! r #`",(car p): ,(cadr p)\r\n")))
-              (slices rest 2))
+    (dolist [cookie cookies]
+      (push! r #`"Set-cookie: ,cookie\r\n"))
+    (dolist [p (slices rest 2)]
+      (when (pair? (cdr p))
+        (push! r #`",(car p): ,(cadr p)\r\n")))
     (push! r "\r\n")
     (reverse r)))
 
@@ -397,7 +414,7 @@
                   (merge-cookies #f)
                   (part-handlers '()))
   (set! (port-buffering (current-error-port)) :line)
-  (guard (e (else (output-proc (on-error e))))
+  (guard (e [else (output-proc (on-error e))])
     (let1 params (cgi-parse-parameters :merge-cookies merge-cookies
                                        :part-handlers part-handlers)
       (output-proc (proc params))))
