@@ -1,23 +1,23 @@
 ;;;
 ;;; file/util.scm - filesystem utility functions
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;;; This module provides convenient utility functions to handle
 ;;; files and directories.   Some functions are provided with
@@ -40,13 +40,9 @@
 
 (define-module file.util
   (use srfi-1)
-  (use srfi-11)
   (use srfi-13)
-  (use util.list)
   (use util.match)
   (use gauche.parameter)
-  (use gauche.experimental.lamb)
-  (use gauche.experimental.ref)
   (export current-directory directory-list directory-list2 directory-fold
           home-directory temporary-directory
           make-directory* create-directory* remove-directory* delete-directory*
@@ -54,7 +50,7 @@
           create-directory-tree check-directory-tree
           build-path resolve-path expand-path simplify-path decompose-path
           absolute-path? relative-path? find-file-in-paths
-	  path-separator
+          path-separator
           path-extension path-sans-extension path-swap-extension
           file-is-readable? file-is-writable? file-is-executable?
           file-is-symlink?
@@ -64,11 +60,13 @@
           file-mtime=? file-mtime<? file-mtime<=? file-mtime>? file-mtime>=?
           file-atime=? file-atime<? file-atime<=? file-atime>? file-atime>=?
           file-ctime=? file-ctime<? file-ctime<=? file-ctime>? file-ctime>=?
-          touch-file copy-file move-file 
+          touch-file touch-files copy-file move-file
           ;copy-files move-files
+          remove-file delete-file
           remove-files delete-files
           null-device console-device
           file->string file->string-list file->list file->sexp-list
+          <lock-file-failure> with-lock-file
           ))
 (select-module file.util)
 
@@ -102,15 +100,12 @@
     (unless (eqv? user (sys-getuid))
       (error "On windows native platforms, getting other user's home directory \
               is not supported (yet)."))
-    ;; NB: I'm not sure if this is a proper way, but it appears to work in
-    ;; typical cases.
-    (or (sys-getenv "HOME")             ; MSYS sets this.
-        (string-append (or (sys-getenv "HOMEDRIVE") "")
-                       (or (sys-getenv "HOMEPATH") "\\")))]
+    ;; See src/system.c for the way to determine the home directory.
+    (sys-normalize-pathname "~" :expand #t)]
    [else
-    (and-let* ([ent (cond ((integer? user) (sys-getpwuid user))
-                          ((string? user)  (sys-getpwnam user))
-                          (else (error "bad user" user)))])
+    (and-let* ([ent (cond [(integer? user) (sys-getpwuid user)]
+                          [(string? user)  (sys-getpwnam user)]
+                          [else (error "bad user" user)])])
       (slot-ref ent 'dir))]))
 
 (define temporary-directory
@@ -129,7 +124,7 @@
 ;; utility for directory-list and directory-list2
 (define (%directory-filter dir pred filter-add-path?)
   (if filter-add-path?
-    (filter (lambda (e) (pred (build-path dir e))) (sys-readdir dir))
+    (filter (^e (pred (build-path dir e))) (sys-readdir dir))
     (filter pred (sys-readdir dir))))
 
 (define (%directory-filter-compose opts)
@@ -137,7 +132,7 @@
                       (filter #f))
     (apply every-pred
            (cond-list
-            [children? (lambda (e) (not (member (sys-basename e) '("." ".."))))]
+            [children? (^e (not (member (sys-basename e) '("." ".."))))]
             [filter]))))
 
 ;; directory-list DIR &keyword ADD-PATH? FILTER-ADD-PATH? CHILDREN? FILTER
@@ -163,7 +158,7 @@
          [entries (sort (%directory-filter dir filters filter-add-path?))])
     (if add-path?
       (partition selector (map (cut build-path dir <>) entries))
-      (partition (lambda (e) (selector (build-path dir e))) entries))))
+      (partition (^e (selector (build-path dir e))) entries))))
 
 ;; directory-fold DIR PROC KNIL &keyword LISTER FOLDER FOLLOW-LINK?
 (define (directory-fold dir proc knil
@@ -243,7 +238,7 @@
     (copy-file src dst :if-exists if-exists :backup-suffix backsfx
                :follow-link? follow-link?
                :keep-timestamp keeptime :keep-mode keepmode))
-  
+
   (define (rec src dst)
     (cond [(file-is-directory? src)
            (let1 exists? (file-exists? dst)
@@ -316,13 +311,13 @@
                    [else sys-chown]))
     (define (walk dir node do-file do-dir)
       (match node
-        [[? name?] (do-file (mkpath dir node))]
+        [[? name?] (do-file (mkpath dir node) #f)]
         [([? name? n] . args)
          (receive (opts content) (collect-options args)
            (if (list? content)
              (apply do-dir (mkpath dir n) content opts)
              (apply do-file (mkpath dir n) content opts)))]
-        [_ (error "invalid tree node:" node)]))  
+        [_ (error "invalid tree node:" node)]))
 
     (define (ensure-file path content
                          :key (mode #f) (owner -1) (group -1) (symlink #f))
@@ -469,7 +464,7 @@
 
 (define (find-file-in-paths name
                             :key (paths (cond [(sys-getenv "PATH")
-                                               => (cut string-split <> 
+                                               => (cut string-split <>
                                                        (cond-expand
                                                         [gauche.os.windows #\;]
                                                         [else #\:]))]
@@ -487,7 +482,7 @@
 
 ;; convenient accessors for file stats.  accepts string file name or
 ;; <sys-stat>.  If the named file doesn't exist, returns #f.
-;; accepts keyword argument FOLLOW-LINK? 
+;; accepts keyword argument FOLLOW-LINK?
 (define-syntax define-stat-accessor
   (syntax-rules ()
     [(_ name slot)
@@ -533,7 +528,7 @@
  [gauche.os.windows
   (define (%stat-compare s1 s2 f1 f2)
     (let ([p1 (sys-normalize-pathname f1 :absolute #t :canonicalize #t)]
-	  [p2 (sys-normalize-pathname f2 :absolute #t :canonicalize #t)])
+          [p2 (sys-normalize-pathname f2 :absolute #t :canonicalize #t)])
       (equal? p1 p2)))
   (define (file-eq? f1 f2)  (%stat-compare #f #f f1 f2))
   (define (file-eqv? f1 f2) (%stat-compare #f #f f1 f2))
@@ -615,15 +610,30 @@
 
 ;;;=============================================================
 ;;; File operation
+;;;
 
-(define (touch-file pathname)
-  (if (sys-access pathname F_OK)
-    (sys-utime pathname)
-    (close-output-port (open-output-file pathname)))
+(define (touch-file pathname :key (time #f) (type #f) (create #t))
+  (unless (or (not time) (and (real? time) (not (negative? time))))
+    (error "bad value for time: #f or nonnegative real is expected, got:" time))
+  (cond
+   [(sys-access pathname F_OK)
+    (case type
+      [(#f) (sys-utime pathname time time)]
+      [(mtime) (let1 s (sys-stat pathname)
+                 (sys-utime pathname (~ s'atime) time))]
+      [(atime) (let1 s (sys-stat pathname)
+                 (sys-utime pathname time (~ s'mtime)))]
+      [else (error "bad value for type: #f, 'atime, or 'mtime is expected, got:"
+                   type)])]
+   [create
+    (close-output-port (open-output-file pathname))
+    (when time (touch-file pathname :time time :type type))])
   (values))
 
-(define (touch-files pathnames)
-  (for-each touch-file pathnames))
+(define (touch-files pathnames :key (time (undefined))
+                                    (type (undefined))
+                                    (create (undefined)))
+  (for-each (cut touch-file <> :time time :type type :create create) pathnames))
 
 ;; copy-file
 ;;  if-exists     - :error :supersede :backup #f
@@ -718,7 +728,7 @@
       (do-symlink)
       (do-copy))
     ))
-  
+
 ;; move-file
 ;;  if-exists  - :error :supersede :backup #f
 ;;  backup-suffix
@@ -736,10 +746,10 @@
       (begin (sys-rename src dst) #t)))
   (define (do-copying)
     (and (copy-file src dst :if-exists if-exists
-                    :backup-suffix backsfx 
+                    :backup-suffix backsfx
                     :safe #t :keep-timestamp #t :keep-mode #t)
          (begin (sys-unlink src) #t)))
-  
+
   ;; body of move-file
   (unless (memq if-exists '(#f :error :supersede :backup))
     (error "argument for :if-exists must be either :error, :supersede, :backup or #f, but got" if-exists))
@@ -774,7 +784,15 @@
 ; (define copy-files (%multifile-cpmv copy-file))
 ; (define move-files (%multifile-cpmv move-file))
 
-;; remove files
+;; removing.
+;; Unlike sys-unlink, we throw an error if FILE doesn't exist,
+;; to align with R7RS.  (Technically we should throw a file-error
+;; object.  We'll fix that later.)
+(define (remove-file file)
+  (or (sys-unlink file)
+      (error <system-error> :errno ENOENT "File does not exist:" file)))
+(define delete-file remove-file)
+
 (define (remove-files . paths)
   (dolist (p paths)
     (cond [(list? p) (apply remove-files p)] ; for the convenience
@@ -782,7 +800,7 @@
           [else (sys-unlink p)])))
 
 (define delete-files remove-files)
-  
+
 ;; file->string, file->list, file->string-list, file->sexp-list
 ;; shortcuts of port->string etc.
 ;; NB: call-with-input-file may pass #f to the proc if :if-does-not-exist #f
@@ -803,3 +821,126 @@
 (define (file->sexp-list file . opts)
   (apply call-with-input-file file (%maybe (cut port->list read <>)) opts))
 
+;;;=============================================================
+;;; File locking
+;;;
+
+;; fcntl lock is unreliable in MT environment (It's process-wide lock,
+;; so we need mutex to avoid inter-thread races.  However, it is not
+;; trivial to provide a mutex per arbitrary file to be locked.)
+;; So we just provide file/directory based locks.
+;;
+;; The drawback is that if a process holding the lock dies, the lock file
+;; may remain, blocking all other processes.  We support 'stealing' the
+;; lock: If the existing lock file is too old to be possibly held by the
+;; application, the attempted locker can grab the lock.
+;;
+;; Stealing makes things a bit complicated, since there can be a race
+;; if more than two processes tries to steal the lock at the same time.
+;; We use a secondary lock file to avoid the race.
+
+;; TODO: If we find the lock is held for a certain time,
+;; we want to check if the process is actually alive---using (sys-kill pid 0)
+;; or something.   If there's no process, we can steal the lock right away,
+;; not waiting for abandon-timeout.   This doesn't work for the lock for
+;; a resource that can be shared between multiple machines, so the feature
+;; should be optional.
+
+(define-condition-type <lock-file-failure> <error> #f
+  (lock-file-name #f))
+
+(define (with-lock-file lock-name proc
+                        :key (type 'file)
+                             (retry-interval 1)
+                             (retry-limit 10)
+                             (abandon-timeout 600)
+                             (secondary-lock-name #`",|lock-name|.2")
+                             (retry2-interval 1)
+                             (retry2-limit 10)
+                             (perms (if (eq? type 'directory)
+                                      #o755
+                                      #o644)))
+  ;; returns #t on success, #f of failure
+  (define (acquire path)
+    (case type
+      [(file)
+       (with-output-to-file path
+         (^() (print (sys-getpid)) (sys-chmod path perms))
+         :if-exists #f)]
+      [(directory)
+       (guard (e [(and (<system-error> e) (eqv? (~ e'errno) EEXIST)) #f])
+         (sys-mkdir path perms) #t)]))
+
+  ;; the simplest way is (remove-files path); but it will eradicate anything
+  ;; under PATH, which is a bit too radical.
+  (define (release path)
+    (guard (e [(<system-error> e)
+               (unless (eq? (~ e'errno) ENOENT)
+                 (errorf <lock-file-failure> :lock-file-name lock-name
+                         "couldn't remove lock file ~s: ~a"
+                         lock-name (sys-strerror (~ e'errno))))])
+      (case type
+        [(file) (sys-unlink path)]
+        [(directory) (sys-rmdir path)])))
+
+  ;; Common retry logic
+  ;;  interval and timeout should be in nanosecs
+  (define (try acquirer releaser interval timeout check success failure)
+    (let loop ([elapsed 0])
+      (cond [(acquirer) (unwind-protect (success) (releaser))]
+            [(< elapsed timeout)
+             (let wait ([w interval])
+               (if-let1 w1 (sys-nanosleep w)
+                 (wait w1)))
+             (check)
+             (loop (+ interval elapsed))]
+            [else (failure)])))
+
+  ;; scale seconds to nanoseconds
+  (define (nsec secs) (* secs 1e9))
+
+  (define (primary-lock)
+    (try (cut acquire secondary-lock-name)
+         (cut release secondary-lock-name)
+         (nsec retry2-interval) (nsec retry2-limit)
+         values
+         (cut acquire lock-name)
+         secondary-lock-failure))
+
+  (define (primary-unlock)
+    (try (cut acquire secondary-lock-name)
+         (cut release secondary-lock-name)
+         (nsec retry2-interval) (nsec retry2-limit)
+         values
+         (cut release lock-name)
+         secondary-lock-failure))
+
+  (define (secondary-lock-failure)
+    (errorf <lock-file-failure> :lock-file-name lock-name
+            "with-lock-file: secondary lock cannot be acquired (~a).  \
+             If no process is locking it, remove it and try again."
+            secondary-lock-name))
+
+  (define (primary-lock-failure)
+    (errorf <lock-file-failure> :lock-file-name lock-name
+            "with-lock-file: couldn't acquire lock file (~a).  \
+             If no process is locking it, remove the file and try again."
+            lock-name))
+
+  (define primary-lock-steal
+    (if abandon-timeout
+      (lambda ()
+        (if (and-let* ([m (file-mtime lock-name)])
+              (< (+ m abandon-timeout) (time->seconds (current-time))))
+          (primary-unlock)))
+      values))
+
+  (unless (memq type '(file directory))
+    (error "unsupported lockfile type:" type))
+  ;; main locker
+  (try primary-lock
+       primary-unlock
+       (nsec retry-interval) (nsec retry-limit)
+       primary-lock-steal
+       proc
+       primary-lock-failure))
