@@ -1,7 +1,7 @@
 ;;;
 ;;; collection.scm - collection generics
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -62,58 +62,65 @@
 
 (define-syntax with-iterator
   (syntax-rules ()
-    ((_ (coll end? next . opts) . body)
-     (call-with-iterator coll (lambda (end? next) . body) . opts))))
+    [(_ (coll end? next . opts) . body)
+     (call-with-iterator coll (^[end? next] . body) . opts)]))
 
-(define-method call-with-iterator ((coll <list>) proc . args)
-  (let* ((start (get-keyword :start args #f))
-         (p (if start (list-tail coll start) coll)))
-    (proc (cut null? p) (lambda () (pop! p)))))
+(define-method call-with-iterator ((coll <list>) proc
+                                   :key (start #f) :allow-other-keys)
+  (let1 p (if start (list-tail coll start) coll)
+    (proc (cut null? p) (^[] (pop! p)))))
 
-(define-method call-with-iterator ((coll <vector>) proc . args)
-  (let ((len (vector-length coll))
-        (i   (get-keyword :start args 0)))
-    (proc (cut >= i len)
-          (lambda () (begin0 (vector-ref coll i) (inc! i))))))
+(define-method call-with-iterator ((coll <vector>) proc
+                                   :key (start 0) :allow-other-keys)
+  (let1 len (vector-length coll)
+    (proc (cut >= start len)
+          (^[] (begin0 (vector-ref coll start) (inc! start))))))
 
-(define-method call-with-iterator ((coll <weak-vector>) proc . args)
-  (let ((len (weak-vector-length coll))
-        (i   (get-keyword :start args 0)))
-    (proc (cut >= i len)
-          (lambda () (begin0 (weak-vector-ref coll i) (inc! i))))))
+(define-method call-with-iterator ((coll <weak-vector>) proc
+                                   :key (start 0) :allow-other-keys)
+  (let1 len (weak-vector-length coll)
+    (proc (cut >= start len)
+          (^[] (begin0 (weak-vector-ref coll start) (inc! start))))))
 
-(define-method call-with-iterator ((coll <string>) proc . args)
-  (let* ((start (get-keyword :start args #f))
-         (s     (open-input-string (if start (string-copy coll start) coll)))
-         (ch    (read-char s)))
+(define-method call-with-iterator ((coll <string>) proc
+                                   :key (start #f) :allow-other-keys)
+  (let* ([s  (open-input-string (if start (string-copy coll start) coll))]
+         [ch (read-char s)])
     (proc (cut eof-object? ch)
-          (lambda () (let ((c ch))
-                       (set! ch (read-char s))
-                       c)))))
+          (^[] (rlet1 c ch
+                 (set! ch (read-char s)))))))
 
-(define-method call-with-iterator ((coll <hash-table>) proc . args)
-  (let ((eof-marker (cons #f #f))
-        (iter (%hash-table-iter coll)))
+(define-method call-with-iterator ((coll <hash-table>) proc :allow-other-keys)
+  (let ([eof-marker (cons #f #f)]
+        [iter (%hash-table-iter coll)])
     (receive (k v) (iter eof-marker)
       (proc (cut eq? k eof-marker)
-            (lambda ()
-              (begin0 (cons k v)
-                      (set!-values (k v) (iter eof-marker))))))))
+            (^[] (begin0 (cons k v)
+                   (set!-values (k v) (iter eof-marker))))))))
 
-(define-method call-with-iterator ((coll <tree-map>) proc . args)
-  (let ((eof-marker (cons #f #f))
-        (iter (%tree-map-iter coll)))
+(define-method call-with-iterator ((coll <tree-map>) proc :allow-other-keys)
+  (let ([eof-marker (cons #f #f)]
+        [iter (%tree-map-iter coll)])
     (receive (k v) (iter eof-marker #f)
       (proc (cut eq? k eof-marker)
-            (lambda ()
-              (begin0 (cons k v)
-                      (set!-values (k v) (iter eof-marker #f))))))))
+            (^[] (begin0 (cons k v)
+                   (set!-values (k v) (iter eof-marker #f))))))))
+
+;; NB: Do not depend on srfi-14.scm.
+(define-method call-with-iterator ((coll <char-set>) proc :allow-other-keys)
+  (let* ([ranges ((with-module gauche.internal %char-set-ranges) coll)]
+         [cursor (if (null? ranges) #f (caar ranges))])
+    (proc (^[] (not cursor))
+          (^[] (rlet1 c (integer->char cursor)
+                 (cond [(< cursor (cdar ranges)) (inc! cursor)]
+                       [(null? (cdr ranges)) (set! cursor #f)]
+                       [else (pop! ranges) (set! cursor (caar ranges))]))))))
 
 ;; n-ary case aux. proc
 (define (call-with-iterators colls proc)
-  (let loop ((colls colls)
-             (eprocs '())
-             (nprocs '()))
+  (let loop ([colls colls]
+             [eprocs '()]
+             [nprocs '()])
     (if (null? colls)
       (proc (reverse! eprocs) (reverse! nprocs))
       (with-iterator ((car colls) end? next)
@@ -125,84 +132,88 @@
 
 (define-syntax with-builder
   (syntax-rules ()
-    ((_ (class add! get . opts) . body)
-     (call-with-builder class (lambda (add! get) . body) . opts))))
+    [(_ (class add! get . opts) . body)
+     (call-with-builder class (^[add! get] . body) . opts)]))
 
-(define-method call-with-builder ((class <list-meta>) proc . args)
-  (let ((q (make-queue)))
+(define-method call-with-builder ((class <list-meta>) proc :allow-other-keys)
+  (let1 q (make-queue)
     (proc (cut enqueue! q <>) (cut queue->list q))))
 
-(define-method call-with-builder ((class <vector-meta>) proc . args)
-  (let ((size (get-keyword :size args #f)))
-    (if size
-        (let ((v (make-vector size))
-              (i 0))
-          (proc (lambda (item)
-                  (when (< i size)
-                    (vector-set! v i item)
-                    (inc! i)))
-                (lambda () v)))
-        (let ((q (make-queue)))
-          (proc (cut enqueue! q <>)
-                (cut list->vector (queue->list q)))))))
+(define-method call-with-builder ((class <vector-meta>) proc
+                                  :key (size #f) :allow-other-keys)
+  (if size
+    (let ([v (make-vector size)]
+          [i 0])
+      (proc (^[item] (when (< i size)
+                       (vector-set! v i item)
+                       (inc! i)))
+            (^[] v)))
+    (let1 q (make-queue)
+      (proc (cut enqueue! q <>)
+            (cut list->vector (queue->list q))))))
 
-(define-method call-with-builder ((class <weak-vector-meta>) proc . args)
-  (let ((size (get-keyword :size args #f)))
-    (if size
-        (let ((v (make-weak-vector size))
-              (i 0))
-          (proc (lambda (item)
-                  (when (< i size)
-                    (weak-vector-set! v i item)
-                    (inc! i)))
-                (lambda () v)))
-        (let ((q (make-queue))
-              (cnt 0))
-          (proc (lambda (item) (enqueue! q item) (inc! cnt))
-                (lambda ()
-                  (let ((v (make-weak-vector cnt)))
-                    (do ((i 0 (+ i 1)))
-                        ((= i cnt) v)
-                      (weak-vector-set! v i (dequeue! q)))))))
-        )))
+(define-method call-with-builder ((class <weak-vector-meta>) proc
+                                  :key (size #f) :allow-other-keys)
+  (if size
+    (let ([v (make-weak-vector size)]
+          [i 0])
+      (proc (^[item] (when (< i size)
+                       (weak-vector-set! v i item)
+                       (inc! i)))
+            (^[] v)))
+    (let ([q (make-queue)]
+          [cnt 0])
+      (proc (^[item] (enqueue! q item) (inc! cnt))
+            (^[] (let1 v (make-weak-vector cnt)
+                   (do ([i 0 (+ i 1)])
+                       [(= i cnt) v]
+                     (weak-vector-set! v i (dequeue! q)))))))))
 
-(define-method call-with-builder ((class <string-meta>) proc . args)
-  (let ((s (open-output-string)))
-    (proc (lambda (item)
+(define-method call-with-builder ((class <string-meta>) proc :allow-other-keys)
+  (let1 s (open-output-string)
+    (proc (^[item]
             (unless (char? item)
               (error "character required to build a string, but got" item))
             (write-char item s))
-          (lambda () (get-output-string s)))))
+          (^[] (get-output-string s)))))
 
-(define-method call-with-builder ((class <hash-table-meta>) proc . args)
-  (let* ((type (get-keyword :type args 'eq?))
-         (h    (make-hash-table type)))
-    (proc (lambda (item)
+(define-method call-with-builder ((class <hash-table-meta>) proc
+                                  :key (type 'eq?) :allow-other-keys)
+  (let1 h (make-hash-table type)
+    (proc (^[item]
             (unless (pair? item)
               (error "pair required to build a hashtable, but got" item))
             (hash-table-put! h (car item) (cdr item)))
-          (lambda () h))))
+          (^[] h))))
 
-(define-method call-with-builder ((class <tree-map-meta>) proc . args)
-  (let-keywords* args ((key=? =)
-                       (key<? <)
-                       (size #f))       ;ignored for now.
-    (let1 tree (make-tree-map key=? key<?)
-      (proc (lambda (item)
-              (unless (pair? item)
-                (error "pair required to build a tree-map, but got" item))
-              (tree-map-put! tree (car item) (cdr item)))
-            (lambda () tree)))))
+(define-method call-with-builder ((class <tree-map-meta>) proc
+                                  :key (key=? =) (key<? <)
+                                  :allow-other-keys)
+  (let1 tree (make-tree-map key=? key<?)
+    (proc (^[item]
+            (unless (pair? item)
+              (error "pair required to build a tree-map, but got" item))
+            (tree-map-put! tree (car item) (cdr item)))
+          (^[] tree))))
+
+;; NB: size key is unused
+(define-method call-with-builder ((class <char-set-meta>) proc
+                                  :key (size #f) :allow-other-keys)
+  (let1 cs (char-set)
+    (proc (^c (unless (char? c)
+                (error "character required to build a char-set, but got" c))
+              ((with-module gauche.internal %char-set-add-chars!) cs (list c)))
+          (^[] cs))))
 
 ;; utility.  return minimum size of collections if it's easily known, or #f.
 (define (maybe-minimum-size col more)
-  (let1 size (and-let* ((siz (lazy-size-of col))
-                        ( (integer? siz) ))
+  (let1 size (and-let* ([siz (lazy-size-of col)]
+                        [ (integer? siz) ])
                siz)
     (if (or (null? more) (not size))
       size  ;; short path
-      (let loop ((cols more)
-                 (r    size))
+      (let loop ([cols more]
+                 [r    size])
         (if (null? cols)
           r
           (let1 size (lazy-size-of (car cols))
@@ -217,7 +228,7 @@
 
 (define-syntax define-fold-k
   (syntax-rules ()
-    ((gen-fold-k name (seed ...))
+    [(gen-fold-k name (seed ...))
      (define-method name (proc seed ... (coll <collection>) . more)
        (if (null? more)
          (with-iterator (coll end? next)
@@ -228,17 +239,16 @@
                  (loop seed ...)))))
          (call-with-iterators
           (cons coll more)
-          (lambda (ends? nexts)
+          (^[ends? nexts]
             (let loop ((seed seed) ...)
               (if (any (cut <>) ends?)
                 (values seed ...)
                 (receive (seed ...)
-                    (apply proc (fold-right (lambda (p r) (cons (p) r))
+                    (apply proc (fold-right (^[p r] (cons (p) r))
                                             (list seed ...)
                                             nexts))
                   (loop seed ...))))))
-         )))
-    ))
+         ))]))
 
 ;; generic way.   This shadows builtin fold.
 (define-fold-k fold (knil))
@@ -256,43 +266,43 @@
 
 ;; partial applied version
 (define-method fold$ (proc)
-  (lambda (knil . lists) (apply fold proc knil lists)))
+  (^[knil . lists] (apply fold proc knil lists)))
 (define-method fold$ (proc knil)
-  (lambda lists (apply fold proc knil lists)))
+  (^ lists (apply fold proc knil lists)))
 
 (define-method fold2$ (proc knil1 knil2)
-  (lambda lists (apply fold2 proc knil1 knil2 lists)))
+  (^ lists (apply fold2 proc knil1 knil2 lists)))
 
 (define-method fold3$ (proc knil1 knil2 knil3)
-  (lambda lists (apply fold3 proc knil1 knil2 knil3 lists)))
+  (^ lists (apply fold3 proc knil1 knil2 knil3 lists)))
 
 ;; map --------------------------------------------------
 
 ;; generic way.  this shadows builtin map.
 (define-method map (proc (coll <collection>) . more)
   (if (null? more)
-      (with-iterator (coll end? next)
-        (do ((q (make-queue)))
-            ((end?) (queue->list q))
-          (enqueue! q (proc (next)))))
-      (let ((%map (with-module gauche map)))
-        (call-with-iterators
-         (cons coll more)
-         (lambda (ends? nexts)
-           (do ((q (make-queue)))
-               ((any (cut <>) ends?)
-                (queue->list q))
-             (enqueue! q (apply proc (%map (cut <>) nexts))))))
-        )))
+    (with-iterator (coll end? next)
+      (do ([q (make-queue)])
+          [(end?) (queue->list q)]
+        (enqueue! q (proc (next)))))
+    (let1 %map (with-module gauche map)
+      (call-with-iterators
+       (cons coll more)
+       (^[ends? nexts]
+         (do ([q (make-queue)])
+             [(any (cut <>) ends?)
+              (queue->list q)]
+           (enqueue! q (apply proc (%map (cut <>) nexts))))))
+      )))
 
 ;; for list arguments, built-in map is much faster.
 (define-method map (proc (coll <list>) . more)
-  (let ((%map (with-module gauche map)))
+  (let1 %map (with-module gauche map)
     (if (null? more)
-        (%map proc coll)
-        (if (every pair? more)
-            (apply %map proc coll more)
-            (next-method)))))
+      (%map proc coll)
+      (if (every pair? more)
+        (apply %map proc coll more)
+        (next-method)))))
 
 ;; redefine map$ to use generic version of map
 (define (map$ proc) (pa$ map proc))
@@ -305,14 +315,14 @@
     (with-builder (class add! get :size (size-of coll))
       (with-iterator (coll end? next)
         (do ()
-            ((end?) (get))
+            [(end?) (get)]
           (add! (proc (next))))))
     (with-builder (class add! get :size (maybe-minimum-size coll more))
       (call-with-iterators
        (cons coll more)
-       (lambda (ends? nexts)
+       (^[ends? nexts]
          (do ()
-             ((any (cut <>) ends?) (get))
+             [(any (cut <>) ends?) (get)]
            (add! (apply proc (map (cut <>) nexts)))))))))
 
 ;; map-to <list> is equivalent to map.
@@ -327,19 +337,19 @@
 (define-method map-accum (proc knil (coll <collection>) . more)
   (if (null? more)
     (receive (res knil)
-        (fold2 (lambda (elt lis knil)
+        (fold2 (^[elt lis knil]
                  (receive (res knil) (proc elt knil)
                    (values (cons res lis) knil)))
                '() knil coll)
       (values (reverse! res) knil))
     (call-with-iterators
      (cons coll more)
-     (lambda (ends? nexts)
-       (let loop ((lis '()) (knil knil))
+     (^[ends? nexts]
+       (let loop ([lis '()] [knil knil])
          (if (any (cut <>) ends?)
            (values (reverse! lis) knil)
            (receive (res knil)
-               (apply proc (fold-right (lambda (p r) (cons (p) r))
+               (apply proc (fold-right (^[p r] (cons (p) r))
                                        (list knil)
                                        nexts))
              (loop (cons res lis) knil))))))
@@ -350,18 +360,18 @@
 ;; generic way.  this shadows builtin for-each.
 (define-method for-each (proc (coll <collection>) . more)
   (if (null? more)
-      (with-iterator (coll end? next)
-        (until (end?) (proc (next))))
-      (let ((%map (with-module gauche map)))
-        (call-with-iterators
-         (cons coll more)
-         (lambda (ends? nexts)
-           (until (any (cut <>) ends?)
-             (apply proc (%map (cut <>) nexts))))))))
+    (with-iterator (coll end? next)
+      (until (end?) (proc (next))))
+    (let1 %map (with-module gauche map)
+      (call-with-iterators
+       (cons coll more)
+       (^[ends? nexts]
+         (until (any (cut <>) ends?)
+           (apply proc (%map (cut <>) nexts))))))))
 
 ;; for list arguments, built-in for-each is much faster.
 (define-method for-each (proc (coll <list>) . more)
-  (let ((%for-each (with-module gauche for-each)))
+  (let1 %for-each (with-module gauche for-each)
     (if (null? more)
         (%for-each proc coll)
         (if (every pair? more)
@@ -375,7 +385,7 @@
 
 ;; generic way
 (define-method size-of ((coll <collection>))
-  (fold (lambda (e r) (+ r 1)) 0 coll))
+  (fold (^[e r] (+ r 1)) 0 coll))
 
 (define-method lazy-size-of ((coll <collection>))
   (delay (size-of coll)))
@@ -385,6 +395,7 @@
 (define-method size-of ((coll <vector>))      (vector-length coll))
 (define-method size-of ((coll <weak-vector>)) (weak-vector-length coll))
 (define-method size-of ((coll <string>))      (string-length coll))
+(define-method size-of ((coll <char-set>))    (char-set-size coll))
 
 (define-method lazy-size-of ((coll <list>))        (length coll))
 (define-method lazy-size-of ((coll <vector>))      (vector-length coll))
@@ -398,9 +409,9 @@
   (with-iterator (coll end? next)
     (let loop ()
       (if (end?)
-          #f
-          (let ((e (next)))
-            (if (pred e) e (loop)))))))
+        #f
+        (let1 e (next)
+          (if (pred e) e (loop)))))))
 
 ;; shortcut
 (define-method find (pred (coll <list>))
@@ -408,70 +419,70 @@
 
 ;; find-min, find-max, find-min&max ---------------------
 
-(define-method find-min ((coll <collection>) . keys)
-  (let-keywords* keys ((key identity)
-                       (compare <)
-                       (default #f))
-    (%find-minmax-1 coll key compare default)))
+(define-method find-min ((coll <collection>)
+                         :key (key identity)
+                              (compare <)
+                              (default #f))
+  (%find-minmax-1 coll key compare default))
 
-(define-method find-max ((coll <collection>) . keys)
-  (let-keywords* keys ((key identity)
-                       (compare <)
-                       (default #f))
-    (%find-minmax-1 coll key (complement compare) default)))
+(define-method find-max ((coll <collection>)
+                         :key (key identity)
+                              (compare <)
+                              (default #f))
+  (%find-minmax-1 coll key (complement compare) default))
 
 (define (%find-minmax-1 coll key compare default)
   (with-iterator (coll end? next)
     (if (end?)
       default
       (let1 elt (next)
-        (let loop ((val (key elt))
-                   (elt elt))
+        (let loop ([val (key elt)]
+                   [elt elt])
           (if (end?)
             elt
-            (let* ((e (next))
-                   (v (key e)))
+            (let* ([e (next)]
+                   [v (key e)])
               (if (compare v val)
                 (loop v e)
                 (loop val elt)))))))))
 
-(define-method find-min&max ((coll <collection>) . keys)
-  (let-keywords* keys ((key identity)
-                       (compare <)
-                       (default #f)
-                       (default-min default)
-                       (default-max default))
-    (with-iterator (coll end? next)
-      (if (end?)
-        (values default-min default-max)
-        (let1 elt (next)
-          (let loop ((minval (key elt))
-                     (minelt elt)
-                     (maxval (key elt))
-                     (maxelt elt))
-            (if (end?)
-              (values minelt maxelt)
-              (let* ((e (next))
-                     (v (key e)))
-                (cond ((compare v minval) (loop v e maxval maxelt))
-                      ((compare maxval v) (loop minval minelt v e))
-                      (else (loop minval minelt maxval maxelt)))))))))))
+(define-method find-min&max ((coll <collection>)
+                             :key (key identity)
+                                  (compare <)
+                                  (default #f)
+                                  (default-min default)
+                                  (default-max default))
+  (with-iterator (coll end? next)
+    (if (end?)
+      (values default-min default-max)
+      (let1 elt (next)
+        (let loop ([minval (key elt)]
+                   [minelt elt]
+                   [maxval (key elt)]
+                   [maxelt elt])
+          (if (end?)
+            (values minelt maxelt)
+            (let* ([e (next)]
+                   [v (key e)])
+              (cond [(compare v minval) (loop v e maxval maxelt)]
+                    [(compare maxval v) (loop minval minelt v e)]
+                    [else (loop minval minelt maxval maxelt)]))))))))
 
 ;; filter -----------------------------------------------
 
 ;; generic way
 (define-method filter (pred (coll <collection>))
-  (let ((q (make-queue)))
+  (let1 q (make-queue)
     (with-iterator (coll end? next)
-      (until (end?) (let ((e (next))) (when (pred e) (enqueue! q e))))
+      (until (end?) (let1 e (next) (when (pred e) (enqueue! q e))))
       (queue->list q))))
 
 (define-method filter-to ((class <class>) pred (coll <collection>))
   (with-builder (class add! get)
     (with-iterator (coll end? next)
       (do ()
-          ((end?) (get))
-        (let ((e (next))) (when (pred e) (add! e)))))))
+          [(end?) (get)]
+        (let1 e (next) (when (pred e) (add! e)))))))
 
 ;; shortcut
 (define-method filter (pred (coll <list>))
@@ -484,17 +495,17 @@
 
 ;; generic way
 (define-method remove (pred (coll <collection>))
-  (let ((q (make-queue)))
+  (let1 q (make-queue)
     (with-iterator (coll end? next)
-      (until (end?) (let ((e (next))) (unless (pred e) (enqueue! q e))))
+      (until (end?) (let1 e (next) (unless (pred e) (enqueue! q e))))
       (queue->list q))))
 
 (define-method remove-to ((class <class>) pred (coll <collection>))
   (with-builder (class add! get)
     (with-iterator (coll end? next)
       (do ()
-          ((end?) (get))
-        (let ((e (next))) (unless (pred e) (add! e)))))))
+          [(end?) (get)]
+        (let1 e (next) (unless (pred e) (add! e)))))))
 
 ;; shortcut
 (define-method remove (pred (coll <list>))
@@ -508,21 +519,21 @@
 ;; generic way
 (define-method partition (pred (coll <collection>))
   (with-iterator (coll end? next)
-    (let loop ((y '()) (n '()))
+    (let loop ([y '()] [n '()])
       (if (end?)
-          (values (reverse y) (reverse n))
-          (let ((e (next)))
-            (if (pred e)
-                (loop (cons e y) n)
-                (loop y (cons e n))))))))
+        (values (reverse y) (reverse n))
+        (let1 e (next)
+          (if (pred e)
+            (loop (cons e y) n)
+            (loop y (cons e n))))))))
 
 (define-method partition-to ((class <class>) pred (coll <collection>))
   (with-builder (class add0! get0)
     (with-builder (class add1! get1)
       (with-iterator (coll end? next)
         (do ()
-            ((end?) (values (get0) (get1)))
-          (let ((e (next)))
+            [(end?) (values (get0) (get1))]
+          (let1 e (next)
             (if (pred e) (add0! e) (add1! e))))))))
 
 ;; shortcut
@@ -538,7 +549,7 @@
   (with-builder (class add! get :size (size-of coll))
     (with-iterator (coll end? next)
       (do ()
-          ((end?) (get))
+          [(end?) (get)]
         (add! (next))))))
 
 ;; shortcut
@@ -557,20 +568,16 @@
 ;;  gather elements with the same key value.
 ;;  cf. group-sequence in gauche.sequence
 
-(define-method group-collection ((col <collection>) . args)
-  (let-keywords args ((key-proc  :key identity)
-                      (test-proc :test  eqv?))
-    (fold (lambda (b r) (cons (reverse! (cdr b)) r))
-          '()
-          (fold (lambda (elt buckets)
-                  (let1 key (key-proc elt)
-                    (cond
-                     ((assoc key buckets test-proc)
-                      => (lambda (p)
-                           (push! (cdr p) elt)
-                           buckets))
-                     (else
-                      (cons (list key elt) buckets)))))
-                '()
-                col))))
+(define-method group-collection ((col <collection>)
+                                 :key ((:key key-proc) identity)
+                                      ((:test test-proc) eqv?))
+  (fold (^[b r] (cons (reverse! (cdr b)) r))
+        '()
+        (fold (^[elt buckets]
+                (let1 key (key-proc elt)
+                  (cond [(assoc key buckets test-proc)
+                         => (^p (push! (cdr p) elt) buckets)]
+                        [else (cons (list key elt) buckets)])))
+              '()
+              col)))
 
