@@ -1,7 +1,7 @@
 ;;;
 ;;; parameter.scm - parameter support
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,7 +34,6 @@
 ;; The API is upper-compatible to ChezScheme and Chicken's.
 
 (define-module gauche.parameter
-  (use gauche.hook)
   (export <parameter> make-parameter parameterize
           parameter-pre-observers
           parameter-post-observers
@@ -43,6 +42,8 @@
   )
 (select-module gauche.parameter)
 
+(autoload gauche.hook make-hook add-hook! delete-hook! run-hook)
+
 (define-class <parameter> ()
   (;; all slots should be private
    (filter :init-keyword :filter :init-value #f)
