@@ -8,7 +8,6 @@
 
 (define-module util.match
   (use srfi-1)
-  (use srfi-2)
   (export match
           match-lambda
           match-lambda*
@@ -227,8 +226,8 @@
                               (fail (and (pair? (cdr c))
                                          (pair? (cadr c))
                                          (equal? (caadr c) '=>)
-                                         (symid? (cadadr c))
                                          (pair? (cdadr c))
+                                         (symid? (cadadr c))
                                          (null? (cddadr c))
                                          (pair? (cddr c))
                                          (cadadr c)))
@@ -278,13 +277,13 @@
          (gs (map (lambda (_) (gensym)) bv)))
     (unreachable plist match-expr)
     `(begin ,@(map (lambda (v) `(define ,v #f)) bv)
-            `(let ((,x ,exp)
-                   (,code (lambda ,gs
-                            ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
-                            (cond (#f #f))))
-                   ,@bindings
-                   ,@(car eb-errf))
-               ,m))))
+            (let ((,x ,exp)
+                  (,code (lambda ,gs
+                           ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
+                           (cond (#f #f))))
+                  ,@bindings
+                  ,@(car eb-errf))
+              ,m))))
 
 (define (symbolize x)
   (cond [(symbol? x) x]
@@ -634,6 +633,11 @@
                  (ks success))
         (cond
          ((eq? '_ p) (ks sf))
+         ((keyword? p)
+          ;;We should make this warning optional, only when keyword-symbol
+          ;;integration mode is turned on.
+          ;;(warn "Unquoted keyword `~s' in match pattern: ~s" p x)
+          (emit `(equal? ,e ,p) sf kf ks))
          ((symid? p) (set! v (cons (cons p e) v))
           (ks sf))
          ((null? p) (emit `(null? ,e) sf kf ks))
@@ -642,7 +646,6 @@
          ((boolean? p) (emit `(equal? ,e ,p) sf kf ks))
          ((char? p) (emit `(equal? ,e ,p) sf kf ks))
          ((number? p) (emit `(equal? ,e ,p) sf kf ks))
-         ((keyword? p) (emit `(equal? ,e ,p) sf kf ks))
          ((and (pair? p) (eq? 'quote (car p)))
           (emit `(equal? ,e ,p) sf kf ks))
          ((and (pair? p) (eq? '? (car p)))
