@@ -1,7 +1,7 @@
 /*
  * boolean.c
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -70,49 +70,98 @@ int Scm_EqvP(ScmObj x, ScmObj y)
     return SCM_EQ(x, y);
 }
 
+/* Equal? needs to deal with circuler structures.
+   We adopt the algorithm in Adams&Dybvig's "Efficient Nondestructive
+   Equality Checking for Trees and Graphs",
+   Proceedings of ICFP 08, pp. 179-188.
+
+   It is much easier to write the algorithm in Scheme, but we don't want
+   the overhead of crossing C-Scheme boundary for trivial cases.
+   So we cover a simple cases (non-aggregates, user-defined objects and
+   flat lists/vectors) in C, and fall back to Scheme routine if we encounter
+   more complex structures.
+
+   Caveat: The cycle may involve user-defined objects.  To detect such
+   cycle, we need to pass down the context info to ScmClass.compare
+   procedure and object-equal? method.  This change would break
+   the backward compatibility, so we'll consider it in future versions.
+   For now, let such cyclic structures explode.
+*/
+
 int Scm_EqualP(ScmObj x, ScmObj y)
 {
-    ScmClass *cx, *cy;
+#define CHECK_AGGREGATE(a, b)                   \
+    do {                                        \
+        if (SCM_PAIRP(a)) {                     \
+            if (SCM_PAIRP(b)) goto fallback;    \
+            return FALSE;                       \
+        }                                       \
+        if (SCM_VECTORP(a)) {                   \
+            if (SCM_VECTORP(b)) goto fallback;  \
+            return FALSE;                       \
+        }                                       \
+        if (!Scm_EqualP(a, b)) return FALSE;    \
+    } while (0)                                 \
 
     if (SCM_EQ(x, y)) return TRUE;
+
+    if (SCM_NUMBERP(x)) {
+        if (!SCM_NUMBERP(y)) return FALSE;
+        return Scm_EqvP(x, y);
+    }
     if (SCM_PAIRP(x)) {
         if (!SCM_PAIRP(y)) return FALSE;
-        do {
-            if (!Scm_EqualP(SCM_CAR(x), SCM_CAR(y))) return FALSE;
-            x = SCM_CDR(x);
-            y = SCM_CDR(y);
-        } while (SCM_PAIRP(x)&&SCM_PAIRP(y));
-        return Scm_EqualP(x, y);
-    }
-    if (SCM_STRINGP(x)) {
-        if (SCM_STRINGP(y)) {
-            return Scm_StringEqual(SCM_STRING(x), SCM_STRING(y));
-        }
-        return FALSE;
-    }
-    if (SCM_NUMBERP(x)) {
-        if (SCM_NUMBERP(y)) {
-            if ((SCM_EXACTP(x) && SCM_EXACTP(y))
-                || (SCM_INEXACTP(x) && SCM_INEXACTP(y))) {
-                return Scm_NumEq(x, y);
+        /* We loop on "spine" of lists, so that the typical long flat list
+           can be compared quickly.  If we find nested lists/vectors, we
+           jump to Scheme routine.  We adopt hare and tortoise to detect
+           loop in the CDR side. */
+        ScmObj xslow = x; ScmObj yslow = y;
+        int xcirc = FALSE; int ycirc = FALSE;
+        for (;;) {
+            ScmObj carx = SCM_CAR(x);
+            ScmObj cary = SCM_CAR(y);
+            CHECK_AGGREGATE(carx, cary);
+
+            x = SCM_CDR(x); y = SCM_CDR(y);
+            if (!SCM_PAIRP(x) || !SCM_PAIRP(y)) return Scm_EqualP(x, y);
+            carx = SCM_CAR(x); cary = SCM_CAR(y);
+
+            CHECK_AGGREGATE(carx, cary);
+
+            if (xslow == x) {
+                if (ycirc) return TRUE;
+                xcirc = TRUE;
             }
+            if (yslow == y) {
+                if (xcirc) return TRUE;
+                ycirc = TRUE;
+            }
+
+            x = SCM_CDR(x); y = SCM_CDR(y);
+            if (!SCM_PAIRP(x) || !SCM_PAIRP(y)) return Scm_EqualP(x, y);
+            xslow = SCM_CDR(xslow); yslow = SCM_CDR(yslow);
         }
-        return FALSE;
     }
     if (SCM_VECTORP(x)) {
-        if (SCM_VECTORP(y)) {
-            int sizx = SCM_VECTOR_SIZE(x);
-            int sizy = SCM_VECTOR_SIZE(y);
-            if (sizx == sizy) {
-                while (sizx--) {
-                    if (!Scm_EqualP(SCM_VECTOR_ELEMENT(x, sizx),
-                                    SCM_VECTOR_ELEMENT(y, sizx)))
-                        break;
-                }
-                if (sizx < 0) return TRUE;
-            }
+        if (!SCM_VECTORP(y)) return FALSE;
+        ScmWord i = 0, len = SCM_VECTOR_SIZE(x);
+        if (SCM_VECTOR_SIZE(y) != len) return FALSE;
+        for (; i < len; i++) {
+            ScmObj xx = SCM_VECTOR_ELEMENT(x, i);
+            ScmObj yy = SCM_VECTOR_ELEMENT(y, i);
+            /* NB: If we detect nested structure in middle of vectors,
+               we run Scheme routine for the entire vector; so we'll test
+               equality of elements before the current one again.  If
+               they are simple objects that's negligible, but there may
+               be objects of user-defined datatypes, for which object-equal?
+               could be expensive; we'll fix it in future. */
+            CHECK_AGGREGATE(xx, yy);
         }
-        return FALSE;
+        return TRUE;
+    }
+    if (SCM_STRINGP(x)) {
+        if (!SCM_STRINGP(y)) return FALSE;
+        return Scm_StringEqual(SCM_STRING(x), SCM_STRING(y));
     }
     /* EXPERIMENTAL: when identifier is compared by equal?,
        we use its symbolic name to compare.  This allows
@@ -134,12 +183,20 @@ int Scm_EqualP(ScmObj x, ScmObj y)
     /* End of EXPERIMENTAL code */
 
     if (!SCM_HPTRP(x)) return (x == y);
-    cx = Scm_ClassOf(x);
-    cy = Scm_ClassOf(y);
-    if (cx == cy && cx->compare) {
-        return (cx->compare(x, y, TRUE) == 0);
+    ScmClass *cx = Scm_ClassOf(x);
+    ScmClass *cy = Scm_ClassOf(y);
+    if (cx == cy && cx->compare) return (cx->compare(x, y, TRUE) == 0);
+    else                         return FALSE;
+
+ fallback: 
+    {
+        /* Fall back to Scheme version. */
+        static ScmObj equal_interleave_proc = SCM_UNDEFINED;
+        SCM_BIND_PROC(equal_interleave_proc, "%interleave-equal?",
+                      Scm_GaucheInternalModule());
+        return !SCM_FALSEP(Scm_ApplyRec2(equal_interleave_proc, x, y));
     }
-    return FALSE;
+#undef CHECK_AGGREGATE
 }
 
 int Scm_EqualM(ScmObj x, ScmObj y, int mode)
