@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,9 +33,8 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/arch.h"
 #include "gauche/paths.h"
-#include "gauche/builtin-syms.h"
+#include "gauche/priv/builtin-syms.h"
 
 /* GC_print_static_roots() is declared in private/gc_priv.h.  It is too much
    hassle to include it with other GC internal baggages, so we just declare
@@ -67,7 +66,6 @@ static struct {
 extern void Scm__InitModule(void);
 extern void Scm__InitModulePost(void);
 extern void Scm__InitSymbol(void);
-extern void Scm__InitKeyword(void);
 extern void Scm__InitNumber(void);
 extern void Scm__InitChar(void);
 extern void Scm__InitClass(void);
@@ -85,16 +83,19 @@ extern void Scm__InitSignal(void);
 extern void Scm__InitSystem(void);
 extern void Scm__InitCode(void);
 extern void Scm__InitVM(void);
-extern void Scm__InitRepl(void);
 extern void Scm__InitAutoloads(void);
 extern void Scm__InitCollection(void);
+extern void Scm__InitComparator(void);
 
 extern void Scm_Init_libalpha(void);
 extern void Scm_Init_libbool(void);
 extern void Scm_Init_libchar(void);
+extern void Scm_Init_libcode(void);
+extern void Scm_Init_libcmp(void);
 extern void Scm_Init_libdict(void);
 extern void Scm_Init_libeval(void);
 extern void Scm_Init_libexc(void);
+extern void Scm_Init_libfmt(void);
 extern void Scm_Init_libio(void);
 extern void Scm_Init_liblazy(void);
 extern void Scm_Init_liblist(void);
@@ -149,7 +150,6 @@ void Scm_Init(const char *signature)
     Scm__InitVM();
     Scm__InitSymbol();
     Scm__InitModule();
-    Scm__InitKeyword();
     Scm__InitNumber();
     Scm__InitChar();
     Scm__InitClass();
@@ -166,14 +166,17 @@ void Scm_Init(const char *signature)
     Scm__InitRead();
     Scm__InitSignal();
     Scm__InitSystem();
-    Scm__InitRepl();
+    Scm__InitComparator();
 
     Scm_Init_libalpha();
     Scm_Init_libbool();
     Scm_Init_libchar();
+    Scm_Init_libcode();
+    Scm_Init_libcmp();
     Scm_Init_libdict();
     Scm_Init_libeval();
     Scm_Init_libexc();
+    Scm_Init_libfmt();
     Scm_Init_libio();
     Scm_Init_liblazy();
     Scm_Init_liblist();
@@ -361,10 +364,6 @@ void Scm_Exit(int code)
 
 void Scm_Cleanup(void)
 {
-    ScmVM *vm = Scm_VM();
-    ScmObj hp;
-    struct cleanup_handler_rec *ch;
-
     if (!cleanup.dirty) return;
     cleanup.dirty = FALSE;
 
@@ -373,13 +372,15 @@ void Scm_Cleanup(void)
        (A handler may intentionally raise an error to skip the rest of
        handlers).  We may change to break from SCM_FOR_EACH in case if
        we detect error in Scm_Apply. */
+    ScmVM *vm = Scm_VM();
+    ScmObj hp;
     SCM_FOR_EACH(hp, vm->handlers) {
         vm->handlers = SCM_CDR(hp);
         Scm_Apply(SCM_CDAR(hp), SCM_NIL, NULL);
     }
 
     /* Call the C-registered cleanup handlers. */
-    for (ch = cleanup.handlers; ch; ch = ch->next) {
+    for (struct cleanup_handler_rec *ch = cleanup.handlers; ch; ch = ch->next) {
         ch->handler(ch->data);
     }
 
@@ -450,13 +451,13 @@ Scm_AddFeature(const char *feature, const char *module)
 static void
 init_cond_features()
 {
-    int i;
     /* The initial cond-features list. */
     static struct {
         const char *feature;
         const char *module;
     } init_features[] = {
         { "gauche", NULL },
+        { "gauche-" GAUCHE_VERSION , NULL },
 
         /* Platform */
 #if   defined(GAUCHE_WINDOWS)
@@ -466,6 +467,29 @@ init_cond_features()
         { "gauche.os.cygwin", NULL },
 #endif
 
+        /* R7RS */
+        /* NB: We should probably make checking 'r7rs' trigger loading
+           r7rs module and setting up the environment. */
+        { "r7rs", NULL },
+            
+        /* R7RS Appendix B */
+        { "exact-closed", NULL },
+        // { "exact-complex", NULL }, /* not yet */
+        { "ieee-float", NULL },
+        { "full-unicode", NULL },
+        { "ratios", NULL },
+#if   defined(GAUCHE_WINDOWS)
+        { "windows", NULL },
+#else
+        { "posix", NULL },
+#endif
+        /* TODO: OS Flags, CPU arch flags and C memory model flags */
+#if   defined(WORDS_BIGENDIAN)
+        { "big-endian", NULL },
+#else
+        { "little-endian", NULL }, /* NB: r7rs say nothing on mixed endian */
+#endif
+
         /* Threads */
 #if   defined(GAUCHE_USE_PTHREADS)
         { "gauche.sys.threads", "gauche.threads" },
@@ -526,21 +550,18 @@ init_cond_features()
         { "srfi-61", NULL },
         { "srfi-62", NULL },
         { "srfi-87", NULL },
+        { "srfi-95", NULL },
         { "srfi-98", "srfi-98" },
         { "srfi-99", "gauche.record" },
+        { "srfi-106", "srfi-106" },
         { NULL, NULL }
     };
 
-    for (i=0; init_features[i].feature; i++) {
+    for (int i=0; init_features[i].feature; i++) {
         Scm_AddFeature(init_features[i].feature, init_features[i].module);
     }
 }
 
-const char *Scm_HostArchitecture(void)
-{
-    return GAUCHE_ARCH;
-}
-
 #ifndef PATH_MAX
 #define PATH_MAX 4096
 #endif
@@ -616,6 +637,21 @@ ScmObj Scm__RuntimeDirectory(void)
     return dir;
 }
 
+/*=============================================================
+ * Command line arguments
+ */
+
+ScmObj Scm_InitCommandLine(int argc, const char *argv[])
+{
+    static ScmObj command_line_proc = SCM_UNDEFINED;
+    ScmObj args = Scm_CStringArrayToList(argv, argc, SCM_STRING_IMMUTABLE);
+    SCM_BIND_PROC(command_line_proc, "command-line", Scm_GaucheModule());
+    Scm_ApplyRec1(command_line_proc, args);
+    SCM_DEFINE(Scm_UserModule(), "*program-name*", SCM_CAR(args));
+    SCM_DEFINE(Scm_UserModule(), "*argv*", SCM_CDR(args));
+    return args;
+}
+
 /*=============================================================
  * 'Main'
  */
@@ -639,22 +675,8 @@ int main(void)
 void Scm_SimpleMain(int argc, const char *argv[],
                     const char *script, u_long flags)
 {
-    ScmModule *user = Scm_UserModule();
-    ScmObj mainproc, args;
-    ScmLoadPacket lpak;
-
     SCM_ASSERT(argc > 0);
-    if (Scm_Load("gauche-init.scm", 0, &lpak)) {
-        Scm_Printf(SCM_CURERR, "%s: Couldn't load gauche-init.scm: %A(%A).\n",
-                   argv[0],
-                   Scm_ConditionMessage(lpak.exception),
-                   Scm_ConditionTypeName(lpak.exception));
-        Scm_Exit(1);
-    }
-
-    args = Scm_CStringArrayToList(argv, argc, SCM_STRING_IMMUTABLE);
-    SCM_DEFINE(user, "*program-name*", SCM_CAR(args));
-    SCM_DEFINE(user, "*argv*", SCM_CDR(args));
+    ScmObj args = Scm_InitCommandLine(argc, argv);
 
     if (script) {
         ScmObj s = SCM_MAKE_STR(script);
@@ -662,13 +684,13 @@ void Scm_SimpleMain(int argc, const char *argv[],
         Scm_LoadFromPort(SCM_PORT(p), SCM_LOAD_PROPAGATE_ERROR, NULL);
     }
 
-    mainproc = Scm_GlobalVariableRef(user, SCM_SYMBOL(SCM_INTERN("main")), 0);
+    ScmModule *user = Scm_UserModule();
+    ScmObj mainproc = Scm_GlobalVariableRef(user, SCM_SYMBOL(SCM_INTERN("main")), 0);
     if (SCM_PROCEDUREP(mainproc)) {
-        ScmObj r = Scm_ApplyRec(mainproc, SCM_LIST1(args));
+        ScmObj r = Scm_ApplyRec1(mainproc, args);
         if (SCM_INTP(r)) Scm_Exit(SCM_INT_VALUE(r));
         else             Scm_Exit(70);
     } else {
         Scm_Exit(70);
     }
 }
-
