@@ -1,7 +1,7 @@
 /*
  * error.c - error handling
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -36,7 +36,7 @@
 #include "gauche/class.h"
 #include "gauche/exception.h"
 #include "gauche/vm.h"
-#include "gauche/builtin-syms.h"
+#include "gauche/priv/builtin-syms.h"
 
 #include <errno.h>
 #include <string.h>
@@ -100,18 +100,66 @@ static ScmObj message_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(e);
 }
 
+/* See comment on gauche/exception.h about hack in 'message' slot.
+   TODO: Remove this hack on 1.0 release. */
 static ScmObj message_get(ScmMessageCondition *obj)
 {
-    return SCM_MESSAGE_CONDITION(obj)->message;
+    ScmObj msglist = obj->message;
+    if (SCM_PAIRP(msglist)) return SCM_CAR(msglist);
+    else return msglist;
 }
 
 static void message_set(ScmMessageCondition *obj, ScmObj val)
 {
-    obj->message = val;
+    ScmObj msglist = obj->message;
+    if (SCM_PAIRP(msglist)) SCM_SET_CAR(msglist, val);
+    else SCM_MESSAGE_CONDITION(obj)->message = SCM_LIST2(val, val);
+}
+
+static ScmObj message_prefix_get(ScmMessageCondition *obj)
+{
+    ScmObj msglist = obj->message;
+    if (SCM_PAIRP(msglist) && SCM_PAIRP(SCM_CDR(msglist))) {
+        return SCM_CADR(msglist);
+    } else {
+        return msglist;
+    }
+}
+
+static void message_prefix_set(ScmMessageCondition *obj, ScmObj val)
+{
+    ScmObj msglist = obj->message;
+    if (SCM_PAIRP(msglist) && SCM_PAIRP(SCM_CDR(msglist))) {
+        SCM_SET_CAR(SCM_CDR(msglist), val);
+    } else {
+        obj->message = SCM_LIST2(msglist, val);
+    }
+}
+
+static ScmObj message_args_get(ScmMessageCondition *obj)
+{
+    ScmObj msglist = obj->message;
+    if (SCM_PAIRP(msglist) && SCM_PAIRP(SCM_CDR(msglist))) {
+        return SCM_CDDR(msglist);
+    } else {
+        return SCM_NIL;
+    }
+}
+
+static void message_args_set(ScmMessageCondition *obj, ScmObj val)
+{
+    ScmObj msglist = obj->message;
+    if (SCM_PAIRP(msglist) && SCM_PAIRP(SCM_CDR(msglist))) {
+        SCM_SET_CDR(SCM_CDR(msglist), val);
+    } else {
+        obj->message = Scm_Cons(msglist, Scm_Cons(msglist, val));
+    }
 }
 
 static ScmClassStaticSlotSpec message_slots[] = {
     SCM_CLASS_SLOT_SPEC("message", message_get, message_set),
+    SCM_CLASS_SLOT_SPEC("message-prefix",   message_prefix_get, message_prefix_set),
+    SCM_CLASS_SLOT_SPEC("message-args", message_args_get, message_args_set),
     SCM_CLASS_SLOT_SPEC_END()
 };
 
@@ -333,7 +381,7 @@ static ScmObj compound_allocate(ScmClass *klass, ScmObj initargs)
 
 ScmObj Scm_MakeCompoundCondition(ScmObj conditions)
 {
-    ScmObj h = SCM_NIL, t = SCM_NIL, cp, cond;
+    ScmObj h = SCM_NIL, t = SCM_NIL;
     int serious = FALSE;
     int nconds = Scm_Length(conditions);
 
@@ -353,6 +401,7 @@ ScmObj Scm_MakeCompoundCondition(ScmObj conditions)
     }
 
     /* collect conditions and creates compound one */
+    ScmObj cp;
     SCM_FOR_EACH(cp, conditions) {
         ScmObj c = SCM_CAR(cp);
         if (!SCM_CONDITIONP(c)) {
@@ -369,10 +418,10 @@ ScmObj Scm_MakeCompoundCondition(ScmObj conditions)
             SCM_APPEND1(h, t, c);
         }
     }
-    cond = compound_allocate((serious?
-                              SCM_CLASS_COMPOUND_CONDITION :
-                              SCM_CLASS_SERIOUS_COMPOUND_CONDITION),
-                             SCM_NIL);
+    ScmObj cond = compound_allocate((serious?
+                                     SCM_CLASS_COMPOUND_CONDITION :
+                                     SCM_CLASS_SERIOUS_COMPOUND_CONDITION),
+                                    SCM_NIL);
     SCM_COMPOUND_CONDITION(cond)->conditions = h;
     return cond;
 }
@@ -418,7 +467,7 @@ ScmObj Scm_MakeThreadException(ScmClass *klass, ScmVM *thread)
 ScmObj Scm_MakeError(ScmObj message)
 {
     ScmError *e = SCM_ERROR(message_allocate(SCM_CLASS_ERROR, SCM_NIL));
-    e->message = message;
+    e->message = SCM_LIST2(message, message);
     return SCM_OBJ(e);
 }
 
@@ -426,7 +475,7 @@ ScmObj Scm_MakeSystemError(ScmObj message, int en)
 {
     ScmSystemError *e =
         SCM_SYSTEM_ERROR(syserror_allocate(SCM_CLASS_SYSTEM_ERROR, SCM_NIL));
-    e->common.message = message;
+    e->common.message = SCM_LIST2(message, message);
     e->error_number = en;
     return SCM_OBJ(e);
 }
@@ -435,7 +484,7 @@ ScmObj Scm_MakeReadError(ScmObj message, ScmPort *port, int line)
 {
     ScmReadError *e =
         SCM_READ_ERROR(readerror_allocate(SCM_CLASS_READ_ERROR, SCM_NIL));
-    e->common.message = message;
+    e->common.message = SCM_LIST2(message, message);
     e->port = port;
     e->line = line;
     return SCM_OBJ(e);
@@ -443,11 +492,11 @@ ScmObj Scm_MakeReadError(ScmObj message, ScmPort *port, int line)
 
 int Scm_ConditionHasType(ScmObj c, ScmObj k)
 {
-    ScmObj cp;
-
     if (!SCM_CONDITIONP(c)) return FALSE;
     if (!SCM_CLASSP(k)) return FALSE;
     if (!SCM_COMPOUND_CONDITION_P(c)) return SCM_ISA(c, SCM_CLASS(k));
+
+    ScmObj cp;
     SCM_FOR_EACH(cp, SCM_COMPOUND_CONDITION(c)->conditions) {
         if (SCM_ISA(SCM_CAR(cp), SCM_CLASS(k))) return TRUE;
     }
@@ -457,12 +506,12 @@ int Scm_ConditionHasType(ScmObj c, ScmObj k)
 ScmObj Scm_ConditionMessage(ScmObj c)
 {
     if (SCM_MESSAGE_CONDITION_P(c)) {
-        return SCM_MESSAGE_CONDITION(c)->message;
+        return message_get(SCM_MESSAGE_CONDITION(c));
     } else if (SCM_COMPOUND_CONDITION_P(c)) {
         ScmObj cp;
         SCM_FOR_EACH(cp, SCM_COMPOUND_CONDITION(c)->conditions) {
             if (SCM_MESSAGE_CONDITION_P(SCM_CAR(cp))) {
-                return SCM_MESSAGE_CONDITION(SCM_CAR(cp))->message;
+                return message_get(SCM_MESSAGE_CONDITION(SCM_CAR(cp)));
             }
         }
     }
@@ -648,8 +697,7 @@ void Scm_TypeError(const char *what, const char *expected, ScmObj got)
  */
 void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
 {
-    ScmObj e, smsg, pe;
-    ScmClass *peclass;
+    ScmObj e;
     ScmVM *vm = Scm_VM();
     va_list args;
     int en = get_errno();
@@ -664,7 +712,8 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
             SCM_PUTZ(": ", -1, ostr);
             SCM_PUTS(syserr, ostr);
         }
-        smsg = Scm_GetOutputString(SCM_PORT(ostr), 0);
+        ScmObj smsg = Scm_GetOutputString(SCM_PORT(ostr), 0);
+        ScmClass *peclass;
 
         switch (reason) {
         case SCM_PORT_ERROR_INPUT:
@@ -678,8 +727,8 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
         default:
             peclass = SCM_CLASS_PORT_ERROR; break;
         }
-        pe = porterror_allocate(peclass, SCM_NIL);
-        SCM_ERROR(pe)->message = smsg;
+        ScmObj pe = porterror_allocate(peclass, SCM_NIL);
+        SCM_ERROR(pe)->message = SCM_LIST2(smsg, smsg);
         SCM_PORT_ERROR(pe)->port = port;
 
         if (en != 0) {
@@ -712,14 +761,10 @@ void Scm_Warn(const char *msg, ...)
     va_end(args);
 }
 
-/* format & warn */
+/* OBSOLETED: 'warn' is now in Scheme. */
 void Scm_FWarn(ScmString *fmt, ScmObj args)
 {
-    ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
-    Scm_Format(SCM_PORT(ostr), fmt, args, TRUE);
-    Scm_Printf(SCM_CURERR, "WARNING: %A\n",
-               Scm_GetOutputString(SCM_PORT(ostr), 0));
-    Scm_Flush(SCM_CURERR);
+    Scm_Error("Scm_FWarn is obsoleted");
 }
 
 /*
@@ -788,7 +833,7 @@ ScmObj Scm_RaiseCondition(ScmObj condition_type, ...)
 
 /*
  * Show stack trace.
- *   stacklite - return value of Scm_GetStackLite
+ *   stacklite - return value of Scm_VMGetStackLite
  *   maxdepth - maximum # of stacks to be shown.
  *              0 to use the default.  -1 for unlimited.
  *   skip     - ignore this number of frames.  Useful to call this from
@@ -869,17 +914,14 @@ void Scm_ShowStackTrace(ScmPort *out, ScmObj stacklite,
    E is a thrown condition, not necessarily an error object. */
 static void Scm_PrintDefaultErrorHeading(ScmObj e, ScmPort *out)
 {
-    ScmObj msg;
-    char *heading, *p;
-
     if (SCM_CONDITIONP(e)) {
-        heading = Scm_GetString(SCM_STRING(Scm_ConditionTypeName(e)));
+        char *heading = Scm_GetString(SCM_STRING(Scm_ConditionTypeName(e)));
         /* TODO: considring that the class name may contain multibyte
            characters, we should use string-upcase here. */
-        for (p=heading; *p; p++) {
+        for (char *p=heading; *p; p++) {
             *p = toupper(*p);
         }
-        msg = Scm_ConditionMessage(e);
+        ScmObj msg = Scm_ConditionMessage(e);
         if (!SCM_FALSEP(msg)) {
             Scm_Printf(out, "*** %s: %A\n", heading, msg);
         } else {
@@ -946,7 +988,6 @@ void Scm_ReportError(ScmObj e)
 void Scm__InitExceptions(void)
 {
     ScmModule *mod = Scm_GaucheModule();
-    ScmClass *cond_meta;
 
     ScmObj mes_ser_supers
         = SCM_LIST2(SCM_OBJ(SCM_CLASS_MESSAGE_CONDITION),
@@ -958,7 +999,7 @@ void Scm__InitExceptions(void)
     Scm_InitStaticClassWithMeta(SCM_CLASS_CONDITION,
                                 "<condition>",
                                 mod, NULL, SCM_FALSE, NULL, 0);
-    cond_meta = Scm_ClassOf(SCM_OBJ(SCM_CLASS_CONDITION));
+    ScmClass *cond_meta = Scm_ClassOf(SCM_OBJ(SCM_CLASS_CONDITION));
     Scm_InitStaticClassWithMeta(SCM_CLASS_SERIOUS_CONDITION,
                                 "<serious-condition>",
                                 mod, cond_meta, SCM_FALSE, NULL, 0);
@@ -1016,4 +1057,3 @@ void Scm__InitExceptions(void)
                                 mod, cond_meta, com_ser_supers,
                                 compound_slots, 0);
 }
-
