@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -42,6 +42,10 @@
 #include <gauche/config.h>
 #include <gauche/config_threads.h>
 
+#if     GAUCHE_API_0_95         /* A provisional API towards 1.0 */
+#define GAUCHE_API_0_9   0
+#endif  /*GAUCHE_API_0_95*/
+
 #ifndef GAUCHE_API_0_9
 #define GAUCHE_API_0_9   1      /* 0.9 API */
 #endif
@@ -58,7 +62,6 @@
 #include <errno.h>
 #include <gauche/int64.h>
 #include <gauche/float.h>
-#include <gauche/arch.h>
 
 #ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -122,6 +125,8 @@ SCM_DECL_BEGIN
 /* Define this to 0 to turn off lazy-pair feature. */
 #define GAUCHE_LAZY_PAIR 1
 
+#undef GAUCHE_UNIFY_SYMBOL_KEYWORD /* not yet */
+
 /* Include appropriate threading interface.  Threading primitives are
    abstracted with SCM_INTERNAL_* macros and ScmInternal* typedefs.
    See gauche/uthread.h for the semantics of these primitives. */
@@ -222,8 +227,8 @@ typedef struct ScmClassRec ScmClass;
 
 /* Type coercer */
 
-#define	SCM_OBJ(obj)      ((ScmObj)(obj))
-#define	SCM_WORD(obj)     ((ScmWord)(obj))
+#define SCM_OBJ(obj)      ((ScmObj)(obj))
+#define SCM_WORD(obj)     ((ScmWord)(obj))
 
 /*
  * PRIMARY TAG IDENTIFICATION
@@ -273,7 +278,7 @@ typedef struct ScmClassRec ScmClass;
  */
 #define SCM_BOOLP(obj)       ((obj) == SCM_TRUE || (obj) == SCM_FALSE)
 #define SCM_BOOL_VALUE(obj)  (!SCM_FALSEP(obj))
-#define	SCM_MAKE_BOOL(obj)   ((obj)? SCM_TRUE:SCM_FALSE)
+#define SCM_MAKE_BOOL(obj)   ((obj)? SCM_TRUE:SCM_FALSE)
 
 #define SCM_EQ(x, y)         ((x) == (y))
 
@@ -299,6 +304,8 @@ SCM_EXTERN int Scm_EqualM(ScmObj x, ScmObj y, int mode);
 #define SCM_MAKE_INT(obj)    SCM_OBJ(((intptr_t)(obj) << 2) + 1)
 
 #define SCM_UINTP(obj)       (SCM_INTP(obj)&&((signed long int)SCM_WORD(obj)>=0))
+typedef long ScmSmallInt;    /* C integer type corresponds to Scheme fixnum
+                                See SCM_SMALL_* macros in gauche/number.h */
 
 /*
  * FLONUM
@@ -321,10 +328,10 @@ typedef struct ScmFlonumRec {
  *  For character cases, I only care about ASCII chars (at least for now)
  */
 
-#define	SCM_CHAR(obj)           ((ScmChar)(obj))
-#define	SCM_CHARP(obj)          ((SCM_WORD(obj)&0xff) == 3)
-#define	SCM_CHAR_VALUE(obj)     SCM_CHAR(((unsigned long)SCM_WORD(obj)) >> 8)
-#define	SCM_MAKE_CHAR(ch)       SCM_OBJ((long)((ch) << 8) + 3)
+#define SCM_CHAR(obj)           ((ScmChar)(obj))
+#define SCM_CHARP(obj)          ((SCM_WORD(obj)&0xff) == 3)
+#define SCM_CHAR_VALUE(obj)     SCM_CHAR(((unsigned long)SCM_WORD(obj)) >> 8)
+#define SCM_MAKE_CHAR(ch)       SCM_OBJ((((unsigned long)(ch))<<8) + 3)
 
 #define SCM_CHAR_INVALID        ((ScmChar)(-1)) /* indicate invalid char */
 #define SCM_CHAR_MAX            (0xffffff)
@@ -337,8 +344,8 @@ typedef struct ScmFlonumRec {
 #define SCM_CHAR_UPCASE(ch)     Scm_CharUpcase(ch)
 #define SCM_CHAR_DOWNCASE(ch)   Scm_CharDowncase(ch)
 
-SCM_EXTERN int Scm_DigitToInt(ScmChar ch, int radix);
-SCM_EXTERN ScmChar Scm_IntToDigit(int n, int radix);
+SCM_EXTERN int Scm_DigitToInt(ScmChar ch, int radix, int extended);
+SCM_EXTERN ScmChar Scm_IntToDigit(int n, int radix, int basechar1, int basechar2);
 SCM_EXTERN int Scm_CharToUcs(ScmChar ch);
 SCM_EXTERN ScmChar Scm_UcsToChar(int ucs);
 SCM_EXTERN ScmObj Scm_CharEncodingName(void);
@@ -455,6 +462,8 @@ typedef struct ScmInstanceRec {
 /* Fundamental allocators */
 #define SCM_MALLOC(size)          GC_MALLOC(size)
 #define SCM_MALLOC_ATOMIC(size)   GC_MALLOC_ATOMIC(size)
+#define SCM_STRDUP(s)             GC_STRDUP(s)
+#define SCM_STRDUP_PARTIAL(s, n)  Scm_StrdupPartial(s, n)
 
 #define SCM_NEW(type)         ((type*)(SCM_MALLOC(sizeof(type))))
 #define SCM_NEW_ARRAY(type, nelts) ((type*)(SCM_MALLOC(sizeof(type)*(nelts))))
@@ -488,7 +497,9 @@ typedef struct ScmTreeMapRec   ScmTreeMap;
 typedef struct ScmModuleRec    ScmModule;
 typedef struct ScmSymbolRec    ScmSymbol;
 typedef struct ScmGlocRec      ScmGloc;
+#if !GAUCHE_UNIFY_SYMBOL_KEYWORD
 typedef struct ScmKeywordRec   ScmKeyword;
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
 typedef struct ScmProcedureRec ScmProcedure;
 typedef struct ScmClosureRec   ScmClosure;
 typedef struct ScmSubrRec      ScmSubr;
@@ -502,6 +513,7 @@ typedef struct ScmRegexpRec    ScmRegexp;
 typedef struct ScmRegMatchRec  ScmRegMatch;
 typedef struct ScmWriteContextRec ScmWriteContext;
 typedef struct ScmAutoloadRec  ScmAutoload;
+typedef struct ScmComparatorRec ScmComparator;
 
 typedef ScmObj ScmSubrProc(ScmObj *, int, void*);
 
@@ -572,9 +584,9 @@ SCM_EXTERN ScmObj Scm_Values(ScmObj args);
 SCM_EXTERN ScmObj Scm_Values2(ScmObj val0, ScmObj val1);
 SCM_EXTERN ScmObj Scm_Values3(ScmObj val0, ScmObj val1, ScmObj val2);
 SCM_EXTERN ScmObj Scm_Values4(ScmObj val0, ScmObj val1, ScmObj val2,
-			      ScmObj val3);
+                              ScmObj val3);
 SCM_EXTERN ScmObj Scm_Values5(ScmObj val0, ScmObj val1, ScmObj val2,
-			      ScmObj val3, ScmObj val4);
+                              ScmObj val3, ScmObj val4);
 
 /* CPS API for evaluating Scheme fragments on VM. */
 SCM_EXTERN ScmObj Scm_VMApply(ScmObj proc, ScmObj args);
@@ -594,7 +606,7 @@ SCM_EXTERN ScmObj Scm_VMDynamicWind(ScmObj pre, ScmObj body, ScmObj post);
 SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
                                      ScmSubrProc *body,
                                      ScmSubrProc *after,
-				     void *data);
+                                     void *data);
 
 SCM_EXTERN ScmObj Scm_VMWithErrorHandler(ScmObj handler, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMWithGuardHandler(ScmObj handler, ScmObj thunk);
@@ -602,7 +614,7 @@ SCM_EXTERN ScmObj Scm_VMWithExceptionHandler(ScmObj handler, ScmObj thunk);
 
 /* Miscellaneous stuff */
 SCM_EXTERN ScmObj Scm_MakeMacroTransformer(ScmSymbol *name,
-					   ScmObj proc);
+                                           ScmObj proc);
 SCM_EXTERN ScmObj Scm_MakeMacroAutoload(ScmSymbol *name,
                                         ScmAutoload *al);
 
@@ -797,8 +809,8 @@ SCM_EXTERN void Scm_InitStaticClassWithMeta(ScmClass *klass,
 
 /* OBSOLETE */
 SCM_EXTERN void Scm_InitBuiltinClass(ScmClass *c, const char *name,
-				     ScmClassStaticSlotSpec *slots,
-				     int withMeta,
+                                     ScmClassStaticSlotSpec *slots,
+                                     int withMeta,
                                      ScmModule *m);
 
 SCM_EXTERN ScmClass *Scm_ClassOf(ScmObj obj);
@@ -938,6 +950,8 @@ SCM_EXTERN ScmClass *Scm_MakeForeignPointerClass(ScmModule *module,
                                                  ScmForeignCleanupProc cleanup,
                                                  int flags);
 SCM_EXTERN ScmObj Scm_MakeForeignPointer(ScmClass *klass, void *ptr);
+SCM_EXTERN ScmObj Scm_MakeForeignPointerWithAttr(ScmClass *klass, void *ptr,
+                                                 ScmObj attr);
 
 /* foreign pointer flags */
 enum {
@@ -1032,31 +1046,31 @@ SCM_CLASS_DECL(Scm_NullClass);
 
 /* Useful macros to manipulate lists. */
 
-#define	SCM_FOR_EACH(p, list) \
+#define SCM_FOR_EACH(p, list) \
     for((p) = (list); SCM_PAIRP(p); (p) = SCM_CDR(p))
 
-#define	SCM_APPEND1(start, last, obj)                           \
+#define SCM_APPEND1(start, last, obj)                           \
     do {                                                        \
-	if (SCM_NULLP(start)) {                                 \
-	    (start) = (last) = Scm_Cons((obj), SCM_NIL);        \
-	} else {                                                \
-	    SCM_SET_CDR((last), Scm_Cons((obj), SCM_NIL));      \
-	    (last) = SCM_CDR(last);                             \
-	}                                                       \
+        if (SCM_NULLP(start)) {                                 \
+            (start) = (last) = Scm_Cons((obj), SCM_NIL);        \
+        } else {                                                \
+            SCM_SET_CDR((last), Scm_Cons((obj), SCM_NIL));      \
+            (last) = SCM_CDR(last);                             \
+        }                                                       \
     } while (0)
 
-#define	SCM_APPEND(start, last, obj)                    \
+#define SCM_APPEND(start, last, obj)                    \
     do {                                                \
         ScmObj list_SCM_GLS = (obj);                    \
-	if (SCM_NULLP(start)) {                         \
-	    (start) = (list_SCM_GLS);                   \
+        if (SCM_NULLP(start)) {                         \
+            (start) = (list_SCM_GLS);                   \
             if (!SCM_NULLP(list_SCM_GLS)) {             \
                 (last) = Scm_LastPair(list_SCM_GLS);    \
             }                                           \
         } else {                                        \
-	    SCM_SET_CDR((last), (list_SCM_GLS));        \
-	    (last) = Scm_LastPair(last);                \
-	}                                               \
+            SCM_SET_CDR((last), (list_SCM_GLS));        \
+            (last) = Scm_LastPair(last);                \
+        }                                               \
     } while (0)
 
 #define SCM_LIST1(a)             Scm_Cons(a, SCM_NIL)
@@ -1084,7 +1098,7 @@ SCM_EXTERN ScmObj Scm_VaCons(va_list elts);
 SCM_EXTERN ScmObj Scm_ArrayToList(ScmObj *elts, int nelts);
 SCM_EXTERN ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail);
 SCM_EXTERN ScmObj *Scm_ListToArray(ScmObj list, int *nelts, ScmObj *store,
-				   int alloc);
+                                   int alloc);
 
 SCM_EXTERN ScmObj Scm_Car(ScmObj obj);
 SCM_EXTERN ScmObj Scm_Cdr(ScmObj obj);
@@ -1095,7 +1109,7 @@ SCM_EXTERN ScmObj Scm_Cddr(ScmObj obj);
 
 SCM_EXTERN int    Scm_Length(ScmObj obj);
 SCM_EXTERN ScmObj Scm_CopyList(ScmObj list);
-SCM_EXTERN ScmObj Scm_MakeList(int len, ScmObj fill);
+SCM_EXTERN ScmObj Scm_MakeList(ScmSmallInt len, ScmObj fill);
 SCM_EXTERN ScmObj Scm_Append2X(ScmObj list, ScmObj obj);
 SCM_EXTERN ScmObj Scm_Append2(ScmObj list, ScmObj obj);
 SCM_EXTERN ScmObj Scm_Append(ScmObj args);
@@ -1103,8 +1117,8 @@ SCM_EXTERN ScmObj Scm_ReverseX(ScmObj list);
 SCM_EXTERN ScmObj Scm_Reverse(ScmObj list);
 SCM_EXTERN ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail);
 SCM_EXTERN ScmObj Scm_Reverse2(ScmObj list, ScmObj tail);
-SCM_EXTERN ScmObj Scm_ListTail(ScmObj list, int i, ScmObj fallback);
-SCM_EXTERN ScmObj Scm_ListRef(ScmObj list, int i, ScmObj fallback);
+SCM_EXTERN ScmObj Scm_ListTail(ScmObj list, ScmSmallInt i, ScmObj fallback);
+SCM_EXTERN ScmObj Scm_ListRef(ScmObj list, ScmSmallInt i, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_LastPair(ScmObj list);
 
 SCM_EXTERN ScmObj Scm_Memq(ScmObj obj, ScmObj list);
@@ -1136,10 +1150,6 @@ SCM_EXTERN ScmObj Scm_PairAttrSet(ScmPair *pair, ScmObj key, ScmObj value);
  * CHARACTERS
  */
 
-SCM_EXTERN ScmChar Scm_ReadXdigitsFromString(const char *, int, const char **);
-SCM_EXTERN ScmChar Scm_ReadXdigitsFromPort(ScmPort *port, int ndigits,
-                                           char *buf, int *nread);
-
 /* Illegal character handling mode.  Used in some APIs that handles
    character conversion, such as input ports and string-incomplete->complete.
 */
@@ -1177,44 +1187,7 @@ typedef enum {
  * WRITE
  */
 
-struct ScmWriteContextRec {
-    short mode;                 /* print mode */
-    short flags;                /* internal */
-    int limit;                  /* internal */
-    int ncirc;                  /* internal */
-    ScmHashTable *table;        /* internal */
-    ScmObj obj;                 /* internal */
-};
-
-/* Print mode flags */
-enum {
-    SCM_WRITE_WRITE = 0,        /* write mode   */
-    SCM_WRITE_DISPLAY = 1,      /* display mode */
-    SCM_WRITE_SHARED = 2,       /* write/ss mode   */
-    SCM_WRITE_WALK = 3,         /* this is a special mode in write/ss */
-    SCM_WRITE_MODE_MASK = 0x3,
-
-    SCM_WRITE_CASE_FOLD = 4,    /* case-fold mode.  need to escape capital
-                                   letters. */
-    SCM_WRITE_CASE_NOFOLD = 8,  /* case-sensitive mode.  no need to escape
-                                   capital letters */
-    SCM_WRITE_CASE_MASK = 0x0c
-};
-
-#define SCM_WRITE_MODE(ctx)   ((ctx)->mode & SCM_WRITE_MODE_MASK)
-#define SCM_WRITE_CASE(ctx)   ((ctx)->mode & SCM_WRITE_CASE_MASK)
-
-SCM_EXTERN void Scm_Write(ScmObj obj, ScmObj port, int mode);
-SCM_EXTERN int Scm_WriteCircular(ScmObj obj, ScmObj port, int mode, int width);
-SCM_EXTERN int Scm_WriteLimited(ScmObj obj, ScmObj port, int mode, int width);
-SCM_EXTERN void Scm_Format(ScmPort *port, ScmString *fmt, ScmObj args, int ss);
-SCM_EXTERN void Scm_Printf(ScmPort *port, const char *fmt, ...);
-SCM_EXTERN void Scm_PrintfShared(ScmPort *port, const char *fmt, ...);
-SCM_EXTERN void Scm_Vprintf(ScmPort *port, const char *fmt, va_list args,
-                            int sharedp);
-SCM_EXTERN ScmObj Scm_Sprintf(const char *fmt, ...);
-SCM_EXTERN ScmObj Scm_SprintfShared(const char *fmt, ...);
-SCM_EXTERN ScmObj Scm_Vsprintf(const char *fmt, va_list args, int sharedp);
+#include <gauche/writer.h>
 
 /*---------------------------------------------------------
  * READ
@@ -1264,12 +1237,6 @@ SCM_EXTERN ScmObj Scm_Vsprintf(const char *fmt, va_list args, int sharedp);
 
 #include <gauche/gloc.h>
 
-/*--------------------------------------------------------
- * KEYWORD
- */
-
-#include <gauche/keyword.h>
-
 /*--------------------------------------------------------
  * NUMBER
  */
@@ -1314,7 +1281,7 @@ struct ScmProcedureRec {
    after argument folding).
 
    This special treatment is to avoid unnecessary consing of argumets;
-   if we know the callee immeidately unfolds the rest argument, it's no 
+   if we know the callee immediately unfolds the rest argument, it's no 
    use to fold excessive arguments anyway.
  */
 
@@ -1335,7 +1302,7 @@ struct ScmProcedureRec {
    
    #f: No inliner associated to this procedure.  (For historical
       reasons, the code that access to this slot expects this slot can be
-      NULL and treats it as SCM_FALSE)
+      NULL and treats it as SCM_FALSE in that case)
 
    <integer>: Only appears in some built-in procedures, and specifies
       the VM instruction number.  This should be considered as a special
@@ -1450,13 +1417,13 @@ struct ScmSubrRec {
                          func, inliner, data)
 
 SCM_EXTERN ScmObj Scm_MakeSubr(ScmSubrProc *func,
-			       void *data,
-			       int required, int optional,
-			       ScmObj info);
+                               void *data,
+                               int required, int optional,
+                               ScmObj info);
 SCM_EXTERN ScmObj Scm_NullProc(void);
 
 SCM_EXTERN ScmObj Scm_SetterSet(ScmProcedure *proc, ScmProcedure *setter,
-				int lock);
+                                int lock);
 SCM_EXTERN ScmObj Scm_Setter(ScmObj proc);
 SCM_EXTERN int    Scm_HasSetter(ScmObj proc);
 
@@ -1486,10 +1453,10 @@ SCM_CLASS_DECL(Scm_GenericClass);
     }
 
 SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
-				       ScmModule *mod);
+                                       ScmModule *mod);
 SCM_EXTERN ScmObj Scm_MakeBaseGeneric(ScmObj name,
-				      ScmObj (*fallback)(ScmObj *, int, ScmGeneric*),
-				      void *data);
+                                      ScmObj (*fallback)(ScmObj *, int, ScmGeneric*),
+                                      void *data);
 SCM_EXTERN ScmObj Scm_NoNextMethod(ScmObj *argv, int argc, ScmGeneric *gf);
 SCM_EXTERN ScmObj Scm_NoOperation(ScmObj *argv, int argc, ScmGeneric *gf);
 SCM_EXTERN ScmObj Scm_InvalidApply(ScmObj *argv, int argc, ScmGeneric *gf);
@@ -1808,6 +1775,7 @@ SCM_EXTERN void Scm_Cleanup(void);
 SCM_EXTERN void Scm_Exit(int code);
 SCM_EXTERN void Scm_Abort(const char *msg);
 SCM_EXTERN void Scm_Panic(const char *msg, ...);
+SCM_EXTERN ScmObj Scm_InitCommandLine(int argc, const char *argv[]);
 
 SCM_EXTERN void Scm_SimpleMain(int argc, const char *argv[],
                                const char *script, u_long flags);
@@ -1838,10 +1806,8 @@ SCM_EXTERN ScmObj Scm_SiteArchitectureDirectory(void);
 SCM_EXTERN ScmObj Scm__RuntimeDirectory(void); /* private */
 
 /* Compare and Sort */
-SCM_EXTERN int Scm_Compare(ScmObj x, ScmObj y);
-SCM_EXTERN void Scm_SortArray(ScmObj *elts, int nelts, ScmObj cmpfn);
-SCM_EXTERN ScmObj Scm_SortList(ScmObj objs, ScmObj fn);
-SCM_EXTERN ScmObj Scm_SortListX(ScmObj objs, ScmObj fn);
+
+#include <gauche/compare.h>
 
 /* Assertion */
 
