@@ -1,12 +1,12 @@
 /*
  * charset.h - Character set implementation
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -74,19 +74,16 @@ SCM_EXTERN ScmObj Scm_CharSetCopy(ScmCharSet *src);
 SCM_EXTERN int    Scm_CharSetEq(ScmCharSet *x, ScmCharSet *y);
 SCM_EXTERN int    Scm_CharSetLE(ScmCharSet *x, ScmCharSet *y);
 SCM_EXTERN ScmObj Scm_CharSetAddRange(ScmCharSet *cs,
-				      ScmChar from, ScmChar to);
+                                      ScmChar from, ScmChar to);
 SCM_EXTERN ScmObj Scm_CharSetAdd(ScmCharSet *dest, ScmCharSet *src);
 SCM_EXTERN ScmObj Scm_CharSetComplement(ScmCharSet *cs);
 SCM_EXTERN ScmObj Scm_CharSetCaseFold(ScmCharSet *cs);
 SCM_EXTERN ScmObj Scm_CharSetRanges(ScmCharSet *cs);
 SCM_EXTERN ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
-				  int error_p, int bracket_syntax);
+                                  int error_p, int bracket_syntax);
 
 SCM_EXTERN int    Scm_CharSetContains(ScmCharSet *cs, ScmChar c);
-
-#if SCM_DEBUG_HELPER
 SCM_EXTERN void   Scm_CharSetDump(ScmCharSet *cs, ScmPort *port);
-#endif
 
 /* predefined character set API */
 enum {
