@@ -1,7 +1,7 @@
 /*
  * module.h - Modules
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -47,13 +47,18 @@ struct ScmModuleRec {
     SCM_HEADER;
     ScmObj name;                /* symbol or #f */
     ScmObj imported;            /* list of imported modules. */
-    ScmObj exported;            /* list of exported symbols */
     int    exportAll;           /* TRUE if (export-all) */
     ScmObj parents;             /* direct parent modules */
     ScmObj mpl;                 /* module precedence list */
     ScmObj depended;            /* list of modules that are depended by this
                                    module for compilation */
-    ScmHashTable *table;        /* binding table */
+    ScmHashTable *internal;     /* Symbol -> GLoc, looked up from this module
+                                   itself and inherited ones */
+    ScmHashTable *external;     /* Symbol -> GLoc, looked up from the modules
+                                   that imports this module.  This table only
+                                   holds exported symbols (it may have
+                                   different name if export renaming is in
+                                   effect) */
     ScmObj origin;              /* if this module is an anonymous wrapper
                                    module, this holds a orginal module.
                                    this isn't used for resolving bindings,
@@ -79,11 +84,15 @@ SCM_EXTERN ScmObj Scm__MakeWrapperModule(ScmModule *origin, ScmObj prefix);
 enum {
     SCM_BINDING_STAY_IN_MODULE = (1L<<0), /*(F,R) do not search parent/imported*/
     SCM_BINDING_CONST = (1L<<1),          /*(M) constant binding */
-    SCM_BINDING_INLINABLE = (1L<<2)       /*(M) inlinable binding */
+    SCM_BINDING_INLINABLE = (1L<<2),      /*(M) inlinable binding */
+    SCM_BINDING_EXTERNAL = (1L<<3)        /*(F) only search externally visible
+                                            bindings, as if we're importing
+                                            the module.  Currently used to
+                                            create alias binding. */
 };
 
 SCM_EXTERN ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
-				    int flags);
+                                    int flags);
 SCM_EXTERN ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
                                     ScmObj value, int flags);
 SCM_EXTERN ScmObj Scm_GlobalVariableRef(ScmModule *module,
@@ -96,7 +105,7 @@ SCM_EXTERN int    Scm_AliasBinding(ScmModule *target, ScmSymbol *targetName,
 /* Convenience API.  Wrapper of Scm_MakeBinding. */
 SCM_EXTERN ScmObj Scm_Define(ScmModule *module,
                              ScmSymbol *symbol,
-			     ScmObj value);
+                             ScmObj value);
 SCM_EXTERN ScmObj Scm_DefineConst(ScmModule *module,
                                   ScmSymbol *symbol,
                                   ScmObj value);
@@ -111,6 +120,7 @@ SCM_EXTERN ScmObj Scm_ExportAll(ScmModule *module);
 SCM_EXTERN ScmModule *Scm_FindModule(ScmSymbol *name, int flags);
 SCM_EXTERN ScmObj Scm_AllModules(void);
 SCM_EXTERN void   Scm_SelectModule(ScmModule *mod);
+SCM_EXTERN ScmObj Scm_ModuleExports(ScmModule *mod);
 
 /* Flags for Scm_FindModule
    NB: Scm_FindModule's second arg has been changed since 0.8.6;
@@ -136,6 +146,9 @@ SCM_EXTERN ScmModule *Scm_SchemeModule(void);
 SCM_EXTERN ScmModule *Scm_GaucheModule(void);
 SCM_EXTERN ScmModule *Scm_UserModule(void);
 SCM_EXTERN ScmModule *Scm_CurrentModule(void);
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+SCM_EXTERN ScmModule *Scm_KeywordModule(void);
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
 
 #define SCM_DEFINE(module, cstr, val)           \
     Scm_Define(SCM_MODULE(module),              \
