@@ -1,12 +1,12 @@
 /*
  * regexp.h - Regexp module internal definitions
  *
- *   Copyright (c) 2006-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2006-2014  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -49,6 +49,11 @@ struct ScmRegexpRec {
     int numSets;         /* # of charsets in sets */
     int flags;           /* internal; CASE_FOLD, BOL_ANCHORED etc. */
     ScmString *mustMatch;
+    ScmObj laset;        /* lookahead set (char-set) or #f.
+                            If not #f, it represents the condition that can
+                            match at the beginning of the regexp.  It can be
+                            used to skip input start position when regexp
+                            isn't BOL_ANCHORED. */
 };
 
 struct ScmRegMatchRec {
