@@ -1,7 +1,7 @@
 /*
  * system.h - Gauche system interface
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -45,16 +45,21 @@
  * hides restarting.
  */
 
-#define SCM_SYSCALL3(result, expr, check)       \
-  do {                                          \
-    (result) = (expr);                          \
-    if ((check) && errno == EINTR) {            \
-      ScmVM *vm__ = Scm_VM();                   \
-      errno = 0;                                \
-      SCM_SIGCHECK(vm__);                       \
-    } else {                                    \
-      break;                                    \
-    }                                           \
+#define SCM_SYSCALL3(result, expr, check)                       \
+  do {                                                          \
+    (result) = (expr);                                          \
+    if ((check) && (errno == EINTR || errno == EPIPE)) {        \
+      ScmVM *vm__ = Scm_VM();                                   \
+      int epipe__ = (errno == EPIPE);                           \
+      errno = 0;                                                \
+      SCM_SIGCHECK(vm__);                                       \
+      if (epipe__) {                                            \
+        errno = EPIPE;                                          \
+        break;                                                  \
+      }                                                         \
+    } else {                                                    \
+      break;                                                    \
+    }                                                           \
   } while (1)
 
 #define SCM_SYSCALL(result, expr) \
@@ -76,6 +81,7 @@ SCM_EXTERN ScmObj Scm_OffsetToInteger(off_t o);
  */
 
 SCM_EXTERN ScmObj Scm_ReadDirectory(ScmString *pathname);
+SCM_EXTERN ScmObj Scm_GetCwd(void);
 
 #define SCM_PATH_ABSOLUTE       (1L<<0)
 #define SCM_PATH_EXPAND         (1L<<1)
@@ -114,6 +120,8 @@ SCM_EXTERN time_t Scm_GetSysTime(ScmObj val);
 
 SCM_EXTERN void Scm_GetTimeOfDay(u_long *sec, u_long *usec);
 SCM_EXTERN long Scm_CurrentMicroseconds();
+SCM_EXTERN int  Scm_ClockGetTimeMonotonic(u_long *sec, u_long *nsec);
+SCM_EXTERN int  Scm_ClockGetResMonotonic(u_long *sec, u_long *nsec);
 
 /* Gauche also has a <time> object, as specified in SRFI-18, SRFI-19
  * and SRFI-21.  It can be constructed from the basic system interface
@@ -247,9 +255,9 @@ SCM_CLASS_DECL(Scm_SysFdsetClass);
 #define SCM_SYS_FDSET_P(obj)    (SCM_XTYPEP(obj, SCM_CLASS_SYS_FDSET))
 
 SCM_EXTERN ScmObj Scm_SysSelect(ScmObj rfds, ScmObj wfds, ScmObj efds,
-				ScmObj timeout);
+                                ScmObj timeout);
 SCM_EXTERN ScmObj Scm_SysSelectX(ScmObj rfds, ScmObj wfds, ScmObj efds,
-				 ScmObj timeout);
+                                 ScmObj timeout);
 #else  /*!HAVE_SELECT*/
 /* dummy definitions */
 typedef struct ScmHeaderRec ScmSysFdset;
@@ -264,6 +272,7 @@ typedef struct ScmHeaderRec ScmSysFdset;
 SCM_EXTERN int    Scm_Mkstemp(char *tmpl);
 SCM_EXTERN ScmObj Scm_SysMkstemp(ScmString *tmpl);
 SCM_EXTERN ScmObj Scm_Environ(void);
+SCM_EXTERN const char *Scm_GetEnv(const char *name);
 SCM_EXTERN void Scm_SetEnv(const char *name, const char *value, int overwrite);
 SCM_EXTERN void Scm_UnsetEnv(const char *name);
 SCM_EXTERN void Scm_ClearEnv(void);
