@@ -1,12 +1,12 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -104,14 +104,31 @@ typedef struct ScmEnvFrameRec {
  * Continuation frame
  *
  *  Continuation is represented as a chain of ScmContFrames.
- *  If argp == NULL && size >= 0, the frame is C continuation.
+ *
+ *   :        :
+ *   +--------+
+ *   |  base  |
+ *   |   pc   |
+ *   |  cpc   |
+ *   | size=N |
+ *   |  env   |
+ *   |..prev..|<--- ScmContFrame* cont
+ *   |arg[N-1]|
+ *   |arg[N-2]|
+ *   :        :
+ *   | arg[0] |
+ *   +--------+
+ *   :        :
+ *
+ *  If env is a special value (&ccEnvMark), this is a C continuation
+ *  and pc contains a C function pointer.
  */
 
 typedef struct ScmContFrameRec {
     struct ScmContFrameRec *prev; /* previous frame */
     ScmEnvFrame *env;             /* saved environment */
-    ScmObj *argp;                 /* saved argument pointer */
     int size;                     /* size of argument frame */
+    SCM_PCTYPE cpc;               /* current PC (for debugging info) */
     SCM_PCTYPE pc;                /* next PC */
     ScmCompiledCode *base;        /* base register value */
 } ScmContFrame;
@@ -120,6 +137,9 @@ typedef struct ScmContFrameRec {
 
 SCM_EXTERN void Scm_CallCC(ScmObj body);
 
+SCM_EXTERN int  Scm__VMProtectStack(ScmVM *vm);
+SCM_EXTERN void Scm__VMUnprotectStack(ScmVM *vm);
+
 /*
  * Syntactic closure
  *
@@ -173,7 +193,7 @@ SCM_EXTERN ScmObj Scm_MakeIdentifier(ScmSymbol *name, ScmModule *mod,
                                      ScmObj env);
 SCM_EXTERN ScmObj Scm_CopyIdentifier(ScmIdentifier *id);
 SCM_EXTERN int    Scm_IdentifierBindingEqv(ScmIdentifier *id, ScmSymbol *sym,
-					   ScmObj env);
+                                           ScmObj env);
 SCM_EXTERN int    Scm_FreeVariableEqv(ScmObj var, ScmObj sym, ScmObj env);
 
 /*
@@ -250,8 +270,8 @@ typedef struct ScmEscapePointRec {
                                    are rewound after an exception is raised
                                    and before the exception handler is called.
                                    If FALSE, the exception handler is called
-                                   first, then the dynamic handlers are 
-                                   rewound.   SRFI-18 model and legacy 
+                                   first, then the dynamic handlers are
+                                   rewound.   SRFI-18 model and legacy
                                    with-error-handler uses the latter model,
                                    but SRFI-34's guard needs the former model.
                                 */
@@ -286,16 +306,36 @@ typedef struct ScmEscapePointRec {
  *  is invoked only once.
  *
  *  The C-level signal handler does count each signal until it is cleared
- *  by VM loop.  If one kind of signal arraives more than a certain limit
+ *  by VM loop.  If one kind of signal arrives more than a certain limit
  *  (set/get by Scm_{Set|Get}SignalPendingLimit), Gauche thinks something
  *  went wrong and bail out.   It is useful, for example, to interrupt
  *  unresponsive program from the terminal.
  */
 
+/* NB: AT least on FreeBSD, RT signals are not counted in NSIG.  We should
+   use _SIG_MAXSIG+1 instead to track all signals. */
+#if defined(_SIG_MAXSIG) && (_SIG_MAXSIG+1 > NSIG)
+#define SCM_NSIG (_SIG_MAXSIG+1)
+#else  /*!_SIG_MAXSIG*/
+#define SCM_NSIG NSIG
+#endif /*!_SIG_MAXSIG*/
+
+/* The following #ifdef is to maintain binary compatibility during 0.9 series.
+ */
+#if defined(GAUCHE_API_0_9)
 typedef struct ScmSignalQueueRec {
-    unsigned char sigcounts[NSIG];
+    union {
+        unsigned char dummy[NSIG];
+        unsigned char *sigcounts;
+    };
     ScmObj pending;        /* pending signal handlers */
 } ScmSignalQueue;
+#else  /*!GAUCHE_API_0_9*/
+typedef union ScmSignalQueueRec {
+    unsigned char sigcounts[SCM_NSIG];
+    ScmObj pending;        /* pending signal handlers */
+} ScmSignalQueue;
+#endif /*!GAUCHE_API_0_9*/
 
 SCM_EXTERN void Scm_SignalQueueInit(ScmSignalQueue* q);
 SCM_EXTERN int  Scm_GetSignalPendingLimit(void);
@@ -327,7 +367,7 @@ SCM_EXTERN ScmObj Scm_VMFinalizerRun(ScmVM *vm);
  * Statistics
  *
  *  Not much stats are collected yet, but will grow in future.
- *  Stats collections are only active if SCM_COLLECT_VM_STATS 
+ *  Stats collections are only active if SCM_COLLECT_VM_STATS
  *  runtime flag is TRUE.
  *  Stats are collected per-VM (i.e. per-thread), but currently
  *  we don't have an API to gather them.
@@ -399,7 +439,10 @@ struct ScmVMRec {
                                    and turned off by Scm_VMFinalizerRun(),
                                    both in core.c */
     intptr_t stopRequest;       /* Flag if there is a pending stop request.
-                                   Turned on by Scm_ThreadStop() in
+                                   See enum ScmThreadStopRequest below
+                                   for the possible values.
+                                   Turned on by Scm_ThreadStop() or
+                                   Scm_ThreadTerminate in
                                    ext/threads/threads.c, and turned off by
                                    process_queued_requests() in vm.c */
 
@@ -456,9 +499,9 @@ struct ScmVMRec {
     ScmObj defaultEscapeHandler;
 
     /* Program information */
-    ScmObj load_next;           /* list of the directories to be searched */
-    ScmObj load_history;        /* history of the nested load */
-    ScmObj load_port;           /* current port from which we are loading */
+    ScmObj dummy0;              /* for the ABI compatibility.  remove on 1.0. */
+    ScmObj dummy1;              /* ditto */
+    ScmObj dummy2;              /* ditto */
     int    evalSituation;       /* eval situation (related to eval-when) */
 
     /* Signal information */
@@ -469,10 +512,15 @@ struct ScmVMRec {
     ScmVMStat stat;
     int profilerRunning;
     ScmVMProfiler *prof;
+
+#if defined(GAUCHE_USE_WTHREADS)
+    ScmWinCleanup *winCleanup; /* mimic pthread_cleanup_* */
+#endif /*defined(GAUCHE_USE_WTHREADS)*/
 };
 
 SCM_EXTERN ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name);
 SCM_EXTERN int    Scm_AttachVM(ScmVM *vm);
+SCM_EXTERN void   Scm_DetachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_VMDump(ScmVM *vm);
 SCM_EXTERN void   Scm_VMDefaultExceptionHandler(ScmObj exc);
 SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc);
@@ -483,8 +531,10 @@ SCM_EXTERN void   Scm_VMSetResult(ScmObj obj);
 SCM_CLASS_DECL(Scm_VMClass);
 #define SCM_CLASS_VM              (&Scm_VMClass)
 
-#ifdef GAUCHE_USE_PTHREADS
+#if   defined(GAUCHE_USE_PTHREADS)
 SCM_EXTERN pthread_key_t Scm_VMKey(void);
+#elif defined(GAUCHE_USE_WTHREADS)
+SCM_EXTERN DWORD Scm_VMKey(void);
 #endif
 
 /* Value of vm->state */
@@ -507,6 +557,11 @@ enum {
     SCM_VM_COMPILING            /* we're batch-compiling the forms */
 };
 
+/* Value of vm->stopRequest.  Zero means no request. */
+typedef enum {
+    SCM_VM_REQUEST_SUSPEND = 1L,   /* Set by Scm_ThreadStop */
+    SCM_VM_REQUEST_TERMINATE = 2L, /* Set by Scm_ThreadTerminate */
+} ScmVMStopRequest;
 
 /*
  * C stack rewinding
@@ -526,14 +581,14 @@ enum {
  *     SCM_NEXT_HANDLER;
  *   } SCM_END_PROTECT;
  *   clean up code on normal situation
- * 
+ *
  *  Note that this construct does not install exception handler or
  *  error handler by itself.   The handler installed by with-error-handler
  *  or with-exception-handler is invoked, and then the SCM_WHEN_ERROR
  *  part is called while the C stack is rewind.
  *  If you want to handle error as well, you should install error handler
  *  by yourself (and deinstall it in the cleanup code).
- * 
+ *
  *  In general, you MUST call SCM_NEXT_HANDLER in the SCM_WHEN_ERROR clause.
  *  In other words, you shouldn't use SCM_UNWIND_PROTECT as "ignore-errors"
  *  construct.  The C stack is rewind not only at the error situation, but
@@ -551,7 +606,7 @@ enum {
        cstack.cont = NULL;                      \
        Scm_VM()->cstack = &cstack;              \
        if (sigsetjmp(cstack.jbuf, FALSE) == 0) {
-           
+
 #define SCM_WHEN_ERROR                          \
        } else {
 
@@ -598,25 +653,13 @@ enum {
  * C-continuation
  */
 
-#define SCM_CCONT_DATA_SIZE 6
-
 typedef ScmObj ScmCContinuationProc(ScmObj result, void **data);
 
-typedef struct ScmCContinuation {
-    SCM_HEADER;
-    ScmCContinuationProc *func;
-    void *data[SCM_CCONT_DATA_SIZE];
-} ScmCContinuation;
-
-#define SCM_CCONT(obj)            ((ScmCContinuation*)(obj))
-#define SCM_CCONTP(obj)           SCM_XTYPEP(obj, SCM_CLASS_CCONT)
-
-SCM_CLASS_DECL(Scm_CContClass);
-#define SCM_CLASS_CCONT           (&Scm_CContClass)
-
 SCM_EXTERN void Scm_VMPushCC(ScmCContinuationProc *func,
-			     void **data,
-			     int datasize);
+                             void **data,
+                             int datasize);
+
+#define SCM_CCONT_DATA_SIZE 6   /* Maximum datasize for VMPushCC */
 
 /*
  * Compiler flags
@@ -630,7 +673,12 @@ enum {
     SCM_COMPILE_SHOWRESULT = (1L<<4),      /* Display each result of
                                               compilation */
     SCM_COMPILE_NOCOMBINE = (1L<<5),       /* Do not combine instructions */
-    SCM_COMPILE_NO_PASS2POST = (1L<<6)     /* Do not run pass2-post. */
+    SCM_COMPILE_NO_POST_INLINE_OPT = (1L<<6), /* Do not run post-inline
+                                                 optimization (pass3). */
+    SCM_COMPILE_NO_LIFTING = (1L<<7),      /* Do not run lambda lifting pass
+                                              (pass4). */
+    SCM_COMPILE_INCLUDE_VERBOSE = (1L<<8), /* Report expansion of 'include' */
+    SCM_COMPILE_ENABLE_CEXPR = (1L<<9)     /* Support C-expressions by reader */
 };
 
 #define SCM_VM_COMPILER_FLAG_IS_SET(vm, flag) ((vm)->compilerFlags & (flag))
