@@ -1,7 +1,7 @@
 /*
  * lazy.c - lazy evaluation constructs
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,6 +33,48 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+/* Workaround for sh4 */
+/*
+ * ABI wise, I'd say SuperH is difficult to support, and libatomic_ops
+ * is not well supported.
+ *
+ * I believe that using gUSA, we could improve libatomic_ops
+ * implementation for SH-4 (compare_and_swap, etc.).  But, those
+ * are imcompatible to SH-4A, SMP machines.
+ *
+ * I believe that SH-4A, architecture wise, breaks SH-4 ABI already.
+ * 
+ * If SH-4A machine insists as if it were SH4 (ABI), we can't use
+ * gUSA, nor ll/sc equivalents (movli.l/movco.l, IIRC), either.
+ * That's totally a mess.
+ *
+ * Only workaround for both of SH-4 and SH-4A is to downgrade to
+ * pthread implementation, so that it will work reliably.
+ * 
+ * -- gniibe  2012-11-27
+ *
+ */
+#if defined(__SH4__)
+#define AO_USE_PTHREAD_DEFS 1
+#endif
+/* Workaround for armel */
+/*
+ * It is unfortunate that libatomic_ops is not well supported
+ * for ARM architectures.  It could be understandable as there
+ * are so many variants in "ARM".
+ *
+ * For __ARMEL__ (which means ARM_ARCH_4T, in Debian), there is no 
+ * hardware support for atomic operations, unfortunatelly.
+ *
+ * NOTE:
+ * It is ARMv6 which introduced LDREX/STREX (exclusives).
+ * It is ARMv7 which introduced DMB/DSB instructions (memory barrier).
+ *
+ *      -- gniibe  2012-11-27
+ */
+#if defined(__ARMEL__)
+#define AO_USE_PTHREAD_DEFS 1
+#endif
 #include "atomic_ops.h"
 
 /*==================================================================
@@ -125,6 +167,7 @@ static ScmObj release_promise(ScmObj *args, int nargs, void *data)
     ScmPromise *p = SCM_PROMISE(data);
     p->content->owner = NULL;
     SCM_INTERNAL_MUTEX_UNLOCK(p->content->mutex);
+    return SCM_UNDEFINED;
 }
 
 static void install_release_thunk(ScmVM *vm, ScmObj promise)
@@ -141,6 +184,7 @@ static void install_release_thunk(ScmVM *vm, ScmObj promise)
 static ScmObj force_cc(ScmObj result, void **data)
 {
     ScmPromise *p = (ScmPromise*)data[0];
+    ScmObj handlers = (ScmObj)data[1];
 
     /* Check if the original promise is forced by evaluating
        the delayed expr to detect recursive force situation */
@@ -163,6 +207,7 @@ static ScmObj force_cc(ScmObj result, void **data)
         p->content->owner = NULL;
         SCM_INTERNAL_MUTEX_UNLOCK(p->content->mutex);
     }
+    Scm_VM()->handlers = handlers;
     SCM_RETURN(Scm_Force(SCM_OBJ(p)));
 }
 
@@ -176,13 +221,14 @@ ScmObj Scm_Force(ScmObj obj)
         if (c->forced) SCM_RETURN(c->code);
         else {
             ScmVM *vm = Scm_VM();
-            void *data[1];
+            void *data[2];
             data[0] = obj;
+            data[1] = vm->handlers;
 
             if (c->owner == vm) {
                 /* we already have the lock and evaluating this promise. */
                 c->count++;
-                Scm_VMPushCC(force_cc, data, 1);
+                Scm_VMPushCC(force_cc, data, 2);
                 SCM_RETURN(Scm_VMApply0(c->code));
             } else {
                 /* TODO: check if the executing thread terminates
@@ -197,7 +243,7 @@ ScmObj Scm_Force(ScmObj obj)
                 install_release_thunk(vm, obj);
                 c->count++;
                 /* mutex is unlocked by force_cc. */
-                Scm_VMPushCC(force_cc, data, 1);
+                Scm_VMPushCC(force_cc, data, 2);
                 SCM_RETURN(Scm_VMApply0(c->code));
             }
         }
