@@ -1,7 +1,7 @@
 ;;;
 ;;; libeval.scm - eval, load and related stuff
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -82,7 +82,7 @@
            [port (guard (e [else e]) (opener path))])
       (%record-load-start path)
       (if (not (input-port? port))
-        (and error-if-not-found (raise e))
+        (and error-if-not-found (raise port))
         (load-from-port (if ignore-coding
                           port
                           (open-coding-aware-port port))
@@ -180,11 +180,13 @@
 ;;   The element in SUFFIXES is directly appended to the FILENAME;
 ;;   so usually it begins with dot.
 ;;
-;;   PATHs may contain a regular file.  In which case, procedures chained
-;;   to *load-path-hooks* are called in turn.  It receives three arguments;
-;;   the regular filename in PATHs, the (partial) filename given to the
-;;   find-load-file, and the list of suffixes.  The hook is mainly intended
-;;   to allow loading from archive files.   If the hook procedure "finds"
+;;   PATHs may contain a regular file, or an empty string "".
+;;   In which case, procedures chained to *load-path-hooks* are called
+;;   in turn.  It receives three arguments; the regular filename in PATHs
+;;   or "", the (partial) filename given to the find-load-file, and the
+;;   list of suffixes.  The hook is mainly intended to allow loading
+;;   from archive files or from special location (e.g. prelinked in the
+;;   executing binary).   If the hook procedure "finds"
 ;;   the searched file in the archive file, it should return a pair of
 ;;   the canonical filename (given filename plus suffix if applicable), and
 ;;   a thunk that opens and returns a port to read the file.  If the hook
@@ -197,10 +199,13 @@
 (define (find-load-file filename paths suffixes
                         :optional (error-if-not-found #f)
                                   (allow-archive #f))
+  (define (file-ok? file)
+    (and (file-exists? file)
+         (not (file-is-directory? file))))
   (define (try-suffixes stem)
-    (cond [(file-is-regular? stem) stem]
+    (cond [(file-ok? stem) stem]
           [else (any (^s (let1 file (string-append stem s)
-                           (and (file-is-regular? file) file)))
+                           (and (file-ok? file) file)))
                      suffixes)]))
   (define (do-absolute stem)
     (if-let1 found (try-suffixes stem)
@@ -216,7 +221,8 @@
       (if-let1 found (try-suffixes (string-append (car ps) "/" filename))
         (list found (cdr ps))
         (do-relative (cdr ps)))]
-     [(and allow-archive (file-is-regular? (car ps)))
+     [(and allow-archive
+           (or (equal? (car ps) "") (file-is-regular? (car ps))))
       (if-let1 r (any (^p (p (car ps) filename suffixes)) *load-path-hooks*)
         (list (car r) (cdr ps) (cdr r))
         (do-relative (cdr ps)))]
@@ -235,6 +241,37 @@
          (do-absolute filename)]
         [else (do-relative paths)]))
 
+;;;
+;;; Repl
+;;;
+
+(select-module gauche.internal)
+
+(define (%repl-print . vals) (for-each (^e (write e) (newline)) vals))
+(define (%repl-prompt) (display "gosh> ") (flush))
+
+(define-in-module gauche (read-eval-print-loop :optional (reader #f)
+                                                         (evaluator #f)
+                                                         (printer #f)
+                                                         (prompter #f))
+  (let ([reader    (or reader read)]
+        [evaluator (or evaluator eval)]
+        [printer   (or printer %repl-print)]
+        [prompter  (or prompter %repl-prompt)])
+    (let loop1 ()
+      (and
+       (with-error-handler
+           (^e (report-error e) #t)
+         (^[]
+           (let loop2 ()
+             (prompter)
+             (let1 exp (reader)
+               (and (not (eof-object? exp))
+                    (receive results (evaluator exp (vm-current-module))
+                      (apply printer results)
+                      (loop2)))))))
+       (loop1)))))
+
 ;;;
 ;;; Macros
 ;;;
@@ -261,7 +298,7 @@
 (define-cproc make-macro-transformer (name::<symbol> proc::<procedure>)
   Scm_MakeMacroTransformerOld)
 
-(define-cproc compile-syntax-rules (name literals rules mod env)
+(define-cproc compile-syntax-rules (name ellipsis literals rules mod env)
   Scm_CompileSyntaxRules)
 
 (define-cproc call-macro-expander (mac::<macro> expr env)
@@ -342,7 +379,7 @@
 (select-module gauche)
 ;; Obtain info about gauche itself
 (define-cproc gauche-version () ::<const-cstring> (result GAUCHE_VERSION))
-(define-cproc gauche-architecture () ::<const-cstring> (result GAUCHE_ARCH))
+(define-cproc gauche-architecture () ::<const-cstring> Scm_HostArchitecture)
 (define-cproc gauche-library-directory () Scm_LibraryDirectory)
 (define-cproc gauche-architecture-directory () Scm_ArchitectureDirectory)
 (define-cproc gauche-site-library-directory () Scm_SiteLibraryDirectory)
@@ -353,6 +390,16 @@
 ;; '@' in the paths embedded in gauche.config.
 (define-cproc %gauche-runtime-directory () Scm__RuntimeDirectory)
 
+;; Command line - R7RS adds 'command-line' procedure.  We provide it as
+;; a predefined parameter.  Like exit-handler, we manually allocate a
+;; parametre slot to avoid importing gauche.parameter.
+(select-module gauche.internal)
+(define-in-module gauche command-line
+  (let1 index (%vm-make-parameter-slot)
+    (^ maybe-arg
+      (rlet1 old (%vm-parameter-ref index '())
+        (when (pair? maybe-arg)
+          (%vm-parameter-set! index #f (car maybe-arg)))))))
 ;;
 ;; External view of VM.
 ;;
