@@ -1,7 +1,7 @@
 ;;;
 ;;; libio.scm - builtin port and I/O procedures
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,6 +35,7 @@
 
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
+                     <gauche/priv/portP.h>
                      <stdlib.h>
                      <fcntl.h>)))
 
@@ -108,6 +109,20 @@
     (result (?: (< i 0) SCM_FALSE (Scm_MakeInteger i)))))
 (define-cproc port-fd-dup! (dst::<port> src::<port>) ::<void> Scm_PortFdDup)
 
+(define-cproc port-attribute-set! (port::<port> key val)
+  Scm_PortAttrSet)
+(define-cproc port-attribute-ref (port::<port> key :optional fallback)
+  (setter port-attribute-set!)
+  Scm_PortAttrGet)
+(define-cproc port-attribute-create! (port::<port> key
+                                      :optional (get #f) (set #f))
+  Scm_PortAttrCreate)
+(define-cproc port-attribute-delete! (port::<port> key)
+  Scm_PortAttrDelete)
+(define-cproc port-attributes (port::<port>)
+  Scm_PortAttrs)
+
+
 (define-cproc port-type (port::<port>)
   (case (SCM_PORT_TYPE port)
     [(SCM_PORT_FILE) (result 'file)]
@@ -119,9 +134,9 @@
   (setter (port::<port> mode) ::<void>
           (unless (== (SCM_PORT_TYPE port) SCM_PORT_FILE)
             (Scm_Error "can't set buffering mode to non-buffered port: %S"port))
-          (set! (ref (-> port src) buf mode)
-                (Scm_BufferingMode mode (-> port direction) -1)))
-  Scm_GetBufferingMode)
+          (Scm_SetPortBufferingMode
+           port (Scm_BufferingMode mode (-> port direction) -1)))
+  (result (Scm_GetPortBufferingModeAsKeyword port)))
 
 (define-cproc port-case-fold-set! (port::<port> flag::<boolean>) ::<void>
   (if flag
@@ -133,8 +148,10 @@
 ;;
 
 (select-module scheme)
-(define-cproc close-input-port (port::<port>)  ::<void> Scm_ClosePort)
-(define-cproc close-output-port (port::<port>) ::<void> Scm_ClosePort)
+(define-cproc close-input-port (port::<input-port>)  ::<void> Scm_ClosePort)
+(define-cproc close-output-port (port::<output-port>) ::<void> Scm_ClosePort)
+(select-module gauche)
+(define-cproc close-port (port::<port>) ::<void> Scm_ClosePort) ;R6RS
 
 (select-module gauche.internal)
 (inline-stub
@@ -174,7 +191,8 @@
                           if-does-not-exist)])
     (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_INPUT
                                             SCM_PORT_BUFFER_FULL)]
-           [o (Scm_OpenFilePort (Scm_GetStringConst path) O_RDONLY bufmode 0)])
+           [o (Scm_OpenFilePort (Scm_GetStringConst path)
+                                O_RDONLY bufmode 0)])
       (when (and (SCM_FALSEP o) (not (%open/allow-noexist? ignerr)))
         (Scm_SysError "couldn't open input file: %S" path))
       (result o))))
@@ -210,7 +228,8 @@
                           if-does-not-exist)])
     (let* ([bufmode::int
             (Scm_BufferingMode buffering SCM_PORT_OUTPUT SCM_PORT_BUFFER_FULL)]
-           [o (Scm_OpenFilePort (Scm_GetStringConst path) flags bufmode mode)])
+           [o (Scm_OpenFilePort (Scm_GetStringConst path)
+                                flags bufmode mode)])
       (when (and (SCM_FALSEP o)
                  (not (%open/allow-noexist? ignerr-noexist))
                  (not (%open/allow-exist? ignerr-exist)))
@@ -346,7 +365,73 @@
                 :optional (whence::<fixnum> (c "SCM_MAKE_INT(SEEK_SET)")))
   Scm_PortSeek)
 
-(define-cproc with-port-locking (port::<port> proc) Scm_VMWithPortLocking)
+;; useful alias
+(define (port-tell p) (port-seek p 0 SEEK_CUR))
+
+;; useful for error messages
+(define (port-position-prefix port)
+  (if-let1 n (port-name port)
+    (let1 l (port-current-line port)
+      (if (positive? l)
+        (format #f "~s:line ~a: " n l)
+        (format #f "~s: " n))
+      "")))
+
+(select-module gauche.internal)
+
+;; Transient flags during circular/shared-aware writing
+(define-cproc %port-walking? (port::<port>) ::<boolean>
+  (setter (port::<port> flag::<boolean>) ::<void>
+          (if flag
+            (logior= (-> port flags) SCM_PORT_WALKING)
+            (logand= (-> port flags) (lognot SCM_PORT_WALKING))))
+  PORT_WALKER_P)
+(define-cproc %port-writing-shared? (port::<port>) ::<boolean>
+  (setter (port::<port> flag::<boolean>) ::<void>
+          (if flag
+            (logior= (-> port flags) SCM_PORT_WRITESS)
+            (logand= (-> port flags) (lognot SCM_PORT_WRITESS))))
+  PORT_WRITESS_P)
+
+(define-cproc %port-recursive-context (port::<port>)
+  (setter (port::<port> obj) ::<void>
+          (set! (-> port recursiveContext) obj))
+  (result (-> port recursiveContext)))
+
+(define-cproc %port-lock! (port::<port>) ::<void>
+  (let* ([vm::ScmVM* (Scm_VM)])
+    (PORT_LOCK port vm)))
+(define-cproc %port-unlock! (port::<port>) ::<void>
+  (PORT_UNLOCK port))
+
+;; Passing extra args is unusual for with-* style, but it can allow avoiding
+;; closure allocation and may be useful for performance-sensitive parts.
+(define-in-module gauche (with-port-locking port proc . args)
+  (unwind-protect
+      (begin (%port-lock! port)
+             (apply proc args))
+    (%port-unlock! port)))
+
+(define-in-module gauche.internal ; used by two-pass output
+  (%with-2pass-setup port walker emitter . args)
+  ;; The caller guarantees to call this when port isn't in two-pass
+  ;; mode.   We lock the port, and call WALKER with setting the port
+  ;; to 'walking' mode, then call EMITTER with setting the port to
+  ;; 'write-ss' mode.
+  (unwind-protect
+      (begin
+        (%port-lock! port)
+        (when (%port-recursive-context port)
+          (error "[internal] %with-2pass-setup called recursively on port:"
+                 port))
+        (set! (%port-recursive-context port) (cons 0 (make-hash-table 'eq?)))
+        (set! (%port-walking? port) #t)
+        (apply walker args)
+        (set! (%port-walking? port) #f)
+        (apply emitter args))
+    (set! (%port-walking? port) #f)
+    (set! (%port-recursive-context port) #f)
+    (%port-unlock! port)))
 
 ;;;
 ;;; Input
@@ -377,9 +462,10 @@
 
 (select-module gauche)
 
-(define-cproc eof-object () (result SCM_EOF)) ;R6RS
+(define-cproc eof-object () :constant (result SCM_EOF)) ;R6RS
 
-(define-cproc byte-ready? (port::<input-port>) ::<boolean> Scm_ByteReady)
+(define-cproc byte-ready? (:optional (port::<input-port> (current-input-port)))
+  ::<boolean> Scm_ByteReady)
 
 (define-cproc read-byte (:optional (port::<input-port> (current-input-port)))
   (let* ([b::int])
@@ -399,6 +485,19 @@
       (Scm_ReadError port "read-line: encountered illegal byte sequence: %S" r))
     (result r)))
 
+(define (read-string n :optional (port (current-input-port)))
+  (define o (open-output-string :private? #t))
+  (let loop ([i 0])
+    (if (>= i n)
+      (get-output-string o)
+      (let1 c (read-char port)
+        (if (eof-object? c)
+          (if (= i 0)
+            (eof-object)
+            (get-output-string o))
+          (begin (write-char c o) (loop (+ i 1))))))))
+
+;; DEPRECATED - read-uvector should be used
 (define-cproc read-block (bytes::<fixnum>
                           :optional (port::<input-port> (current-input-port)))
   (when (< bytes 0)
@@ -426,6 +525,12 @@
             (Scm_DStringPutz (& ds) buf nbytes)))
     (result (Scm_DStringGet (& ds) SCM_STRING_INCOMPLETE))))
 
+;; Reader parameters
+(define-cproc reader-lexical-mode (:optional k)
+  (if (SCM_UNBOUNDP k)
+    (result (Scm_ReaderLexicalMode))
+    (result (Scm_SetReaderLexicalMode k))))
+
 ;; Read time constructor (srfi-10)
 (select-module gauche)
 
@@ -460,6 +565,27 @@
 (define-in-module gauche read-with-shared-structure read)
 (define-in-module gauche read/ss read)
 
+;; srfi-105
+(select-module gauche.internal)
+(define (%xform-cexpr cex)
+  (define (simple-cexpr? op tail)
+    (cond [(not (pair? tail)) #f]
+          [(null? (cdr tail)) #t]
+          [(and (pair? (cdr tail)) (eq? op (cadr tail)))
+           (simple-cexpr? op (cddr tail))]
+          [else #f]))
+  (define (gather-args args) ; args is a list with even elements
+    (if (null? (cdr args))
+      args
+      (cons (car args) (gather-args (cddr args)))))
+  (cond [(not (pair? cex)) cex]         ;includes {} -> ()
+        [(null? (cdr cex)) (car cex)]   ; {e} -> e
+        [(and (pair? (cdr cex)) (null? (cddr cex))) cex] ; {x y} -> (x y)
+        [(and (pair? (cdr cex)) (pair? (cddr cex)) (symbol? (cadr cex))
+              (simple-cexpr? (cadr cex) (cddr cex)))
+         (cons* (cadr cex) (car cex) (gather-args (cddr cex)))]
+        [else (cons 'nfx cex)]))
+
 ;;;
 ;;; Output
 ;;;
@@ -469,6 +595,16 @@
 (define-cproc write (obj :optional (port::<output-port> (current-output-port)))
   ::<void> (Scm_Write obj (SCM_OBJ port) SCM_WRITE_WRITE))
 
+(define-cproc write-simple (obj :optional (port::<output-port>
+                                           (current-output-port)))
+  ::<void>
+  (Scm_Write obj (SCM_OBJ port) SCM_WRITE_SIMPLE))
+
+(define-cproc write-shared (obj :optional (port::<output-port>
+                                           (current-output-port)))
+  ::<void>
+  (Scm_Write obj (SCM_OBJ port) SCM_WRITE_SHARED))
+
 (define-cproc display
   (obj :optional (port::<output-port> (current-output-port)))
   ::<void> (Scm_Write obj (SCM_OBJ port) SCM_WRITE_DISPLAY))
@@ -495,20 +631,52 @@
                                  :optional (port (current-output-port)))
   ::<int> (result (Scm_WriteLimited obj port SCM_WRITE_WRITE limit)))
 
-(define-cproc write* (obj :optional (port (current-output-port)))
-  ::<int> (result (Scm_WriteCircular obj port SCM_WRITE_WRITE 0)))
+(define write* write-shared)
 
 (define-cproc flush (:optional (oport::<output-port> (current-output-port)))
   ::<void> Scm_Flush)
 
 (define-cproc flush-all-ports () ::<void> (Scm_FlushAllPorts FALSE))
 
-
+;;
+;; Internal recusive writer
+;;
 (select-module gauche.internal)
 
-(define-cproc %format
-  (port::<output-port> fmt::<string> args shared::<boolean>) ::<void>
-  Scm_Format)
+(define-cproc write-need-recurse? (obj) ::<boolean>
+  (result (not (or (not (SCM_PTRP obj))
+                   (SCM_NUMBERP obj)
+                   (SCM_KEYWORDP obj)
+                   (and (SCM_SYMBOLP obj) (SCM_SYMBOL_INTERNED obj)) 
+                   (and (SCM_STRINGP obj) (== (SCM_STRING_SIZE obj) 0))
+                   (and (SCM_VECTORP obj) (== (SCM_VECTOR_SIZE obj) 0))))))
+
+(define (write-walk obj port)
+  (if-let1 ctx (%port-recursive-context port)
+    (%write-walk-rec obj port (cdr ctx))))
+
+(define (%write-walk-rec obj port tab)
+  (when (write-need-recurse? obj)
+    (if (hash-table-exists? tab obj)
+      (hash-table-update! tab obj (cut + <> 1))   ; seen more than once
+      (begin
+        (hash-table-put! tab obj 1) ; seen once
+        (cond
+         [(symbol? obj)] ; uninterned symbols
+         [(string? obj)]
+         [(pair? obj)
+          (%write-walk-rec (car obj) port tab)
+          (%write-walk-rec (cdr obj) port tab)]
+         [(vector? obj)
+          (dotimes [i (vector-length obj)]
+            (%write-walk-rec (vector-ref obj i) port tab))]
+         [else ; generic objects.  we go walk pass via write-object
+          (write-object obj port)])
+        ;; If circular-only, we don't count non-circular objects.
+        (unless (%port-writing-shared? port)
+          (when (eqv? (hash-table-get tab obj #f) 1)
+            (hash-table-delete! tab obj)))
+        ))))
 
 (select-module gauche.internal)
 
@@ -519,25 +687,6 @@
 
 (define-in-module gauche (print . args) (for-each display args) (newline))
 
-(define (%format-common port fmt args shared?)
-  (cond [(eqv? port #f)
-         (let ((out (open-output-string :private? #t)))
-           (%format out fmt args shared?)
-           (get-output-string out))]
-        [(eqv? port #t)
-         (%format (current-output-port) fmt args shared?)]
-        [else (%format port fmt args shared?)]))
-
-(define-in-module gauche (format fmt . args)
-  (if (string? fmt)
-    (%format-common #f fmt args #f) ;; srfi-28 compatible behavior
-    (%format-common fmt (car args) (cdr args) #f)))
-
-(define-in-module gauche (format/ss fmt . args)
-  (if (string? fmt)
-    (%format-common #f fmt args #t) ;; srfi-28 compatible behavior
-    (%format-common fmt (car args) (cdr args) #t)))
-
 ;;;
 ;;; With-something
 ;;;
@@ -561,6 +710,12 @@
     (apply %open-output-file/conv filename args)
     (apply %open-output-file filename args)))
 
+;; R6RS call-with-port
+;; Make sure to close PORT when proc returns or throws an error
+(define-in-module gauche (call-with-port port proc)
+  (unwind-protect (proc port)
+    (close-port port)))
+
 ;; File ports.
 
 (define-in-module scheme (call-with-input-file filename proc . flags)
@@ -603,8 +758,8 @@
   (proc (open-input-string str)))
 
 (define-in-module gauche (call-with-string-io str proc)
-  (let ((out (open-output-string))
-        (in  (open-input-string str)))
+  (let ([out (open-output-string)]
+        [in  (open-input-string str)])
     (proc in out)
     (get-output-string out)))
 
@@ -670,4 +825,15 @@
     (port-case-fold-set! port #f)
     (values)))
 
+(define-reader-directive 'gauche-legacy
+  (^[sym port ctx] (reader-lexical-mode 'legacy) (values)))
 
+(define-reader-directive 'r7rs
+  (^[sym port ctx] (reader-lexical-mode 'strict-r7) (values)))
+
+;; HIGHLY EXPERIMENTAL
+(define-reader-directive 'c-expr
+  (^[sym port ctx]
+    ((with-module gauche.internal vm-compiler-flag-set!)
+     SCM_COMPILE_ENABLE_CEXPR)
+    (values)))
