@@ -1,7 +1,7 @@
 ;;;
 ;;; liblist.scm - builtin list procedures
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -116,13 +116,25 @@
   (let* ([len::long (Scm_Length list)])
     (when (< len 0) (Scm_Error "bad list: %S" list))
     (result len)))
+(define-cproc length<=? (list k::<fixnum>) ::<boolean> :constant
+  (result TRUE)
+  (dolist [_ list] (when (<= (post-- k) 0) (result FALSE) (break))))
 
 (define-cproc append (:rest lists) (inliner APPEND) Scm_Append)
 (define-cproc reverse (list::<list> :optional (tail ())) Scm_Reverse2)
 
 (define-cproc list-tail (list k::<fixnum> :optional fallback) :constant
   Scm_ListTail)
+;;We need to define list-set! as cproc in order to use it in the setter clause
+;;of list-ref.  This limitation of cgen.stub should be removed in future.
+;;(define (list-set! lis k v) (set-car! (list-tail lis k) v))
+(define-cproc list-set! (lis k::<fixnum> v) ::<void>
+  (let* ([p (Scm_ListTail lis k SCM_FALSE)])
+    (if (SCM_PAIRP p)
+      (SCM_SET_CAR p v)
+      (Scm_Error "list-set!: index out of bound: %d" k))))
 (define-cproc list-ref (list k::<fixnum> :optional fallback) :constant
+  (setter list-set!)
   Scm_ListRef)
 
 (define-cproc memq (obj list::<list>) :constant (inliner MEMQ) Scm_Memq)
@@ -221,12 +233,16 @@
 (define (last lis) (car (last-pair lis))) ;srfi-1
 
 (define (iota count :optional (start 0) (step 1)) ;srfi-1
-  (when (< count 0) (error "count must be nonnegative: " count))
+  (unless (and (integer? count) (>= count 0))
+    (error "count must be nonnegative integer: " count))
   (if (and (exact? start) (exact? step))
-    (do ([c count (- c 1)]
-         [v (+ start (* (- count 1) step)) (- v step)]
-         [r '() (cons v r)])
-        [(<= c 0) r])
+    ;; we allow inexact integer as 'count', for the consistency of
+    ;; giota and liota in which we can also accept +inf.0 as count.
+    (let1 count (exact count)
+      (do ([c count (- c 1)]
+           [v (+ start (* (- count 1) step)) (- v step)]
+           [r '() (cons v r)])
+          [(<= c 0) r]))
     ;; for inexact numbers, we use multiplication to avoid error accumulation.
     (do ([c count (- c 1)]
          [r '() (cons (+ start (* (- c 1) step)) r)])
@@ -382,7 +398,7 @@
             [else (loop (cdr lis))])))
   (let restart ([ans lis])
     (cond [(null-list? ans) ans]
-	  [(not (pred (car ans))) (restart (cdr ans))]
+          [(not (pred (car ans))) (restart (cdr ans))]
           [else (keep! ans (cdr ans)) ans])))
 
 (define (remove  pred l) (filter  (^x (not (pred x))) l))
@@ -435,6 +451,40 @@
           (apply kons (append! cars (list (rec cdrs))))
           knil)))))
 
+(define (count pred lis . more)
+  (if (null? more)
+    (let rec ([lis lis] [cnt 0])
+      (if (null-list? lis)
+        cnt
+        (rec (cdr lis) (if (pred (car lis)) (+ cnt 1) cnt))))
+    (let rec ([liss (cons lis more)] [cnt 0])
+      (receive (cars cdrs) ((with-module gauche.internal %zip-nary-args) liss)
+        (if cars
+          (rec cdrs (if (apply pred cars) (+ cnt 1) cnt))
+          cnt)))))
+
+(define (reduce f ridentity lis)
+  (if (null-list? lis)
+    ridentity
+    (fold f (car lis) (cdr lis))))
+
+(define (reduce-right f ridentity lis)
+  (if (null-list? lis)
+    ridentity
+    (let rec ([head (car lis)] [lis (cdr lis)])
+      (if (pair? lis)
+        (f head (rec (car lis) (cdr lis)))
+        head))))
+
+(define (append-reverse list tail)  (reverse list tail)) ;srfi-1 compat
+(define (append-reverse! list tail) (reverse! list tail));srfi-1 compat
+
+(define (concatenate  lists) (reduce-right append  '() lists))
+(define (concatenate! lists) (reduce-right append! '() lists))
+
+(define (append-map f lis . lists)  (concatenate  (apply map f lis lists)))
+(define (append-map! f lis . lists) (concatenate! (apply map f lis lists)))
+
 (define (map* fn tail-fn lis . more)
   (if (null? more)
     (let rec ([xs lis] [rs '()])
@@ -473,6 +523,17 @@
           [(null? rest) (error "given list is too short:" lis)]
           [else (loop (- i 1) (cdr rest) rest)])))
 
+;; partition is here, for gauche.procedure has partition$ and we don't
+;; want it to depend on srfi-1.  partition! is left in srfi-1, for its
+;; optimized version is rather complicated.
+(define (partition pred lis)
+  (let rec ([lis lis] [xs '()] [ys '()])
+    (if (null-list? lis)
+      (values (reverse! xs) (reverse! ys))
+      (if (pred (car lis))
+        (rec (cdr lis) (cons (car lis) xs) ys)
+        (rec (cdr lis) xs (cons (car lis) ys))))))
+
 (define (take list k)
   (let loop ([lis list] [r '()] [j k])
     (cond [(= j 0) (reverse! r)]
@@ -531,13 +592,37 @@
           [(null? lis) '()]
           [else (loop (+ i 1) (cdr lis))])))
 
+(with-module gauche.internal
+  ;; A tolerant version of list-tail.  If LIS is shorter than K, returns
+  ;; (- k (length lis)) as the second value.
+  (define (%list-tail* lis k)
+    (let loop ([lis lis] [k k])
+      (cond [(<= k 0) (values lis 0)]
+            [(null? lis) (values lis k)]
+            [else (loop (cdr lis) (- k 1))])))
+  )
+
 (define (take-right* lis k :optional (fill? #f) (filler #f))
-  (when (or (not (integer? k)) (negative? k))
-    (error "index must be non-negative integer" k))
-  (let1 len (length lis)
-    (cond [(<= k len) (drop lis (- len k))]
-          [fill? (append! (make-list (- k len) filler) lis)]
-          [else lis])))
+  (when (or (not (integer? k)) (negative? k) (inexact? k))
+    (error "index must be non-negative exact integer" k))
+  ;; NB: This procedure can be used to take the last K elements of
+  ;; a huge lazy list.  (Not so much in take-right, with which you need
+  ;; to know the length of list is greater than K beforehand.)
+  ;; The naive implementation (drop lis (- (length lis) k)) would require
+  ;; to realize entire list on memory, which we want to avoid.
+  ;; We overwrite LIS and TAIL in each iteration instead of rebinding it,
+  ;; in order to release reference to the head of list.
+  (receive (tail j) ((with-module gauche.internal %list-tail*) lis k)
+    (if (= j 0)
+      (let loop ()
+        (if (pair? tail)
+          (begin (set! lis (cdr lis))
+                 (set! tail (cdr tail))
+                 (loop))
+          lis))
+      (if fill?
+        (append! (make-list j filler) lis)
+        lis))))
 
 (define (drop-right* lis k)
   (let1 len (length lis)
@@ -547,12 +632,12 @@
 (define (slices lis k . args)
   (unless (and (integer? k) (positive? k))
     (error "index must be positive integer" k))
-  (let loop ((lis lis)
-             (r '()))
+  (let loop ([lis lis]
+             [r '()])
     (if (null? lis)
-        (reverse! r)
-        (receive (h t) (apply split-at* lis k args)
-          (loop t (cons h r))))))
+      (reverse! r)
+      (receive (h t) (apply split-at* lis k args)
+        (loop t (cons h r))))))
 
 ;; intersperse - insert ITEM between elements in the list.
 ;; (the order of arguments is taken from Haskell's intersperse)
@@ -609,19 +694,19 @@
   (cond [(assoc key alist eq) => cdr]
         [else default]))
 
-(define (assq-ref alist key . opts)
-  (assoc-ref alist key (get-optional opts #f) eq?))
-(define (assv-ref alist key . opts)
-  (assoc-ref alist key (get-optional opts #f) eqv?))
+(define (assq-ref alist key :optional (default #f))
+  (assoc-ref alist key default eq?))
+(define (assv-ref alist key :optional (default #f))
+  (assoc-ref alist key default eqv?))
 
 (define (rassoc-ref alist key :optional (default #f) (eq equal?))
   (cond [(rassoc key alist eq) => car]
         [else default]))
 
-(define (rassq-ref alist key . opts)
-  (rassoc-ref alist key (get-optional opts #f) eq?))
-(define (rassv-ref alist key . opts)
-  (rassoc-ref alist key (get-optional opts #f) eqv?))
+(define (rassq-ref alist key :optional (default #f))
+  (rassoc-ref alist key default eq?))
+(define (rassv-ref alist key :optional (default #f))
+  (rassoc-ref alist key default eqv?))
 
 ;; 'assoc-set!'
 (define (assoc-set! alist key val :optional (eq equal?))
