@@ -1,7 +1,7 @@
 ;;;
-;;; libmod.scm - modules and bindings
+;;; libmod.scm - modules, bindings, GLOCs
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,6 +36,10 @@
 (inline-stub
  (declcode (.include <gauche/vminsn.h>)))
 
+;;;
+;;; Modules API
+;;;
+
 (define-cproc module? (obj) ::<boolean> :constant SCM_MODULEP)
 
 (define-cproc module-name (mod::<module>)
@@ -44,8 +48,8 @@
 (define-cproc module-parents (mod::<module>) (result (-> mod parents)))
 (define-cproc module-precedence-list (mod::<module>) (result (-> mod mpl)))
 (define-cproc module-imports (mod::<module>) (result (-> mod imported)))
-(define-cproc module-exports (mod::<module>) (result (-> mod exported)))
-(define-cproc module-table (mod::<module>) (result (SCM_OBJ (-> mod table))))
+(define-cproc module-exports (mod::<module>) Scm_ModuleExports)
+(define-cproc module-table (mod::<module>) (result (SCM_OBJ (-> mod internal))))
 
 (define-cproc find-module (name::<symbol>) ::<module>?
   (result (Scm_FindModule name SCM_FIND_MODULE_QUIET)))
@@ -59,18 +63,52 @@
           [else (Scm_TypeError ":if-exists" ":error or #f" if_exists)])
     (result (Scm_MakeModule name error_if_exists))))
 
-(define-cproc module-name->path (name)
-  (let* ([s::ScmSymbol* NULL])
-    (cond [(SCM_SYMBOLP name) (set! s (SCM_SYMBOL name))]
-          [(SCM_IDENTIFIERP name) (set! s (-> (SCM_IDENTIFIER name) name))]
-          [else (SCM_TYPE_ERROR name "symbol or identifier")])
-    (result (Scm_ModuleNameToPath s))))
+;; (use x.y.z) ==> (requrie "x/y/z") (import x.y.z)
+(define-macro (use module . options)
+  ;; Kludge - remove these once we have low-level hygienic macro.
+  (define begin.   ((with-module gauche.internal make-identifier)
+                    'begin (find-module 'gauche) '()))
+  (define require. ((with-module gauche.internal make-identifier)
+                    'require (find-module 'gauche) '()))
+  (define import.  ((with-module gauche.internal make-identifier)
+                    'import (find-module 'gauche) '()))
+  `(,begin.
+     (,require. ,(module-name->path module))
+     (,import. (,module ,@options))))
 
-(define-cproc path->module-name (path::<string>) Scm_PathToModuleName)
+;; foo.bar.baz <=> "foo/bar/baz"
+;;  - Two consecutive dots in module name becomes one dot in path
+;;    foo..bar.baz <=> "foo.bar/baz".   This is to support R7RS library
+;;    whose name is (foo.bar baz).
+;;  - Todo: Escape unsafe characters in name.
+(define (module-name->path name)
+  (define (path-comp p)
+    (rlet1 s (list->string (reverse p))
+      (cond [(equal? s "")
+             (error "Invalid module name (it can't end with #\\.):" name)]
+            [(or (equal? s ".") (equal? s ".."))
+             (error "Invalid module name (component can't be \".\" or \"..\"):"
+                    name)])))
+  (with-input-from-string
+   (cond [(symbol? name) (symbol->string name)]
+         [(identifier? name) (symbol->string (identifier-name name))]
+         [else (error "symbol or identifier expected, but got:" name)])
+   (^[] (let loop ([c (read-char)] [p '()] [ps '()])
+          (cond
+           [(eof-object? c)
+            (string-join (reverse (cons (path-comp p) ps)) "/")]
+           [(eqv? c #\.)
+            (let1 c2 (read-char)
+              (if (eqv? c2 #\.)
+                (loop (read-char) (cons c2 p) ps)
+                (loop c2 '() (cons (path-comp p) ps))))]
+           [else (loop (read-char) (cons c p) ps)])))))
 
-(define-cproc %export-all (module::<module>) Scm_ExportAll)
-(define-cproc %extend-module (module::<module> supers::<list>)
-  Scm_ExtendModule)
+(define (path->module-name path)
+  (unless (string? path) (error "string required, but got:" path))
+  ($ string->symbol $ (cut string-join <> ".")
+     $ map (cut regexp-replace-all #/\./ <> "..")
+     $ string-split path "/"))
 
 (inline-stub
  (define-cfn get-module-from-mod-or-name (id) ::ScmModule* :static
@@ -81,12 +119,12 @@
                (return NULL)]))
  )
 
+;; Global bindind access, public API
 (define-cproc global-variable-bound? (mod-or-name name::<symbol>) ::<boolean>
   (result
    (not (SCM_UNBOUNDP
          (Scm_GlobalVariableRef (get-module-from-mod-or-name mod-or-name)
                                 name 0)))))
-
 (define-cproc global-variable-ref (mod_or_name name::<symbol>
                                                :optional
                                                fallback
@@ -100,32 +138,14 @@
                  name module))
     (result r2)))
 
+(define-in-module gauche (symbol-bound? name :optional (module #f)) ; Deprecated
+  (global-variable-bound? module name))
+
+;; Module import/export internal APIs.  Not public.
 (select-module gauche.internal)
-(inline-stub
- (define-type <gloc> "ScmGloc*" "GLOC"
-   "SCM_GLOCP" "SCM_GLOC" "SCM_OBJ")
- )
-
-;; Returns GLOC object or #f
-(define-cproc find-binding (mod::<module> name::<symbol>
-                                          stay-in-module::<boolean>)
-  ::<gloc>?
-  (result (Scm_FindBinding mod name
-                           (?: stay_in_module SCM_BINDING_STAY_IN_MODULE 0))))
-
-;; This small piece of code encapsulates the common procedure in
-;; pass1/variable to find whether the variable reference is a constant
-;; or not.
-(define-cproc find-const-binding (id::<identifier>)
-  (let* ([g::ScmGloc* (Scm_FindBinding (-> id module) (-> id name) 0)])
-    (cond [(or (not g)
-               (not (SCM_GLOC_CONST_P g))
-               (SCM_VM_COMPILER_FLAG_IS_SET (Scm_VM)
-                                            SCM_COMPILE_NOINLINE_CONSTS))
-           (result SCM_FALSE)]
-          [else
-           (result (SCM_GLOC_GET g))])))
-
+(define-cproc %export-all (module::<module>) Scm_ExportAll)
+(define-cproc %extend-module (module::<module> supers::<list>)
+  Scm_ExtendModule)
 (define-cproc %insert-binding (mod::<module> name::<symbol> value
                                              :optional (flags '()))
   (let* ([z::int 0])
@@ -156,22 +176,85 @@
 (define-cproc %import-modules (mod::<module> mods) ;deprecated
   Scm_ImportModules)
 
+;;;
+;;; Universal import
+;;;
+
+(select-module gauche.internal)
+
+(declare (keep-private-macro import))
+
+;; Kludge: We let user#import handle both gauche#import and r7rs#import, so
+;; that R7RS script can be run by gosh without any special treatment.
+;; If we see r7rs syntax, not only we import the specified module, but
+;; we switch the current module to r7rs.user.
+(define (%expand-user-import args)
+  (if (any (^[import-spec]
+             (or (symbol? import-spec)
+                 (and (list? import-spec)
+                      (pair? (cdr import-spec))
+                      (keyword? (cadr import-spec)))))
+           args)
+    `((with-module gauche import) ,@args)
+    (begin
+      ;; Some black magic needed here to inject the imported module to
+      ;; r7rs.user.  We do this here instead of expanding into forms
+      ;; to do the work, for "select-module" thingy is tricky.
+      (unless (provided? "r7rs")
+        (load "r7rs")
+        (provide "r7rs"))
+      (eval `(import ,@args) (find-module 'r7rs.user))
+      '(select-module r7rs.user))))
+
+(select-module user)
+
+(define-macro (import . import-specs)
+  ((with-module gauche.internal %expand-user-import) import-specs))
+
+;;;
+;;; GLOCs
+;;;
+
+(select-module gauche.internal)
+
+;; GLOCs are not for public use, so all APIs are internal.
+
+(inline-stub
+ (define-type <gloc> "ScmGloc*" "GLOC"
+   "SCM_GLOCP" "SCM_GLOC" "SCM_OBJ")
+ )
+
+;; Returns GLOC object or #f.
+(define-cproc find-binding (mod::<module> name::<symbol>
+                                          stay-in-module::<boolean>)
+  ::<gloc>?
+  (result (Scm_FindBinding mod name
+                           (?: stay_in_module SCM_BINDING_STAY_IN_MODULE 0))))
+
+;; This small piece of code encapsulates the common procedure in
+;; pass1/variable to find whether the variable reference is a constant
+;; or not.
+(define-cproc find-const-binding (id::<identifier>)
+  (let* ([g::ScmGloc* (Scm_FindBinding (-> id module) (-> id name) 0)])
+    (cond [(or (not g)
+               (not (SCM_GLOC_CONST_P g))
+               (SCM_VM_COMPILER_FLAG_IS_SET (Scm_VM)
+                                            SCM_COMPILE_NOINLINE_CONSTS))
+           (result SCM_FALSE)]
+          [else
+           (result (SCM_GLOC_GET g))])))
+
 (define-cproc gloc-bound? (gloc::<gloc>) ::<boolean>
   (result (not (SCM_UNBOUNDP (SCM_GLOC_GET gloc)))))
 (define-cproc gloc-ref (gloc::<gloc> :optional fallback)
   (let* ([v::ScmObj (SCM_GLOC_GET gloc)])
     (if (SCM_UNBOUNDP v)
-      (if (SCM_UNBOUNDP fallback)
-        (Scm_Error "gloc %S doesn't have a value" (SCM_OBJ gloc))
+      (begin
+        (when (SCM_UNBOUNDP fallback)
+          (Scm_Error "gloc %S doesn't have a value" (SCM_OBJ gloc)))
         (result fallback))
       (result v))))
 (define-cproc gloc-set! (gloc::<gloc> value) SCM_GLOC_SET)
 (define-cproc gloc-const? (gloc::<gloc>) ::<boolean> Scm_GlocConstP)
 (define-cproc gloc-inlinable? (gloc::<gloc>) ::<boolean> Scm_GlocInlinableP)
 
-;;;
-;;; symbol-bound? (deprecated)
-;;;
-
-(define-in-module gauche (symbol-bound? name . maybe-module)
-  (global-variable-bound? (get-optional maybe-module #f) name))
