@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -716,6 +716,19 @@
 (define-method (setter ref) ((obj <string>) (index <integer>) val)
   (string-set! obj index val))
 
+;; Universal accessor
+(define ~
+  (getter-with-setter
+   (case-lambda
+     [(obj selector) (ref obj selector)]
+     [(obj selector . more) (apply ~ (ref obj selector) more)])
+   (case-lambda
+     [(obj selector val) ((setter ref) obj selector val)]
+     [(obj selector selector2 . rest)
+      (apply (setter ~) (ref obj selector) selector2 rest)])))
+
+(define ref* ~)                         ;for the backward compatibility
+
 ;;----------------------------------------------------------------
 ;; Generalized application hooks
 ;;  (should this be in separate file, e.g. apply.scm?)
@@ -738,24 +751,32 @@
      (errorf "bad symbol argument to ~s: ~s: must be either 'before or 'after"
              self s)]))
 
+;; Char-set membership
+(define-method object-apply ((self <char-set>) (c <char>))
+  (char-set-contains? self c))
+
 ;; A trick to let a condition type behave its own predicate
 (define-method object-apply ((type <condition-meta>) obj)
   (condition-has-type? obj type))
 
 ;; Regexp printer.  It doesn't need speed, and it's easier to write in Scheme.
 ;; This is defined here since we can only use it after define-method support.
-;; TODO: eventually we need a converter from internal AST to string regexp
-;; so that we can print the regexp which is created procedurally (via
-;; regexp-compile), hence we have complete read/write invariance.
+;; NB: case-folding regexp is expressed by (?i:...) in regexp->string.
 (define-method write-object ((obj <regexp>) out)
-  (cond [(regexp->string obj)
-         => (^s (format out "#/~a/~a"
-                        (regexp-replace-all*
-                         s #/\// "\\\\/" #/[\x00-\x1f\x7f]/
-                         (lambda (m)
-                           (format "\\x~2,'0x" (char->integer (string-ref (m 0) 0)))))
-                        (if (regexp-case-fold? obj) "i" "")))]
-        [else (format out "#<regexp>")]))
+  (with-input-from-string (regexp->string obj)
+    (^[]
+      (display "#/" out)
+      (let loop ([c (read-char)])
+        (unless (eof-object? c)
+          (cond [(char=? #\/ c) (display #\\ out) (display c out)]
+                [(memq (char-general-category c) '(Mn Mc Me Cc Cf Cs Co Cn))
+                 (let1 code (char->ucs c)
+                   (if (< code #x10000)  
+                     (format out "\\u~4,'0x" code)
+                     (format out "\\U~8,'0x" code)))]
+                [else (display c out)])
+          (loop (read-char))))
+      (display "/" out))))
 
 ;;;
 ;;; Make exported symbol visible from outside
@@ -785,6 +806,7 @@
                 slot-definition-setter slot-definition-accessor
                 class-slot-definition class-slot-accessor
                 x->string x->integer x->number ref |setter of ref|
+                ~ ref*
 
                 ;; These shouldn't be necessary to be injected into gauche
                 ;; module; unfortunately, the current define-method and
