@@ -1,7 +1,7 @@
 ;;;
 ;;; librx.scm - builtin regular-expression procedures
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -91,7 +91,10 @@
   (if (SCM_FALSEP match)
     (result (SCM_MAKE_INT 0))
     (rxmatchop (SCM_MAKE_INT (-> (SCM_REGMATCH match) numMatches)))))
-
+(define-cproc rxmatch-named-groups (match)
+  (if (SCM_FALSEP match)
+    (result SCM_NIL)
+    (rxmatchop (-> (SCM_REGMATCH match) grpNames))))
 
 (select-module gauche.internal)
 (define-cproc %regexp-dump (rx::<regexp>) ::<void> Scm_RegDump)
@@ -164,7 +167,7 @@
   (^[string rx sub . more]
     (cond [(null? more) (func-1 rx string sub)]
           [else
-           (unless (zero? (modulo (length more) 2))
+           (unless (even? (length more))
              (errorf "~a: regexp and subsitution don't pair up" name))
            (let loop ([s (func-1 rx string sub)]
                       [args more])
