@@ -1,7 +1,7 @@
 ;;;
 ;;; libsym.scm - built-in symbol and keyword procedures
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -54,6 +54,13 @@
 (define-cproc symbol-sans-prefix (s::<symbol> p::<symbol>)
   Scm_SymbolSansPrefix)
 
+;; R7RS
+(define (symbol=? x y . rest)
+  (if-let1 z (find ($ not $ symbol? $) (list* x y rest))
+    (error "symbol required, but got:" z))
+  (and (eq? x y) (or (null? rest) (every (cut eq? x <>) rest))))
+
+
 ;;;
 ;;;  Keywords
 ;;;
@@ -75,7 +82,9 @@
 (define-cproc delete-keyword! (key list) Scm_DeleteKeywordX)
 
 (define-cproc keyword->string (key::<keyword>)
-  (result (SCM_OBJ (SCM_KEYWORD_NAME key))))
+  (.if "defined(GAUCHE_UNIFY_SYMBOL_KEYWORD)"
+       (result (Scm_Substring (SCM_KEYWORD_NAME key) 1 -1 FALSE))
+       (result (SCM_OBJ (SCM_KEYWORD_NAME key)))))
 
 (define-in-module gauche (delete-keywords ks kvlist)
   (define (rec kvs)
