@@ -1,7 +1,7 @@
 ;;;
 ;;; libsys.scm - builtin system inteface
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -51,12 +51,6 @@
 
   (.if "defined(GAUCHE_WINDOWS)"
        (.undef _SC_CLK_TCK)) ;; avoid undefined reference to sysconf
-  (.if "defined(_MSC_VER)"
-       ;; This is a kludge to workaround the configure misdetection
-       (begin
-         (.undef HAVE_GETDOMAINNAME)
-         (.undef HAVE_UNSETENV)
-         (.undef HAVE_GETPGID)))
   ))
 
 ;;---------------------------------------------------------------------
@@ -87,137 +81,159 @@
 
 ;; We won't (and can't) cover every possible errnos, including system
 ;; specific ones.  The following list is taken from Linux asm/errno.h.
+
 (inline-stub
- (define-enum-conditionally E2BIG)
- (define-enum-conditionally EACCES)
- (define-enum-conditionally EADDRINUSE)
- (define-enum-conditionally EADDRNOTAVAIL)
- (define-enum-conditionally EADV)
- (define-enum-conditionally EAFNOSUPPORT)
- (define-enum-conditionally EAGAIN)
- (define-enum-conditionally EALREADY)
- (define-enum-conditionally EBADE)
- (define-enum-conditionally EBADF)
- (define-enum-conditionally EBADFD)
- (define-enum-conditionally EBADMSG)
- (define-enum-conditionally EBADR)
- (define-enum-conditionally EBADRQC)
- (define-enum-conditionally EBADSLT)
- (define-enum-conditionally EBFONT)
- (define-enum-conditionally EBUSY)
- (define-enum-conditionally ECANCELED)
- (define-enum-conditionally ECHILD)
- (define-enum-conditionally ECHRNG)
- (define-enum-conditionally ECOMM)
- (define-enum-conditionally ECONNABORTED)
- (define-enum-conditionally ECONNREFUSED)
- (define-enum-conditionally ECONNRESET)
- (define-enum-conditionally EDEADLK)
- (define-enum-conditionally EDEADLOCK)
- (define-enum-conditionally EDESTADDRREQ)
- (define-enum-conditionally EDOM)
- (define-enum-conditionally EDOTDOT)
- (define-enum-conditionally EDQUOT)
- (define-enum-conditionally EEXIST)
- (define-enum-conditionally EFAULT)
- (define-enum-conditionally EFBIG)
- (define-enum-conditionally EHOSTDOWN)
- (define-enum-conditionally EHOSTUNREACH)
- (define-enum-conditionally EIDRM)
- (define-enum-conditionally EILSEQ)
- (define-enum-conditionally EINPROGRESS)
- (define-enum-conditionally EINTR)
- (define-enum-conditionally EINVAL)
- (define-enum-conditionally EIO)
- (define-enum-conditionally EISCONN)
- (define-enum-conditionally EISDIR)
- (define-enum-conditionally EISNAM)
- (define-enum-conditionally EKEYEXPIRED)
- (define-enum-conditionally EKEYREJECTED)
- (define-enum-conditionally EKEYREVOKED)
- (define-enum-conditionally EL2HLT)
- (define-enum-conditionally EL2NSYNC)
- (define-enum-conditionally EL3HLT)
- (define-enum-conditionally EL3RST)
- (define-enum-conditionally ELIBACC)
- (define-enum-conditionally ELIBBAD)
- (define-enum-conditionally ELIBEXEC)
- (define-enum-conditionally ELIBMAX)
- (define-enum-conditionally ELIBSCN)
- (define-enum-conditionally ELNRNG)
- (define-enum-conditionally ELOOP)
- (define-enum-conditionally EMEDIUMTYPE)
- (define-enum-conditionally EMFILE)
- (define-enum-conditionally EMLINK)
- (define-enum-conditionally EMSGSIZE)
- (define-enum-conditionally EMULTIHOP)
- (define-enum-conditionally ENAMETOOLONG)
- (define-enum-conditionally ENAVAIL)
- (define-enum-conditionally ENETDOWN)
- (define-enum-conditionally ENETRESET)
- (define-enum-conditionally ENETUNREACH)
- (define-enum-conditionally ENFILE)
- (define-enum-conditionally ENOANO)
- (define-enum-conditionally ENOBUFS)
- (define-enum-conditionally ENOCSI)
- (define-enum-conditionally ENODATA)
- (define-enum-conditionally ENODEV)
- (define-enum-conditionally ENOENT)
- (define-enum-conditionally ENOEXEC)
- (define-enum-conditionally ENOKEY)
- (define-enum-conditionally ENOLCK)
- (define-enum-conditionally ENOLINK)
- (define-enum-conditionally ENOMEDIUM)
- (define-enum-conditionally ENOMEM)
- (define-enum-conditionally ENOMSG)
- (define-enum-conditionally ENONET)
- (define-enum-conditionally ENOPKG)
- (define-enum-conditionally ENOPROTOOPT)
- (define-enum-conditionally ENOSPC)
- (define-enum-conditionally ENOSR)
- (define-enum-conditionally ENOSTR)
- (define-enum-conditionally ENOSYS)
- (define-enum-conditionally ENOTBLK)
- (define-enum-conditionally ENOTCONN)
- (define-enum-conditionally ENOTDIR)
- (define-enum-conditionally ENOTEMPTY)
- (define-enum-conditionally ENOTNAM)
- (define-enum-conditionally ENOTSOCK)
- (define-enum-conditionally ENOTTY)
- (define-enum-conditionally ENOTUNIQ)
- (define-enum-conditionally ENXIO)
- (define-enum-conditionally EOPNOTSUPP)
- (define-enum-conditionally EOVERFLOW)
- (define-enum-conditionally EPERM)
- (define-enum-conditionally EPFNOSUPPORT)
- (define-enum-conditionally EPIPE)
- (define-enum-conditionally EPROTO)
- (define-enum-conditionally EPROTONOSUPPORT)
- (define-enum-conditionally EPROTOTYPE)
- (define-enum-conditionally ERANGE)
- (define-enum-conditionally EREMCHG)
- (define-enum-conditionally EREMOTE)
- (define-enum-conditionally EREMOTEIO)
- (define-enum-conditionally ERESTART)
- (define-enum-conditionally EROFS)
- (define-enum-conditionally ESHUTDOWN)
- (define-enum-conditionally ESOCKTNOSUPPORT)
- (define-enum-conditionally ESPIPE)
- (define-enum-conditionally ESRCH)
- (define-enum-conditionally ESRMNT)
- (define-enum-conditionally ESTALE)
- (define-enum-conditionally ESTRPIPE)
- (define-enum-conditionally ETIME)
- (define-enum-conditionally ETIMEDOUT)
- (define-enum-conditionally ETOOMANYREFS)
- (define-enum-conditionally ETXTBSY)
- (define-enum-conditionally EUCLEAN)
- (define-enum-conditionally EUNATCH)
- (define-enum-conditionally EUSERS)
- (define-enum-conditionally EWOULDBLOCK)
- (define-enum-conditionally EXDEV)
- (define-enum-conditionally EXFULL)
- )
+ "ScmHashTable *errno_n2y;"             ;integer -> symbol
+ "ScmHashTable *errno_y2n;"             ;symbol -> integer
+ (initcode
+  (set! errno_n2y (SCM_HASH_TABLE (Scm_MakeHashTableSimple SCM_HASH_EQV 0)))
+  (set! errno_y2n (SCM_HASH_TABLE (Scm_MakeHashTableSimple SCM_HASH_EQ 0)))))
+  
+
+(define-macro (define-errno symbol)
+  `(inline-stub
+    (initcode
+     (.if ,#`"defined(,symbol)"
+          (begin
+            (Scm_Define (Scm_GaucheModule)
+                        (SCM_SYMBOL ',symbol) (SCM_MAKE_INT ,symbol))
+            (Scm_HashTableSet errno_n2y (SCM_MAKE_INT ,symbol) ',symbol 0)
+            (Scm_HashTableSet errno_y2n ',symbol (SCM_MAKE_INT ,symbol) 0))))))
+
+(define-cproc sys-errno->symbol (num::<fixnum>)
+  (result (Scm_HashTableRef errno_n2y (SCM_MAKE_INT num) SCM_FALSE)))
+(define-cproc sys-symbol->errno (name::<symbol>)
+  (result (Scm_HashTableRef errno_y2n (SCM_OBJ name) SCM_FALSE)))
+
+(define-errno E2BIG)
+(define-errno EACCES)
+(define-errno EADDRINUSE)
+(define-errno EADDRNOTAVAIL)
+(define-errno EADV)
+(define-errno EAFNOSUPPORT)
+(define-errno EAGAIN)
+(define-errno EALREADY)
+(define-errno EBADE)
+(define-errno EBADF)
+(define-errno EBADFD)
+(define-errno EBADMSG)
+(define-errno EBADR)
+(define-errno EBADRQC)
+(define-errno EBADSLT)
+(define-errno EBFONT)
+(define-errno EBUSY)
+(define-errno ECANCELED)
+(define-errno ECHILD)
+(define-errno ECHRNG)
+(define-errno ECOMM)
+(define-errno ECONNABORTED)
+(define-errno ECONNREFUSED)
+(define-errno ECONNRESET)
+(define-errno EDEADLK)
+(define-errno EDEADLOCK)
+(define-errno EDESTADDRREQ)
+(define-errno EDOM)
+(define-errno EDOTDOT)
+(define-errno EDQUOT)
+(define-errno EEXIST)
+(define-errno EFAULT)
+(define-errno EFBIG)
+(define-errno EHOSTDOWN)
+(define-errno EHOSTUNREACH)
+(define-errno EIDRM)
+(define-errno EILSEQ)
+(define-errno EINPROGRESS)
+(define-errno EINTR)
+(define-errno EINVAL)
+(define-errno EIO)
+(define-errno EISCONN)
+(define-errno EISDIR)
+(define-errno EISNAM)
+(define-errno EKEYEXPIRED)
+(define-errno EKEYREJECTED)
+(define-errno EKEYREVOKED)
+(define-errno EL2HLT)
+(define-errno EL2NSYNC)
+(define-errno EL3HLT)
+(define-errno EL3RST)
+(define-errno ELIBACC)
+(define-errno ELIBBAD)
+(define-errno ELIBEXEC)
+(define-errno ELIBMAX)
+(define-errno ELIBSCN)
+(define-errno ELNRNG)
+(define-errno ELOOP)
+(define-errno EMEDIUMTYPE)
+(define-errno EMFILE)
+(define-errno EMLINK)
+(define-errno EMSGSIZE)
+(define-errno EMULTIHOP)
+(define-errno ENAMETOOLONG)
+(define-errno ENAVAIL)
+(define-errno ENETDOWN)
+(define-errno ENETRESET)
+(define-errno ENETUNREACH)
+(define-errno ENFILE)
+(define-errno ENOANO)
+(define-errno ENOBUFS)
+(define-errno ENOCSI)
+(define-errno ENODATA)
+(define-errno ENODEV)
+(define-errno ENOENT)
+(define-errno ENOEXEC)
+(define-errno ENOKEY)
+(define-errno ENOLCK)
+(define-errno ENOLINK)
+(define-errno ENOMEDIUM)
+(define-errno ENOMEM)
+(define-errno ENOMSG)
+(define-errno ENONET)
+(define-errno ENOPKG)
+(define-errno ENOPROTOOPT)
+(define-errno ENOSPC)
+(define-errno ENOSR)
+(define-errno ENOSTR)
+(define-errno ENOSYS)
+(define-errno ENOTBLK)
+(define-errno ENOTCONN)
+(define-errno ENOTDIR)
+(define-errno ENOTEMPTY)
+(define-errno ENOTNAM)
+(define-errno ENOTSOCK)
+(define-errno ENOTTY)
+(define-errno ENOTUNIQ)
+(define-errno ENXIO)
+(define-errno EOPNOTSUPP)
+(define-errno EOVERFLOW)
+(define-errno EPERM)
+(define-errno EPFNOSUPPORT)
+(define-errno EPIPE)
+(define-errno EPROTO)
+(define-errno EPROTONOSUPPORT)
+(define-errno EPROTOTYPE)
+(define-errno ERANGE)
+(define-errno EREMCHG)
+(define-errno EREMOTE)
+(define-errno EREMOTEIO)
+(define-errno ERESTART)
+(define-errno EROFS)
+(define-errno ESHUTDOWN)
+(define-errno ESOCKTNOSUPPORT)
+(define-errno ESPIPE)
+(define-errno ESRCH)
+(define-errno ESRMNT)
+(define-errno ESTALE)
+(define-errno ESTRPIPE)
+(define-errno ETIME)
+(define-errno ETIMEDOUT)
+(define-errno ETOOMANYREFS)
+(define-errno ETXTBSY)
+(define-errno EUCLEAN)
+(define-errno EUNATCH)
+(define-errno EUSERS)
+(define-errno EWOULDBLOCK)
+(define-errno EXDEV)
+(define-errno EXFULL)
 
 ;;---------------------------------------------------------------------
 ;; grp.h - groups
@@ -281,18 +297,7 @@
 ;;---------------------------------------------------------------------
 ;; math.h
 
-(define-cproc fmod (x::<real> y::<real>) ::<double> fmod)
-
-(define-cproc frexp (d::<real>) ::(<double> <int>)
-  (set! SCM_RESULT0 (frexp d (& SCM_RESULT1))))
-
-(define-cproc modf (x::<real>) ::(<double> <double>)
-  (set! SCM_RESULT0 (modf x (& SCM_RESULT1))))
-
-(define-cproc ldexp (x::<real> exp::<int>) ::<real> ldexp)
-
-(define-cproc log10 (x::<real>) ::<real> log10)
-
+;; fmod, frexp, modf, ldexp, log10 - in libnum.scm
 
 ;;---------------------------------------------------------------------
 ;; pwd.h - passwords
@@ -360,7 +365,7 @@
 
 (inline-stub
  (initcode (.if "defined HAVE_SIGWAIT"
-                "Scm_AddFeature(\"gauche.sys.sigwait\", NULL);")))
+                (Scm_AddFeature "gauche.sys.sigwait" NULL))))
 
 (define (sys-sigset . signals)
   (if (null? signals)
@@ -375,7 +380,8 @@
     (SCM_SYSCALL r (remove filename))
     (when (< r 0) (Scm_SysError "remove failed on %s" filename))))
 
-(define-cproc sys-rename (oldname::<const-cstring> newname::<const-cstring>)
+(define-cproc sys-rename (oldname::<const-cstring>
+                          newname::<const-cstring>)
   ::<void>
   (let* ([r::int])
     (.if "defined(GAUCHE_WINDOWS)"
@@ -414,7 +420,7 @@
 
 (define-cproc sys-exit (code::<fixnum>) ::<void> _exit)
 
-(define-cproc sys-getenv (name::<const-cstring>) ::<const-cstring>? getenv)
+(define-cproc sys-getenv (name::<const-cstring>) ::<const-cstring>? Scm_GetEnv)
 
 (define-cproc sys-abort () ::<void> abort)
 
@@ -465,11 +471,11 @@
 (inline-stub
  (initcode
   (.if "defined(HAVE_PUTENV) || defined(HAVE_SETENV)"
-       "Scm_AddFeature(\"gauche.sys.setenv\", NULL);")
+       (Scm_AddFeature "gauche.sys.setenv" NULL))
   (.if "defined HAVE_UNSETENV"
-       "Scm_AddFeature(\"gauche.sys.unsetenv\", NULL);")
+       (Scm_AddFeature "gauche.sys.unsetenv" NULL))
   (.if "defined HAVE_CLEARENV"
-       "Scm_AddFeature(\"gauche.sys.clearenv\", NULL);")))
+       (Scm_AddFeature "gauche.sys.clearenv" NULL))))
 
 (define (sys-environ->alist :optional (envlist (sys-environ)))
   (map (^[envstr] (receive (pre post) (string-scan envstr #\= 'both)
@@ -522,7 +528,7 @@
 
 (inline-stub
  (initcode (.if "defined HAVE_GETLOADAVG"
-                "Scm_AddFeature(\"gauche.sys.getloadavg\", NULL);")))
+                (Scm_AddFeature "gauche.sys.getloadavg" NULL))))
 
 ;;---------------------------------------------------------------------
 ;; sys/resource.h
@@ -561,7 +567,7 @@
        (SCM_SYSCALL ret (setrlimit rsrc (& limit)))
        (when (< ret 0) (Scm_SysError "setrlimit failed"))))
 
-   (initcode "Scm_AddFeature(\"gauche.sys.getrlimit\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.getrlimit" NULL))
 
    (define-constant RLIM_INFINITY (c "MAKERLIMIT(RLIM_INFINITY)"))
    (define-enum-conditionally RLIMIT_AS)
@@ -627,7 +633,8 @@
    (define-cfn check-trailing-separator (path::(const char*))
      ::(const char*) :static (return path)))
 
- (define-cproc sys-stat (path::<const-cstring>) ::<sys-stat> (stat-common stat))
+ (define-cproc sys-stat (path::<const-cstring>)
+   ::<sys-stat> (stat-common stat))
 
  ;; On Windows we don't have lstat.  Omitting sys-lstat from Windows is
  ;; a bit inconvenient, however, to write a portable code if we do so.
@@ -764,6 +771,26 @@
 (define-cproc current-microseconds ()   ;EXPERIMENTAL
   ::<long> Scm_CurrentMicroseconds)
 
+;; Returns #f and #f if the system doesn't provide monotonic time.
+(define-cproc sys-clock-gettime-monotonic () ::(<top> <top>)
+  (let* ([sec::u_long] [nsec::u_long]
+         [r::int (Scm_ClockGetTimeMonotonic (& sec) (& nsec))])
+    (if r
+      (begin (set! SCM_RESULT0 (Scm_MakeIntegerU sec))
+             (set! SCM_RESULT1 (Scm_MakeIntegerU nsec)))
+      (begin (set! SCM_RESULT0 SCM_FALSE)
+             (set! SCM_RESULT1 SCM_FALSE)))))
+
+;; Returns #f and #f if the system doesn't provide monotonic time.
+(define-cproc sys-clock-getres-monotonic () ::(<top> <top>)
+  (let* ([sec::u_long] [nsec::u_long]
+         [r::int (Scm_ClockGetResMonotonic (& sec) (& nsec))])
+    (if r
+      (begin (set! SCM_RESULT0 (Scm_MakeIntegerU sec))
+             (set! SCM_RESULT1 (Scm_MakeIntegerU nsec)))
+      (begin (set! SCM_RESULT0 SCM_FALSE)
+             (set! SCM_RESULT1 SCM_FALSE)))))
+
 (define-cproc current-time ()           ;SRFI-18, SRFI-19, SRFI-21
   Scm_CurrentTime)
 
@@ -812,7 +839,8 @@
  (define-enum F_OK)
  )
 
-(define-cproc sys-access (pathname::<const-cstring> amode::<int>) ::<boolean>
+(define-cproc sys-access (pathname::<const-cstring> amode::<int>)
+  ::<boolean>
   (let* ([r::int])
     (when (Scm_IsSugid)
       (Scm_Error "cannot use sys-access in suid/sgid program."))
@@ -853,7 +881,7 @@
      ::<int>
      (SCM_SYSCALL SCM_RESULT (lchown path owner group))
      (when (< SCM_RESULT 0) (Scm_SysError "lchown failed on %S" path)))
-   (initcode "Scm_AddFeature(\"gauche.sys.lchown\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.lchown" NULL))
    )
  )
 
@@ -906,11 +934,7 @@
       (set! flags (logior flags SCM_EXEC_DETACHED)))
     (result (Scm_SysExec command args iomap sigmask directory flags))))
 
-(define-cproc sys-getcwd () ::<const-cstring>
-  (let* ([p::(.array char [1024])]) ; TODO: size needs to be configured
-    (when (== (getcwd p 1023) NULL) (Scm_SysError "getcwd failed"))
-    (result p)))
-
+(define-cproc sys-getcwd () Scm_GetCwd)
 (define-cproc sys-getegid () ::<int> getegid)
 (define-cproc sys-getgid ()  ::<int> getgid)
 (define-cproc sys-geteuid () ::<int> geteuid)
@@ -942,7 +966,7 @@
      (define-cproc sys-getpgid (pid::<int>) ::<int>
        (SCM_SYSCALL SCM_RESULT (cast int (getpgid pid)))
        (when (< SCM_RESULT 0) (Scm_SysError "getpgid failed")))
-     (initcode "Scm_AddFeature(\"gauche.sys.getpgid\", NULL);")
+     (initcode (Scm_AddFeature "gauche.sys.getpgid" NULL))
      )
 
    (define-cproc sys-getpgrp () ::<int>
@@ -981,7 +1005,8 @@
 
 (define-cproc sys-getlogin () ::<const-cstring>? getlogin)
 
-(define-cproc sys-link (existing::<const-cstring> newpath::<const-cstring>)
+(define-cproc sys-link (existing::<const-cstring>
+                        newpath::<const-cstring>)
   ::<void>
   (let* ([r::int])
     (SCM_SYSCALL r (link existing newpath))
@@ -1068,7 +1093,7 @@
        (if (and (== (ref rem tv_sec) 0) (== (ref rem tv_nsec) 0))
          (result '#f)
          (result (Scm_MakeTime '#f (ref rem tv_sec) (ref rem tv_nsec))))))
-   (initcode "Scm_AddFeature(\"gauche.sys.nanosleep\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.nanosleep" NULL))
    ) ; defined(HAVE_NANOSLEEP)||defined(GAUCHE_WINDOWS)
  )
 
@@ -1081,9 +1106,10 @@
            (chmod pathname #o600)))
     (SCM_SYSCALL r (unlink pathname))
     (if (< r 0)
-      (if (== errno ENOENT)
-        (result '#f)
-        (Scm_SysError "unlink failed on %s" pathname))
+      (begin
+        (unless (== errno ENOENT)
+          (Scm_SysError "unlink failed on %s" pathname))
+        (result '#f))
       (result '#t))))
 
 (define-cproc sys-isatty (port_or_fd) ::<boolean>
@@ -1094,7 +1120,8 @@
   (let* ([fd::int (Scm_GetPortFd port_or_fd FALSE)])
     (result (?: (< fd 0) NULL (ttyname fd)))))
 
-(define-cproc sys-truncate (path::<const-cstring> length::<integer>) ::<void>
+(define-cproc sys-truncate (path::<const-cstring> length::<integer>)
+  ::<void>
   (let* ([r::int])
     (SCM_SYSCALL r (truncate path (Scm_IntegerToOffset length)))
     (when (< r 0) (Scm_SysError "truncate failed on %s" path))))
@@ -1111,7 +1138,7 @@
  (when "defined(HAVE_CRYPT)"
    (define-cproc sys-crypt (key::<const-cstring> salt::<const-cstring>)
      ::<const-cstring> (result (cast (const char *) (crypt key salt))))
-   (initcode "Scm_AddFeature(\"gauche.sys.crypt\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.crypt" NULL))
    )
  )
 
@@ -1146,13 +1173,14 @@
 
 (inline-stub
  (when "defined(HAVE_SYMLINK)"
-   (define-cproc sys-symlink (existing::<const-cstring> newpath::<const-cstring>)
+   (define-cproc sys-symlink (existing::<const-cstring>
+                              newpath::<const-cstring>)
      ::<void>
      (let* ([r::int])
        (SCM_SYSCALL r (symlink existing newpath))
        (when (< r 0)
          (Scm_SysError "symlink from %s to %s failed" newpath existing))))
-   (initcode "Scm_AddFeature(\"gauche.sys.symlink\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.symlink" NULL))
    )
 
  (when "defined(HAVE_READLINK)"
@@ -1163,7 +1191,7 @@
        (when (< n 0) (Scm_SysError "readlink failed on %s" path))
        (when (== n 1024) (Scm_Error "readlink result too long on %s" path))
        (result (Scm_MakeString buf n -1 SCM_STRING_COPYING))))
-   (initcode "Scm_AddFeature(\"gauche.sys.readlink\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.readlink" NULL))
    )
  )
 
@@ -1227,7 +1255,7 @@
    (define-cproc sys-select! (rfds wfds efds :optional (timeout #f))
      Scm_SysSelectX)
 
-   (initcode "Scm_AddFeature(\"gauche.sys.select\", NULL);")
+   (initcode (Scm_AddFeature "gauche.sys.select" NULL))
    ) ;; when defined(HAVE_SELECT)
  )
 
