@@ -1,7 +1,7 @@
 /*
  * list.c - List related functions
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -81,13 +81,11 @@ ScmObj Scm_Acons(ScmObj caar, ScmObj cdar, ScmObj cdr)
 
 ScmObj Scm_List(ScmObj elt, ...)
 {
-    va_list pvar;
-    ScmObj cdr;
-
     if (elt == NULL) return SCM_NIL;
 
+    va_list pvar;
     va_start(pvar, elt);
-    cdr = Scm_VaList(pvar);
+    ScmObj cdr = Scm_VaList(pvar);
     va_end(pvar);
     return Scm_Cons(elt, cdr);
 }
@@ -95,13 +93,11 @@ ScmObj Scm_List(ScmObj elt, ...)
 
 ScmObj Scm_Conses(ScmObj elt, ...)
 {
-    va_list pvar;
-    ScmObj cdr;
-
     if (elt == NULL) return SCM_NIL;
 
+    va_list pvar;
     va_start(pvar, elt);
-    cdr = Scm_VaCons(pvar);
+    ScmObj cdr = Scm_VaCons(pvar);
     va_end(pvar);
     if (cdr == NULL) return elt;
     else             return Scm_Cons(elt, cdr);
@@ -113,10 +109,10 @@ ScmObj Scm_VaList(va_list pvar)
     ScmObj start = SCM_NIL, cp = SCM_NIL, obj;
 
     for (obj = va_arg(pvar, ScmObj);
-	 obj != NULL;
-	 obj = va_arg(pvar, ScmObj))
+         obj != NULL;
+         obj = va_arg(pvar, ScmObj))
     {
-	if (SCM_NULLP(start)) {
+        if (SCM_NULLP(start)) {
             start = SCM_OBJ(SCM_NEW(ScmPair));
             SCM_SET_CAR(start, obj);
             SCM_SET_CDR(start, SCM_NIL);
@@ -128,7 +124,7 @@ ScmObj Scm_VaList(va_list pvar)
             SCM_SET_CAR(item, obj);
             SCM_SET_CDR(item, SCM_NIL);
             cp = item;
-	}
+        }
     }
     return start;
 }
@@ -149,8 +145,7 @@ ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail)
 {
     ScmObj h = SCM_NIL, t = SCM_NIL;
     if (elts) {
-        int i;
-        for (i=0; i<nelts; i++) SCM_APPEND1(h, t, *elts++);
+        for (int i=0; i<nelts; i++) SCM_APPEND1(h, t, *elts++);
     }
     if (!SCM_NULLP(tail)) SCM_APPEND(h, t, tail);
     return h;
@@ -158,9 +153,10 @@ ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail)
 
 ScmObj *Scm_ListToArray(ScmObj list, int *nelts, ScmObj *store, int alloc)
 {
-    ScmObj *array, lp;
-    int len = Scm_Length(list), i;
+    int len = Scm_Length(list);
     if (len < 0) Scm_Error("proper list required, but got %S", list);
+
+    ScmObj *array;
     if (store == NULL) {
         array = SCM_NEW_ARRAY(ScmObj, len);
     } else {
@@ -172,7 +168,8 @@ ScmObj *Scm_ListToArray(ScmObj list, int *nelts, ScmObj *store, int alloc)
             array = store;
         }
     }
-    for (i=0, lp=list; i<len; i++, lp=SCM_CDR(lp)) {
+    int i = 0;
+    for (ScmObj lp=list; i<len; i++, lp=SCM_CDR(lp)) {
         array[i] = SCM_CAR(lp);
     }
     *nelts = len;
@@ -181,7 +178,7 @@ ScmObj *Scm_ListToArray(ScmObj list, int *nelts, ScmObj *store, int alloc)
 
 /* cXr stuff */
 
-#define	CXR(cname, sname, body)                 \
+#define CXR(cname, sname, body)                 \
 ScmObj cname (ScmObj obj)                       \
 {                                               \
    ScmObj obj2 = obj;                           \
@@ -189,11 +186,11 @@ ScmObj cname (ScmObj obj)                       \
    return obj2;                                 \
 }
 
-#define	A                                                       \
+#define A                                                       \
    if (!SCM_PAIRP(obj2)) Scm_Error("bad object: %S", obj);      \
    obj2 = SCM_CAR(obj2);
 
-#define	D                                                       \
+#define D                                                       \
    if (!SCM_PAIRP(obj2)) Scm_Error("bad object: %S", obj);      \
    obj2 = SCM_CDR(obj2);
 
@@ -222,15 +219,15 @@ int Scm_Length(ScmObj obj)
         if (SCM_NULLP(obj)) break;
         if (!SCM_PAIRP(obj)) return SCM_LIST_DOTTED;
 
-	obj = SCM_CDR(obj);
-	len++;
+        obj = SCM_CDR(obj);
+        len++;
         if (SCM_NULLP(obj)) break;
         if (!SCM_PAIRP(obj)) return SCM_LIST_DOTTED;
 
-	obj = SCM_CDR(obj);
-	slow = SCM_CDR(slow);
-	if (obj == slow) return SCM_LIST_CIRCULAR;
-	len++;
+        obj = SCM_CDR(obj);
+        slow = SCM_CDR(slow);
+        if (obj == slow) return SCM_LIST_CIRCULAR;
+        len++;
     }
     return len;
 }
@@ -242,10 +239,9 @@ int Scm_Length(ScmObj obj)
 
 ScmObj Scm_CopyList(ScmObj list)
 {
-    ScmObj start = SCM_NIL, last = SCM_NIL;
-
     if (!SCM_PAIRP(list)) return list;
 
+    ScmObj start = SCM_NIL, last = SCM_NIL;
     SCM_FOR_EACH(list, list) {
         SCM_APPEND1(start, last, SCM_CAR(list));
     }
@@ -258,12 +254,12 @@ ScmObj Scm_CopyList(ScmObj list)
  *    Note that <len> is C-integer.
  */
 
-ScmObj Scm_MakeList(int len, ScmObj fill)
+ScmObj Scm_MakeList(ScmSmallInt len, ScmObj fill)
 {
-    ScmObj start = SCM_NIL, last = SCM_NIL;
     if (len < 0) {
         Scm_Error("make-list: negative length given: %d", len);
     }
+    ScmObj start = SCM_NIL, last = SCM_NIL;
     while (len--) {
         SCM_APPEND1(start, last, fill);
     }
@@ -280,7 +276,7 @@ ScmObj Scm_Append2X(ScmObj list, ScmObj obj)
 {
     ScmObj cp;
     SCM_FOR_EACH(cp, list) {
-	if (SCM_NULLP(SCM_CDR(cp))) {
+        if (SCM_NULLP(SCM_CDR(cp))) {
             SCM_SET_CDR(cp, obj);
             return list;
         }
@@ -295,12 +291,11 @@ ScmObj Scm_Append2X(ScmObj list, ScmObj obj)
 
 ScmObj Scm_Append2(ScmObj list, ScmObj obj)
 {
-    ScmObj start = SCM_NIL, last = SCM_NIL;
-
     if (!SCM_PAIRP(list)) return obj;
 
+    ScmObj start = SCM_NIL, last = SCM_NIL;
     SCM_FOR_EACH(list, list) {
-	SCM_APPEND1(start, last, SCM_CAR(list));
+        SCM_APPEND1(start, last, SCM_CAR(list));
     }
     SCM_SET_CDR(last, obj);
 
@@ -336,17 +331,15 @@ ScmObj Scm_Append(ScmObj args)
 
 ScmObj Scm_Reverse2(ScmObj list, ScmObj tail)
 {
-    ScmObj cp, result;
-    ScmPair *p;
-
     if (!SCM_PAIRP(list)) return tail;
 
-    p = SCM_NEW(ScmPair);
+    ScmPair *p = SCM_NEW(ScmPair);
     SCM_SET_CAR(p, SCM_NIL);
     SCM_SET_CDR(p, tail);
-    result = SCM_OBJ(p);
+    ScmObj result = SCM_OBJ(p);
+    ScmObj cp;
     SCM_FOR_EACH(cp, list) {
-	SCM_SET_CAR(result, SCM_CAR(cp));
+        SCM_SET_CAR(result, SCM_CAR(cp));
         p = SCM_NEW(ScmPair);
         SCM_SET_CAR(p, SCM_NIL);
         SCM_SET_CDR(p, result);
@@ -370,8 +363,8 @@ ScmObj Scm_Reverse(ScmObj list)
 
 ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail)
 {
+    if (!SCM_PAIRP(list)) return tail;
     ScmObj first, next, result = tail;
-    if (!SCM_PAIRP(list)) return result;
     for (first = list; SCM_PAIRP(first); first = next) {
         next = SCM_CDR(first);
         SCM_SET_CDR(first, result);
@@ -390,25 +383,24 @@ ScmObj Scm_ReverseX(ScmObj list)
  *    Note that i is C-INTEGER.  If i is out of bound, signal error.
  */
 
-ScmObj Scm_ListTail(ScmObj list, int i, ScmObj fallback)
+ScmObj Scm_ListTail(ScmObj list, ScmSmallInt i, ScmObj fallback)
 {
-    int cnt = i;
     if (i < 0) goto err;
+    ScmSmallInt cnt = i;
     while (cnt-- > 0) {
         if (!SCM_PAIRP(list)) goto err;
         list = SCM_CDR(list);
     }
     return list;
   err:
-    if (SCM_UNBOUNDP(fallback)) Scm_Error("argument out of range: %d", i);
+    if (SCM_UNBOUNDP(fallback)) Scm_Error("argument out of range: %ld", i);
     return fallback;
 }
 
-ScmObj Scm_ListRef(ScmObj list, int i, ScmObj fallback)
+ScmObj Scm_ListRef(ScmObj list, ScmSmallInt i, ScmObj fallback)
 {
-    int k;
     if (i < 0) goto err;
-    for (k=0; k<i; k++) {
+    for (ScmSmallInt k=0; k<i; k++) {
         if (!SCM_PAIRP(list)) goto err;
         list = SCM_CDR(list);
     }
@@ -416,7 +408,7 @@ ScmObj Scm_ListRef(ScmObj list, int i, ScmObj fallback)
     return SCM_CAR(list);
   err:
     if (SCM_UNBOUNDP(fallback)) {
-        Scm_Error("argument out of range: %d", i);
+        Scm_Error("argument out of range: %ld", i);
     }
     return fallback;
 }
@@ -428,12 +420,12 @@ ScmObj Scm_ListRef(ScmObj list, int i, ScmObj fallback)
 
 ScmObj Scm_LastPair(ScmObj l)
 {
-    ScmObj cp;
-
     if (!SCM_PAIRP(l)) Scm_Error("pair required: %S", l);
+
+    ScmObj cp;
     SCM_FOR_EACH(cp, l) {
-	ScmObj cdr = SCM_CDR(cp);
-	if (!SCM_PAIRP(cdr)) return cp;
+        ScmObj cdr = SCM_CDR(cp);
+        if (!SCM_PAIRP(cdr)) return cp;
     }
     return SCM_UNDEFINED;       /* NOTREACHED */
 }
@@ -470,9 +462,9 @@ ScmObj Scm_Member(ScmObj obj, ScmObj list, int cmpmode)
 /* delete. */
 ScmObj Scm_Delete(ScmObj obj, ScmObj list, int cmpmode)
 {
-    ScmObj start = SCM_NIL, last = SCM_NIL, cp, prev = list;
-
     if (SCM_NULLP(list)) return SCM_NIL;
+
+    ScmObj start = SCM_NIL, last = SCM_NIL, cp, prev = list;
     SCM_FOR_EACH(cp, list) {
         if (Scm_EqualM(obj, SCM_CAR(cp), cmpmode)) {
             for (; prev != cp; prev = SCM_CDR(prev))
@@ -512,8 +504,8 @@ ScmObj Scm_DeleteX(ScmObj obj, ScmObj list, int cmpmode)
 
 ScmObj Scm_Assq(ScmObj obj, ScmObj alist)
 {
-    ScmObj cp;
     if (!SCM_LISTP(alist)) Scm_Error("assq: list required, but got %S", alist);
+    ScmObj cp;
     SCM_FOR_EACH(cp,alist) {
         ScmObj entry = SCM_CAR(cp);
         if (!SCM_PAIRP(entry)) continue;
@@ -524,20 +516,20 @@ ScmObj Scm_Assq(ScmObj obj, ScmObj alist)
 
 ScmObj Scm_Assv(ScmObj obj, ScmObj alist)
 {
-    ScmObj cp;
     if (!SCM_LISTP(alist)) Scm_Error("assv: list required, but got %S", alist);
+    ScmObj cp;
     SCM_FOR_EACH(cp,alist) {
-	ScmObj entry = SCM_CAR(cp);
-	if (!SCM_PAIRP(entry)) continue;
-	if (Scm_EqvP(obj, SCM_CAR(entry))) return entry;
+        ScmObj entry = SCM_CAR(cp);
+        if (!SCM_PAIRP(entry)) continue;
+        if (Scm_EqvP(obj, SCM_CAR(entry))) return entry;
     }
     return SCM_FALSE;
 }
 
 ScmObj Scm_Assoc(ScmObj obj, ScmObj alist, int cmpmode)
 {
-    ScmObj cp;
     if (!SCM_LISTP(alist)) Scm_Error("assoc: list required, but got %S", alist);
+    ScmObj cp;
     SCM_FOR_EACH(cp,alist) {
         ScmObj entry = SCM_CAR(cp);
         if (!SCM_PAIRP(entry)) continue;
@@ -549,12 +541,12 @@ ScmObj Scm_Assoc(ScmObj obj, ScmObj alist, int cmpmode)
 /* Assoc-delete */
 ScmObj Scm_AssocDelete(ScmObj elt, ScmObj alist, int cmpmode)
 {
-    ScmObj start = SCM_NIL, last = SCM_NIL, cp, p, prev = alist;
     if (!SCM_LISTP(alist)) {
         Scm_Error("assoc-delete: list required, but got %S", alist);
     }
     if (SCM_NULLP(alist)) return SCM_NIL;
 
+    ScmObj start = SCM_NIL, last = SCM_NIL, cp, p, prev = alist;
     SCM_FOR_EACH(cp, alist) {
         p = SCM_CAR(cp);
         if (SCM_PAIRP(p)) {
@@ -573,10 +565,10 @@ ScmObj Scm_AssocDelete(ScmObj elt, ScmObj alist, int cmpmode)
 
 ScmObj Scm_AssocDeleteX(ScmObj elt, ScmObj alist, int cmpmode)
 {
-    ScmObj cp, prev = SCM_NIL;
     if (!SCM_LISTP(alist)) {
         Scm_Error("assoc-delete!: list required, but got %S", alist);
     }
+    ScmObj cp, prev = SCM_NIL;
     SCM_FOR_EACH(cp, alist) {
         ScmObj e = SCM_CAR(cp);
         if (SCM_PAIRP(e)) {
@@ -656,28 +648,30 @@ ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences)
 /* WILL RENAME IN 1.0 */
 ScmObj Scm_MonotonicMerge1(ScmObj sequences)
 {
-    ScmObj result = SCM_NIL, next, h;
-    ScmObj *seqv, *sp, *tp;
+    ScmObj result = SCM_NIL;
     int nseqs = Scm_Length(sequences);
-
     if (nseqs < 0) Scm_Error("bad list of sequences: %S", sequences);
-    seqv = SCM_NEW_ARRAY(ScmObj, nseqs);
-    for (sp=seqv; SCM_PAIRP(sequences); sp++, sequences=SCM_CDR(sequences)) {
+    ScmObj *seqv = SCM_NEW_ARRAY(ScmObj, nseqs);
+    for (ScmObj *sp=seqv;
+         SCM_PAIRP(sequences);
+         sp++, sequences=SCM_CDR(sequences)) {
         *sp = SCM_CAR(sequences);
     }
 
     for (;;) {
         /* have we consumed all the inputs? */
+        ScmObj *sp;
         for (sp=seqv; sp<seqv+nseqs; sp++) {
             if (!SCM_NULLP(*sp)) break;
         }
         if (sp == seqv+nseqs) return Scm_ReverseX(result);
 
         /* select candidate */
-        next = SCM_FALSE;
+        ScmObj next = SCM_FALSE;
         for (sp = seqv; sp < seqv+nseqs; sp++) {
             if (!SCM_PAIRP(*sp)) continue;
-            h = SCM_CAR(*sp);
+            ScmObj h = SCM_CAR(*sp);
+            ScmObj *tp;
             for (tp = seqv; tp < seqv+nseqs; tp++) {
                 if (!SCM_PAIRP(*tp)) continue;
                 if (!SCM_FALSEP(Scm_Memq(h, SCM_CDR(*tp)))) {
@@ -726,12 +720,11 @@ ScmObj Scm_ExtendedCons(ScmObj car, ScmObj cdr)
 
 ScmObj Scm_PairAttrGet(ScmPair *pair, ScmObj key, ScmObj fallback)
 {
-    ScmObj p;
     if (!SCM_EXTENDED_PAIR_P(pair)) {
         goto fallback;
     }
 
-    p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
+    ScmObj p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
     if (SCM_PAIRP(p)) return SCM_CDR(p);
   fallback:
     if (fallback == SCM_UNBOUND)
@@ -742,13 +735,12 @@ ScmObj Scm_PairAttrGet(ScmPair *pair, ScmObj key, ScmObj fallback)
 
 ScmObj Scm_PairAttrSet(ScmPair *pair, ScmObj key, ScmObj value)
 {
-    ScmObj p;
     if (!SCM_EXTENDED_PAIR_P(pair)) {
         Scm_Error("Cannot set pair attribute (%S) to non-extended pair: %S",
                   key, SCM_OBJ(pair));
     }
 
-    p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
+    ScmObj p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
     if (SCM_PAIRP(p)) SCM_SET_CDR(p, value);
     else SCM_EXTENDED_PAIR(pair)->attributes
         = Scm_Acons(key, value, SCM_EXTENDED_PAIR(pair)->attributes);
