@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,9 +33,10 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/arch.h"
 #include "gauche/port.h"
-#include "gauche/builtin-syms.h"
+#include "gauche/priv/builtin-syms.h"
+#include "gauche/priv/readerP.h"
+#include "gauche/priv/portP.h"
 
 #include <ctype.h>
 #include <fcntl.h>
@@ -76,10 +77,17 @@ static struct {
     ScmParameterLoc load_main_script;  /* yields #t during loading main script.
                                           see SCM_LOAD_MAIN_SCRIPT flag
                                           in load.h. */
-
+    
     /* Dynamic linking */
     ScmObj dso_suffixes;
     dlobj *dso_list;              /* List of dynamically loaded objects. */
+    ScmObj dso_prelinked;         /* List of 'prelinked' DSOs, that is, they
+                                     are already linked but pretened to be
+                                     DSOs.  dynamic-load won't do anything.
+                                     NB: We assume initfns of prelinked DSOs
+                                     are already called by the application,
+                                     but we may change this design in future.
+                                  */
     ScmInternalMutex dso_mutex;
 } ldinfo = { (ScmGloc*)&ldinfo, };  /* trick to put ldinfo in .data section */
 
@@ -153,11 +161,12 @@ void Scm_LoadPacketInit(ScmLoadPacket *p)
 struct load_info {
     ScmPort *port;
     ScmModule *prev_module;
-    ScmReadContext *ctx;
+    ScmReadContext *prev_ctx;
     ScmObj prev_port;
     ScmObj prev_history;
     ScmObj prev_next;
     ScmObj prev_main_script;
+    ScmObj prev_reader_lexical_mode;
     int    prev_situation;
 };
 
@@ -184,6 +193,8 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
     PARAM_SET(vm, load_history, p->prev_history);
     PARAM_SET(vm, load_next, p->prev_next);
     PARAM_SET(vm, load_main_script, p->prev_main_script);
+    Scm_SetReaderLexicalMode(p->prev_reader_lexical_mode);
+    Scm_SetCurrentReadContext(p->prev_ctx);
     vm->evalSituation = p->prev_situation;
     return SCM_UNDEFINED;
 }
@@ -192,7 +203,7 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
 static ScmObj load_cc(ScmObj result, void **data)
 {
     struct load_info *p = (struct load_info*)(data[0]);
-    ScmObj expr = Scm_ReadWithContext(SCM_OBJ(p->port), p->ctx);
+    ScmObj expr = Scm_Read(SCM_OBJ(p->port));
 
     if (!SCM_EOFP(expr)) {
         Scm_VMPushCC(load_cc, data, 1);
@@ -210,8 +221,6 @@ static ScmObj load_body(ScmObj *args, int nargs, void *data)
 ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
                           ScmObj env, int flags)
 {
-    struct load_info *p;
-    ScmObj port_info;
     ScmVM *vm = Scm_VM();
     ScmModule *module = vm->module;
 
@@ -228,23 +237,26 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
         Scm_Error("bad load environment (must be a module or #f): %S", env);
     }
 
-    p = SCM_NEW(struct load_info);
+    struct load_info *p = SCM_NEW(struct load_info);
     p->port = port;
     p->prev_module    = vm->module;
     p->prev_port      = PARAM_REF(vm, load_port);
     p->prev_history   = PARAM_REF(vm, load_history);
     p->prev_next      = PARAM_REF(vm, load_next);
     p->prev_main_script = PARAM_REF(vm, load_main_script);
+    p->prev_reader_lexical_mode = Scm_ReaderLexicalMode();
     p->prev_situation = vm->evalSituation;
 
-    p->ctx = Scm_MakeReadContext(NULL);
-    p->ctx->flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
+    ScmReadContext *newctx = Scm_MakeReadContext(NULL);
+    newctx->flags |= RCTX_LITERAL_IMMUTABLE | RCTX_SOURCE_INFO;
+    p->prev_ctx = Scm_SetCurrentReadContext(newctx);
 
     PARAM_SET(vm, load_next, next_paths);
     PARAM_SET(vm, load_port, SCM_OBJ(port));
     PARAM_SET(vm, load_main_script, SCM_MAKE_BOOL(flags&SCM_LOAD_MAIN_SCRIPT));
     vm->module = module;
     vm->evalSituation = SCM_VM_LOADING;
+    ScmObj port_info;
     if (SCM_PORTP(p->prev_port)) {
         port_info = SCM_LIST2(p->prev_port,
                               Scm_MakeInteger(Scm_PortLine(SCM_PORT(p->prev_port))));
@@ -260,9 +272,7 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
 int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *packet)
 {
     static ScmObj load_from_port = SCM_UNDEFINED;
-    ScmEvalPacket eresult;
     ScmObj args = SCM_NIL;
-    int r;
     SCM_BIND_PROC(load_from_port, "load-from-port", Scm_GaucheModule());
     load_packet_prepare(packet);
 
@@ -277,7 +287,8 @@ int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *packet)
         if (packet) packet->loaded = TRUE;
         return 0;
     } else {
-        r = Scm_Apply(load_from_port, args, &eresult);
+        ScmEvalPacket eresult;
+        int r = Scm_Apply(load_from_port, args, &eresult);
         if (packet) {
             packet->exception = eresult.exception;
             packet->loaded = (r >= 0);
@@ -353,7 +364,6 @@ int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
     static ScmObj load_proc = SCM_UNDEFINED;
     ScmObj f = SCM_MAKE_STR_COPYING(cpath);
     ScmObj opts = SCM_NIL;
-    ScmEvalPacket eresult;
     SCM_BIND_PROC(load_proc, "load", Scm_SchemeModule());
 
     if (flags&SCM_LOAD_QUIET_NOFILE) {
@@ -374,6 +384,7 @@ int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
         }
         return 0;
     } else {
+        ScmEvalPacket eresult;
         int r = Scm_Apply(load_proc, Scm_Cons(f, opts), &eresult);
         if (packet) {
             packet->exception = eresult.exception;
@@ -389,39 +400,37 @@ int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
 
 ScmObj Scm_GetLoadPath(void)
 {
-    ScmObj paths;
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    paths = Scm_CopyList(ldinfo.load_path_rec->value);
+    ScmObj paths = Scm_CopyList(ldinfo.load_path_rec->value);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
     return paths;
 }
 
 ScmObj Scm_GetDynLoadPath(void)
 {
-    ScmObj paths;
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    paths = Scm_CopyList(ldinfo.dynload_path_rec->value);
+    ScmObj paths = Scm_CopyList(ldinfo.dynload_path_rec->value);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
     return paths;
 }
 
 static ScmObj break_env_paths(const char *envname)
 {
-    const char *e = getenv(envname);
+    const char *e = Scm_GetEnv(envname);
 #ifndef GAUCHE_WINDOWS
     char delim = ':';
 #else  /*GAUCHE_WINDOWS*/
     char delim = ';';
 #endif /*GAUCHE_WINDOWS*/
 
-    if (e == NULL) {
-	return SCM_NIL;
+    if (e == NULL || strlen(e) == 0) {
+        return SCM_NIL;
     } else if (Scm_IsSugid()) {
         /* don't trust env when setugid'd */
         return SCM_NIL;
     } else {
-	return Scm_StringSplitByChar(SCM_STRING(SCM_MAKE_STR_COPYING(e)),
-				     delim);
+        return Scm_StringSplitByChar(SCM_STRING(SCM_MAKE_STR_COPYING(e)),
+                                     delim);
     }
 }
 
@@ -448,12 +457,10 @@ static ScmObj add_list_item(ScmObj orig, ScmObj item, int afterp)
 ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
 {
     ScmObj spath = SCM_MAKE_STR_COPYING(cpath);
-    ScmObj dpath;
-    ScmObj r;
     struct stat statbuf;
 
     /* check dynload path */
-    dpath = Scm_StringAppendC(SCM_STRING(spath), "/", 1, 1);
+    ScmObj dpath = Scm_StringAppendC(SCM_STRING(spath), "/", 1, 1);
     dpath = Scm_StringAppendC(SCM_STRING(dpath), Scm_HostArchitecture(),-1,-1);
     if (stat(Scm_GetStringConst(SCM_STRING(dpath)), &statbuf) < 0
         || !S_ISDIR(statbuf.st_mode)) {
@@ -468,7 +475,7 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
     ADD_LIST_ITEM(ldinfo.load_path_rec->value, spath, afterp);
     ADD_LIST_ITEM(ldinfo.dynload_path_rec->value, dpath, afterp);
-    r = ldinfo.load_path_rec->value;
+    ScmObj r = ldinfo.load_path_rec->value;
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
 
     return r;
@@ -582,6 +589,29 @@ struct dlobj_rec {
     ScmInternalCond  cv;
 };
 
+#if 0 /* for debug */
+static void dump_dlobj(dlobj *dlo)
+{
+    printf("{\n  dlobj \"%s\"%s\n",
+           dlo->path, dlo->loaded? " (loaded)" : " (not loaded)");
+    printf("  initfns:\n");
+    for (dlobj_initfn *ifn = dlo->initfns; ifn; ifn = ifn->next) {
+        printf("    name=\"%s\", fn=%p %s}\n",
+               ifn->name, ifn->fn,
+               ifn->initialized? "(initialized) ":"");
+    }
+    printf("}\n");
+}
+
+static void dump_dlobjs(void)
+{
+    printf("Registerd dlobjs:\n");
+    for (dlobj *dlo = ldinfo.dso_list; dlo; dlo = dlo->next) {
+        dump_dlobj(dlo);
+    }
+}
+#endif
+
 /* NB: we rely on dlcompat library for dlopen instead of using dl_darwin.c
    for now; Boehm GC requires dlopen when compiled with pthread, so there's
    not much point to avoid dlopen here. */
@@ -644,18 +674,16 @@ static void unlock_dlobj(dlobj *dlo)
 
 static const char *derive_dynload_initfn(const char *filename)
 {
-    const char *head, *tail, *s;
-    char *name, *d;
-
-    head = strrchr(filename, '/');
+    const char *head = strrchr(filename, '/');
     if (head == NULL) head = filename;
     else head++;
-    tail = strchr(head, '.');
+    const char *tail = strchr(head, '.');
     if (tail == NULL) tail = filename + strlen(filename);
 
-    name = SCM_NEW_ATOMIC2(char *, sizeof(DYNLOAD_PREFIX) + tail - head);
+    char *name = SCM_NEW_ATOMIC2(char *, sizeof(DYNLOAD_PREFIX) + tail - head);
     strcpy(name, DYNLOAD_PREFIX);
-    for (s = head, d = name + sizeof(DYNLOAD_PREFIX) - 1; s < tail; s++, d++) {
+    char *d = name + sizeof(DYNLOAD_PREFIX) - 1;
+    for (const char *s = head; s < tail; s++, d++) {
         if (isalnum(*s)) *d = tolower(*s);
         else *d = '_';
     }
@@ -683,12 +711,11 @@ static dlobj_initfn *find_initfn(dlobj *dlo, const char *name)
 /* From the given DSO name, find out the path of the actual DSO */
 static const char *find_dso_path(ScmString *dsoname)
 {
-    ScmObj spath;
     static ScmObj find_file = SCM_UNDEFINED;
     SCM_BIND_PROC(find_file, "find-load-file", Scm_GaucheInternalModule());
 
-    spath = Scm_ApplyRec5(find_file, SCM_OBJ(dsoname), Scm_GetDynLoadPath(),
-                          ldinfo.dso_suffixes, SCM_FALSE, SCM_FALSE);
+    ScmObj spath = Scm_ApplyRec5(find_file, SCM_OBJ(dsoname), Scm_GetDynLoadPath(),
+                                 ldinfo.dso_suffixes, SCM_FALSE, SCM_FALSE);
     if (SCM_FALSEP(spath)) {
         Scm_Error("can't find dlopen-able module %S", dsoname);
     }
@@ -771,15 +798,65 @@ static void call_initfn(dlobj *dlo, const char *name)
     ifn->initialized = TRUE;
 }
 
-/* Dynamically load the specified object by FILENAME.
-   FILENAME must not contain the system's suffix (.so, for example).
+/* Experimental: Prelink feature---we allow the extension module to be
+   statically linked, and (dynamic-load DSONAME) merely calls initfn.
+   The application needs to call Scm_RegisterPrelinked to tell the system
+   which DSO is statically linked.  We pretend that the named DSO is
+   already loaded from a pseudo pathname "@/DSONAME" (e.g. for
+   "gauche--collection", we use "@/gauche--collection".) */
+
+static const char *pseudo_pathname_for_prelinked(ScmString *dsoname)
+{
+    ScmDString ds;
+    Scm_DStringInit(&ds);
+    Scm_DStringPutz(&ds, "@/", 2);
+    Scm_DStringAdd(&ds, dsoname);
+    return Scm_DStringGetz(&ds);
+}
+
+/* Register DSONAME as prelinked.  DSONAME shoudn't have system's suffix.
+   INITFNS is an array of function pointers, NULL terminated.
+   INITFN_NAMES should have prefixed with '_', for call_initfn() searches
+   names with '_' first. */
+void Scm_RegisterPrelinked(ScmString *dsoname,
+                           const char *initfn_names[],
+                           ScmDynLoadInitFn initfns[])
+{
+    const char *path = pseudo_pathname_for_prelinked(dsoname);
+    dlobj *dlo = find_dlobj(path);
+    dlo->loaded = TRUE;
+
+    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
+    for (int i=0; initfns[i] && initfn_names[i]; i++) {
+        dlobj_initfn *ifn = find_initfn(dlo, initfn_names[i]);
+        SCM_ASSERT(ifn->fn == NULL);
+        ifn->fn = initfns[i];
+    }
+    ldinfo.dso_prelinked = Scm_Cons(SCM_OBJ(dsoname), ldinfo.dso_prelinked);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
+}
+
+static const char *find_prelinked(ScmString *dsoname)
+{
+    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
+    /* in general it is dangerous to invoke equal?-comparison during lock,
+       but in this case we know they're string comparison and won't raise
+       an error. */
+    ScmObj z = Scm_Member(SCM_OBJ(dsoname), ldinfo.dso_prelinked, SCM_CMP_EQUAL);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
+    return SCM_FALSEP(z) ? NULL : pseudo_pathname_for_prelinked(dsoname);
+}
+
+/* Dynamically load the specified object by DSONAME.
+   DSONAME must not contain the system's suffix (.so, for example).
    The same name of DSO can be only loaded once.
    A DSO may contain multiple initialization functions (initfns), in
    which case each initfn is called at most once.
 */
-ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, u_long flags/*reserved*/)
+ScmObj Scm_DynLoad(ScmString *dsoname, ScmObj initfn, u_long flags/*reserved*/)
 {
-    const char *dsopath = find_dso_path(filename);
+    const char *dsopath = find_prelinked(dsoname);
+    if (!dsopath) dsopath = find_dso_path(dsoname);
     const char *initname = get_initfn_name(initfn, dsopath);
     dlobj *dlo = find_dlobj(dsopath);
 
@@ -810,6 +887,8 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, u_long flags/*reserved*/)
    For now, I allow only a string. */
 /* Note that require and provide is recognized at compile time. */
 
+static int do_require(ScmObj, int, ScmModule *, ScmLoadPacket *);
+
 /* [Preventing Race Condition]
  *
  *   Besides the list of provided features (ldinfo.provided), the
@@ -860,12 +939,35 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, u_long flags/*reserved*/)
  *   we know that the first matching entry is the current one.)
  */
 
+/* NB: It has never been explicit, but 'require' and 'extend' are expected to
+   work as if we load the module into #<module gauche>.  Those forms only loads
+   the file once, so it doesn't make much sense to allow it to load into
+   different modules for each time, since you never know whether the file
+   is loaded at this time or it has already been loaded.  With the same
+   reason, it doesn't make much sense to use the current module.
+
+   Until 0.9.4 we didn't actually force 'require' to use #<module gauche>
+   as the base module.  Most of the time, 'require' is called in a module
+   that inherits gauche, so the problem didn't appear.  However, if one
+   requires a file from a module that doesn't inherit gauche, most likely
+   accidentally, he sees confusing errors---because "define-module" form
+   isn't recognized!
+
+   As of 0.9.4 we fix the base module to #<module gauche> when require is
+   called.   Note that autoload needs different requirements, so we have
+   a subroutine do_require that takes the desired base module.
+ */
 int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
+{
+    return do_require(feature, flags, Scm_GaucheModule(), packet);
+}
+
+int do_require(ScmObj feature, int flags, ScmModule *base_mod,
+               ScmLoadPacket *packet)
 {
     ScmVM *vm = Scm_VM();
-    ScmObj provided, providing, p, q;
-    int loop = FALSE, r;
-    ScmLoadPacket xresult;
+    ScmObj provided;
+    int loop = FALSE;
 
     load_packet_prepare(packet);
     if (!SCM_STRINGP(feature)) {
@@ -882,16 +984,16 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
     for (;;) {
         provided = Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL);
         if (!SCM_FALSEP(provided)) break;
-        providing = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
+        ScmObj providing = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
         if (SCM_FALSEP(providing)) break;
 
         /* Checks for dependencies */
-        p = providing;
+        ScmObj p = providing;
         SCM_ASSERT(SCM_PAIRP(p) && SCM_PAIRP(SCM_CDR(p)));
         if (SCM_CADR(p) == SCM_OBJ(vm)) { loop = TRUE; break; }
 
         for (;;) {
-            q = Scm_Assq(SCM_CDR(p), ldinfo.waiting);
+            ScmObj q = Scm_Assq(SCM_CDR(p), ldinfo.waiting);
             if (SCM_FALSEP(q)) break;
             SCM_ASSERT(SCM_PAIRP(q));
             p = Scm_Assoc(SCM_CDR(q), ldinfo.providing, SCM_CMP_EQUAL);
@@ -919,7 +1021,13 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
     }
 
     if (!SCM_FALSEP(provided)) return 0; /* no work to do */
-    r = Scm_Load(Scm_GetStringConst(SCM_STRING(feature)), 0, &xresult);
+    /* Make sure to load the file into base_mod.   We don't need UNWIND_PROTECT
+       here, since errors are caught in Scm_Load. */
+    ScmLoadPacket xresult;
+    ScmModule *prev_mod = vm->module;
+    vm->module = base_mod;
+    int r = Scm_Load(Scm_GetStringConst(SCM_STRING(feature)), 0, &xresult);
+    vm->module = prev_mod;
     if (packet) packet->exception = xresult.exception;
 
     if (r < 0) {
@@ -934,7 +1042,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
 
     /* Success */
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
-    p = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
+    ScmObj p = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
     ldinfo.providing = Scm_AssocDeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
     /* `Autoprovide' feature */
     if (SCM_NULLP(SCM_CDDR(p))
@@ -949,7 +1057,6 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
 
 ScmObj Scm_Provide(ScmObj feature)
 {
-    ScmObj cp;
     ScmVM *self = Scm_VM();
 
     if (!SCM_STRINGP(feature)&&!SCM_FALSEP(feature)) {
@@ -960,6 +1067,7 @@ ScmObj Scm_Provide(ScmObj feature)
         && SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL))) {
         ldinfo.provided = Scm_Cons(feature, ldinfo.provided);
     }
+    ScmObj cp;
     SCM_FOR_EACH(cp, ldinfo.providing) {
         if (SCM_CADR(SCM_CAR(cp)) == SCM_OBJ(self)) {
             SCM_SET_CDR(SCM_CDR(SCM_CAR(cp)), SCM_LIST1(feature));
@@ -973,9 +1081,8 @@ ScmObj Scm_Provide(ScmObj feature)
 
 int Scm_ProvidedP(ScmObj feature)
 {
-    int r;
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
-    r = !SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL));
+    int r = !SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL));
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.prov_mutex);
     return r;
 }
@@ -1018,7 +1125,6 @@ void Scm_DefineAutoload(ScmModule *where,
 {
     ScmString *path = NULL;
     ScmSymbol *import_from = NULL;
-    ScmObj ep;
 
     if (SCM_STRINGP(file_or_module)) {
         path = SCM_STRING(file_or_module);
@@ -1029,6 +1135,7 @@ void Scm_DefineAutoload(ScmModule *where,
         Scm_Error("autoload: string or symbol required, but got %S",
                   file_or_module);
     }
+    ScmObj ep;
     SCM_FOR_EACH(ep, list) {
         ScmObj entry = SCM_CAR(ep);
         if (SCM_SYMBOLP(entry)) {
@@ -1053,7 +1160,6 @@ void Scm_DefineAutoload(ScmModule *where,
 ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
 {
     int circular = FALSE;
-    ScmModule *prev_module;
     ScmVM *vm = Scm_VM();
 
     /* shortcut in case if somebody else already did the job. */
@@ -1111,24 +1217,21 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
                   adata->module, adata->name);
     }
 
-    prev_module = vm->module;
     SCM_UNWIND_PROTECT {
-        vm->module = adata->module;
-        Scm_Require(SCM_OBJ(adata->path), SCM_LOAD_PROPAGATE_ERROR, NULL);
-        vm->module = prev_module;
+        do_require(SCM_OBJ(adata->path), SCM_LOAD_PROPAGATE_ERROR,
+                   adata->module, NULL);
 
         if (adata->import_from) {
             /* autoloaded file defines import_from module.  we need to
                import the binding individually. */
             ScmModule *m = Scm_FindModule(adata->import_from,
                                           SCM_FIND_MODULE_QUIET);
-            ScmGloc *f, *g;
             if (m == NULL) {
                 Scm_Error("Trying to autoload module %S from file %S, but the file doesn't define such a module",
                           adata->import_from, adata->path);
             }
-            f = Scm_FindBinding(SCM_MODULE(m), adata->name, 0);
-            g = Scm_FindBinding(adata->module, adata->name, 0);
+            ScmGloc *f = Scm_FindBinding(SCM_MODULE(m), adata->name, 0);
+            ScmGloc *g = Scm_FindBinding(adata->module, adata->name, 0);
             SCM_ASSERT(f != NULL);
             SCM_ASSERT(g != NULL);
             adata->value = SCM_GLOC_GET(f);
@@ -1150,7 +1253,6 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
         }
     } SCM_WHEN_ERROR {
         adata->locker = NULL;
-        vm->module = prev_module;
         SCM_INTERNAL_COND_BROADCAST(adata->cv);
         SCM_NEXT_HANDLER;
     } SCM_END_PROTECT;
@@ -1198,19 +1300,19 @@ void Scm__InitLoad(void)
 {
     ScmModule *m = Scm_SchemeModule();
     ScmVM *vm = Scm_VM();
-    ScmObj init_load_path, init_dynload_path, init_load_suffixes, t;
+    ScmObj t;
 
-    init_load_path = t = SCM_NIL;
+    ScmObj init_load_path = t = SCM_NIL;
     SCM_APPEND(init_load_path, t, break_env_paths("GAUCHE_LOAD_PATH"));
     SCM_APPEND1(init_load_path, t, Scm_SiteLibraryDirectory());
     SCM_APPEND1(init_load_path, t, Scm_LibraryDirectory());
 
-    init_dynload_path = t = SCM_NIL;
+    ScmObj init_dynload_path = t = SCM_NIL;
     SCM_APPEND(init_dynload_path, t, break_env_paths("GAUCHE_DYNLOAD_PATH"));
     SCM_APPEND1(init_dynload_path, t, Scm_SiteArchitectureDirectory());
     SCM_APPEND1(init_dynload_path, t, Scm_ArchitectureDirectory());
 
-    init_load_suffixes = t = SCM_NIL;
+    ScmObj init_load_suffixes = t = SCM_NIL;
     SCM_APPEND1(init_load_suffixes, t, SCM_MAKE_STR(".sci"));
     SCM_APPEND1(init_load_suffixes, t, SCM_MAKE_STR(".scm"));
 
@@ -1246,6 +1348,7 @@ void Scm__InitLoad(void)
     ldinfo.dso_suffixes = SCM_LIST2(SCM_MAKE_STR(".la"),
                                     SCM_MAKE_STR("." SHLIB_SO_SUFFIX));
     ldinfo.dso_list = NULL;
+    ldinfo.dso_prelinked = SCM_NIL;
 
 #define PARAM_INIT(name, val) Scm_InitParameterLoc(vm, &ldinfo.name, val)
     PARAM_INIT(load_history, SCM_NIL);
