@@ -1,7 +1,7 @@
 /*
  * module.c - module implementation
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,18 +33,24 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/builtin-syms.h"
+#include "gauche/priv/builtin-syms.h"
 #include "gauche/class.h"
 
 /*
  * Modules
  *
- *  A module maps symbols to global locations.
+ *  A module maps symbols to global locations (GLOCs).
  *  The mapping is resolved at the compile time.
  *  Scheme's current-module is therefore a syntax, instead of
  *  a procedure, to capture compile-time information.
  *
- *  Modules are registered to global hash table using their names
+ *  Each module has two hashtables; the 'internal' table keeps all the
+ *  bindings in the module, while the 'external' table keeps only the
+ *  bindings that are exported.  In most cases, the latter is a subset
+ *  of the former.  If a binding is renamed on export, however,
+ *  two tables map different symbols on the same GLOC.
+ *
+ *  Modules are registered to a global hash table using their names
  *  as keys, so that the module is retrieved by its name.  The exception
  *  is "anonymous modules", which have #f as the name field
  *  and not registered in the global table.   Anonymous modules are especially
@@ -104,6 +110,9 @@ DEFINE_STATIC_MODULE(gaucheModule);   /* #<module gauche> */
 DEFINE_STATIC_MODULE(internalModule); /* #<module gauche.internal> */
 DEFINE_STATIC_MODULE(gfModule);       /* #<module gauche.gf> */
 DEFINE_STATIC_MODULE(userModule);     /* #<module user> */
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+DEFINE_STATIC_MODULE(keywordModule);  /* #<module keyword> */
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
 
 static ScmObj defaultParents = SCM_NIL; /* will be initialized */
 static ScmObj defaultMpl =     SCM_NIL; /* will be initialized */
@@ -115,19 +124,19 @@ static ScmObj defaultMpl =     SCM_NIL; /* will be initialized */
 static void init_module(ScmModule *m, ScmObj name)
 {
     m->name = name;
-    m->imported = m->exported = m->depended = SCM_NIL;
+    m->imported = m->depended = SCM_NIL;
     m->exportAll = FALSE;
     m->parents = defaultParents;
     m->mpl = Scm_Cons(SCM_OBJ(m), defaultMpl);
-    m->table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
+    m->internal = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
+    m->external = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     m->origin = m->prefix = SCM_FALSE;
 }
 
 /* Internal */
 static ScmObj make_module(ScmObj name)
 {
-    ScmModule *m;
-    m = SCM_NEW(ScmModule);
+    ScmModule *m = SCM_NEW(ScmModule);
     SCM_SET_CLASS(m, SCM_CLASS_MODULE);
     init_module(m, name);
     return SCM_OBJ(m);
@@ -136,9 +145,8 @@ static ScmObj make_module(ScmObj name)
 /* Internal.  Lookup module with name N from the table. */
 static ScmModule *lookup_module(ScmSymbol *name)
 {
-    ScmObj v;
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-    v = Scm_HashTableRef(modules.table, SCM_OBJ(name), SCM_UNBOUND);
+    ScmObj v = Scm_HashTableRef(modules.table, SCM_OBJ(name), SCM_UNBOUND);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     if (SCM_UNBOUNDP(v)) return NULL;
     else return SCM_MODULE(v);
@@ -147,11 +155,10 @@ static ScmModule *lookup_module(ScmSymbol *name)
 /* Internal.  Lookup module, and if there's none, create one. */
 static ScmModule *lookup_module_create(ScmSymbol *name, int *created)
 {
-    ScmDictEntry *e;
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-    e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(modules.table),
-                           (intptr_t)name,
-                           SCM_DICT_CREATE);
+    ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(modules.table),
+                                         (intptr_t)name,
+                                         SCM_DICT_CREATE);
     if (e->value == 0) {
         (void)SCM_DICT_SET_VALUE(e, make_module(SCM_OBJ(name)));
         *created = TRUE;
@@ -164,20 +171,17 @@ static ScmModule *lookup_module_create(ScmSymbol *name, int *created)
 
 ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists)
 {
-    ScmObj r;
     if (name == NULL) {
-        r = make_module(SCM_FALSE);
-    } else {
-        int created;
-        r = SCM_OBJ(lookup_module_create(name, &created));
-        if (!created) {
-            if (error_if_exists) {
-                Scm_Error("couldn't create module '%S': named module already exists",
-                          SCM_OBJ(name));
-            } else {
-                r = SCM_FALSE;
-            }
+        return make_module(SCM_FALSE);
+    }
+    int created;
+    ScmObj r = SCM_OBJ(lookup_module_create(name, &created));
+    if (!created) {
+        if (error_if_exists) {
+            Scm_Error("couldn't create module '%S': named module already exists",
+                      SCM_OBJ(name));
         }
+        return SCM_FALSE;
     }
     return r;
 }
@@ -210,6 +214,7 @@ typedef struct {
     ScmObj more_searched;
 } module_cache;
 
+
 static inline void init_module_cache(module_cache *c)
 {
     c->num_searched = 0;
@@ -218,8 +223,7 @@ static inline void init_module_cache(module_cache *c)
 
 static inline int module_visited_p(module_cache *c, ScmObj m)
 {
-    int i;
-    for (i=0; i<c->num_searched; i++) {
+    for (int i=0; i<c->num_searched; i++) {
         if (SCM_EQ(m, c->searched[i])) return TRUE;
     }
     if (!SCM_NULLP(c->more_searched)) {
@@ -237,79 +241,107 @@ static inline void module_add_visited(module_cache *c, ScmObj m)
     }
 }
 
-ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol, int flags)
+/* The main logic of global binding search.  We factored this out since
+   we need recursive searching in case of phantom binding (see gloc.h
+   about phantom bindings).  The flags stay_in_module and external_only
+   corresponds to the flags passed to Scm_FindBinding.  The exclude_self
+   flag is only used in recursive search. */
+static ScmGloc *search_binding(ScmModule *module, ScmSymbol *symbol,
+                               int stay_in_module, int external_only,
+                               int exclude_self)
 {
-    ScmModule *m = module;
-    ScmObj v, p, mp;
-    ScmGloc *gloc = NULL;
-    int stay_in_module = flags&SCM_BINDING_STAY_IN_MODULE;
     module_cache searched;
-
     init_module_cache(&searched);
-    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
 
-    /* first, search from the specified module.
-       NB: we directly check gloc->value instead of calling
-       SCM_GLOC_GET, since this check is merely to eliminate
-       the GLOC inserted by export. */
-    v = Scm_HashTableRef(m->table, SCM_OBJ(symbol), SCM_FALSE);
-    if (SCM_GLOCP(v)) {
-        gloc = SCM_GLOC(v);
-        if (!SCM_GLOC_PHANTOM_BINDING_P(gloc)) goto out;
+    /* First, search from the specified module.  In this phase, we just ignore
+       phantom bindings, for we'll search imported bindings later anyway. */
+    if (!exclude_self) {
+        ScmObj v = Scm_HashTableRef(
+            external_only? module->external : module->internal,
+            SCM_OBJ(symbol), SCM_FALSE);
+        if (SCM_GLOCP(v)) {
+            if (SCM_GLOC_PHANTOM_BINDING_P(SCM_GLOC(v))) {
+                /* If we're here, the symbol is external to MODULE but
+                   the real GLOC is somewhere in imported or inherited
+                   modules.  We turn off external_only switch so that
+                   when we search inherited modules we look into it's
+                   internal bindings. */
+                external_only = FALSE;
+            } else {
+                return SCM_GLOC(v);
+            }
+        }
+        if (stay_in_module) return NULL;
+        module_add_visited(&searched, SCM_OBJ(module));
     }
-    if (stay_in_module) goto out;
 
+    ScmObj p, mp;
     /* Next, search from imported modules */
     SCM_FOR_EACH(p, module->imported) {
         ScmObj elt = SCM_CAR(p);
-        ScmModule *mod = NULL;
         ScmObj sym = SCM_OBJ(symbol);
 
         SCM_ASSERT(SCM_MODULEP(elt));
-        mod = SCM_MODULE(elt);
-
-        SCM_FOR_EACH(mp, mod->mpl) {
+        SCM_FOR_EACH(mp, SCM_MODULE(elt)->mpl) {
             ScmGloc *g;
 
             SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
 
             if (module_visited_p(&searched, SCM_CAR(mp))) continue;
-            m = SCM_MODULE(SCM_CAR(mp));
+            ScmModule *m = SCM_MODULE(SCM_CAR(mp));
             if (SCM_SYMBOLP(m->prefix)) {
                 sym = Scm_SymbolSansPrefix(SCM_SYMBOL(sym),
                                            SCM_SYMBOL(m->prefix));
-                if (!SCM_SYMBOLP(sym)) goto skip;
+                if (!SCM_SYMBOLP(sym)) break;
             }
 
-            v = Scm_HashTableRef(m->table, SCM_OBJ(sym), SCM_FALSE);
-            /* see above comment about the check of gloc->value */
+            ScmObj v = Scm_HashTableRef(m->external, SCM_OBJ(sym), SCM_FALSE);
             if (SCM_GLOCP(v)) {
                 g = SCM_GLOC(v);
                 if (g->hidden) break;
-                if (g->exported && !SCM_UNBOUNDP(g->value)) {
-                    gloc = g;
-                    goto out;
+                if (SCM_GLOC_PHANTOM_BINDING_P(g)) {
+                    g = search_binding(m, symbol, FALSE, FALSE, TRUE);
+                    if (g) return g;
+                } else {
+                    return g;
                 }
             }
             module_add_visited(&searched, SCM_OBJ(m));
         }
-    skip:;
     }
 
     /* Then, search from parent modules */
     SCM_ASSERT(SCM_PAIRP(module->mpl));
     SCM_FOR_EACH(mp, SCM_CDR(module->mpl)) {
         SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
-        m = SCM_MODULE(SCM_CAR(mp));
+        ScmModule *m = SCM_MODULE(SCM_CAR(mp));
+
         if (SCM_SYMBOLP(m->prefix)) {
             ScmObj sym = Scm_SymbolSansPrefix(symbol, SCM_SYMBOL(m->prefix));
-            if (!SCM_SYMBOLP(sym)) goto out;
+            if (!SCM_SYMBOLP(sym)) return NULL;
             symbol = SCM_SYMBOL(sym);
         }
-        v = Scm_HashTableRef(m->table, SCM_OBJ(symbol), SCM_FALSE);
-        if (SCM_GLOCP(v)) { gloc = SCM_GLOC(v); goto out; }
+        ScmObj v = Scm_HashTableRef(external_only?m->external:m->internal,
+                                    SCM_OBJ(symbol), SCM_FALSE);
+        if (SCM_GLOCP(v)) {
+            if (SCM_GLOC_PHANTOM_BINDING_P(SCM_GLOC(v))) {
+                external_only = FALSE; /* See above comment */
+            } else {
+                return SCM_GLOC(v);
+            }
+        }
     }
- out:
+    return NULL;
+}
+
+ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol, int flags)
+{
+    int stay_in_module = flags&SCM_BINDING_STAY_IN_MODULE;
+    int external_only = flags&SCM_BINDING_EXTERNAL;
+    ScmGloc *gloc = NULL;
+
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
+    gloc = search_binding(module, symbol, stay_in_module, external_only, FALSE);
     SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
     return gloc;
 }
@@ -319,10 +351,9 @@ ScmObj Scm_GlobalVariableRef(ScmModule *module,
                              int flags)
 {
     ScmGloc *g = Scm_FindBinding(module, symbol, flags);
-    ScmObj val;
 
     if (g == NULL) return SCM_UNBOUND;
-    val = SCM_GLOC_GET(g);
+    ScmObj val = SCM_GLOC_GET(g);
     if (SCM_AUTOLOADP(val)) {
         /* NB: Scm_ResolveAutoload may return SCM_UNBOUND */
         val = Scm_ResolveAutoload(SCM_AUTOLOAD(val), 0);
@@ -337,7 +368,6 @@ ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
                          ScmObj value, int flags)
 {
     ScmGloc *g;
-    ScmObj v;
     ScmObj oldval = SCM_UNDEFINED;
     int prev_kind = 0;
     int kind = ((flags&SCM_BINDING_CONST)
@@ -347,7 +377,7 @@ ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
                    : 0));
 
     SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
-    v = Scm_HashTableRef(module->table, SCM_OBJ(symbol), SCM_FALSE);
+    ScmObj v = Scm_HashTableRef(module->internal, SCM_OBJ(symbol), SCM_FALSE);
     /* NB: this function bypasses check of gloc setter */
     if (SCM_GLOCP(v)) {
         g = SCM_GLOC(v);
@@ -356,11 +386,10 @@ ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
         oldval = g->value;
     } else {
         g = SCM_GLOC(Scm_MakeGloc(symbol, module));
-        Scm_HashTableSet(module->table, SCM_OBJ(symbol), SCM_OBJ(g), 0);
+        Scm_HashTableSet(module->internal, SCM_OBJ(symbol), SCM_OBJ(g), 0);
         /* If module is marked 'export-all', export this binding by default */
         if (module->exportAll) {
-            g->exported = TRUE;
-            module->exported = Scm_Cons(SCM_OBJ(g->name), module->exported);
+            Scm_HashTableSet(module->external, SCM_OBJ(symbol), SCM_OBJ(g), 0);
         }
     }
     SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
@@ -399,26 +428,27 @@ ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
  *   This is not for genreral use.  It is intended to be used for
  *   intermediate anonymous modules, created by import handling
  *   routine to implement :except and :rename qualifiers.
+ *   Since we assume MODULE is for intermediate modules, we only
+ *   insert bindings to the external table, for those modules are
+ *   only searched in the 'import' path.
  */
 void Scm_HideBinding(ScmModule *module, ScmSymbol *symbol)
 {
-    ScmGloc *g;
-    ScmObj v;
     int err_exists = FALSE;
 
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-    v = Scm_HashTableRef(module->table, SCM_OBJ(symbol), SCM_FALSE);
+    ScmObj v = Scm_HashTableRef(module->external, SCM_OBJ(symbol), SCM_FALSE);
     if (!SCM_FALSEP(v)) {
         err_exists = TRUE;
     } else {
-        g = SCM_GLOC(Scm_MakeGloc(symbol, module));
+        ScmGloc *g = SCM_GLOC(Scm_MakeGloc(symbol, module));
         g->hidden = TRUE;
-        Scm_HashTableSet(module->table, SCM_OBJ(symbol), SCM_OBJ(g), 0);
+        Scm_HashTableSet(module->external, SCM_OBJ(symbol), SCM_OBJ(g), 0);
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
 
     if (err_exists) {
-        Scm_Error("hide-binding: binding already exists: %S", SCM_OBJ(symbol));
+        Scm_Error("hide-binding: binding already exists: %S (exports=%S)", SCM_OBJ(symbol), Scm_ModuleExports(module));
     }
 }
 
@@ -450,13 +480,11 @@ void Scm_HideBinding(ScmModule *module, ScmSymbol *symbol)
 int Scm_AliasBinding(ScmModule *target, ScmSymbol *targetName,
                      ScmModule *origin, ScmSymbol *originName)
 {
-    ScmObj v;
-    ScmGloc *g = Scm_FindBinding(origin, originName, 0);
-
-    if (g == NULL || !(g->exported)) return FALSE;
+    ScmGloc *g = Scm_FindBinding(origin, originName, SCM_BINDING_EXTERNAL);
+    if (g == NULL) return FALSE;
     SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
-    Scm_HashTableSet(target->table, SCM_OBJ(targetName), SCM_OBJ(g), 0);
-    target->exported = Scm_Cons(SCM_OBJ(targetName), target->exported);
+    Scm_HashTableSet(target->external, SCM_OBJ(targetName), SCM_OBJ(g), 0);
+    Scm_HashTableSet(target->internal, SCM_OBJ(targetName), SCM_OBJ(g), 0);
     SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
     return TRUE;
 }
@@ -470,7 +498,6 @@ ScmObj Scm_ImportModule(ScmModule *module,
                         u_long flags) /* reserved for future use */
 {
     ScmModule *imp = NULL;
-    ScmObj p;
     if (SCM_MODULEP(imported)) {
         imp = SCM_MODULE(imported);
     } else if (SCM_SYMBOLP(imported)) {
@@ -486,7 +513,7 @@ ScmObj Scm_ImportModule(ScmModule *module,
     }
 
     /* Preallocate a pair, so that we won't call malloc during locking */
-    p = Scm_Cons(SCM_OBJ(imp), SCM_NIL);
+    ScmObj p = Scm_Cons(SCM_OBJ(imp), SCM_NIL);
 
     /* Prepend imported module to module->imported list. */
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
@@ -525,57 +552,92 @@ ScmObj Scm_ImportModules(ScmModule *module, ScmObj list)
 /*
  * Export
  */
-ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
+/* <spec>  :: <name> | (rename <name> <exported-name>) */
+ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj specs)
 {
-    ScmObj lp, syms, badsym = SCM_FALSE;
-    int error = FALSE;
-    ScmSymbol *s;
-    ScmDictEntry *e;
-    ScmGloc *g;
+    ScmObj lp;
+    ScmObj overwritten = SCM_NIL; /* list of (exported-name orig-internal-name
+                                     new-internal-name). */
+    /* Check input first */
+    SCM_FOR_EACH(lp, specs) {
+        ScmObj spec = SCM_CAR(lp);
+        if (!(SCM_SYMBOLP(spec)
+              || (SCM_PAIRP(spec) && SCM_PAIRP(SCM_CDR(spec))
+                  && SCM_PAIRP(SCM_CDDR(spec))
+                  && SCM_NULLP(SCM_CDR(SCM_CDDR(spec)))
+                  && SCM_EQ(SCM_CAR(spec), SCM_SYM_RENAME)
+                  && SCM_SYMBOLP(SCM_CADR(spec))
+                  && SCM_SYMBOLP(SCM_CAR(SCM_CDDR(spec)))))) {
+            Scm_Error("Invalid export-spec; a symbol, or (rename <symbol> <symbol>) is expected, but got %S", spec);
+        }
+    }
 
-    /* We used to do something like
-     *  (set! (module-exports module)
-     *        (delete-duplicates (union (module-exports module) list)))
-     * This was slow when we exported lots of symbols.  As of 0.8.6,
-     * each GLOC has exported flag, so we can check whether a binding
-     * is exported or not in O(1).   Module-exports list is kept
-     * for backward compatibility.
-     */
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-    syms = module->exported;
-    SCM_FOR_EACH(lp, list) {
-        if (!SCM_SYMBOLP(SCM_CAR(lp))) {
-            error = TRUE;
-            badsym = SCM_CAR(lp);
-            break;
+    SCM_FOR_EACH(lp, specs) {
+        ScmObj spec = SCM_CAR(lp);
+        ScmSymbol *name, *exported_name;
+        if (SCM_SYMBOLP(spec)) {
+            name = exported_name = SCM_SYMBOL(spec);
+        } else {
+            /* we already knew those are symbols */
+            name = SCM_SYMBOL(SCM_CADR(spec));
+            exported_name = SCM_SYMBOL(SCM_CAR(SCM_CDDR(spec)));
         }
-        s = SCM_SYMBOL(SCM_CAR(lp));
-        e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(module->table),
-                               (intptr_t)s, SCM_DICT_CREATE);
-        if (e->value) {         /* e->value must be GLOC. */
-            g = SCM_GLOC(e->value);
-            if (!g->exported) {
-                syms = Scm_Cons(SCM_OBJ(s), syms);
-                g->exported = TRUE;
+        ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(module->external),
+                                             (intptr_t)exported_name, SCM_DICT_GET);
+        if (e) {
+            /* If we have e, it's already exported.  Check if
+               the previous export is for the same binding. */
+            SCM_ASSERT(SCM_DICT_VALUE(e) && SCM_GLOCP(SCM_DICT_VALUE(e)));
+            ScmGloc *g = SCM_GLOC(SCM_DICT_VALUE(e));
+            if (!SCM_EQ(name, g->name)) {
+                /* exported_name got a different meaning. we record it to warn
+                   later, then 'unexport' the old one. */
+                overwritten = Scm_Cons(SCM_LIST3(SCM_OBJ(exported_name),
+                                                 SCM_OBJ(g->name),
+                                                 SCM_OBJ(name)),
+                                       overwritten);
+                Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(module->external),
+                                   (intptr_t)exported_name, SCM_DICT_DELETE);
+                e = NULL;
             }
-        } else {
-            g = SCM_GLOC(Scm_MakeGloc(s, module));
-            g->exported = TRUE;
-            (void)SCM_DICT_SET_VALUE(e, SCM_OBJ(g));
-            syms = Scm_Cons(SCM_OBJ(s), syms);
+        }
+        /* we check again, for the symbol may be unexported above. */
+        if (e == NULL) {
+            /* This symbol hasn't been exported.  Either it only has an
+               internal binding, or there's no binding at all.  In the latter
+               case, we create a new binding (without value). */
+            e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(module->internal),
+                                   (intptr_t)name, SCM_DICT_CREATE);
+            if (!e->value) {
+                ScmGloc *g = SCM_GLOC(Scm_MakeGloc(name, module));
+                (void)SCM_DICT_SET_VALUE(e, SCM_OBJ(g));
+            }
+            Scm_HashTableSet(module->external, SCM_OBJ(exported_name),
+                             SCM_DICT_VALUE(e), 0);
         }
     }
-    if (!error) module->exported = syms;
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
-    if (error) Scm_Error("symbol required, but got %S", badsym);
-    return syms;
+
+    /* Now, if this export changes the meaning of exported symbols, we
+       warn it.  We expect this only happens at the development time, when
+       one is fiddling exports incrementally, so we just use Scm_Warn -
+       a library ready to be used shouldn't cause this warning. */
+    if (!SCM_NULLP(overwritten)) {
+        ScmObj lp;
+        SCM_FOR_EACH(lp, overwritten) {
+            ScmObj p = SCM_CAR(lp);
+            Scm_Warn("Exporting %S from %S as %S overrides the previous export of %S",
+                     SCM_CAR(SCM_CDDR(p)), SCM_OBJ(module), SCM_CAR(p),
+                     SCM_CADR(p));
+        }
+    }
+
+    return SCM_UNDEFINED;  /* we might want to return something more useful...*/
 }
 
 ScmObj Scm_ExportAll(ScmModule *module)
 {
-    ScmHashIter iter;
-    ScmDictEntry *e;
-
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     if (!module->exportAll) {
         /* Mark the module 'export-all' so that the new bindings would get
@@ -583,13 +645,15 @@ ScmObj Scm_ExportAll(ScmModule *module)
         module->exportAll = TRUE;
 
         /* Scan the module and mark all existing bindings as exported. */
-        Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(module->table));
+        ScmHashIter iter;
+        Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(module->internal));
+        ScmDictEntry *e;
         while ((e = Scm_HashIterNext(&iter)) != NULL) {
-            ScmGloc *g = SCM_GLOC(SCM_DICT_VALUE(e));
-            if (!g->exported) {
-                g->exported = TRUE;
-                module->exported =
-                    Scm_Cons(SCM_OBJ(g->name), module->exported);
+            ScmDictEntry *ee;
+            ee = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(module->external),
+                                    e->key, SCM_DICT_CREATE);
+            if (!ee->value) {
+                (void)SCM_DICT_SET_VALUE(ee, SCM_DICT_VALUE(e));
             }
         }
     }
@@ -597,6 +661,24 @@ ScmObj Scm_ExportAll(ScmModule *module)
     return SCM_OBJ(module);
 }
 
+/* Returns list of exported symbols.   We assume this is infrequent
+   operation, so we build the list every call.  If it becomes a problem,
+   we can cache the result. */
+ScmObj Scm_ModuleExports(ScmModule *module)
+{
+    ScmObj h = SCM_NIL, t = SCM_NIL;
+
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+    ScmHashIter iter;
+    Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(module->external));
+    ScmDictEntry *e;
+    while ((e = Scm_HashIterNext(&iter)) != NULL) {
+        SCM_APPEND1(h, t, SCM_DICT_KEY(e));
+    }
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+    return h;
+}
+
 /*----------------------------------------------------------------------
  * Extending (inheriting) modules
  */
@@ -608,8 +690,9 @@ ScmObj Scm_ExportAll(ScmModule *module)
 
 ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers)
 {
-    ScmObj mpl, seqh = SCM_NIL, seqt = SCM_NIL, sp;
+    ScmObj seqh = SCM_NIL, seqt = SCM_NIL;
 
+    ScmObj sp;
     SCM_FOR_EACH(sp, supers) {
         if (!SCM_MODULEP(SCM_CAR(sp))) {
             Scm_Error("non-module object found in the extend syntax: %S",
@@ -619,7 +702,7 @@ ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers)
     }
     SCM_APPEND1(seqh, seqt, supers);
     module->parents = supers;
-    mpl = Scm_MonotonicMerge1(seqh);
+    ScmObj mpl = Scm_MonotonicMerge1(seqh);
     if (SCM_FALSEP(mpl)) {
         Scm_Error("can't extend those modules simultaneously because of inconsistent precedence lists: %S", supers);
     }
@@ -633,15 +716,13 @@ ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers)
 
 ScmModule *Scm_FindModule(ScmSymbol *name, int flags)
 {
-    ScmModule *m;
-    int created;
-
     if (flags & SCM_FIND_MODULE_CREATE) {
-        m = lookup_module_create(name, &created);
+        int created;
+        ScmModule *m = lookup_module_create(name, &created);
         SCM_ASSERT(m != NULL);
         return m;
     } else {
-        m = lookup_module(name);
+        ScmModule *m = lookup_module(name);
         if (m == NULL) {
             if (!(flags & SCM_FIND_MODULE_QUIET)) {
                 Scm_Error("no such module: %S", name);
@@ -679,42 +760,19 @@ void Scm_SelectModule(ScmModule *mod)
  */
 
 /* Convert module name and pathname (mod load-path) and vice versa.
-   The default conversion is pretty straightforward, e.g.
-   util.list <=> "util/list"  etc.  However, modules and files can
-   have many-to-many mapping, and I'd like to reserve the room
-   of future extensions.   Eventually there will be some special
-   mapping table so the programmer can register exceptional mappings. */
-
+   We moved the definition in Scheme.  These are just stubs to call them. */
 ScmObj Scm_ModuleNameToPath(ScmSymbol *name)
 {
-    const ScmStringBody *b = SCM_STRING_BODY(SCM_SYMBOL_NAME(name));
-    char *buf = SCM_NEW_ATOMIC2(char *, SCM_STRING_BODY_SIZE(b)+1);
-    char *p = buf, *e = buf + SCM_STRING_BODY_SIZE(b);
-    memcpy(buf, SCM_STRING_BODY_START(b), SCM_STRING_BODY_SIZE(b));
-    while (p < e) {
-        int n = SCM_CHAR_NFOLLOWS(*p);
-        if (*p == '.') *p++ = '/';
-        else p += n+1;
-    }
-    *e = '\0';
-    return Scm_MakeString(buf, SCM_STRING_BODY_SIZE(b),
-                          SCM_STRING_BODY_LENGTH(b), 0);
+    static ScmObj module_name_to_path_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(module_name_to_path_proc, "module-name->path", Scm_GaucheModule());
+    return Scm_ApplyRec1(module_name_to_path_proc, SCM_OBJ(name));
 }
 
 ScmObj Scm_PathToModuleName(ScmString *path)
 {
-    const ScmStringBody *b = SCM_STRING_BODY(path);
-    char *buf = SCM_NEW_ATOMIC2(char *, SCM_STRING_BODY_SIZE(b)+1);
-    char *p = buf, *e = buf + SCM_STRING_BODY_SIZE(b);
-    memcpy(buf, SCM_STRING_BODY_START(b), SCM_STRING_BODY_SIZE(b));
-    while (p < e) {
-        int n = SCM_CHAR_NFOLLOWS(*p);
-        if (*p == '/') *p++ = '.';
-        else if (*p == '.') Scm_Error("bad pathname for module path: %S", path);
-        else p += n+1;
-    }
-    *e = '\0';
-    return SCM_INTERN(buf);
+    static ScmObj path_to_module_name_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(path_to_module_name_proc, "path->module-name", Scm_GaucheModule());
+    return Scm_ApplyRec1(path_to_module_name_proc, SCM_OBJ(path));
 }
 
 /*----------------------------------------------------------------------
@@ -733,7 +791,7 @@ static ScmObj module_imported(ScmObj m)
 
 static ScmObj module_exported(ScmObj m)
 {
-    return SCM_MODULE(m)->exported;
+    return Scm_ModuleExports(SCM_MODULE(m));
 }
 
 static ScmObj module_exportAll(ScmObj m)
@@ -758,7 +816,7 @@ static ScmObj module_depended(ScmObj m)
 
 static ScmObj module_table(ScmObj m)
 {
-    return SCM_OBJ(SCM_MODULE(m)->table);
+    return SCM_OBJ(SCM_MODULE(m)->internal);
 }
 
 static ScmObj module_origin(ScmObj m)
@@ -814,6 +872,13 @@ ScmModule *Scm_UserModule(void)
     return &userModule;
 }
 
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+ScmModule *Scm_KeywordModule(void)
+{
+    return &keywordModule;
+}
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
+
 ScmModule *Scm_CurrentModule(void)
 {
     return Scm_VM()->module;
@@ -831,14 +896,27 @@ ScmModule *Scm_CurrentModule(void)
 
 void Scm__InitModule(void)
 {
-    ScmObj mpl = SCM_NIL;
+    /* List of builtin modules.  We create these so that 'use' or r7rs 'import'
+       won't try to search the file. */
+    static const char *builtin_modules[] = {
+        "srfi-2", "srfi-6", "srfi-8", "srfi-10", "srfi-16", "srfi-17",
+        "srfi-22", "srfi-23", "srfi-26", "srfi-28", "srfi-31", "srfi-34",
+        "srfi-35", "srfi-36", "srfi-38", "srfi-45", "srfi-55", "srfi-61",
+        "srfi-62", "srfi-87", "srfi-95",
+        "gauche.vm.debugger",   /* for the backward compatibility */
+        NULL };
+    const char **modname;
 
     (void)SCM_INTERNAL_MUTEX_INIT(modules.mutex);
     modules.table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 64));
 
     /* standard module chain */
+    ScmObj mpl = SCM_NIL;
     INIT_MOD(nullModule, SCM_SYM_NULL, mpl);
     INIT_MOD(schemeModule, SCM_SYM_SCHEME, mpl);
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+    INIT_MOD(keywordModule, SCM_SYM_KEYWORD, mpl);
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
     INIT_MOD(gaucheModule, SCM_SYM_GAUCHE, mpl);
     INIT_MOD(gfModule, SCM_SYM_GAUCHE_GF, mpl);
     INIT_MOD(userModule, SCM_SYM_USER, mpl);
@@ -850,6 +928,11 @@ void Scm__InitModule(void)
     /* other modules */
     mpl = defaultMpl;
     INIT_MOD(internalModule, SCM_SYM_GAUCHE_INTERNAL, mpl);
+
+    /* create predefined moudles */
+    for (modname = builtin_modules; *modname; modname++) {
+        (void)SCM_FIND_MODULE(*modname, SCM_FIND_MODULE_CREATE);
+    }
 }
 
 void Scm__InitModulePost(void)
