@@ -1,7 +1,7 @@
 /*
  * symbol.c - symbol implementation
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/builtin-syms.h"
+#include "gauche/priv/builtin-syms.h"
 
 /*-----------------------------------------------------------
  * Symbols
@@ -42,47 +42,96 @@
 static void symbol_print(ScmObj obj, ScmPort *port, ScmWriteContext *);
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_SymbolClass, symbol_print);
 
-static ScmSymbol *make_sym(ScmObj name, int interned)
-{
-    ScmSymbol *sym = SCM_NEW(ScmSymbol);
-    SCM_SET_CLASS(sym, SCM_CLASS_SYMBOL);
-    sym->name = SCM_STRING(name);
-    sym->flags = interned? SCM_SYMBOL_FLAG_INTERNED : 0;
-    return sym;
-}
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+static ScmClass *keyword_cpl[] = {
+    SCM_CLASS_STATIC_PTR(Scm_SymbolClass),
+    SCM_CLASS_STATIC_PTR(Scm_TopClass),
+    NULL
+};
+
+SCM_DEFINE_BUILTIN_CLASS(Scm_KeywordClass, symbol_print,
+                         NULL, NULL, NULL, keyword_cpl);
+#else  /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
+SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_KeywordClass, symbol_print);
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
 
 /* name -> symbol mapper */
 static ScmInternalMutex obtable_mutex = SCM_INTERNAL_MUTEX_INITIALIZER;
 static ScmHashTable *obtable = NULL;
 
-/* Intern */
-ScmObj Scm_MakeSymbol(ScmString *name, int interned)
-{
-    ScmObj e;
-    ScmObj sname;
-    ScmSymbol *sym;
+#if !GAUCHE_UNIFY_SYMBOL_KEYWORD
+/* Global keyword table. */
+static struct {
+    ScmHashTable *table;
+    ScmInternalMutex mutex;
+} keywords = { NULL };
 
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
+
+/* internal constructor.  NAME must be an immutable string. */
+static ScmSymbol *make_sym(ScmClass *klass, ScmString *name, int interned)
+{
     if (interned) {
         /* fast path */
         SCM_INTERNAL_MUTEX_LOCK(obtable_mutex);
-        e = Scm_HashTableRef(obtable, SCM_OBJ(name), SCM_FALSE);
+        ScmObj e = Scm_HashTableRef(obtable, SCM_OBJ(name), SCM_FALSE);
         SCM_INTERNAL_MUTEX_UNLOCK(obtable_mutex);
-        if (!SCM_FALSEP(e)) return e;
+        if (!SCM_FALSEP(e)) return SCM_SYMBOL(e);
     }
 
-    sname = Scm_CopyStringWithFlags(name, SCM_STRING_IMMUTABLE,
-                                    SCM_STRING_IMMUTABLE);
-    sym = make_sym(sname, interned);
-    if (!interned) return SCM_OBJ(sym);
+    ScmSymbol *sym = SCM_NEW(ScmSymbol);
+    SCM_SET_CLASS(sym, klass);
+    sym->name = name;
+    sym->flags = interned? SCM_SYMBOL_FLAG_INTERNED : 0;
 
-    /* Using SCM_DICT_NO_OVERWRITE ensures that if another thread interns
-       the same name symbol between above HashTableRef and here, we'll
-       get the already interned symbol. */
-    SCM_INTERNAL_MUTEX_LOCK(obtable_mutex);
-    e = Scm_HashTableSet(obtable, SCM_OBJ(name), SCM_OBJ(sym),
+    if (!interned) {
+        return sym;
+    } else {
+        /* Using SCM_DICT_NO_OVERWRITE ensures that if another thread interns
+           the same name symbol between above HashTableRef and here, we'll
+           get the already interned symbol. */
+        SCM_INTERNAL_MUTEX_LOCK(obtable_mutex);
+        ScmObj e = Scm_HashTableSet(obtable, SCM_OBJ(name), SCM_OBJ(sym),
+                                    SCM_DICT_NO_OVERWRITE);
+        SCM_INTERNAL_MUTEX_UNLOCK(obtable_mutex);
+        return SCM_SYMBOL(e);
+    }
+}
+
+/* Intern */
+ScmObj Scm_MakeSymbol(ScmString *name, int interned)
+{
+    ScmObj sname = Scm_CopyStringWithFlags(name, SCM_STRING_IMMUTABLE,
+                                           SCM_STRING_IMMUTABLE);
+    return SCM_OBJ(make_sym(SCM_CLASS_SYMBOL, SCM_STRING(sname), interned));
+}
+
+/* In unified keyword, we include preceding ':' to the name. */
+ScmObj Scm_MakeKeyword(ScmString *name)
+{
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+    /* We could optimize this later. */
+    ScmObj prefix = Scm_MakeString(":", 1, 1, SCM_STRING_IMMUTABLE);
+    ScmObj sname = Scm_StringAppend2(SCM_STRING(prefix), name);
+    ScmSymbol *s = make_sym(SCM_CLASS_KEYWORD, SCM_STRING(sname), TRUE);
+    Scm_DefineConst(Scm_KeywordModule(), s, SCM_OBJ(s));
+    return SCM_OBJ(s);
+#else  /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
+    (void)SCM_INTERNAL_MUTEX_LOCK(keywords.mutex);
+    ScmObj r = Scm_HashTableRef(keywords.table, SCM_OBJ(name), SCM_FALSE);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(keywords.mutex);
+
+    if (SCM_KEYWORDP(r)) return r;
+
+    ScmKeyword *k = SCM_NEW(ScmKeyword);
+    SCM_SET_CLASS(k, SCM_CLASS_KEYWORD);
+    k->name = SCM_STRING(Scm_CopyString(name));
+    (void)SCM_INTERNAL_MUTEX_LOCK(keywords.mutex);
+    r = Scm_HashTableSet(keywords.table, SCM_OBJ(name), SCM_OBJ(k),
                          SCM_DICT_NO_OVERWRITE);
-    SCM_INTERNAL_MUTEX_UNLOCK(obtable_mutex);
-    return e;
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(keywords.mutex);
+    return r;
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
 }
 
 /* Default prefix string. */
@@ -91,20 +140,17 @@ static SCM_DEFINE_STRING_CONST(default_prefix, "G", 1, 1);
 /* Returns uninterned symbol.   PREFIX can be NULL */
 ScmObj Scm_Gensym(ScmString *prefix)
 {
-    ScmObj name;
-    ScmSymbol *sym;
     char numbuf[50];
-    int nc;
     /* We don't need mutex for this variable, since a race on it is
        tolerated---multiple threads may be get the same name symbols,
        but they are uninterned and never be eq? to each other. */
     static intptr_t gensym_count = 0;
 
     if (prefix == NULL) prefix = &default_prefix;
-    nc = snprintf(numbuf, 49, "%"PRIdPTR, gensym_count++);
+    int nc = snprintf(numbuf, 49, "%"PRIdPTR, gensym_count++);
     numbuf[49] = '\0';
-    name = Scm_StringAppendC(prefix, numbuf, nc, nc);
-    sym = make_sym(name, FALSE);
+    ScmObj name = Scm_StringAppendC(prefix, numbuf, nc, nc);
+    ScmSymbol *sym = make_sym(SCM_CLASS_SYMBOL, SCM_STRING(name), FALSE);
     return SCM_OBJ(sym);
 }
 
@@ -134,7 +180,7 @@ ScmObj Scm_SymbolSansPrefix(ScmSymbol *s, ScmSymbol *p)
    bit 3: bad char for symbol, and should be written as \c
    bit 4: may be escaped when case fold mode
  */
-static char special[] = {
+static const char special[] = {
  /* NUL .... */
     7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
  /* .... */
@@ -161,10 +207,10 @@ void Scm_WriteSymbolName(ScmString *snam, ScmPort *port, ScmWriteContext *ctx,
     /* TODO: For now, we regard chars over 0x80 is all "printable".
        Need a more consistent mechanism. */
     const ScmStringBody *b = SCM_STRING_BODY(snam);
-    const char *p = SCM_STRING_BODY_START(b), *q;
-    int siz = SCM_STRING_BODY_SIZE(b), i;
+    const char *p = SCM_STRING_BODY_START(b);
+    int siz = SCM_STRING_BODY_SIZE(b);
     int escape = FALSE;
-    int spmask = ((SCM_WRITE_CASE(ctx) == SCM_WRITE_CASE_FOLD)? 0x12 : 0x02);
+    int spmask = (Scm_WriteContextCase(ctx) == SCM_WRITE_CASE_FOLD)? 0x12 : 0x02;
 
     if (siz == 0) {         /* special case */
         if (!(flags & SCM_SYMBOL_WRITER_NOESCAPE_EMPTY)) {
@@ -178,10 +224,14 @@ void Scm_WriteSymbolName(ScmString *snam, ScmPort *port, ScmWriteContext *ctx,
     }
     if ((unsigned int)*p < 128
         && (special[(unsigned int)*p]&1)
+#if GAUCHE_UNIFY_SYMBOL_KEYWORD
+        && (*p != ':')
+#endif
         && (!(flags & SCM_SYMBOL_WRITER_NOESCAPE_INITIAL))) {
         escape = TRUE;
     } else {
-        for (i=0, q=p; i<siz; i++, q++) {
+        const char *q = p;
+        for (int i=0; i<siz; i++, q++) {
             if ((unsigned int)*q < 128
                 && (special[(unsigned int)*q]&spmask)) {
                 escape = TRUE;
@@ -191,7 +241,7 @@ void Scm_WriteSymbolName(ScmString *snam, ScmPort *port, ScmWriteContext *ctx,
     }
     if (escape) {
         SCM_PUTC('|', port);
-        for (q=p; q<p+siz; ) {
+        for (const char *q=p; q<p+siz; ) {
             unsigned int ch;
             SCM_CHAR_GET(q, ch);
             q += SCM_CHAR_NBYTES(ch);
@@ -200,7 +250,7 @@ void Scm_WriteSymbolName(ScmString *snam, ScmPort *port, ScmWriteContext *ctx,
                     SCM_PUTC('\\', port);
                     SCM_PUTC(ch, port);
                 } else if (special[ch] & 4) {
-                    Scm_Printf(port, "\\x%02x", ch);
+                    Scm_Printf(port, "\\x%02x;", ch);
                 } else {
                     SCM_PUTC(ch, port);
                 }
@@ -222,14 +272,106 @@ void Scm_WriteSymbolName(ScmString *snam, ScmPort *port, ScmWriteContext *ctx,
  */
 static void symbol_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    if (SCM_WRITE_MODE(ctx) == SCM_WRITE_DISPLAY) {
+    if (Scm_WriteContextMode(ctx) == SCM_WRITE_DISPLAY) {
         SCM_PUTS(SCM_SYMBOL_NAME(obj), port);
     } else {
+#if !GAUCHE_UNIFY_SYMBOL_KEYWORD
+        if (SCM_KEYWORDP(obj)) {
+            SCM_PUTC(':', port);
+            /* We basically print keyword names in the same way as symbols
+               (i.e. using |-escape if necessary).  However, as a convention,
+               two things are different from the default symbol writer.
+               (1) We don't check the noninitials; :1 is unambiguously a
+               keyword, so we don't need to print :|1|.
+               (2) A keyword with an empty name can be printed just as :,
+               instead of :||.
+               These conventions are useful if we pass the S-expression with
+               these keywords to other Scheme implementations that don't support
+               CL-style keywords; they would just read those ones as symbols.
+            */
+            Scm_WriteSymbolName(SCM_KEYWORD(obj)->name, port, ctx,
+                                (SCM_SYMBOL_WRITER_NOESCAPE_INITIAL
+                                 |SCM_SYMBOL_WRITER_NOESCAPE_EMPTY));
+            return;
+        }
+#endif /*!GAUCHE_UNIFY_SYMBOL_KEYWORD*/
         if (!SCM_SYMBOL_INTERNED(obj)) SCM_PUTZ("#:", -1, port);
         Scm_WriteSymbolName(SCM_SYMBOL_NAME(obj), port, ctx, 0);
     }
 }
 
+/*
+ * Keyword Utilities
+ *   The names are historical; KEY doesn't need to be a keyword at all;
+ *   anything that can be compared by eq? do.
+ */
+
+ScmObj Scm_GetKeyword(ScmObj key, ScmObj list, ScmObj fallback)
+{
+    ScmObj cp;
+    SCM_FOR_EACH(cp, list) {
+        if (!SCM_PAIRP(SCM_CDR(cp))) {
+            Scm_Error("incomplete key list: %S", list);
+        }
+        if (key == SCM_CAR(cp)) return SCM_CADR(cp);
+        cp = SCM_CDR(cp);
+    }
+    if (SCM_UNBOUNDP(fallback)) {
+        Scm_Error("value for key %S is not provided: %S", key, list);
+    }
+    return fallback;
+}
+
+ScmObj Scm_DeleteKeyword(ScmObj key, ScmObj list)
+{
+    ScmObj cp;
+    SCM_FOR_EACH(cp, list) {
+        if (!SCM_PAIRP(SCM_CDR(cp))) {
+            Scm_Error("incomplete key list: %S", list);
+        }
+        if (key == SCM_CAR(cp)) {
+            /* found */
+            ScmObj h = SCM_NIL, t = SCM_NIL;
+            ScmObj tail = Scm_DeleteKeyword(key, SCM_CDR(SCM_CDR(cp)));
+            ScmObj cp2;
+            SCM_FOR_EACH(cp2, list) {
+                if (cp2 == cp) {
+                    SCM_APPEND(h, t, tail);
+                    return h;
+                } else {
+                    SCM_APPEND1(h, t, SCM_CAR(cp2));
+                }
+            }
+        }
+        cp = SCM_CDR(cp);
+    }
+    return list;
+}
+
+ScmObj Scm_DeleteKeywordX(ScmObj key, ScmObj list)
+{
+    ScmObj cp, prev = SCM_FALSE;
+    SCM_FOR_EACH(cp, list) {
+        if (!SCM_PAIRP(SCM_CDR(cp))) {
+            Scm_Error("incomplete key list: %S", list);
+        }
+        if (key == SCM_CAR(cp)) {
+            /* found */
+            if (SCM_FALSEP(prev)) {
+                /* we're at the head of list */
+                return Scm_DeleteKeywordX(key, SCM_CDR(SCM_CDR(cp)));
+            } else {
+                ScmObj tail = Scm_DeleteKeywordX(key, SCM_CDR(SCM_CDR(cp)));
+                SCM_SET_CDR(prev, tail);
+                return list;
+            }
+        }
+        cp = SCM_CDR(cp);
+        prev = cp;
+    }
+    return list;
+}
+
 /*
  * Initialization
  */
@@ -241,4 +383,12 @@ void Scm__InitSymbol(void)
     SCM_INTERNAL_MUTEX_INIT(obtable_mutex);
     obtable = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_STRING, 4096));
     init_builtin_syms();
+#if !GAUCHE_UNIFY_SYMBOL_KEYWORD
+    (void)SCM_INTERNAL_MUTEX_INIT(keywords.mutex);
+    keywords.table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_STRING, 256));
+#else
+    /* Temporary: In order to compile 0.9.4 source by 0.9.3.  Should be
+       removed after 0.9.4 release.  See lib/gauche/cgen/cise.scm */
+    Scm_AddFeature("gauche.unified-keyword-symbol", NULL);
+#endif /*GAUCHE_UNIFY_SYMBOL_KEYWORD*/
 }
