@@ -1,7 +1,7 @@
 /*
  * vector.c - vector implementation
  *
- *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -44,11 +44,11 @@
 
 static void vector_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    int i;
     SCM_PUTZ("#(", -1, port);
-    for (i=0; i<SCM_VECTOR_SIZE(obj); i++) {
+    for (int i=0; i<SCM_VECTOR_SIZE(obj); i++) {
         if (i != 0) SCM_PUTC(' ', port);
-        Scm_Write(SCM_VECTOR_ELEMENT(obj, i), SCM_OBJ(port), ctx->mode);
+        Scm_Write(SCM_VECTOR_ELEMENT(obj, i), SCM_OBJ(port),
+                  Scm_WriteContextMode(ctx));
     }
     SCM_PUTZ(")", -1, port);
 }
@@ -56,7 +56,7 @@ static void vector_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 SCM_DEFINE_BUILTIN_CLASS(Scm_VectorClass, vector_print, NULL, NULL, NULL,
                          SCM_CLASS_SEQUENCE_CPL);
 
-static ScmVector *make_vector(int size)
+static ScmVector *make_vector(ScmSmallInt size)
 {
     ScmVector *v = SCM_NEW2(ScmVector *,
                             sizeof(ScmVector) + sizeof(ScmObj)*(size-1));
@@ -65,27 +65,23 @@ static ScmVector *make_vector(int size)
     return v;
 }
 
-ScmObj Scm_MakeVector(int size, ScmObj fill)
+ScmObj Scm_MakeVector(ScmSmallInt size, ScmObj fill)
 {
-    int i;
-    ScmVector *v;
     if (size < 0) {
         Scm_Error("vector size must be a positive integer, but got %d", size);
     }
-    v = make_vector(size);
+    ScmVector *v = make_vector(size);
     if (SCM_UNBOUNDP(fill)) fill = SCM_UNDEFINED;
-    for (i=0; i<size; i++) v->elements[i] = fill;
+    for (ScmSmallInt i=0; i<size; i++) v->elements[i] = fill;
     return SCM_OBJ(v);
 }
 
-ScmObj Scm_ListToVector(ScmObj l, int start, int end)
+ScmObj Scm_ListToVector(ScmObj l, ScmSmallInt start, ScmSmallInt end)
 {
     ScmVector *v;
-    ScmObj e;
-    int i;
 
     if (end < 0) {
-        int size = Scm_Length(l);
+        ScmSmallInt size = Scm_Length(l);
         if (size < 0) Scm_Error("bad list: %S", l);
         SCM_CHECK_START_END(start, end, size);
         v = make_vector(size - start);
@@ -93,8 +89,8 @@ ScmObj Scm_ListToVector(ScmObj l, int start, int end)
         SCM_CHECK_START_END(start, end, end);
         v = make_vector(end - start);
     }
-    e = Scm_ListTail(l, start, SCM_UNBOUND);
-    for (i=0; i<end-start; i++, e=SCM_CDR(e)) {
+    ScmObj e = Scm_ListTail(l, start, SCM_UNBOUND);
+    for (ScmSmallInt i=0; i<end-start; i++, e=SCM_CDR(e)) {
         if (!SCM_PAIRP(e)) {
             Scm_Error("list too short: %S", l);
         }
@@ -103,9 +99,9 @@ ScmObj Scm_ListToVector(ScmObj l, int start, int end)
     return SCM_OBJ(v);
 }
 
-ScmObj Scm_VectorToList(ScmVector *v, int start, int end)
+ScmObj Scm_VectorToList(ScmVector *v, ScmSmallInt start, ScmSmallInt end)
 {
-    int len = SCM_VECTOR_SIZE(v);
+    ScmWord len = SCM_VECTOR_SIZE(v);
     SCM_CHECK_START_END(start, end, len);
     return Scm_ArrayToList(SCM_VECTOR_ELEMENTS(v)+start,
                            end-start);
@@ -119,31 +115,33 @@ ScmObj Scm_VectorToList(ScmVector *v, int start, int end)
    check (for Scheme routines) should be done in the stub file, since
    Scheme version may receive bignum, which can't be passed to C API. */
 
-ScmObj Scm_VectorRef(ScmVector *vec, int i, ScmObj fallback)
+ScmObj Scm_VectorRef(ScmVector *vec, ScmSmallInt i, ScmObj fallback)
 {
     if (i < 0 || i >= vec->size) return fallback;
     return vec->elements[i];
 }
 
-ScmObj Scm_VectorSet(ScmVector *vec, int i, ScmObj obj)
+ScmObj Scm_VectorSet(ScmVector *vec, ScmSmallInt i, ScmObj obj)
 {
     if (i >= 0 && i < vec->size) vec->elements[i] = obj;
     return obj;
 }
 
-ScmObj Scm_VectorFill(ScmVector *vec, ScmObj fill, int start, int end)
+ScmObj Scm_VectorFill(ScmVector *vec, ScmObj fill,
+                      ScmSmallInt start, ScmSmallInt end)
 {
-    int i, len = SCM_VECTOR_SIZE(vec);
+    ScmSmallInt len = SCM_VECTOR_SIZE(vec);
     SCM_CHECK_START_END(start, end, len);
-    for (i=start; i < end; i++) {
+    for (ScmSmallInt i=start; i < end; i++) {
         SCM_VECTOR_ELEMENT(vec, i) = fill;
     }
     return SCM_OBJ(vec);
 }
 
-ScmObj Scm_VectorCopy(ScmVector *vec, int start, int end, ScmObj fill)
+ScmObj Scm_VectorCopy(ScmVector *vec,
+                      ScmSmallInt start, ScmSmallInt end, ScmObj fill)
 {
-    int i, len = SCM_VECTOR_SIZE(vec);
+    ScmSmallInt len = SCM_VECTOR_SIZE(vec);
     ScmVector *v = NULL;
     if (end < 0) end = len;
     if (end < start) {
@@ -154,7 +152,7 @@ ScmObj Scm_VectorCopy(ScmVector *vec, int start, int end, ScmObj fill)
     } else {
         if (SCM_UNBOUNDP(fill)) fill = SCM_UNDEFINED;
         v = make_vector(end - start);
-        for (i=0; i<end-start; i++) {
+        for (ScmSmallInt i=0; i<end-start; i++) {
             if (i+start < 0 || i+start >= len) {
                 SCM_VECTOR_ELEMENT(v, i) = fill;
             } else {
@@ -244,8 +242,8 @@ const char *Scm_UVectorTypeName(int type) /* for error msgs etc. */
 /* Returns the size of element of the uvector of given class */
 int Scm_UVectorElementSize(ScmClass *klass)
 {
-    static int sizes[] = { 1, 1, 2, 2, 4, 4, 8, 8,
-                           2, sizeof(float), sizeof(double) };
+    static const int sizes[] = { 1, 1, 2, 2, 4, 4, 8, 8,
+                                 2, sizeof(float), sizeof(double) };
     int ind = (int)Scm_UVectorType(klass);
     if (ind >= 0) return sizes[ind];
     return -1;
@@ -258,26 +256,29 @@ int Scm_UVectorSizeInBytes(ScmUVector *uv)
 }
 
 /* Generic constructor */
-ScmObj Scm_MakeUVectorFull(ScmClass *klass, int size, void *init,
+ScmObj Scm_MakeUVectorFull(ScmClass *klass, ScmSmallInt size, void *init,
                            int immutable, void *owner)
 {
-    ScmUVector *vec;
     int eltsize = Scm_UVectorElementSize(klass);
     SCM_ASSERT(eltsize >= 1);
-    vec = SCM_NEW(ScmUVector);
+    ScmUVector *vec = SCM_NEW(ScmUVector);
     SCM_SET_CLASS(vec, klass);
     if (init) {
         vec->elements = init;   /* trust the caller */
     } else {
         vec->elements = SCM_NEW_ATOMIC2(void*, size*eltsize);
     }
+#if GAUCHE_API_0_95
+    vec->size_flags = (size << 1)|(immutable?1:0);
+#else  /*!GAUCHE_API_0_95*/
     vec->size = size;
     vec->immutable = immutable;
+#endif /*!GAUCHE_API_0_95*/
     vec->owner = owner;
     return SCM_OBJ(vec);
 }
 
-ScmObj Scm_MakeUVector(ScmClass *klass, int size, void *init)
+ScmObj Scm_MakeUVector(ScmClass *klass, ScmSmallInt size, void *init)
 {
     return Scm_MakeUVectorFull(klass, size, init, FALSE, NULL);
 }
@@ -286,7 +287,7 @@ ScmObj Scm_MakeUVector(ScmClass *klass, int size, void *init)
    (As the 'VM' in the name suggests, the return value of this API
    should immediately be passed to VM.  See comments on FFX in gauche/number.h)
  */
-ScmObj Scm_VMUVectorRef(ScmUVector *v, int t, int k, ScmObj fallback)
+ScmObj Scm_VMUVectorRef(ScmUVector *v, int t, ScmSmallInt k, ScmObj fallback)
 {
     SCM_ASSERT(Scm_UVectorType(SCM_CLASS_OF(v)) == t);
     if (k < 0 || k >= SCM_UVECTOR_SIZE(v)) return fallback;
@@ -315,42 +316,72 @@ ScmObj Scm_VMUVectorRef(ScmUVector *v, int t, int k, ScmObj fallback)
 /*
  * Inidividual constructors for convenience
  */
-#define DEF_UVCTOR(tag, T) \
-ScmObj SCM_CPP_CAT3(Scm_Make,tag,Vector)(int size, T fill)              \
+#define DEF_UVCTOR_FILL(tag, T) \
+ScmObj SCM_CPP_CAT3(Scm_Make,tag,Vector)(ScmSmallInt size, T fill)      \
 {                                                                       \
     ScmUVector *u =                                                     \
         (ScmUVector*)Scm_MakeUVector(SCM_CPP_CAT3(SCM_CLASS_,tag,VECTOR),\
                                      size, NULL);                       \
-    int i;                                                              \
-    for (i=0; i<size; i++) {                                            \
-        SCM_CPP_CAT3(SCM_,tag,VECTOR_ELEMENTS)(u)[i] = fill;            \
-    }                                                                   \
+    T *elts = SCM_CPP_CAT3(SCM_,tag,VECTOR_ELEMENTS)(u);                \
+    for (ScmSmallInt i=0; i<size; i++) *elts++ = fill;                  \
     return SCM_OBJ(u);                                                  \
-}                                                                       \
-ScmObj SCM_CPP_CAT3(Scm_Make,tag,VectorFromArray)(int size, const T array[])\
+}
+
+#define DEF_UVCTOR_ARRAY(tag, T) \
+ScmObj SCM_CPP_CAT3(Scm_Make,tag,VectorFromArray)(ScmSmallInt size,     \
+                                                  const T array[])      \
 {                                                                       \
     T *z = SCM_NEW_ATOMIC_ARRAY(T, size);                               \
     memcpy(z, array, size*sizeof(T));                                   \
     return Scm_MakeUVector(SCM_CPP_CAT3(SCM_CLASS_,tag,VECTOR),         \
                            size, (void*)z);                             \
 }                                                                       \
-ScmObj SCM_CPP_CAT3(Scm_Make,tag,VectorFromArrayShared)(int size, T array[])\
+ScmObj SCM_CPP_CAT3(Scm_Make,tag,VectorFromArrayShared)(ScmSmallInt size,\
+                                                        T array[])      \
 {                                                                       \
     return Scm_MakeUVector(SCM_CPP_CAT3(SCM_CLASS_,tag,VECTOR),         \
                            size, (void*)array);                         \
 }
 
-DEF_UVCTOR(S8, signed char)
-DEF_UVCTOR(U8, unsigned char)
-DEF_UVCTOR(S16, short)
-DEF_UVCTOR(U16, u_short)
-DEF_UVCTOR(S32, ScmInt32)
-DEF_UVCTOR(U32, ScmUInt32)
-DEF_UVCTOR(S64, ScmInt64)
-DEF_UVCTOR(U64, ScmUInt64)
-DEF_UVCTOR(F16, ScmHalfFloat)
-DEF_UVCTOR(F32, float)
-DEF_UVCTOR(F64, double)
+/* NB: For u8vector and s8vector we can let memset() to fill the
+   contents, expecting it's optimized. */
+ScmObj Scm_MakeS8Vector(ScmSmallInt size, signed char fill)
+{
+    ScmUVector *u =
+        (ScmUVector*)Scm_MakeUVector(SCM_CLASS_S8VECTOR, size, NULL);
+    (void)memset(SCM_S8VECTOR_ELEMENTS(u), fill, size);
+    return SCM_OBJ(u);
+}
+
+ScmObj Scm_MakeU8Vector(ScmSmallInt size, unsigned char fill)
+{
+    ScmUVector *u =
+        (ScmUVector*)Scm_MakeUVector(SCM_CLASS_U8VECTOR, size, NULL);
+    (void)memset(SCM_U8VECTOR_ELEMENTS(u), fill, size);
+    return SCM_OBJ(u);
+}
+
+DEF_UVCTOR_FILL(S16, short)
+DEF_UVCTOR_FILL(U16, u_short)
+DEF_UVCTOR_FILL(S32, ScmInt32)
+DEF_UVCTOR_FILL(U32, ScmUInt32)
+DEF_UVCTOR_FILL(S64, ScmInt64)
+DEF_UVCTOR_FILL(U64, ScmUInt64)
+DEF_UVCTOR_FILL(F16, ScmHalfFloat)
+DEF_UVCTOR_FILL(F32, float)
+DEF_UVCTOR_FILL(F64, double)
+
+DEF_UVCTOR_ARRAY(S8, signed char)
+DEF_UVCTOR_ARRAY(U8, unsigned char)
+DEF_UVCTOR_ARRAY(S16, short)
+DEF_UVCTOR_ARRAY(U16, u_short)
+DEF_UVCTOR_ARRAY(S32, ScmInt32)
+DEF_UVCTOR_ARRAY(U32, ScmUInt32)
+DEF_UVCTOR_ARRAY(S64, ScmInt64)
+DEF_UVCTOR_ARRAY(U64, ScmUInt64)
+DEF_UVCTOR_ARRAY(F16, ScmHalfFloat)
+DEF_UVCTOR_ARRAY(F32, float)
+DEF_UVCTOR_ARRAY(F64, double)
 
 /*
  * Class-dependent functions
@@ -363,9 +394,8 @@ static void SCM_CPP_CAT3(print_,tag,vector)(ScmObj obj,                 \
                                             ScmPort *out,               \
                                             ScmWriteContext *ctx)       \
 {                                                                       \
-    int i;                                                              \
     Scm_Printf(out, "#"#tag"(");                                        \
-    for (i=0; i<SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(obj); i++) {         \
+    for (int i=0; i<SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(obj); i++) {     \
         T elt = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(obj)[i];         \
         if (i != 0) Scm_Printf(out, " ");                               \
         pr(out, elt);                                                   \
@@ -426,12 +456,11 @@ DEF_PRINT(F64, f64, double, fpr)
 #define DEF_CMP(TAG, tag, T, eq)                                        \
 static int SCM_CPP_CAT3(compare_,tag,vector)(ScmObj x, ScmObj y, int equalp) \
 {                                                                       \
-    int len = SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(x), i;                 \
-    T xx, yy;                                                           \
+    ScmSmallInt len = SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(x);            \
     if (SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(y) != len) return -1;        \
-    for (i=0; i<len; i++) {                                             \
-        xx = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(x)[i];              \
-        yy = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(y)[i];              \
+    for (ScmSmallInt i=0; i<len; i++) {                                 \
+        T xx = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(x)[i];            \
+        T yy = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(y)[i];            \
         if (!eq(xx,yy)) {                                               \
             return -1;                                                  \
         }                                                               \
