@@ -43,6 +43,9 @@
 
 (prim-test "case" #t (lambda ()  (case (+ 2 3) ((1 3 5 7 9) #t) ((0 2 4 6 8) #f))))
 (prim-test "case" #t (lambda () (undefined? (case 1 ((2 3) #t)))))
+(prim-test "case" #t (lambda () (case 1 (() #f) ((1) #t))))
+(prim-test "case" #t (lambda () (case 1 (() #f) (else #t))))
+(prim-test "case" #t (lambda () (undefined? (case 1 (() #t)))))
 (prim-test "case (srfi-87)" 0 (lambda () (case (+ 2 3) ((1 3 5) 0) (else => values))))
 (prim-test "case (srfi-87)" 6 (lambda () (case (+ 2 3) ((1 3 5) => (cut + 1 <>)) (else => values))))
 (prim-test "case (srfi-87)" 5 (lambda () (case (+ 2 3) ((2 4 6) 0) (else => values))))
@@ -357,6 +360,23 @@
           (lambda ()
             (eval '(apply car foo '()) (find-module 'primsyn.test))))))
 
+;;----------------------------------------------------------------
+(test-section "max literal arguments")
+
+;; Fix this after we have separate compile-error condition.
+(define (test-max-literal-args msg expr)
+  (prim-test (string-append "max literal arguments for " msg)
+             'caught 
+             (lambda ()
+               (with-error-handler (lambda (e) 'caught)
+                 (lambda () (eval expr (interaction-environment)))))))
+
+(test-max-literal-args "inliner" `(list ,@(make-list 10000 #f)))
+(test-max-literal-args "global proc" `(make ,@(make-list 10000 #f)))
+(test-max-literal-args "local proc"
+                       `(let ((foo (lambda x x)))
+                          (foo ,@(make-list 10000 #f))))
+
 ;;----------------------------------------------------------------
 (test-section "local procedure optimization")
 
@@ -483,5 +503,27 @@
               ((> x 3) x)
             #f))))
 
+;;----------------------------------------------------------------
+(test-section "letrec and letrec*")
+
+(prim-test "letrec reordering" '((1 3) . (2 3 1))
+           (lambda ()
+             (let ((r '()))
+               (cons (letrec ((a (begin (set! r (cons 1 r)) 1))
+                              (b (begin (set! r (cons 2 r)) 2))
+                              (c (begin (set! r (cons 3 r)) 3)))
+                       (list a c))
+                     r))))
+
+(prim-test "letrec* non-reordering" '((1 3) . (3 2 1))
+           (lambda ()
+             (let ((r '()))
+               (cons (letrec* ((a (begin (set! r (cons 1 r)) 1))
+                               (b (begin (set! r (cons 2 r)) 2))
+                               (c (begin (set! r (cons 3 r)) 3)))
+                       (list a c))
+                     r))))
+
+
 (test-end)
 
