@@ -58,20 +58,20 @@
 
 (define (cmds . args)
   (let1 cmdlist (apply cmd args)
-    (string-concatenate (apply append (map (lambda (x) `(,x " ")) cmdlist)))))
+    (string-concatenate (apply append (map (^x `(,x " ")) cmdlist)))))
 
 (define (rmrf . files)
   ;; shorthand of normalizing pathname.  this doesn't do anything on
-  ;; unix, but on Windows the separator in PATHNAME is replaced.
+  ;; unix, but on Windows separators in PATHNAME are replaced.
   (define (n pathname) (sys-normalize-pathname pathname))
 
   (dolist [f files]
     (cond-expand
      [gauche.os.windows
-      (sys-system #`"rmdir /q /s ,(n f) > NUL 2>&1")
-      (sys-system #`"del /q ,(n f) > NUL 2>&1")]
+      (sys-system #"rmdir /q /s ~(n f) > NUL 2>&1")
+      (sys-system #"del /q ~(n f) > NUL 2>&1")]
      [else
-      (sys-system #`"rm -rf ,f > /dev/null")])))
+      (sys-system #"rm -rf ~f > /dev/null")])))
 
 (define (touch file)
   (with-output-to-file file (cut values)))
@@ -513,4 +513,30 @@
 
 (rmrf "testc.o")
 
+;;-------------------------------
+(test-section "unwind-protect upon exit")
+
+;; unwind-protect is tested in exception.scm, but we needed to do this test
+;; after we test gauche.process.
+
+(rmrf "test.o" "test1.o")
+
+(with-output-to-file "test.o"
+  (cut write '(define (main args)
+                (unwind-protect (begin
+                                  (with-output-to-file "test1.o"
+                                    (^[] (display "foo\n")))
+                                  (exit 1))
+                  (sys-unlink "test1.o")))))
+
+(test* "unwind-protect upon exit" '(1 #f)
+       ;; we haven't tested file.util yet, so using a clumsy way.
+       (let* ([gosh (string-append (sys-dirname (current-load-path))
+                                   "/../src/gosh")]
+              [p (run-process `(,gosh "-ftest" "test.o") :wait #t)])
+         (list (sys-wait-exit-status (process-exit-status p))
+               (file-exists? "test1.o"))))
+
+(rmrf "test.o" "test1.o")
+
 (test-end)
