@@ -11,14 +11,9 @@
 ;; Some test examples are taken from the test suite of Henry Spencer's
 ;; regexp package and PCRE.
 
-(define (match&list rx input)
-  (cond [(rxmatch rx input)
-         => (^[match] (map (^n (rxmatch-substring match n))
-                           (iota (rxmatch-num-matches match))))]
-        [else #f]))
-
 (define (test-re re str expect)
-  (test* (write-to-string `(,re ,str)) expect (match&list re str)))
+  (test* (write-to-string `(,re ,str)) expect
+         (rxmatch-substrings (rxmatch re str))))
 
 (define (test-before re str expect . opts)
   (test* (write-to-string `(rxmatch-before (,re ,str))) expect
@@ -33,7 +28,7 @@
 (define-syntax test-parse
   (syntax-rules ()
     ((_ re ast)
-     (test* #`"regexp-parse \",re\"" ast (regexp-parse re)))))
+     (test* #"regexp-parse \"~re\"" ast (regexp-parse re)))))
 
 ;;-------------------------------------------------------------------------
 (test-section "regexp-parse")
@@ -50,6 +45,11 @@
 (test-parse "a|b" '(0 #f (alt #\a #\b)))
 (test-parse "[ab]" '(0 #f #[ab]))
 (test-parse "[^ab]" '(0 #f (comp . #[ab])))
+(test-parse "\\x0d;\\x0a;" '(0 #f #\return #\newline))
+(test-parse "\\x0dbc\\x0acd" ;legacy fallback
+            '(0 #f (seq #\return #\b #\c) (seq #\newline #\c #\d)))
+(test-parse "\\u000dbc;" '(0 #f (seq #\return #\b #\c #\;)))
+(test-parse "\x0d;[\x0a;-\x0d]" '(0 #f #\return #[\u000a-\u000d]))
 (test-parse "." '(0 #f any))
 (test-parse "^" '(0 #f bol))
 (test-parse "$" '(0 #f eol))
@@ -66,11 +66,11 @@
 (test-parse "(?<=ab)" '(0 #f (assert (lookbehind #\a #\b))))
 (test-parse "(?<!ab)" '(0 #f (nassert (lookbehind #\a #\b))))
 (test-parse "(?<name>a)" '(0 #f (1 name #\a)))
-(test-parse "(?(?=)y)" '(0 #f (cpat (assert) (#\y) #f)))
+(test-parse "(?(?=)y)" '(0 #f (cpat (assert) (#\y) ())))
 (test-parse "(?(?=)y|n)" '(0 #f (cpat (assert) (#\y) (#\n))))
-(test-parse "(?(?<=)y)" '(0 #f (cpat (assert (lookbehind)) (#\y) #f)))
+(test-parse "(?(?<=)y)" '(0 #f (cpat (assert (lookbehind)) (#\y) ())))
 (test-parse "(?(?<=)y|n)" '(0 #f (cpat (assert (lookbehind)) (#\y) (#\n))))
-(test-parse "()(?(1)y)" '(0 #f (1 #f) (cpat 1 (#\y) #f)))
+(test-parse "()(?(1)y)" '(0 #f (1 #f) (cpat 1 (#\y) ())))
 (test-parse "()(?(1)y|n)"'(0 #f (1 #f) (cpat 1 (#\y) (#\n))))
 (test-parse "()\\1" '(0 #f (1 #f) (backref . 1)))
 (test-parse "(?<name>)\\k<name>" '(0 #f (1 name) (backref . 1)))
@@ -97,7 +97,7 @@
             "(a)(b)" "(a|b)c(d)" "(a(b(c|d)|e))f"
             "(?:abc)*" "(ab(?:cd)?e{3,4})" "(?i:ab(?-i:cd)ef)"
             ;; backref
-            "(a)bc\\1" "ab(?<foo>c)(d)\\k<foo>"
+            "(a)bc\\1" "ab(?<foo>c)(d)\\k<foo>" "((.)\\2)"
             ;; once, lookahead, lookbehind
             "(?>abc)" "(?=a*b*c)" "(?!a*b*c)" "(?<=a[bc])" "(?<!a[bc])"
             "(a)(?(1)b)" "(a)(?(1)b|c)" "(a)(?(1)|c)"
@@ -105,33 +105,21 @@
             "(?(?<=ab)cd|ef)" "(?(?<!ab)cd|ef)" 
             ))
 
-;;-------------------------------------------------------------------------
-(test-section "regexp-writer")
-
-(define-syntax test-regexp-writer
-  (syntax-rules ()
-    [(_ exp pat)
-     (test* exp exp (write-to-string (string->regexp pat)))]))
-
-(test-regexp-writer "#/abc/" "abc")
-(test-regexp-writer "#/a[\\d]/" "a[\\d]")
-(test-regexp-writer "#/a[a-z]/" "a[a-z]")
-(test-regexp-writer "#/a\\/b/"  "a/b")
-(test-regexp-writer "#/\\\\/"  "\\\\")
-(test-regexp-writer "#/ /"  "\x20")
-(test-regexp-writer "#/\\x20/"  "\\x20")
-
-(test* "#/abc/i" "#/abc/i"
-       (write-to-string (string->regexp "abc" :case-fold #t)))
-
 ;;-------------------------------------------------------------------------
 (test-section "compile")
 
 (define-syntax test-regexp-compile
   (syntax-rules ()
     [(_ pat)
-     (test* #`"regexp-compile \",|pat|\"" #t
-            (regexp? (regexp-compile (regexp-parse pat))))]))
+     (test* #"regexp-compile \"~|pat|\""
+            (let1 orig (string->regexp pat)
+              (list (regexp->string orig)
+                    (regexp-num-groups orig)
+                    (regexp-named-groups orig)))
+            (let1 compiled (regexp-compile (regexp-parse pat))
+              (list (regexp->string compiled)
+                    (regexp-num-groups compiled)
+                    (regexp-named-groups compiled))))]))
 
 (test-regexp-compile "a")
 (test-regexp-compile "ab")
@@ -171,11 +159,19 @@
 (test-regexp-compile "(?<name>)\\k<name>")
 (test-regexp-compile "(?<name>)(?<name>)\\k<name>")
 
+;; Renumbering groups
+(test* "regexp-compile group renumbering"
+       '(5 (0 #f (1 #f (2 #f #\a) (3 #f #\b)) (4 #f #\c)))
+       (let1 c (regexp-compile
+                '(0 #f (3 #f (100 #f #\a) (0 #f #\b)) (3 #f #\c)))
+         (list (regexp-num-groups c)
+               (regexp-ast c))))
+
 (define %regexp-laset (with-module gauche.internal %regexp-laset))
 (define-syntax test-regexp-laset
   (syntax-rules ()
     [(_ pat exp)
-     (test* #`"regexp-laset \",|pat|\"" exp
+     (test* #"regexp-laset \"~|pat|\"" exp
             (%regexp-laset (regexp-compile (regexp-parse pat))))]))
 
 (test-regexp-laset "abc" #[a])
@@ -199,20 +195,20 @@
 (test-section "basics")
 
 (test-re #/a/ "a"   '("a"))
-(test-re #/a/ "A"   #f)
+(test-re #/a/ "A"   '())
 (test-re #/a/ "ba"  '("a"))
 (test-re #/a/ "bac" '("a"))
-(test-re #/a/ ""    #f) ;; input null str
+(test-re #/a/ ""    '()) ;; input null str
 (test-re #/a/ (string (integer->char 0) #\a #\b) '("a")) ;; NUL character
 (test-re #/abc/ "abc" '("abc"))
-(test-re #/abc/ "abbc" #f)
+(test-re #/abc/ "abbc" '())
 (test-re #/abc/ "babcd" '("abc"))
 (test-re #/abc|de/ "dabce" '("abc"))
 (test-re #/abc|de/ "abdec" '("de"))
-(test-re #/abc|de/ "abe" #f)
+(test-re #/abc|de/ "abe" '())
 (test-re #/a|b|c/ "abc" '("a"))
 (test-re #/a|b|c/ "bac" '("b"))
-(test-re #/a|b|c/ "def" #f)
+(test-re #/a|b|c/ "def" '())
 (test-re #/|abc/ "abc" '(""))
 (test-re #/abc|/ "abc" '("abc"))
 (test-re #/abc|/ "abd" '(""))
@@ -223,7 +219,7 @@
 (test-re #/a(b)c/        "abc"     '("abc" "b"))
 (test-re #/a((b)(c))/    "abc"     '("abc" "bc" "b" "c"))
 (test-re #/a((((b))))c/  "abc"     '("abc" "b" "b" "b" "b"))
-(test-re #/a((((b))))c/  "a(b)c"   '#f)
+(test-re #/a((((b))))c/  "a(b)c"   '())
 (test-re #/a\(/          "a("      '("a("))
 (test-re #/a()b/         "ab"      '("ab" ""))
 (test-re #/a()()b/       "ab"      '("ab" "" ""))
@@ -244,57 +240,62 @@
 (test-re #/(.)*/         "abc"     '("abc" "c"))
 (test-re #/(a([^a])*)*/  "abcaBC"  '("abcaBC" "aBC" "C"))
 (test-re #/b|()|a/       "cac"     '("" ""))
+(test-re #/(a)*a/        "a"       '("a" #f))
+(test-re #/(a)*a/        "aa"      '("aa" "a"))
+(test-re #/(a)*a/        "aaa"     '("aaa" "a"))
+(test-re #/(a)*?a/       "a"       '("a" #f))
+(test-re #/(a)*?a/       "aa"      '("a" #f))
 
 ;;-------------------------------------------------------------------------
 (test-section "simple meta")
 
 (test-re #/a.c/ "abc" '("abc"))
 (test-re #/a../ "abc" '("abc"))
-(test-re #/a../ "ab"  #f)
-(test-re #/.../ "ab"  #f)
+(test-re #/a../ "ab"  '())
+(test-re #/.../ "ab"  '())
 (test-re #/./   "abc" '("a"))
-(test-re #/./   ""    #f)
+(test-re #/./   ""    '())
 
 ;;-------------------------------------------------------------------------
 (test-section "anchors")
 
 (test-re #/^abc/       "abcd" '("abc"))
-(test-re #/^abc/       "aabcd" #f)
+(test-re #/^abc/       "aabcd" '())
 (test-re #/^^/         "^^abc" '("^"))
-(test-re #/^^/         "a^^c" #f)
+(test-re #/^^/         "a^^c" '())
 (test-re #/^abc|def/   "abc" '("abc"))
-(test-re #/^abc|def/   "zabc" #f)
+(test-re #/^abc|def/   "zabc" '())
 (test-re #/^abc|def/   "zabcdef" '("def"))
 (test-re #/abc|^def/   "defabc" '("def"))
 (test-re #/abc|^def/   "abcdef" '("abc"))
 (test-re #/abc|^def/   "defabbc" '("def"))
-(test-re #/abc|^def/   "adefbc" #f)
+(test-re #/abc|^def/   "adefbc" '())
 (test-re #/^(abc|def)/ "abc" '("abc" "abc"))
-(test-re #/^(abc|def)/ "aabc" #f)
+(test-re #/^(abc|def)/ "aabc" '())
 (test-re #/(^abc|def)/ "abcdef" '("abc" "abc"))
 (test-re #/(^abc|def)/ "^abcdef" '("def" "def"))
 (test-re #/a(^bc|def)/ "a^bcdef" '("a^bc" "^bc"))
-(test-re #/a(^bc|def)/ "abcdef" #f)
+(test-re #/a(^bc|def)/ "abcdef" '())
 (test-re #/^/          "hoge" '(""))
 (test-re #/$/          "hoge" '(""))
 (test-re #/abc$/       "bcabc" '("abc"))
-(test-re #/abc$/       "abcab" #f)
+(test-re #/abc$/       "abcab" '())
 (test-re #/^abc$/      "abc" '("abc"))
-(test-re #/abc$$/      "abc" #f)
+(test-re #/abc$$/      "abc" '())
 (test-re #/abc$$/      "abc$" '("abc$"))
 (test-re #/$$/         "abc$" '("$"))
 (test-re #/^$/         "" '(""))
-(test-re #/^$/         "a" #f)
+(test-re #/^$/         "a" '())
 (test-re #/^^$$/       "^$" '("^$"))
 (test-re #/abc$|def/   "abc" '("abc"))
 (test-re #/abc$|def/   "defabc" '("def"))
 (test-re #/^abc|def$/  "abcdef" '("abc"))
-(test-re #/^abc|def$/  "defabc" #f)
-(test-re #/^abc|def$/  "defabc" #f)
+(test-re #/^abc|def$/  "defabc" '())
+(test-re #/^abc|def$/  "defabc" '())
 (test-re #/(^abc|def$)/ "aaadef" '("def" "def"))
-(test-re #/(^abc|def$)$/ "aaadef" #f)
+(test-re #/(^abc|def$)$/ "aaadef" '())
 (test-re #/(^abc|def$)$/ "aaadef$" '("def$" "def$"))
-(test-re #/(abc$|def)$/ "aaabc" #f)
+(test-re #/(abc$|def)$/ "aaabc" '())
 (test-re #/(abc$|def)$/ "aaabc$" '("abc$" "abc$"))
 (test-re #/a$b/         "aa$bb" '("a$b"))
 (test-re #/ab\$/        "ab$cd" '("ab$"))
@@ -304,7 +305,7 @@
 
 (test-re #/a\*c/        "a*c" '("a*c"))
 (test-re #/a\.c/        "a.c" '("a.c"))
-(test-re #/a\.c/        "abc" #f)
+(test-re #/a\.c/        "abc" '())
 (test-re #/a\\b/        "a\\b" '("a\\b"))
 (test-re #/a\\\*b/      "a\\*b" '("a\\*b"))
 (test-re #/a\jc/        "ajc" '("ajc"))
@@ -336,15 +337,15 @@
 (test-re #/a(b*|c)d/ "abbd"                 '("abbd" "bb"))
 (test-re #/a(b*|c)d/ "ad"                   '("ad" ""))
 (test-re #/a(b*|c)d/ "acd"                  '("acd" "c"))
-(test-re #/a(b*|c)d/ "abcd"                 #f)
+(test-re #/a(b*|c)d/ "abcd"                 '())
 (test-re #/a.*c/     "bacbababbbbadbaba"    '("ac"))
-(test-re #/a.*c/     "abaaaabababbadbabdba" #f)
+(test-re #/a.*c/     "abaaaabababbadbabdba" '())
 
 ;; Tests for input skipping using laset
 (test-re #/a+b+/     "aaaacccccccbaaabbccc" '("aaabb"))
 (test-re #/a*b+/     "aaaacccccccbaaabbccc" '("b"))
 
-(test-re #/a+@a+\.a+/ "aaaaaaaaaaa@aaaaaaaa@@a.aaaaa@aaaaa" #f)
+(test-re #/a+@a+\.a+/ "aaaaaaaaaaa@aaaaaaaa@@a.aaaaa@aaaaa" '())
 (test-re #/a+@a+\.a+/ "aaaaaaaaaaa@aaaaaaaa@@a.aaaaa@aaaaa.a" '("aaaaa@aaaaa.a"))
  
 ;;-------------------------------------------------------------------------
@@ -374,24 +375,24 @@
 
 (test-re #/a[bc]d/   "abd" '("abd"))
 (test-re #/a[bc]d/   "acd" '("acd"))
-(test-re #/a[bc]d/   "aed" #f)
+(test-re #/a[bc]d/   "aed" '())
 (test-re #/a[a-z]d/  "aed" '("aed"))
-(test-re #/a[a-z]d/  "aEd" #f)
+(test-re #/a[a-z]d/  "aEd" '())
 (test-re #/a[]]d/    "a]d" '("a]d"))
 (test-re #/a[]-]d/   "a-d" '("a-d"))
-(test-re #/a[]-^]d/  "a-d" #f)
+(test-re #/a[]-^]d/  "a-d" '())
 (test-re #/a[]-^]d/  "a]d" '("a]d"))
 (test-re #/a[a-z-]d/ "a-d" '("a-d"))
 (test-re #/a[a-z-]d/ "afd" '("afd"))
 (test-re #/a[az-]d/  "a-d" '("a-d"))
 (test-re #/a[a-]d/   "a-d" '("a-d"))
-(test-re #/a[az-]d/  "afd" #f)
+(test-re #/a[az-]d/  "afd" '())
 (test-re #/a[az-]d/  "azd" '("azd"))
 (test-re #/a[^ab]c/  "abacc" '("acc"))
 (test-re #/a[^]]c/   "abc" '("abc"))
-(test-re #/a[^]]c/   "a]c" #f)
+(test-re #/a[^]]c/   "a]c" '())
 (test-re #/a[^^]c/   "abc" '("abc"))
-(test-re #/a[^^]c/   "a^c" #f)
+(test-re #/a[^^]c/   "a^c" '())
 (test-re #/a[Bc]*d/  "aBccBd" '("aBccBd"))
 (test-re #/[a]b[c]/  "abc" '("abc"))
 (test-re #/[abc]b[abc]/ "abc" '("abc"))
@@ -409,7 +410,7 @@
 (test-re #/\s/        "  "   '(" "))
 (test-re #/\s\s/      "  "   '("  "))
 (test-re #/\s/        "\t "  '("\t"))
-(test-re #/\s\s/      "\\s"  #f)
+(test-re #/\s\s/      "\\s"  '())
 (test-re #/\\s\s/     "\\s " '("\\s "))
 (test-re #/(\s*)(\S+)(\s*)/ "\txyz   abc" '("\txyz   " "\t" "xyz" "   "))
 (test-re #/(\d+)(\D+)(\d+)(\D+)(\d+)/ " (1234) 5678-9012 "
@@ -520,27 +521,33 @@
 ;;-------------------------------------------------------------------------
 (test-section "backreference")
 (test-re #/^(.)\1$/ "aa" '("aa" "a"))
-(test-re #/^(.)\1$/ "ab" #f)
+(test-re #/^(.)\1$/ "ab" '())
 (test-re #/(.+)\1/ "a123123j" '("123123" "123"))
 (test-re #/(.+)\1/i "AbCaBC" '("AbCaBC" "AbC"))
-(test-re #/(.+)\1/ "AbCAb1" #f)
+(test-re #/(.+)\1/ "AbCAb1" '())
+(test-re #/((.)\2)/ "aa" '("aa" "aa" "a"))
+(test-re #/(a)*\1/ "aa" '("aa" "a"))
+(test-re #/(aa)*\1/ "aaa" '())
+(test-re #/(aa)*\1/ "aaaaa" '("aaaa" "aa"))
 (test* "^\\1(.)$" (test-error) (string->regexp "^\\1(.)"))
 (test* "^(\\1)$" (test-error) (string->regexp "^(\\1)$"))
+(test* "(.)\\2" (test-error) (string->regexp "(.)\\2"))
+(test* "((.)\\1)" (test-error) (string->regexp "((.)\\1)"))
 
 ;;-------------------------------------------------------------------------
 (test-section "independent subexpression")
-(test-re #/(?>.*\/)foo/ "/this/is/a/long/line/" #f)
+(test-re #/(?>.*\/)foo/ "/this/is/a/long/line/" '())
 (test-re #/(?>.*\/)foo/ "/this/is/a/long/line/foo" '("/this/is/a/long/line/foo"))
 (test-re #/(?>(\.\d\d[1-9]?))\d+/ "1.230003938" '(".230003938" ".23"))
 (test-re #/(?>(\.\d\d[1-9]?))\d+/ "1.875000282" '(".875000282" ".875"))
-(test-re #/(?>(\.\d\d[1-9]?))\d+/ "1.235" #f)
+(test-re #/(?>(\.\d\d[1-9]?))\d+/ "1.235" '())
 (test-re #/^((?>\w+)|(?>\s+))*$/ "foo bar" '("foo bar" "bar"))
-(test-re #/a*+a/ "aaa" #f)
+(test-re #/a*+a/ "aaa" '())
 (test-re #/a*+b/ "aab" '("aab"))
-(test-re #/a++a/ "aaa" #f)
+(test-re #/a++a/ "aaa" '())
 (test-re #/a++b/ "aab" '("aab"))
-(test-re #/a?+a/ "a" #f)
-(test-re #/a?+b/ "ab" '("ab"))
+(test-re #/a?+a/ "a"   '())
+(test-re #/a?+b/ "ab"  '("ab"))
 
 ;;-------------------------------------------------------------------------
 (test-section "lookahead assertion")
@@ -555,57 +562,57 @@
 (test-re #/(?:(?!foo)...|^.{0,2})bar(.*)/ "2barrel" '("2barrel" "rel"))
 (test-re #/(?:(?!foo)...|^.{0,2})bar(.*)/ "A barrel" '("A barrel" "rel"))
 (test-re #/^(\D*)(?=\d)(?!123)/ "abc456" '("abc" "abc"))
-(test-re #/^(\D*)(?=\d)(?!123)/ "abc123" #f)
+(test-re #/^(\D*)(?=\d)(?!123)/ "abc123" '())
 (test-re #/(?!^)abc/ "the abc" '("abc"))
-(test-re #/(?!^)abc/ "abc" #f)
+(test-re #/(?!^)abc/ "abc" '())
 (test-re #/(?!^^).../ "^^abc" '("^ab"))
 (test-re #/(?!^^).../ "^abc" '("abc"))
 (test-re #/(?=^)abc/ "abc" '("abc"))
-(test-re #/(?=^)abc/ "the abc" #f)
+(test-re #/(?=^)abc/ "the abc" '())
 (test-re #/(?=^^).../ "^abc" '("^ab"))
 (test-re #/(?=^^).../ "^^abc" '("^^a"))
 (test-re #/^(?=$).*/ "" '(""))
-(test-re #/^(?!$).*/ "" #f)
-(test-re #/^(?=$).*/ "a" #f)
+(test-re #/^(?!$).*/ "" '())
+(test-re #/^(?=$).*/ "a" '())
 (test-re #/^(?=$$).*/ "$" '("$"))
 (test-re #/^(?!$).*/ "a" '("a"))
 
 (test-re #/(\.\d\d((?=0)|\d(?=\d)))/ "1.230003938" '(".23" ".23" ""))
 (test-re #/(\.\d\d((?=0)|\d(?=\d)))/ "1.875000282" '(".875" ".875" "5"))
-(test-re #/(\.\d\d((?=0)|\d(?=\d)))/ "1.235" #f)
+(test-re #/(\.\d\d((?=0)|\d(?=\d)))/ "1.235" '())
 (test-re #/^\D*(?!123)/ "ABC123" '("AB"))
 (test-re #/^(\D*)(?=\d)(?!123)/ "ABC445" '("ABC" "ABC"))
-(test-re #/^(\D*)(?=\d)(?!123)/ "ABC123" #f)
+(test-re #/^(\D*)(?=\d)(?!123)/ "ABC123" '())
 (test-re #/a(?!b)./ "abad" '("ad"))
 (test-re #/a(?!^b)./ "a^bad" '("a^"))
 (test-re #/a(?!b)/ "abad" '("a"))
 (test-re #/a(?=d)./ "abad" '("ad"))
-(test-re #/a(?=^b)./ "a^bad" #f)
+(test-re #/a(?=^b)./ "a^bad" '())
 (test-re #/a(?=c|d)./ "abad" '("ad"))
 
 ;;-------------------------------------------------------------------------
 (test-section "lookbehind assertion")
-(test-re #/(?<=a)b/ "b" #f)
+(test-re #/(?<=a)b/ "b" '())
 (test-re #/(?<=a)b/ "ab" '("b"))
 (test-re #/(?<=a+)b/ "aab" '("b"))
 (test-re #/(?<=x[yz])b/ "xzb" '("b"))
-(test-re #/(?<=zyx)b/ "xyzb" #f)
+(test-re #/(?<=zyx)b/ "xyzb" '())
 (test-re #/(?<=[ab]+)c/ "abc" '("c"))
-(test-re #/(?<!<[^>]*)foo/ "<foo>" #f)
+(test-re #/(?<!<[^>]*)foo/ "<foo>" '())
 (test-re #/(?<!<[^>]*)foo/ "<bar>foo" '("foo"))
 (test-re #/(?<=^a)b/ "ab" '("b"))
-(test-re #/(?<=^)b/ "ab" #f)
+(test-re #/(?<=^)b/ "ab" '())
 (test-re #/(?<=^)b/ "b" '("b"))
-(test-re #/.(?<=^)b/ "a^b" #f)
+(test-re #/.(?<=^)b/ "a^b" '())
 (test-re #/(?<=^a$)/ "a" '(""))
-(test-re #/(?<=^a$)b/ "a$b" #f)
+(test-re #/(?<=^a$)b/ "a$b" '())
 (test-re #/(?<=(a))b/ "ab" '("b" "a"))
 (test-re #/(?<=(a)(b))c/ "abc" '("c" "a" "b"))
 (test-re #/(?<=(a)|(b))c/ "bc" '("c" #f "b"))
 (test-re #/(?<=(?<!foo)bar)baz/ "abarbaz" '("baz"))
-(test-re #/(?<=(?<!foo)bar)baz/ "foobarbaz" #f)
+(test-re #/(?<=(?<!foo)bar)baz/ "foobarbaz" '())
 (test-re #/(?<=\d{3})(?<!999)foo/ "865foo" '("foo"))
-(test-re #/(?<=\d{3})(?<!999)foo/ "999foo" #f)
+(test-re #/(?<=\d{3})(?<!999)foo/ "999foo" '())
 (test* "(?<=(?>a*))" (test-error) (string->regexp "(?<=(?>a*))"))
 (test-re #/(abc)...(?<=\1)/ "abcabc" '("abcabc" "abc"))
 (test-re #/(abC)...(?<=\1)/i "abCAbc" '("abCAbc" "abC"))
@@ -627,7 +634,7 @@
        (let1 m (#/(?<foo>a)/ "abcd")
          (m 'bar)))
 (test-re #/(?<foo>^a$)/ "a" '("a" "a"))
-(test-re #/(?<foo>^a$)/ "ab" #f)
+(test-re #/(?<foo>^a$)/ "ab" '())
 (test-re #/(?<name-with-hyphen>a)/ "a" '("a" "a"))
 (test* "(?<host>\d+.\d+.\d+.\d+)|(?<host>[\w.]+)"
        '("127.0.0.1" "127.0.0.1" #f "127.0.0.1")
@@ -638,7 +645,7 @@
        (let1 m (#/(?<host>\d+.\d+.\d+.\d+)|(?<host>[\w.]+)/ "foo.com")
          (list (m 0) (m 1) (m 2) (m 'host))))
 (test-re #/(?<foo>.+)\k<foo>/ "abcabc" '("abcabc" "abc"))
-(test-re #/(?<foo>.+)\k<foo>/ "abcdef" #f)
+(test-re #/(?<foo>.+)\k<foo>/ "abcdef" '())
 
 (test-parse  "\\k<foo>" (test-error))
 (test-before #/(?<foo>def)/ "abcdefghi" "abc" 'foo)
@@ -650,15 +657,15 @@
 ;;-------------------------------------------------------------------------
 (test-section "conditional subexpression")
 (test-re #/(a)(?(1)b)/ "ab" '("ab" "a"))
-(test-re #/(a)(?(1)b)/ "aa" #f)
-(test-re #/(a)(?(1)b)/ "ac" #f)
-(test-re #/(a)?(?(1)b|c)/ "xb" #f)
+(test-re #/(a)(?(1)b)/ "aa" '())
+(test-re #/(a)(?(1)b)/ "ac" '())
+(test-re #/(a)?(?(1)b|c)/ "xb" '())
 (test-re #/(a)?(?(1)b|c)/ "xc" '("c" #f))
-(test-re #/(?(?<=a)b)/ "ab" '("b"))
-(test-re #/(?(?<=a)b)/ "ac" #f)
-(test-re #/(?(?<=a)b)/ "xb" #f)
-(test-re #/(?(?<=a)b)/ "ab" '("b"))
-(test-re #/(?(?<=a)b)/ "ac" #f)
+(test-re #/(<)?[^<>]+(?(1)>)/ "<foo>" '("<foo>" "<"))
+(test-re #/(<)?[^<>]+(?(1)>)/ "foo" '("foo" #f))
+(test-re #/(?(?<=a)b)/ "ab" '(""))
+(test-re #/(?(?<=a)b)/ "ac" '(""))
+(test-re #/(?(?<=a)b)/ "xb" '(""))
 (test-parse "(?(?a)b|c)" (test-error))
 (test-re #/()(?(1))/ "" '("" ""))
 
@@ -896,6 +903,28 @@
 (test* "equal #/abc/i #/abc/i" #t
        (equal? #/abc/i #/abc/i))
 
+;;-------------------------------------------------------------------------
+(test-section "regexp printer")
+
+(let ()
+  (define (test-regexp-writer exp pat)
+    (test* exp exp (write-to-string (string->regexp pat))))
+
+  (test-regexp-writer "#/abc/" "abc")
+  (test-regexp-writer "#/a[0-9]/" "a[\\d]")
+  (test-regexp-writer "#/a[a-z]/" "a[a-z]")
+  (test-regexp-writer "#/a\\/b/"  "a/b")
+  (test-regexp-writer "#/\\\\/"  "\\\\")
+  (test-regexp-writer "#/ /"  "\x20")
+  (test-regexp-writer "#/ /"  "\\x20")
+  (test-regexp-writer "#/\\//"  "\\/") ; single '/'
+  (test-regexp-writer "#/\\\\\\//"  "\\\\/") ; backslash, then '/'
+  (test-regexp-writer "#/\\\\\\//"  "\\\\\\/") ; backslash, then '/'
+  )
+
+(test* "#/(?i:abc)/" "#/(?i:abc)/"
+       (write-to-string (string->regexp "abc" :case-fold #t)))
+
 ;;-------------------------------------------------------------------------
 (test-section "regexp from AST")
 
