@@ -18,10 +18,10 @@
 (define rfc822-header1
   "Received: by foo.bar.com id ZZZ55555; Thu, 31 May 2001 16:38:04 -1000 (HST)
 Received: from ooo.ooo.com (ooo.ooo.com [1.2.3.4])
-	by foo.bar.com (9.9.9+3.2W/3.7W-) with ESMTP id ZZZ55555
-	for <yoo@bar.com>; Thu, 31 May 2001 16:38:02 -1000 (HST)
+\tby foo.bar.com (9.9.9+3.2W/3.7W-) with ESMTP id ZZZ55555
+\tfor <yoo@bar.com>; Thu, 31 May 2001 16:38:02 -1000 (HST)
 Received: from zzz ([1.2.3.5]) by ooo.ooo.com  with Maccrosoft SMTPSVC(5.5.1877.197.19);
-	 Thu, 31 May 2001 22:33:16 -0400
+\t Thu, 31 May 2001 22:33:16 -0400
 Message-ID: <beefbeefbeefbeef@ooo.ooo.com>
 Subject: Bogus Tester
 From: Bogus Sender <bogus@ooo.com>
@@ -38,8 +38,8 @@ Content-Length: 4349
 
 (define rfc822-header1-list
   '(("received" "by foo.bar.com id ZZZ55555; Thu, 31 May 2001 16:38:04 -1000 (HST)")
-    ("received" "from ooo.ooo.com (ooo.ooo.com [1.2.3.4])	by foo.bar.com (9.9.9+3.2W/3.7W-) with ESMTP id ZZZ55555	for <yoo@bar.com>; Thu, 31 May 2001 16:38:02 -1000 (HST)")
-    ("received" "from zzz ([1.2.3.5]) by ooo.ooo.com  with Maccrosoft SMTPSVC(5.5.1877.197.19);	 Thu, 31 May 2001 22:33:16 -0400")
+    ("received" "from ooo.ooo.com (ooo.ooo.com [1.2.3.4])\tby foo.bar.com (9.9.9+3.2W/3.7W-) with ESMTP id ZZZ55555\tfor <yoo@bar.com>; Thu, 31 May 2001 16:38:02 -1000 (HST)")
+    ("received" "from zzz ([1.2.3.5]) by ooo.ooo.com  with Maccrosoft SMTPSVC(5.5.1877.197.19);\t Thu, 31 May 2001 22:33:16 -0400")
     ("message-id" "<beefbeefbeefbeef@ooo.ooo.com>")
     ("subject" "Bogus Tester")
     ("from" "Bogus Sender <bogus@ooo.com>")
@@ -106,7 +106,7 @@ Content-Length: 4349
         (lambda (ind wday)
           (receive (y m d H M S tz wd)
               (rfc822-parse-date
-               #`",|wday|, ,(+ 2 ind) Jan 2000 00:00:00 +0000")
+               #"~|wday|, ~(+ 2 ind) Jan 2000 00:00:00 +0000")
             wd))
         '("Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat" "Znn")))
 
@@ -115,7 +115,7 @@ Content-Length: 4349
        (map (lambda (mon)
               (receive (y m d H M S tz wd)
                   (rfc822-parse-date
-                   #`"1 ,mon 1999 00:00:00 +0000")
+                   #"1 ~mon 1999 00:00:00 +0000")
                 m))
             '("Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug"
               "Sep" "Oct" "Nov" "Dec" "Zzz")))
@@ -201,7 +201,7 @@ Content-Length: 4349
                           (with-output-to-string
                             (cut rfc822-write-headers hdrs
                                  :check (lambda (name body reason)
-                                          (values #`"x-,name"
+                                          (values #"x-~name"
                                                   (x->string reason))))))
                    )])])
 
@@ -276,6 +276,11 @@ Content-Length: 4349
 (test* "decode" "BAR0er9" (base64-decode-string "QkFSMGVyOQ=="))
 (test* "decode" "BAR0er9" (base64-decode-string "QkFS\r\nMGVyOQ\r\n=="))
 
+(test* "standard encode" "YTA+YTA/" (base64-encode-string "a0>a0?"))
+(test* "standard decode" "a0>a0?" (base64-decode-string "YTA+YTA/"))
+(test* "url-safe encode" "YTA-YTA_" (base64-encode-string "a0>a0?" :url-safe #t))
+(test* "url-safe decode" "a0>a0?" (base64-decode-string "YTA-YTA_" :url-safe #t))
+
 ;;--------------------------------------------------------------------
 (test-section "rfc.quoted-printable")
 (use rfc.quoted-printable)
@@ -444,6 +449,8 @@ Content-Length: 4349
 (test-section "rfc.mime")
 (use rfc.mime)
 (test-module 'rfc.mime)
+(use rfc.mime-port)
+(test-module 'rfc.mime-port)
 
 (test* "mime-parse-version" '((1 0) (1 0) (1 0) (1 0) #f)
        (map mime-parse-version
@@ -591,15 +598,15 @@ Content-Length: 4349
 
 ;; NB: this assumes the test is run either under src/ or test/
 (define (mime-message-tester num headers)
-  (let ((src #`"../test/data/rfc-mime-,|num|.txt")
-        (res (call-with-input-file #`"../test/data/rfc-mime-,|num|.res.txt"
+  (let ((src #"../test/data/rfc-mime-~|num|.txt")
+        (res (call-with-input-file #"../test/data/rfc-mime-~|num|.res.txt"
                read)))
     (call-with-input-file src
       (lambda (inp)
         (let* ((title (read-line inp)) ;; test title
                (expl  (read-line inp)) ;; explanation (ignored)
                (headers (or headers (rfc822-read-headers inp))))
-          (test* #`"mime-parse-message (,|num| - ,|title|)"
+          (test* #"mime-parse-message (~|num| - ~|title|)"
                  res
                  (and (equal? (mime-parse-version
                                (rfc822-header-ref headers "mime-version"))
@@ -661,20 +668,20 @@ Content-Length: 4349
        `(,(mime-parse-content-type ctype) '() ,body)]
       [("message/rfc822" _ ("text/plain" _ body))
        `(("message" "rfc822") '()
-         ,#`"content-type: text/plain\r\n\r\n,body")]
+         ,#"content-type: text/plain\r\n\r\n~body")]
       [(ctype _ children ...)
        `(,(mime-parse-content-type ctype) '()
          (subparts ,@(map gen-parts children)))]))
-  (let1 src (call-with-input-file #`"../test/data/rfc-mime-,|num|.res.txt" read)
+  (let1 src (call-with-input-file #"../test/data/rfc-mime-~|num|.res.txt" read)
     (receive (composed boundary)
         (mime-compose-message-string (list (gen-parts src)))
-      (test* #`"mime-roundtrip (,num)"
+      (test* #"mime-roundtrip (~num)"
              `("multipart/mixed" 0 ,src)
              (mime-message-resolver
               (call-with-input-string composed
                 (cut mime-parse-message <>
                      `(("mime-version" "1.0")
-                       ("content-type" ,#`"multipart/mixed; boundary=\",|boundary|\""))
+                       ("content-type" ,#"multipart/mixed; boundary=\"~|boundary|\""))
                      (cut mime-body->string <> <>)))
               #f)))))
                      
@@ -685,14 +692,14 @@ Content-Length: 4349
 (use rfc.uri)
 (test-module 'rfc.uri)
 
-(test* "encode" "abc%3c%20%3e%20%22%20%23%25%7b%7c%7d%5c%5e"
+(test* "encode" "abc%3C%20%3E%20%22%20%23%25%7B%7C%7D%5C%5E"
        (uri-encode-string "abc< > \" #%{|}\\^"))
-(test* "encode (noescape)" ".a%21%2ap"
+(test* "encode (noescape)" ".a%21%2Ap"
        (uri-encode-string ".a!*p" :noescape *rfc3986-unreserved-char-set*))
 (test* "decode" "abc< > \" #%?{|}\\^"
-       (uri-decode-string "abc%3c%20%3e%20%22%20%23%25%3f%7b%7c%7d%5c%5e"))
+       (uri-decode-string "abc%3c%20%3E%20%22%20%23%25%3f%7B%7C%7d%5c%5e"))
 (test* "decode" "abc<+>+\"+#%?{|}\\^"
-       (uri-decode-string "abc%3c+%3e+%22+%23%25%3f%7b%7c%7d%5c%5e"))
+       (uri-decode-string "abc%3C+%3e+%22+%23%25%3F%7b%7c%7D%5C%5E"))
 (test* "decode" "abc< > \" #%?{|}\\^"
        (uri-decode-string "abc%3c+%3e+%22+%23%25%3f%7b%7c%7d%5c%5e"
                           :cgi-decode #t))
@@ -837,6 +844,21 @@ Content-Length: 4349
   (t "http://example.com"  "../foo" "http://example.com/foo")
   )
 
+;; data uri scheme.  NB: this uses gauche.vport internally (for now),
+;; so it depends on ext/vport.
+
+(let ([d0 "data:text/plain;charset=utf-8,%21%40%23%24%25%5E%26%2A%28%29_abc"]
+      [d1 "data:application/octed-stream;base64,IUAjJCVeJiooKV9hYmM="])
+  (test* "data uri decode->encode (text)" d0
+         (receive (ct data) (uri-decompose-data d0)
+           (cond-expand
+            [gauche.ces.utf8 (uri-compose-data data)]
+            [else (uri-compose-data data :content-type ct)])))
+  (test* "data uri decode->encode (binary)" d1
+         (receive (ct data) (uri-decompose-data d1)
+           (uri-compose-data data :content-type ct)))
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "rfc.http")
 (use rfc.http)
@@ -844,7 +866,7 @@ Content-Length: 4349
 
 (use gauche.parameter)
 
-(test* "http-user-agent" #`"gauche.http/,(gauche-version)"
+(test* "http-user-agent" #"gauche.http/~(gauche-version)"
        (and (is-a? http-user-agent <parameter>)
             (http-user-agent)))
 
@@ -871,7 +893,7 @@ Content-Length: 4349
       (let1 ht (make-hash-table 'string=?)
         (hash-table-put! ht "/redirect01"
                          `("HTTP/1.x 302 Moved Temporarily\n"
-                           ,#`"Location: http://localhost:,|*http-port*|/redirect02\n\n"))
+                           ,#"Location: http://localhost:~|*http-port*|/redirect02\n\n"))
         (hash-table-put! ht "/redirect11"
                          '("HTTP/1.x 302 Moved Temporarily\n"
                            "Location: /redirect12\n\n"))
@@ -900,12 +922,12 @@ Content-Length: 4349
             (let* ([headers (rfc822-read-headers in)]
                    [bodylen
                     (cond [(assoc-ref headers "content-length")
-                           => (lambda (e) (string->number (car e)))]
+                           => (^e (string->number (car e)))]
                           [else 0])]
                    [body (read-block bodylen in)])
               (cond
                [(equal? request-uri "/exit")
-		(socket-close client)
+                (socket-close client)
                 (sys-exit 0)]
                [(hash-table-get %predefined-contents request-uri #f)
                 => (cut for-each (cut display <> out) <>)]
@@ -937,10 +959,10 @@ Content-Length: 4349
 (let ([expected `(("method" "GET")
                   ("request-uri" "/get")
                   ("request-body" "")
-                  ("host" ,#`"localhost:,*http-port*")
-                  ("user-agent" ,#`"gauche.http/,(gauche-version)")
+                  ("host" ,#"localhost:~*http-port*")
+                  ("user-agent" ,#"gauche.http/~(gauche-version)")
                   ("my-header" "foo"))]
-      [host #`"localhost:,*http-port*"])
+      [host #"localhost:~*http-port*"])
   (define (req-body . args)
     (values-ref (apply http-request args) 2))
 
@@ -1016,25 +1038,25 @@ Content-Length: 4349
 
 (test* "http-get (redirect)" "/redirect02"
        (receive (code headers body)
-           (http-request 'GET #`"localhost:,*http-port*" "/redirect01")
+           (http-request 'GET #"localhost:~*http-port*" "/redirect01")
          (cond ((assoc-ref (read-from-string body) "request-uri")
                 => car))))
 
 (test* "http-get (redirect)" "/redirect12"
        (receive (code headers body)
-           (http-request 'GET #`"localhost:,*http-port*" "/redirect11")
+           (http-request 'GET #"localhost:~*http-port*" "/redirect11")
          (cond ((assoc-ref (read-from-string body) "request-uri")
                 => car))))
 
 (test* "http-get (no redirect)" "/redirect12"
        (receive (code headers body)
-           (http-request 'GET #`"localhost:,*http-port*" "/redirect11"
+           (http-request 'GET #"localhost:~*http-port*" "/redirect11"
                          :redirect-handler #f)
          (rfc822-header-ref headers "location")))
 
 (test* "http-get (custom redirect)" "/foofoo"
        (receive (code headers body)
-           (http-request 'GET #`"localhost:,*http-port*" "/redirect11"
+           (http-request 'GET #"localhost:~*http-port*" "/redirect11"
                          :redirect-handler (^[meth code hdrs body]
                                              `(GET . "/foofoo")))
          (cond ((assoc-ref (read-from-string body) "request-uri")
@@ -1042,22 +1064,22 @@ Content-Length: 4349
 
 (test* "http-get (custom redirect to HEAD)" #f
        (receive (code headers body)
-           (http-request 'GET #`"localhost:,*http-port*" "/redirect11"
+           (http-request 'GET #"localhost:~*http-port*" "/redirect11"
                          :redirect-handler (^[meth code hdrs body]
                                              `(HEAD . "/foofoo")))
          body))
 
 (test* "http-get (loop)" (test-error <http-error>)
-       (http-request 'GET #`"localhost:,*http-port*" "/loop1"))
+       (http-request 'GET #"localhost:~*http-port*" "/loop1"))
 
 (test* "http-get (chunked body)" "OK"
        (receive (code headers body)
-           (http-request 'GET #`"localhost:,*http-port*" "/chunked")
+           (http-request 'GET #"localhost:~*http-port*" "/chunked")
          body))
 
 (test* "http-head" #t
        (receive (code headers body)
-           (http-request 'HEAD #`"localhost:,*http-port*" "/")
+           (http-request 'HEAD #"localhost:~*http-port*" "/")
          (and (equal? code "200")
               (equal? headers '(("content-type" "text/plain")))
               (not body))))
@@ -1065,11 +1087,11 @@ Content-Length: 4349
 (let1 expected `(("method" "POST")
                  ("request-uri" "/post")
                  ("content-length" "4")
-                 ("host" ,#`"localhost:,*http-port*")
-                 ("user-agent" ,#`"gauche.http/,(gauche-version)")
+                 ("host" ,#"localhost:~*http-port*")
+                 ("user-agent" ,#"gauche.http/~(gauche-version)")
                  ("request-body" "data"))
   (define (tester msg thunk)
-    (test* #`"http-post ,msg" expected
+    (test* #"http-post ~msg" expected
            (receive (code headers body) (thunk)
              (and (equal? code "200")
                   (equal? headers '(("content-type" "text/plain")))
@@ -1078,17 +1100,17 @@ Content-Length: 4349
 
   (tester "(new API)"
           (lambda ()
-            (http-request 'POST #`"localhost:,*http-port*" "/post"
+            (http-request 'POST #"localhost:~*http-port*" "/post"
                           :sender (http-string-sender "data"))))
 
   (tester "(old API)"
           (lambda ()
-            (http-post #`"localhost:,*http-port*" "/post" "data")))
+            (http-post #"localhost:~*http-port*" "/post" "data")))
   )
 
 (let1 expected '(("a" "b") ("c" "d"))
   (define (tester msg thunk)
-    (test* #`"http-post (multipart/form-data) ,msg" expected
+    (test* #"http-post (multipart/form-data) ~msg" expected
            (receive (code headers body) (thunk)
              
              (and-let* ([ (equal? code "200") ]
@@ -1111,18 +1133,18 @@ Content-Length: 4349
 
   (tester "(new API)"
           (lambda ()
-            (http-request 'POST #`"localhost:,*http-port*" "/post"
+            (http-request 'POST #"localhost:~*http-port*" "/post"
                           :sender
                           (http-multipart-sender '(("a" "b") ("c" "d"))))))
   (tester "(old API)"
           (lambda ()
-            (http-post #`"localhost:,*http-port*" "/post"
+            (http-post #"localhost:~*http-port*" "/post"
                        '(("a" "b") ("c" "d")))))
   )
 
 (test* "<http-error>" #t
        (guard (e (else (is-a? e <http-error>)))
-         (http-request 'GET #`"localhost:,*http-port*" "/exit")))
+         (http-request 'GET #"localhost:~*http-port*" "/exit")))
 
 (sys-waitpid -1)
 
