@@ -33,10 +33,6 @@
        (string->list "abcdefg" 0 1)) ;srfi-13 extension
 (test* "string->list" '() (string->list ""))
 
-;; this should be switched by native encoding.
-;(test* "string w/ char >= \\x80" (integer->char #xa1)
-;       (string-ref (string (integer->char #xa1)) 0))
-
 (test* "string-copy" '("abcde" #f)
        (let* ((x "abcde") (y (string-copy x)))
          (list y (eq? x y))))
@@ -47,6 +43,28 @@
 (define x (string-copy "abcde"))
 (test* "string-set!" "abZde" (begin (string-set! x 2 #\Z) x))
 
+(test* "string w/ char >= \\x80" #\u00a1
+       (string-ref (string #\u00a1) 0))
+
+(test* "string-reader hex-escape" '(1 2 3)
+       (let1 s "\x1;\x2;\x00003;"
+         (map (^i (char->integer (string-ref s i))) '(0 1 2))))
+(test* "string-reader hex-escape legacy support" '(1 2 0 48 48 51)
+       (let1 s "\x01\x02\x00003"
+         (map (^i (char->integer (string-ref s i))) '(0 1 2 3 4 5))))
+(test* "string-reader hex-escape error cases 1 (no hexdigits)"
+       (test-error)
+       (read-from-string "\"\\x;\""))
+(test* "string-reader hex-escape error cases 2 (integer overflow)"
+       (test-error)
+       (read-from-string "\"\\x11111111111111111111111111111111;\""))
+(test* "string-reader hex-escape error cases 3 (out of unicode range)"
+       (test-error)
+       (read-from-string "\"\\x111111;\""))
+(test* "string-reader hex-escape legacy fallback" "\x11;1111"
+       (read-from-string "\"\\x111111\""))
+
+
 (test* "string-fill!" "ZZZZZZ"
        (string-fill! (string-copy "000000") #\Z))
 (test* "string-fill!" "000ZZZ"
@@ -92,6 +110,9 @@
 
   (test-string-scan 3 "abcdefghi" "def")
   (test-string-scan 3 "abcdefghi" "def" 'index)
+  (test-string-scan 6 "abcdefghi" "ghi")
+  (test-string-scan 0 "abcdefghi" "abc")
+  (test-string-scan '(0 6) "abcabcabc" "abc")
   (test-string-scan "abc" "abcdefghi" "def" 'before)
   (test-string-scan "ghi" "abcdefghi" "def" 'after)
   (test-string-scan2 "abc" "defghi" "abcdefghi" "def" 'before*)
@@ -99,6 +120,9 @@
   (test-string-scan2 "abc" "ghi" "abcdefghi" "def" 'both)
 
   (test-string-scan 4 "abcdefghi" #\e)
+  (test-string-scan 8 "abcdefghi" #\i)
+  (test-string-scan 0 "abcdefghi" #\a)
+  (test-string-scan '(0 8) "abcdefgha" #\a)
   (test-string-scan "abcd" "abcdefghi" #\e 'before)
   (test-string-scan "fghi" "abcdefghi" #\e 'after)
   (test-string-scan2 "abcd" "efghi" "abcdefghi" #\e 'before*)
@@ -109,6 +133,9 @@
   (test-string-scan 216
                     "abracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabr"
                     "abracadabra")
+  (test-string-scan '(0 365)
+                    "axaxcadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababaxaxax"
+                    "axax")
 
   ;; results differ between leftmost and rightmost
   (test-string-scan '(1 8) "abracadabra" "br")
@@ -199,6 +226,99 @@
 (test* "string-split (predicate)" '("" "---" "***" "&" "")
        (string-split "aa---bbb***c&d" char-alphabetic?))
 
+(test-section "string-split(with limit)")
+
+(test* "string-split (char)" '("aa*bbb*c**")
+       (string-split "aa*bbb*c**" #\* 0))
+(test* "string-split (char)" '("aa" "bbb*c**")
+       (string-split "aa*bbb*c**" #\* 1))
+(test* "string-split (char)" '("aa" "bbb" "c**")
+       (string-split "aa*bbb*c**" #\* 2))
+
+(test* "string-split (char)" '("aa")
+       (string-split "aa" #\* 1))
+(test* "string-split (char)" '("")
+       (string-split "" #\* 2))
+(test* "string-split (char)" '("" "")
+       (string-split "*" #\* 3))
+(test* "string-split (char)" '("*")
+       (string-split "*" #\* 0))
+
+(test* "string-split (1-char string)" '("aa" "bbb*c")
+       (string-split "aa*bbb*c" "*" 1))
+(test* "string-split (1-char string)" '("aa" "bbb" "c")
+       (string-split "aa*bbb*c" "*" 2))
+
+(test* "string-split (string)" '("aa**bbb**c*c")
+       (string-split "aa**bbb**c*c" "**" 0))
+(test* "string-split (string)" '("aa" "bbb**c*c")
+       (string-split "aa**bbb**c*c" "**" 1))
+(test* "string-split (string)" '("aa" "bbb" "c*c")
+       (string-split "aa**bbb**c*c" "**" 2))
+(test* "string-split (string)" '("aa" "bbb" "c*c")
+       (string-split "aa**bbb**c*c" "**" 3))
+(test* "string-split (string)" '("aa**bbb**c*c")
+       (string-split "aa**bbb**c*c" "--" 2))
+(test* "string-split (string)" '("aa**bbb**c*c**")
+       (string-split "aa**bbb**c*c**" "**" 0))
+(test* "string-split (string)" '("aa" "bbb**c*c**")
+       (string-split "aa**bbb**c*c**" "**" 1))
+(test* "string-split (string)" '("aa" "bbb" "c*c**")
+       (string-split "aa**bbb**c*c**" "**" 2))
+(test* "string-split (string)" '("aa" "bbb" "c*c" "")
+       (string-split "aa**bbb**c*c**" "**" 3))
+(test* "string-split (string)" '("")
+       (string-split "" "**" 0))
+(test* "string-split (string)" '("")
+       (string-split "" "**" 1))
+(test* "string-split (string)" '("**")
+       (string-split "**" "**" 0))
+(test* "string-split (string)" '("" "")
+       (string-split "**" "**" 1))
+
+(test* "string-split (regexp)" '("aa--bbb--c-c")
+       (string-split "aa--bbb--c-c" #/-+/ 0))
+(test* "string-split (regexp)" '("aa" "bbb--c-c")
+       (string-split "aa--bbb--c-c" #/-+/ 1))
+(test* "string-split (regexp)" '("aa" "bbb" "c-c")
+       (string-split "aa--bbb--c-c" #/-+/ 2))
+(test* "string-split (regexp)" '("aa" "bbb" "c" "c")
+       (string-split "aa--bbb--c-c" #/-+/ 3))
+(test* "string-split (regexp)" '("aa" "bbb" "c" "c")
+       (string-split "aa--bbb--c-c" #/-+/ 4))
+(test* "string-split (regexp)" '("aa--bbb---c-c")
+       (string-split "aa--bbb---c-c" #/--/ 0))
+(test* "string-split (regexp)" '("aa" "bbb---c-c")
+       (string-split "aa--bbb---c-c" #/--/ 1))
+(test* "string-split (regexp)" '("aa" "bbb" "-c-c")
+       (string-split "aa--bbb---c-c" #/--/ 2))
+(test* "string-split (regexp)" '("aa" "bbb" "-c-c")
+       (string-split "aa--bbb---c-c" #/--/ 3))
+(test* "string-split (regexp)" '("--aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 0))
+(test* "string-split (regexp)" '("" "aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 1))
+(test* "string-split (regexp)" '("" "aa" "bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 2))
+(test* "string-split (regexp)" '("" "aa" "bbb" "c-c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 3))
+(test* "string-split (regexp)" '("" "aa" "bbb" "c" "c-")
+       (string-split "--aa--bbb---c-c-" #/-+/ 4))
+(test* "string-split (regexp)" '("--aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 0))
+(test* "string-split (regexp)" '("--" "--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 1))
+(test* "string-split (regexp)" '("--" "--" "---c-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 2))
+(test* "string-split (regexp)" '("--" "--" "---" "-c-")
+       (string-split "--aa--bbb---c-c-" #/\w+/ 3))
+(test* "string-split (regexp)" '("--aa--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/z+/ 2))
+(test* "string-split (regexp)" (test-error) ;; test detection of infinite loop
+       (string-split "--aa--bbb---c-c-" #/-*/ 2))
+(test* "string-split (bad limit)" (test-error)
+       (string-split "--aa--bbb---c-c-" #/-+/ 'a))
+
 ;;-------------------------------------------------------------------
 (test-section "incomplete strings")
 
@@ -454,7 +574,7 @@
 
 (define (string-port-tester . args)
   (let ((out (open-output-string)))
-    (for-each (lambda (s) (display s out)) args)
+    (for-each (^s (display s out)) args)
     (get-output-string out)))
 
 (define (test-string-port signature total seg)
@@ -500,16 +620,16 @@
 
 (test* "string interpolation" "string interpolation"
        (let ((x "inter") (y "polation"))
-         #`"string ,|x|,|y|"))
+         #"string ~|x|~|y|"))
 (test "string interpolation" "string interpolation"
       (lambda ()
         (define (x) "inter")
         (define (y) "polation")
-        #`"string ,(x),(y)"))
+        #"string ~(x)~(y)"))
 (test "string interpolation" "string interpolation"
       (lambda ()
         (define (x a)
           (if a "inter" "polation"))
-        #`"string ,(x #t),(x #f)"))
+        #"string ~(x #t)~(x #f)"))
 
 (test-end)
