@@ -6,87 +6,91 @@
 (test-start "treemap")
 
 ;; Basic stuff
-(let1 tree1 #f
-  (test* "make-tree-map" #t
-         (begin (set! tree1 (make-tree-map = <))
-                (tree-map? tree1)))
-  (test* "tree-map-get" (test-error)
-         (tree-map-get #f 0 'foo))
-  (test* "tree-map-get" 'not-found
-         (tree-map-get tree1 0 'not-found))
-  (test* "tree-map-get" (test-error)
-         (tree-map-get tree1 0))
-  (test* "tree-map-put!" (test-error)
-         (tree-map-put! #f 0 'foo))
-  (test* "tree-map-put!" "0"
-         (begin (tree-map-put! tree1 0 "0")
-                (tree-map-get tree1 0)))
-  (test* "tree-map-put!" '("0" "1")
-         (begin (tree-map-put! tree1 1 "1")
-                (list (tree-map-get tree1 0)
-                      (tree-map-get tree1 1))))
-  (test* "tree-map-put!" 'bar
-         (begin (tree-map-put! tree1 2 'foo)
-                (tree-map-put! tree1 2 'bar)
-                (tree-map-get tree1 2)))
-  '(test* "tree-map-check" #t
-         (tree-map-check tree1))
-  (test* "tree-map-fold" '(2 bar 1 "1" 0 "0")
-         (tree-map-fold tree1 list* '()))
-  (test* "tree-map-fold-right" '(0 "0" 1 "1" 2 bar)
-         (tree-map-fold-right tree1 list* '()))
-  (test* "tree-map-delete! (exiting key)" '(#t not-found)
-         (let1 r (tree-map-delete! tree1 1)
-           (list r (tree-map-get tree1 1 'not-found))))
-  (test* "tree-map-delete! (non-existing key)" #f
-         (tree-map-delete! tree1 1))
-  (test* "tree-map-delete!" 'no-error
-         (begin (tree-map-delete! tree1 1)
-                'no-error))
-  (test* "tree-map->alist" '()
-         (tree-map->alist (make-tree-map = <)))
-  (test* "tree-map->alist" '((0 . "0") (1 . "1") (2 . "2"))
-         (let1 tree (make-tree-map = <)
-           (for-each (lambda (p) (tree-map-put! tree (car p) (cdr p)))
-                     '((0 . "0") (1 . "1") (2 . "2")))
-           (tree-map->alist tree)))
-  (test* "alist->tree-map" '((0 . "0") (1 . "1") (2 . "2"))
-         (tree-map->alist
-          (alist->tree-map '((0 . "0") (1 . "1") (2 . "2")) = <)))
-  (test* "tree-map-empty?" #f
-         (tree-map-empty? tree1))
-  (test* "tree-map-empty?" #t
-         (tree-map-empty? (make-tree-map = <)))
-  (test* "tree-map-empty?" (test-error)
-         (tree-map-empty? 'wrong-arg))
-  (test* "tree-map-exists?" '(#t #f)
-         (let1 tree (make-tree-map = <)
-           (tree-map-put! tree 1 'foo)
-           (map (cut tree-map-exists? tree <>)
-                '(1 2))))
-  (test* "tree-map-num-entries" '(0 1 0)
-         (let* ((t (make-tree-map = <))
-                (a (tree-map-num-entries t))
-                (b (begin (tree-map-put! t 7 7)
-                          (tree-map-num-entries t)))
-                (c (begin (tree-map-delete! t 7)
-                          (tree-map-num-entries t))))
-           (list a b c)))
-  (test* "tree-map-push!" '(bar foo)
-         (let1 tree (make-tree-map = <)
-           (tree-map-push! tree 1 'foo)
-           (tree-map-push! tree 1 'bar)
-           (tree-map-get tree 1)))
-  (test* "tree-map-pop!" '(foo bar)
-         (let1 tree (alist->tree-map '((1 foo bar)) = <)
-           (let1 r (tree-map-pop! tree 1)
-             (list r (tree-map-pop! tree 1)))))
-  (test* "tree-map-update!" 2
-         (let1 tree (make-tree-map = <)
-           (tree-map-update! tree 1 (cut + 1 <>) 0)
-           (tree-map-update! tree 1 (cut + 1 <>) 0)
-           (tree-map-get tree 1)))
-  )
+(define (do-tree-map ctor)
+  (let1 tree1 #f
+    (test* "make-tree-map" #t
+           (begin (set! tree1 (ctor)) (tree-map? tree1)))
+    (test* "tree-map-get" (test-error)
+           (tree-map-get #f 0 'foo))
+    (test* "tree-map-get" 'not-found
+           (tree-map-get tree1 0 'not-found))
+    (test* "tree-map-get" (test-error)
+           (tree-map-get tree1 0))
+    (test* "tree-map-put!" (test-error)
+           (tree-map-put! #f 0 'foo))
+    (test* "tree-map-put!" "0"
+           (begin (tree-map-put! tree1 0 "0")
+                  (tree-map-get tree1 0)))
+    (test* "tree-map-put!" '("0" "1")
+           (begin (tree-map-put! tree1 1 "1")
+                  (list (tree-map-get tree1 0)
+                        (tree-map-get tree1 1))))
+    (test* "tree-map-put!" 'bar
+           (begin (tree-map-put! tree1 2 'foo)
+                  (tree-map-put! tree1 2 'bar)
+                  (tree-map-get tree1 2)))
+    '(test* "tree-map-check" #t
+            (tree-map-check tree1))
+    (test* "tree-map-fold" '(2 bar 1 "1" 0 "0")
+           (tree-map-fold tree1 list* '()))
+    (test* "tree-map-fold-right" '(0 "0" 1 "1" 2 bar)
+           (tree-map-fold-right tree1 list* '()))
+    (test* "tree-map-delete! (exiting key)" '(#t not-found)
+           (let1 r (tree-map-delete! tree1 1)
+             (list r (tree-map-get tree1 1 'not-found))))
+    (test* "tree-map-delete! (non-existing key)" #f
+           (tree-map-delete! tree1 1))
+    (test* "tree-map-delete!" 'no-error
+           (begin (tree-map-delete! tree1 1)
+                  'no-error))
+    (test* "tree-map->alist" '()
+           (tree-map->alist (ctor)))
+    (test* "tree-map->alist" '((0 . "0") (1 . "1") (2 . "2"))
+           (let1 tree (ctor)
+             (for-each (^p (tree-map-put! tree (car p) (cdr p)))
+                       '((0 . "0") (1 . "1") (2 . "2")))
+             (tree-map->alist tree)))
+    (test* "alist->tree-map" '((0 . "0") (1 . "1") (2 . "2"))
+           (tree-map->alist
+            (alist->tree-map '((0 . "0") (1 . "1") (2 . "2")) = <)))
+    (test* "tree-map-empty?" #f
+           (tree-map-empty? tree1))
+    (test* "tree-map-empty?" #t
+           (tree-map-empty? (ctor)))
+    (test* "tree-map-empty?" (test-error)
+           (tree-map-empty? 'wrong-arg))
+    (test* "tree-map-exists?" '(#t #f)
+           (let1 tree (ctor)
+             (tree-map-put! tree 1 'foo)
+             (map (cut tree-map-exists? tree <>)
+                  '(1 2))))
+    (test* "tree-map-num-entries" '(0 1 0)
+           (let* ((t (ctor))
+                  (a (tree-map-num-entries t))
+                  (b (begin (tree-map-put! t 7 7)
+                            (tree-map-num-entries t)))
+                  (c (begin (tree-map-delete! t 7)
+                            (tree-map-num-entries t))))
+             (list a b c)))
+    (test* "tree-map-push!" '(bar foo)
+           (let1 tree (ctor)
+             (tree-map-push! tree 1 'foo)
+             (tree-map-push! tree 1 'bar)
+             (tree-map-get tree 1)))
+    (test* "tree-map-pop!" '(foo bar)
+           (let1 tree (alist->tree-map '((1 foo bar)) = <)
+             (let1 r (tree-map-pop! tree 1)
+               (list r (tree-map-pop! tree 1)))))
+    (test* "tree-map-update!" 2
+           (let1 tree (ctor)
+             (tree-map-update! tree 1 (cut + 1 <>) 0)
+             (tree-map-update! tree 1 (cut + 1 <>) 0)
+             (tree-map-get tree 1)))
+    ))
+
+(do-tree-map (cut make-tree-map = <))
+(do-tree-map (cut make-tree-map (^[a b] (cond [(< a b) -1][(= a b) 0][else 1]))))
+(do-tree-map (cut make-tree-map))
 
 ;; Min, max, iterators
 (let ((empty (make-tree-map = <))
@@ -383,18 +387,18 @@
   (define (get-val key) (if (string? key) (cdr (assoc key alist)) key))
 
   (define (test-key-val name key expected-key both-proc key-proc val-proc)
-    (test* #`",name (,key)" (list expected-key (get-val expected-key))
+    (test* #"~name (~key)" (list expected-key (get-val expected-key))
            (receive (k v) (both-proc tree key) (list k v)))
-    (test* #`",|name|-key (,key)" expected-key
+    (test* #"~|name|-key (~key)" expected-key
            (key-proc tree key))
-    (test* #`",|name|-value (,key)" (get-val expected-key)
+    (test* #"~|name|-value (~key)" (get-val expected-key)
            (val-proc tree key))
     (unless expected-key
-      (test* #`",name (,key) / fallback" (list 0 1)
+      (test* #"~name (~key) / fallback" (list 0 1)
              (receive (k v) (both-proc tree key 0 1) (list k v)))
-      (test* #`",|name|-key (,key) / fallback" 0
+      (test* #"~|name|-key (~key) / fallback" 0
              (key-proc tree key 0))
-      (test* #`",|name|-value (,key) / fallback" 0
+      (test* #"~|name|-value (~key) / fallback" 0
              (val-proc tree key 0))))
 
   (define (tester key floor ceil pred succ)
@@ -422,10 +426,10 @@
 
 (let1 tree (make-tree-map = <)
   (test* "tree-map-pop-min! and num-entries" 0 
-	 (begin
-	   (tree-map-put! tree 0 0)
-	   (tree-map-pop-min! tree)
-	   (tree-map-num-entries tree))))
+         (begin
+           (tree-map-put! tree 0 0)
+           (tree-map-pop-min! tree)
+           (tree-map-num-entries tree))))
 
 (test-end)
 
