@@ -84,47 +84,47 @@
 (test* "permutations-for-each"
        '()
        (let1 r '()
-         (permutations-for-each (lambda (p) (push! r p)) '())
+         (permutations-for-each (^p (push! r p)) '())
          (reverse r)))
 (test* "permutations-for-each"
        '((a))
        (let1 r '()
-         (permutations-for-each (lambda (p) (push! r p)) '(a))
+         (permutations-for-each (^p (push! r p)) '(a))
          (reverse r)))
 (test* "permutations-for-each"
        '((a b c) (a c b) (b a c) (b c a) (c a b) (c b a))
        (let1 r '()
-         (permutations-for-each (lambda (p) (push! r p)) '(a b c))
+         (permutations-for-each (^p (push! r p)) '(a b c))
          (reverse r)))
 (test* "permutations*-for-each"
        '()
        (let1 r '()
-         (permutations*-for-each (lambda (p) (push! r p)) '())
+         (permutations*-for-each (^p (push! r p)) '())
          (reverse r)))
 (test* "permutations*-for-each"
        '((a))
        (let1 r '()
-         (permutations*-for-each (lambda (p) (push! r p)) '(a))
+         (permutations*-for-each (^p (push! r p)) '(a))
          (reverse r)))
 (test* "permutations*-for-each"
        '((a b c) (a c b) (b a c) (b c a) (c a b) (c b a))
        (let1 r '()
-         (permutations*-for-each (lambda (p) (push! r p)) '(a b c))
+         (permutations*-for-each (^p (push! r p)) '(a b c))
          (reverse r)))
 (test* "permutations*-for-each"
        '((a a b) (a b a) (b a a))
        (let1 r '()
-         (permutations*-for-each (lambda (p) (push! r p)) '(a a b))
+         (permutations*-for-each (^p (push! r p)) '(a a b))
          (reverse r)))
 (test* "permutations*-for-each"
        '((a a a))
        (let1 r '()
-         (permutations*-for-each (lambda (p) (push! r p)) '(a a a))
+         (permutations*-for-each (^p (push! r p)) '(a a a))
          (reverse r)))
 (test* "permutations*-for-each"
        '(("a" "a" "b") ("a" "b" "a") ("b" "a" "a"))
        (let1 r '()
-         (permutations*-for-each (lambda (p) (push! r p)) '("a" "a" "b")
+         (permutations*-for-each (^p (push! r p)) '("a" "a" "b")
                                  string=?)
          (reverse r)))
 
@@ -170,80 +170,80 @@
 
 (test* "combinations-for-each" '(())
        (let1 r '()
-         (combinations-for-each (lambda (c) (push! r c)) '() 0)
+         (combinations-for-each (^c (push! r c)) '() 0)
          (reverse! r)))
 (test* "combinations-for-each" '((a))
        (let1 r '()
-         (combinations-for-each (lambda (c) (push! r c)) '(a) 1)
+         (combinations-for-each (^c (push! r c)) '(a) 1)
          (reverse! r)))
 (test* "combinations-for-each" '((a) (b) (c) (d))
        (let1 r '()
-         (combinations-for-each (lambda (c) (push! r c)) '(a b c d) 1)
+         (combinations-for-each (^c (push! r c)) '(a b c d) 1)
          (reverse! r)))
 (test* "combinations-for-each" '((a b) (a c) (b c))
        (let1 r '()
-         (combinations-for-each (lambda (c) (push! r c)) '(a b c) 2)
+         (combinations-for-each (^c (push! r c)) '(a b c) 2)
          (reverse! r)))
 (test* "combinations-for-each" '((a b c))
        (let1 r '()
-         (combinations-for-each (lambda (c) (push! r c)) '(a b c) 3)
+         (combinations-for-each (^c (push! r c)) '(a b c) 3)
          (reverse! r)))
 (test* "combinations-for-each" '((a b c) (a b d) (a c d) (b c d))
        (let1 r '()
-         (combinations-for-each (lambda (c) (push! r c)) '(a b c d) 3)
+         (combinations-for-each (^c (push! r c)) '(a b c d) 3)
          (reverse! r)))
 
 (test* "combinations*-for-each" '(())
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '() 0)
+         (combinations*-for-each (^c (push! r c)) '() 0)
          (reverse! r)))
 (test* "combinations*-for-each" '((a))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a) 1)
+         (combinations*-for-each (^c (push! r c)) '(a) 1)
          (reverse! r)))
 (test* "combinations*-for-each" '((a) (b) (c) (d))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b c d) 1)
+         (combinations*-for-each (^c (push! r c)) '(a b c d) 1)
          (reverse! r)))
 (test* "combinations*-for-each" '((a b) (a c) (b c))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b c) 2)
+         (combinations*-for-each (^c (push! r c)) '(a b c) 2)
          (reverse! r)))
 (test* "combinations*-for-each" '((a b c))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b c) 3)
+         (combinations*-for-each (^c (push! r c)) '(a b c) 3)
          (reverse! r)))
 (test* "combinations*-for-each" '((a b c) (a b d) (a c d) (b c d))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b c d) 3)
+         (combinations*-for-each (^c (push! r c)) '(a b c d) 3)
          (reverse! r)))
 (test* "combinations*-for-each" '((a) (b))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a a b) 1)
+         (combinations*-for-each (^c (push! r c)) '(a a b) 1)
          (reverse! r)))
 (test* "combinations*-for-each" '((a a) (a b))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a a b) 2)
+         (combinations*-for-each (^c (push! r c)) '(a a b) 2)
          (reverse! r)))
 (test* "combinations*-for-each" '((a a b))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a a b) 3)
+         (combinations*-for-each (^c (push! r c)) '(a a b) 3)
          (reverse! r)))
 (test* "combinations*-for-each" '((a b) (a a))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b a a) 2)
+         (combinations*-for-each (^c (push! r c)) '(a b a a) 2)
          (reverse! r)))
 (test* "combinations*-for-each" '((a b a) (a a a))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b a a) 3)
+         (combinations*-for-each (^c (push! r c)) '(a b a a) 3)
          (reverse! r)))
 (test* "combinations*-for-each" '((a b b) (a b a))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '(a b b a) 3)
+         (combinations*-for-each (^c (push! r c)) '(a b b a) 3)
          (reverse! r)))
 (test* "combinations*-for-each" '(("a" "b" "b") ("a" "b" "a"))
        (let1 r '()
-         (combinations*-for-each (lambda (c) (push! r c)) '("a" "b" "b" "a") 3
+         (combinations*-for-each (^c (push! r c)) '("a" "b" "b" "a") 3
                                  string=?)
          (reverse! r)))
 
@@ -272,32 +272,32 @@
 
 (test* "power-set-for-each" '(())
        (let1 r '()
-         (power-set-for-each (lambda (s) (push! r s)) '())
+         (power-set-for-each (^s (push! r s)) '())
          (reverse! r)))
 (test* "power-set-for-each" '(() (a))
        (let1 r '()
-         (power-set-for-each (lambda (s) (push! r s))  '(a))
+         (power-set-for-each (^s (push! r s))  '(a))
          (reverse! r)))
 (test* "power-set-for-each" '(() (a) (b) (c) (a b) (a c) (b c) (a b c))
        (let1 r '()
-         (power-set-for-each (lambda (s) (push! r s))  '(a b c))
+         (power-set-for-each (^s (push! r s))  '(a b c))
          (reverse! r)))
 
 (test* "power-set*-for-each" '(())
        (let1 r '()
-         (power-set*-for-each (lambda (s) (push! r s)) '())
+         (power-set*-for-each (^s (push! r s)) '())
          (reverse! r)))
 (test* "power-set*-for-each" '(() (a))
        (let1 r '()
-         (power-set*-for-each (lambda (s) (push! r s))  '(a))
+         (power-set*-for-each (^s (push! r s))  '(a))
          (reverse! r)))
 (test* "power-set*-for-each" '(() (a) (b) (a a) (a b) (a a b))
        (let1 r '()
-         (power-set*-for-each (lambda (s) (push! r s))  '(a a b))
+         (power-set*-for-each (^s (push! r s))  '(a a b))
          (reverse! r)))
 (test* "power-set*-for-each" '(() ("a") ("b") ("a" "a") ("a" "b") ("a" "a" "b"))
        (let1 r '()
-         (power-set*-for-each (lambda (s) (push! r s))  '("a" "a" "b")
+         (power-set*-for-each (^s (push! r s))  '("a" "a" "b")
                               string=?)
          (reverse! r)))
 
@@ -468,7 +468,7 @@
          (rbtree->alist (make-rbtree = <)))
   (test* "rbtree->alist" '((0 . "0") (1 . "1") (2 . "2"))
          (let1 tree (make-rbtree = <)
-           (for-each (lambda (p) (rbtree-put! tree (car p) (cdr p)))
+           (for-each (^p (rbtree-put! tree (car p) (cdr p)))
                      '((0 . "0") (1 . "1") (2 . "2")))
            (rbtree->alist tree)))
   (test* "alist->rbtree" '((0 . "0") (1 . "1") (2 . "2"))
@@ -534,7 +534,7 @@
 
   (test* "for-each" (rbtree->alist tree2)
          (let1 r '()
-           (for-each (lambda (e) (push! r e)) tree2)
+           (for-each (^e (push! r e)) tree2)
            (reverse! r)))
 
   (test* "map" (rbtree->alist tree2)
@@ -777,7 +777,7 @@
 (test* "stream-map/stream-ref" '(2 4 6 8)
        (let1 s
            (stream-map (cut * 2 <>)
-                       (stream-unfoldn (lambda (s) (values (+ s 1) (list s)))
+                       (stream-unfoldn (^s (values (+ s 1) (list s)))
                                        0 1))
          (map (cut stream-ref s <>) '(1 2 3 4))))
 
@@ -790,7 +790,7 @@
          (reverse r)))
 
 (test* "stream-filter/take/drop" '(12 15 18 21)
-       (let1 s (stream-filter (lambda (x) (zero? (modulo x 3)))
+       (let1 s (stream-filter (^x (zero? (modulo x 3)))
                               (stream-iota #f))
          (stream->list (stream-take (stream-drop s 4) 4))))
 
@@ -887,6 +887,15 @@
          (topological-sort (map (cut map symbol->string <>) input) string=?))
   )
 
+;; cycle
+(test* "topological-sort: error" (test-error)
+       (topological-sort '((watch tie)
+                           (tie watch))))
+(test* "topological-sort: error" (test-error)
+       (topological-sort '((shirt watch)
+                           (watch tie)
+                           (tie watch))))
+
 ;;-----------------------------------------------
 (test-section "util.trie")
 (use util.trie)
@@ -931,7 +940,7 @@
     (test* "trie: exists? more" #t
            (every (cut trie-exists? t1 <>) strs))
     (test* "trie: common-prefix" '(19 12 8 4 4 3)
-           (map (lambda (p) (length (trie-common-prefix t1 p)))
+           (map (^p (length (trie-common-prefix t1 p)))
                 '("" "k" "ka" "ku" "li" "lili")))
     (test* "trie: common-prefix" '(("kua" . 3)
                                       ("kua`aina" . 8)
@@ -970,7 +979,7 @@
                  0 strs)
            (let1 c 0 (trie-for-each t1 (lambda (k v) (inc! c v))) c))
     (test* "trie: trie->list"
-           (map (lambda (s) (cons s (string-length s))) strs)
+           (map (^s (cons s (string-length s))) strs)
            (trie->list t1)
            (cut lset= equal? <> <>))
     (test* "trie: trie-keys"
@@ -1042,7 +1051,7 @@
                       (lambda (t f s) (fold f s t)))
     (test* "trie(custom): put!" (- (length strs) 1)
            (begin
-             (for-each (lambda (s) (trie-put! t5 s (string-length s))) strs)
+             (for-each (^s (trie-put! t5 s (string-length s))) strs)
              (trie-num-entries t5)))
     (test* "trie(custom): get" 99
            (begin
